Package: SimuRg
Title: Building, Fitting and Evaluating PK/PD Modeles
Version: 0.2.0
Authors@R: c(
    person("Victor", "Sokolov", , "victor.sokolov@msdecisions.tech", role =  c("cph", "aut")),
    person("Anna", "Mikhailova", , "anna.mikhailova@msdecisions.tech", role = "cre"),
    person("Yaroslav", "Ugolkov", , "yaroslav.ugolkov@msdecisions.tech", role = "aut"),
    person("Anatoly", "Pokladyuk", , "anatoly.pokladyuk@msdecisions.tech", role = "aut"),
    person("Alina", "Melnikova", , "alina.melnikova@msdecisions.tech", role = "aut"),
    person("Victoria", "Kulesh", , "victoria.kulesh@msdecisions.tech", role = "aut")
  )
Description: Provides a unified workflow for building, fitting using external engines, and evaluating ordinary differential equation (ODE)-based pharmacokinetic/pharmacodynamic (PK/PD) models. Supports generation of estimation scenarios and control files for external engines (e.g., 'Monolix'), simulation of models using 'rxode2', and creation of goodness-of-fit diagnostics. Includes tools for covariate modeling, virtual population design, and local and global sensitivity analyses.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: GGally, testthat (>= 3.0.0)
Imports: cluster, dplyr, fastDummies, forcats, ggplot2, jsonlite, MASS,
        philentropy, purrr, readr, recipes, rlang, rxode2, scales,
        stringr, synthpop, sys, tibble, tidyr, uwot, zoo, lhs,
        sensitivity, ppcor, withr
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-15 11:15:10 UTC; MikhailovaAnna
Author: Victor Sokolov [cph, aut],
  Anna Mikhailova [cre],
  Yaroslav Ugolkov [aut],
  Anatoly Pokladyuk [aut],
  Alina Melnikova [aut],
  Victoria Kulesh [aut]
Maintainer: Anna Mikhailova <anna.mikhailova@msdecisions.tech>
Repository: CRAN
Date/Publication: 2026-05-19 09:40:10 UTC
Built: R 4.5.2; ; 2026-05-19 10:36:54 UTC; unix

Package: SimuRg
Title: Building, Fitting and Evaluating PK/PD Modeles
Version: 0.2.0
Authors@R: c(
    person("Victor", "Sokolov", , "victor.sokolov@msdecisions.tech", role =  c("cph", "aut")),
    person("Anna", "Mikhailova", , "anna.mikhailova@msdecisions.tech", role = "cre"),
    person("Yaroslav", "Ugolkov", , "yaroslav.ugolkov@msdecisions.tech", role = "aut"),
    person("Anatoly", "Pokladyuk", , "anatoly.pokladyuk@msdecisions.tech", role = "aut"),
    person("Alina", "Melnikova", , "alina.melnikova@msdecisions.tech", role = "aut"),
    person("Victoria", "Kulesh", , "victoria.kulesh@msdecisions.tech", role = "aut")
  )
Description: Provides a unified workflow for building, fitting using external engines, and evaluating ordinary differential equation (ODE)-based pharmacokinetic/pharmacodynamic (PK/PD) models. Supports generation of estimation scenarios and control files for external engines (e.g., 'Monolix'), simulation of models using 'rxode2', and creation of goodness-of-fit diagnostics. Includes tools for covariate modeling, virtual population design, and local and global sensitivity analyses.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: GGally, testthat (>= 3.0.0)
Imports: cluster, dplyr, fastDummies, forcats, ggplot2, jsonlite, MASS,
        philentropy, purrr, readr, recipes, rlang, rxode2, scales,
        stringr, synthpop, sys, tibble, tidyr, uwot, zoo, lhs,
        sensitivity, ppcor, withr
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-15 11:15:10 UTC; MikhailovaAnna
Author: Victor Sokolov [cph, aut],
  Anna Mikhailova [cre],
  Yaroslav Ugolkov [aut],
  Anatoly Pokladyuk [aut],
  Alina Melnikova [aut],
  Victoria Kulesh [aut]
Maintainer: Anna Mikhailova <anna.mikhailova@msdecisions.tech>
Repository: CRAN
Date/Publication: 2026-05-19 09:40:10 UTC
Built: R 4.6.0; ; 2026-05-19 10:44:00 UTC; unix

# Generated by roxygen2: do not edit by hand

export(read_smrg_obj)
export(sg_converter)
export(sg_covsens_sim)
export(sg_covsens_vis)
export(sg_fit)
export(sg_globalsens_sim)
export(sg_globalsens_vis)
export(sg_gof_obpr)
export(sg_gof_par_cov)
export(sg_gof_par_dist)
export(sg_gof_res)
export(sg_gof_res_dist)
export(sg_gof_tp)
export(sg_localsens_sim)
export(sg_localsens_vis)
export(sg_modbuild)
export(sg_modcomp)
export(sg_multistart)
export(sg_parsum)
export(sg_predist_sim)
export(sg_sim)
export(sg_sim_tp)
export(sg_translator)
export(sg_vpc_sim)
export(sg_vpc_vis)
export(sg_vpop_est)
import(dplyr)
import(ggplot2)
import(ppcor)
import(purrr)
import(rxode2)
import(stringr)
import(sys)
import(tibble)
import(tidyr)
importFrom(MASS,mvrnorm)
importFrom(cluster,daisy)
importFrom(fastDummies,dummy_cols)
importFrom(forcats,fct_inorder)
importFrom(grDevices,recordPlot)
importFrom(graphics,abline)
importFrom(graphics,box)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(grid,unit)
importFrom(jsonlite,fromJSON)
importFrom(lhs,randomLHS)
importFrom(philentropy,distance)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(readr,cols)
importFrom(readr,parse_number)
importFrom(readr,read_csv)
importFrom(readr,read_tsv)
importFrom(readr,write_csv)
importFrom(recipes,bake)
importFrom(recipes,prep)
importFrom(recipes,recipe)
importFrom(recipes,step_center)
importFrom(recipes,step_dummy)
importFrom(recipes,step_scale)
importFrom(rlang,.data)
importFrom(rlang,expr)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(scales,math_format)
importFrom(scales,number_format)
importFrom(scales,pretty_breaks)
importFrom(scales,trans_breaks)
importFrom(scales,trans_format)
importFrom(sensitivity,fast99)
importFrom(sensitivity,tell)
importFrom(stats,lm)
importFrom(stats,runif)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(synthpop,syn)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(uwot,umap)
importFrom(uwot,umap_transform)
importFrom(zoo,na.locf)

# SimuRg

## Overview

**SimuRg** provides a comprehensive workflow for non-linear mixed-effects model
development in pharmacometrics. The package provides the entire modeling pipeline:
from model calibration with Monolix fitter(2023) and output processing to goodness-of-fit 
visualization, simulation, and sensitivity analysis. To use Monolix, it should 
be installed. 

Key features:
- **Model calibration** via the Monolix fitter
- **Automated output conversion** into a generalized fit output
- **Diagnostic visualization** for model assessment
- **Simulation capabilities** for model predictions
- **Sensitivity analysis** tools for parameter exploration

## Installation

### From CRAN
```r
install.packages("SimuRg")
```

## Illustrated example

First of all, the model should be calibrated with Monolix fitter. For this goal,
Monolix should be installed on the computer. As this software have commercial 
license, we start our example with the conversion from the Monolix output files 
into the generalized fit output.

```r
library("SimuRg")
library(stringr)
# Convert Monolix project results
test_folder <- system.file("extdata", "Monolix_objects", package = "SimuRg")
if (substr(test_folder, nchar(test_folder), nchar(test_folder)) != "/")
   test_folder <- str_c(test_folder, "/")
pro_name <- "proj-solo"
result <- sg_converter(folder_path = test_folder, proj_name = pro_name)

# Running goodness-of-fit objects
sg_gof_obpr(result)
sg_gof_res(
  fpath_i = result,
  res_type = "IWRES",
  vs_time = TRUE
)
sg_gof_par_cov(
  fpath_i = result,
  ptype = "IndParvsCov",
  cont_cov = cont_cov,
  cat_cov = cat_cov
)
sg_gof_par_dist(fpath_i = result)
sg_gof_res_dist(fpath_i = result, res_type = "IWRES")
sg_gof_res(
  fpath_i = result,
  res_type = "IWRES",
  vs_time = TRUE
)
```

## Author: Ugolkov Yaroslav
## First created: 2026-02-27
## Description: RxODE example model code
## Keywords: SimuRg, model, RxODE

mod_ex <- RxODE({
  # Doses in mg
  # Time in hours

  ### Parameter values
  # Typical
  POPCL = 5;
  POPVC = 180;
  POPQ = 7;
  POPVP = 52;
  POPKTR = 6;
  FBIOPAR = 1;

  # Covariate coefficients
  POPIGFRCOV = 0.8;
  POPPATCOVCLGR1 = 0.85;
  POPPATCOVCLGR2 = 0.85;
  POPPATCOVCLGR3 = 0.85;
  POPPATCOVCLGR4 = 0.85;

  # Random effects
  PPVCL = 0;
  PPVVC = 0;
  PPVKTR = 0;

  # Residual error
  RUV = 0;

  ### Covariates
  PATCOVCL = 1
  if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
  if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
  if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
  if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
  IGFRCOV = (IGFR/112)^POPIGFRCOV

  ### Parameters
  CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
  VC = POPVC * exp(PPVVC);
  Q = POPQ;
  VP = POPVP;
  KTR = POPKTR * exp(PPVKTR);

  ### Explicit functions
  Cc = Ac/VC;                 # nmol/L
  Cp = Ap/VP;                 # nmol/L

  ### Initial conditions
  At1(0) = 0;         # mg
  At2(0) = 0;         # mg
  At3(0) = 0;         # mg
  At4(0) = 0;         # mg
  At5(0) = 0;         # mg
  At6(0) = 0;         # mg
  Ad(0) = 0;          # mg
  Ac(0) = 0;          # mg
  Ap(0) = 0;          # mg

  ### ODEs
  d/dt(At1) = - KTR*At1;
  d/dt(At2) = KTR*At1 - KTR*At2;
  d/dt(At3) = KTR*At2 - KTR*At3;
  d/dt(At4) = KTR*At3 - KTR*At4;
  d/dt(At5) = KTR*At4 - KTR*At5;
  d/dt(At6) = KTR*At5 - KTR*At6;
  d/dt(Ad) = KTR*At6 - KTR*Ad;
  d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
  d/dt(Ap) = Q*(Cc - Cp);

  FBIO = FBIOPAR
  f(At1) = FBIO*1000000/505;
  CHECKRUV = RUV;
  Cc_ResErr = Cc + RUV*Cc;
})

## Author: Alina Melnikova
## First created: 2025-07-17
## Description: Covariate simulation from original dataset
## Keywords: covariates, simulations
## Version: v1.1 - add JS and KL metrics, covariate matrix, merge umap


# Comparison of correlation matrices
compare_cor_matrices <- function(data_obs, data_syn, vars, method = "kendall") {

  R_obs <- cor(data_obs[, vars, drop = FALSE],
               use = "pairwise.complete.obs",
               method = method)

  R_syn <- cor(data_syn[, vars, drop = FALSE],
               use = "pairwise.complete.obs",
               method = method)

  idx <- upper.tri(R_obs, diag = FALSE)

  # mean_abs_diff = mean(abs(R_obs[idx] - R_syn[idx]))
  # return(mean_abs_diff )
   list(
     mean_abs_diff = mean(abs(R_obs[idx] - R_syn[idx])),
     max_abs_diff  = max(abs(R_obs[idx] - R_syn[idx])),
  #   #frobenius     = sqrt(sum((R_obs - R_syn)^2)),
     R_obs = R_obs,
     R_syn = R_syn,
    R_diff = R_syn - R_obs
  )
}

# Create optimal visit sequence based on correlations

create_optimal_visit_sequence <- function(data, var_cont, var_cat) {
  var_all <- c(var_cont, var_cat)

  # Handle edge cases
  if (length(var_all) == 0) {
    warning("No variables provided for visit sequence")
    return(character(0))
  }

  if (length(var_cont) == 0 && length(var_cat) > 0) {
    # Only categorical variables, return as-is
    return(var_cat)
  }

  if (length(var_cont) < 2) {
    # If too few continuous variables, use default order
    return(var_all)
  }

  # Calculate correlation matrix for continuous variables
  if (length(var_cont) >= 2) {
    cor_matrix <- cor(data[, var_cont, drop = FALSE],
                      use = "pairwise.complete.obs")
    # Replace NA with 0 (for variables with no variance)
    cor_matrix[is.na(cor_matrix)] <- 0

    # Calculate average absolute correlation for each variable
    avg_abs_corr <- rowMeans(abs(cor_matrix))
    names(avg_abs_corr) <- var_cont

    # Order continuous variables by average correlation (highest first)
    var_cont_ordered <- names(sort(avg_abs_corr, decreasing = TRUE))
  } else {
    var_cont_ordered <- var_cont
  }

  # Combine: continuous (ordered) + categorical
  # Handle both single and multiple categorical variables
  if (length(var_cat) > 0) {
    visit_seq <- c(var_cont_ordered, var_cat)
  } else {
    visit_seq <- var_cont_ordered
  }

  return(visit_seq)
}

# Remove exact duplicates between synthetic and original data by adding noise
remove_exact_duplicates <- function(data_syn, data_orig, var_cont, var_cat,
                                     noise_level = 0.10, seed = 123) {

  # Find exact duplicates between synthetic and original
  common_cols <- intersect(names(data_syn), names(data_orig))
  data_syn$row_id <- seq_len(nrow(data_syn))

  # Identify which synthetic rows are exact duplicates of original rows
  dupl_indices <- data_syn %>%
    dplyr::semi_join(data_orig, by = common_cols) %>%
    pull(row_id)

  n_duplicates <- length(dupl_indices)

  if (n_duplicates == 0) {
    data_syn$row_id <- NULL
    return(list(
      data_cleaned = data_syn,
      n_duplicates_removed = 0,
      duplicate_indices = integer(0)
    ))
  }

  message("Found", n_duplicates, "exact duplicates. Adding noise to remove them...\n")

  # Add noise to continuous variables for duplicate rows
  if (length(var_cont) > 0) {
    for (var in var_cont) {
      if (var %in% names(data_syn)) {
        # Calculate SD from original data
        sd_val <- sd(data_orig[[var]], na.rm = TRUE)

        # Add noise only to duplicate rows
        if (is.null(seed)) {
          noise <- rnorm(n_duplicates, mean = 0, sd = noise_level * sd_val)
        } else {
          withr::with_seed(seed, {
            noise <- rnorm(n_duplicates, mean = 0, sd = noise_level * sd_val)
          })
        }
        data_syn[dupl_indices, var] <- data_syn[dupl_indices, var] + noise

        # Ensure values stay within reasonable bounds (min/max of original)
        min_val <- min(data_orig[[var]], na.rm = TRUE)
        max_val <- max(data_orig[[var]], na.rm = TRUE)
        data_syn[dupl_indices, var] <- pmin(pmax(data_syn[dupl_indices, var], min_val), max_val)
      }
    }
  }

  # For binary/categorical variables, randomly flip a small proportion
  if (length(var_cat) > 0) {
    for (var in var_cat) {
      if (var %in% names(data_syn)) {
        levels_var <- levels(data_syn[[var]])
        n_levels <- length(levels_var)

        # Only perturb if there are at least 2 levels
        if (n_levels >= 2) {
          # Randomly select ~20% of duplicate rows to flip for this variable
          n_to_flip <- max(1, ceiling(0.2 * n_duplicates))
          if (is.null(seed)) {
            rows_to_flip <- sample(dupl_indices, size = n_to_flip)
          } else {
            withr::with_seed(seed, {
              rows_to_flip <- sample(dupl_indices, size = n_to_flip)
            })
          }

          # For each selected row, change to a different level
          for (idx in rows_to_flip) {
            current_level <- as.character(data_syn[idx, var])
            other_levels <- setdiff(levels_var, current_level)
            if (length(other_levels) > 0) {
              data_syn[idx, var] <- factor(sample(other_levels, 1), levels = levels_var)
            }
          }
        }
      }
    }
  }

  data_syn$row_id <- NULL

  return(list(
    data_cleaned = data_syn,
    n_duplicates_removed = n_duplicates,
    duplicate_indices = dupl_indices
  ))
}

#' Perform generation of synthetic datasets for an empirical distribution
#'
#' The function operates in two modes:
#' \itemize{
#'   \item \strong{Fixed seed mode} (when \code{seed} is specified): Generates a single dataset using the provided seed
#'   \item \strong{Search mode} (when \code{seed = NA}): Iteratively searches for \code{nds} datasets that meet the correlation difference threshold
#' }
#'
#' @returns A list of lists, where each element contains results for one generated dataset:
#' \itemize{
#'   \item \code{datagen} - Synthetic dataset returned by \code{synthpop::syn()} (data.frame)
#'   \item \code{seed} - Random seed used to generate this dataset (integer or NA)
#'   \item \code{exact_dupl_check} - Logical indicating if exact duplicates exist between original and synthetic data
#'   \item \code{dplot_umap} - ggplot object with combined UMAP visualization comparing original and synthetic data (or \code{NULL} if diag_plots=FALSE)
#'   \item \code{ks_test} - Tibble with Kolmogorov-Smirnov p-values and statuses for continuous variables (or \code{NULL} if no continuous variables)
#'   \item \code{jsd_res} - Weighted mean Jensen-Shannon divergence (JSD) value for categorical variables (numeric or \code{NULL} if no categorical variables)
#'   \item \code{corr_diff_mean} - Mean absolute difference between original and synthetic correlation matrices (numeric or \code{NULL} if no continuous variables)
#'   \item \code{corr_diff_max} - Maximum absolute difference between original and synthetic correlation matrices (numeric or \code{NULL} if no continuous variables)
#'   \item \code{dplot_corr_diff} - ggplot heatmap object showing correlation difference matrix (Synthetic - Original) (or \code{NULL} if diag_plots=FALSE or no continuous variables)
#'   \item \code{dplot_cont} - List of ggplot histograms for continuous variables (or \code{NULL} if diag_plots=FALSE or no continuous variables)
#'   \item \code{dplot_cat} - List of ggplot barplots for categorical variables (or \code{NULL} if diag_plots=FALSE or no categorical variables)
#' }
#' @inheritParams sg_dummy
#' @param data_i data frame. Input data frame containing the original dataset to be synthesized (required)
#' @param nobj integer. Specify the exact number of rows to generate in the synthetic dataset. When provided, overrides \code{npop} (optional, default: \code{NA})
#' @param minnumlev integer. Threshold; numeric variables with <= \code{minnumlev} unique values are converted to factors (optional, default: \code{3})
#' @param seed integer. Random seed for synthetic data generation. If provided (not \code{NA}), generates a single dataset with this seed (fixed seed mode). If \code{NA}, uses search mode to find \code{nds} datasets meeting correlation threshold (optional, default: \code{NA})
#' @param seed_umap integer. Random seed for UMAP algorithm reproducibility (optional, default: \code{123})
#' @param palette character vector. Contains color codes (hex format) for custom plot color schemes. If provided, should contain at least 2 colors. Used for histograms, bar plots, and UMAP visualizations (optional, default: \code{c("#3a6eba", "#efdd3c", "#1a1866", "#f2b93b")})
#' @param diag_plots logical flag. If \code{TRUE}, generates diagnostic plots and UMAP visualizations (optional, default: \code{TRUE})
#' @param remove_duplicates logical flag. If \code{TRUE}, automatically removes exact duplicates between original and synthetic data by adding controlled noise (optional, default: \code{TRUE})
#' @param noise_level numeric. Proportion of standard deviation to use when adding noise to continuous variables for duplicate removal. E.g., 0.10 means 10% of SD (optional, default: \code{0.10})
#' @param nds integer. Number of synthetic datasets to generate in search mode. Ignored in fixed seed mode (optional, default: \code{1})
#' @param tg_corrdif numeric. Target maximum absolute correlation difference threshold for dataset selection in search mode. Ignored in fixed seed mode (optional, default: \code{0.1})
#' @examples
#' \donttest{
#' library(dplyr)
#'
#' # Generate example dataset
#' data <- data.frame(
#'   id = 1:150,
#'   ALB = rnorm(150, mean = 3.5, sd = 0.5),
#'   ALT = rnorm(150, mean = 50, sd = 20),
#'   SEX = factor(sample(c("M", "F"), 150, replace = TRUE)),
#'   RACE = factor(sample(c("White", "Black", "Asian", "Other"), 150, replace = TRUE))
#' )
#'
#' # EXAMPLE 1: Fixed seed mode - generate single dataset with specified seed
#' output_fixed <- sg_vpop_est(data_i = data,
#'                             id_col = "id",
#'                             seed = 123,        # Fixed seed mode
#'                             seed_umap = 40,
#'                             diag_plots = TRUE)
#'
#' # Access the dataset and its diagnostics
#' print(head(output_fixed[[1]]$datagen, 10))
#' print(output_fixed[[1]]$ks_test)
#' print(output_fixed[[1]]$seed)           # Will be 123
#' print(output_fixed[[1]]$corr_diff_max)
#' print(output_fixed[[1]]$dplot_umap)
#'
#' # EXAMPLE 2: Search mode - generate 3 datasets meeting correlation threshold
#' output_search <- sg_vpop_est(data_i = data,
#'                              id_col = "id",
#'                              seed = NA,        # Search mode (default)
#'                              seed_umap = 40,
#'                              diag_plots = TRUE,
#'                              nds = 3,          # Generate 3 datasets
#'                              tg_corrdif = 0.1) # Target correlation difference
#'
#' # Compare metrics across all datasets
#' sapply(output_search, function(x) x$corr_diff_max)
#' sapply(output_search, function(x) x$jsd_res)
#' sapply(output_search, function(x) x$seed)  # Different seeds for each dataset
#'
#'}
#' @import dplyr
#' @import ggplot2
#' @importFrom purrr map_dfr map
#' @importFrom stringr str_c
#' @importFrom cluster daisy
#' @importFrom fastDummies dummy_cols
#' @importFrom synthpop syn
#' @importFrom uwot umap umap_transform
#' @importFrom tidyr drop_na
#' @importFrom tibble tibble as_tibble
#' @importFrom recipes recipe step_dummy step_center step_scale prep bake
#' @importFrom philentropy distance
#' @export
sg_vpop_est <-  function(data_i, nobj = NA, id_col = NULL, minnumlev = 3,npop = 1,
                       excl_col = NULL,seed = NA,
                       seed_umap = 123, palette = NULL,
                       diag_plots = FALSE,
                       #show_info=FALSE,
                       remove_duplicates = TRUE,
                       noise_level = 0.10,
                       nds = 1,#number of datasets to generate
                       tg_corrdif = 0.1){

  theme_set(theme_bw())
  theme_update(panel.grid.minor = element_blank())
  MSDcol_cut <- c("#3a6eba","#efdd3c", "#1a1866", "#b73b58")

  if(!is.null(palette) & length(palette)>1){
    color_p = rep(palette,2)
  } else {color_p = MSDcol_cut
    message("Default color palette is used")}

  #Control parameters for Random Forest (optimized for correlation preservation)
  num_trees = 300          # Increased from 100 for better stability
  max_depth = 12           # Increased from 7 to capture more interactions
  min_node_size = 5        # Decreased from 30 for finer splits
  #use_smoothing = TRUE     # Enable smoothing for continuous variables
  optimize_visit_seq = TRUE # Use correlation-based visit sequence

  ### Comment!!!! - Testing code below (comment out for production)
  # data_orig <- read.csv("functions/datasets/data_gbsg2.csv") %>% head(300)  # Change to your dataset path
  # data_i = data_orig; nobj = NA; id_col = NULL; minnumlev = 3;npop = 1
  # excl_col = NULL;
  # seed = 98;
  # seed_umap = 42; palette = NULL;
  # diag_plots = F; remove_duplicates = TRUE;
  # noise_level = 0.10; tg_corrdif = 0.1; nds = 5

  #####--------------- Processing ---------------#####
  #Exclude ID column and Exclusion columns

  if (!is.null(id_col)){
    if (!(id_col %in% names(data_i))){
      warning("`id_col` = '", id_col, "' is not present in `data`.")
    }
    data_i <- data_i %>%
      dplyr::select(-any_of(id_col))
  }

  if (!is.null(excl_col)){
    missing_excl <- setdiff(excl_col, names(data_i))
    if (length(missing_excl) > 0){
      warning("The following `excl_col` values are not present in `data`: ",
              paste(missing_excl, collapse = ", "))
    }
    data_i <- data_i %>%
      dplyr::select(-any_of(excl_col))
  }


  #Exclude rows with NAs.
  data_i <- data_i %>%
    drop_na()

  var_char = NULL

  # Convert character columns to factor
  is_char  <- vapply(data_i, is.character, logical(1))
  var_char  <- names(data_i)[is_char]

  if (!is.na(minnumlev)){minnum = minnumlev} else {minnum = 3}

  # Convert numeric variables with <= minnumlev unique values to factor
  is_low_level_num <- vapply(
    data_i,
    function(x) is.numeric(x) && length(unique(na.omit(x))) <= minnum,
    logical(1)
  )
  var_low_level <- names(data_i)[is_low_level_num]


  data_i <- data_i %>%
    mutate(across(any_of(c(var_char, var_low_level)), as.factor))

  is_cont <- vapply(data_i, is.numeric, logical(1))
  is_cat  <- vapply(data_i, is.factor, logical(1))

  # Define names for continuous and categorical (ordered/factor) columns
  var_cont <- names(data_i)[is_cont]
  var_cat  <- names(data_i)[is_cat]
  var_all <- c(var_cont, var_cat)



  n_orig <- nrow(data_i)


  # if (show_info){
  # # Print dataset summary
  #  message("Number of rows in the data_orig:", nrow(data_i), "\n")
  #  message("Number and names of continuous columns - ", length(var_cont), ":", paste(var_cont, collapse = ", "), "\n")
  #  message("Number and names of categorical columns - ", length(var_cat), ":", paste(var_cat, collapse = ", "), "\n")
  #}
  data_orig <- data_i

  # Ensure data_i is a tibble to satisfy synthpop list-argument requirements
  data_i <- dplyr::as_tibble(data_i)

  # Convert all integer columns to numeric to prevent smoothing/casting errors
  # (Smoothing produces continuous values, which can fail if cast back to integer)
  data_i <- data_i %>% mutate(across(where(is.integer), as.numeric))

  if (!is.na(nobj)){n_new = nobj} else if(!is.na(npop)&(npop>0)){
    n_new = as.integer(round(n_orig * npop))
    } else {n_new = n_orig}

  #Synthpop Method
  # if (!is.na(seed)){seed_i = seed} else {seed_i = 123}
  # message("Seed for synthetic data generation:", seed_i, "\n")

  # Create optimal visit sequence based on correlations (if enabled)
  if (optimize_visit_seq && length(var_cont) >= 2) {
    message("Creating correlation-based visit sequence...\n")
    visit_sequence <- create_optimal_visit_sequence(data_i, var_cont, var_cat)
  } else {
    # Use default order: all variables in their original order
    visit_sequence <- var_all
  }

  # Prepare method vector: "rf" for all variables
  method_vector <- rep("rf", length(var_all))
  names(method_vector) <- var_all



  message(sprintf("RF parameters: trees = %.2f, max_depth = %.2f, , min_node_size = %.2f\n", num_trees, max_depth,
      min_node_size))
  message("Visit sequence:", paste(visit_sequence, collapse = ", "), "\n")
  message("Variables to synthesize:", length(visit_sequence),
      "(", length(var_cont), "continuous,", length(var_cat), "categorical )\n")


  # Generate synthetic data with optimized parameters
  # Pass RF parameters directly to syn() function

  # Initialize results list
  results_list <- list()

  # Check mode: fixed seed or search mode
  if (!is.na(seed)) {
    # === MODE 1: Fixed seed mode ===
    message("\n=== Fixed seed mode: generating dataset with seed = ", seed, "===\n")
    mode_fixed_seed <- TRUE
    n_datasets <- 1
  } else {
    # === MODE 2: Search mode (original logic) ===
    message("\n=== Search mode: generating", nds, "dataset(s) with optimal seeds ===\n")
    mode_fixed_seed <- FALSE
    n_datasets <- nds

    # Initialize search parameters
    n_iterations <- 100
    diff_lim <- tg_corrdif
    seed_values <- 1:n_iterations
    i_init <- 0
    i <- 0
    res_seeds <- c()
  }

  # Main loop for dataset generation and diagnostics
  for (k in 1:n_datasets) {

    # === GENERATION PHASE ===
    if (mode_fixed_seed) {
      # Fixed seed mode: generate single dataset with specified seed
      message("\n=== Generating dataset with fixed seed =", seed, "===\n")

      syn_obj <- syn(data_i,
                     method = method_vector,
                     visit.sequence = visit_sequence,
                     #smoothing = smoothing_vector,
                     k = n_new,
                     seed = seed,
                     ranger.num.trees = num_trees,
                     ranger.max.depth = max_depth,
                     ranger.min.node.size = min_node_size,
                     ranger.respect.unordered.factors = "order")

      data_syn <- syn_obj$syn
      target_seed <- seed

    } else {
      # Search mode: iterate through seeds until target corr_diff is achieved
      message("\n=== Generating dataset", k, "of", n_datasets, "===\n")

      corr_diff_contr <- diff_lim + 0.05
      p <- 0

      while ((corr_diff_contr > diff_lim) & (p < n_iterations)) {
        i <- i + 1
        p <- p + 1
        syn_obj <- syn(data_i,
                       method = method_vector,
                       visit.sequence = visit_sequence,
                       #smoothing = smoothing_vector,
                       k = n_new,
                       seed = seed_values[p],
                       ranger.num.trees = num_trees,
                       ranger.max.depth = max_depth,
                       ranger.min.node.size = min_node_size,
                       ranger.respect.unordered.factors = "order")
        data_syn <- syn_obj$syn
        corr_diff <- compare_cor_matrices(data_i, data_syn, var_cont, method = "kendall")
        corr_diff_contr <- corr_diff$max_abs_diff
      }

      target_seed <- seed_values[[p]]

      if ((p == n_iterations) & (corr_diff_contr > diff_lim)) {
        message("Target correlation coefficient difference is not obtained\n")
        target_seed <- NA
      }

      res_seeds <- c(res_seeds, target_seed)
      message("Dataset", k, "generated with seed:", target_seed, "\n")

      # Update seed range for next iteration
      i_init <- i
      seed_values <- (i_init + 1):(n_iterations + i_init)
    }

    # === DIAGNOSTICS PHASE (common for both modes) ===
    message("Computing diagnostics for dataset", k, "...\n")

    # Check for duplicates *between* real and synthetic data (not within each)
    common_cols <- intersect(names(data_i), names(data_syn))
    dupl_check_before <- nrow(dplyr::semi_join(data_syn, data_i, by = common_cols)) > 0
    n_dupl_removed <- 0

    if (dupl_check_before){
      #warning("Exact duplicates found between original and synthetic data.")

      # Remove duplicates if requested
      if (remove_duplicates) {
        dedup_result <- remove_exact_duplicates(
          data_syn = data_syn,
          data_orig = data_i,
          var_cont = var_cont,
          var_cat = var_cat,
          noise_level = noise_level,
          #seed = seed_i
        )

        data_syn <- dedup_result$data_cleaned
        n_dupl_removed <- dedup_result$n_duplicates_removed

        # Check again after deduplication
        dupl_check_after <- nrow(dplyr::semi_join(data_syn, data_i, by = common_cols)) > 0

        if (dupl_check_after) {
          message("Warning: Some duplicates may still remain after noise addition.\n")
        } else {
          message("Successfully removed all", n_dupl_removed, "exact duplicates.\n")
        }
      }
    }

    dupl_check <- nrow(dplyr::semi_join(data_syn, data_i, by = common_cols)) > 0

    # Initialize diagnostic variables
    ks_results <- NULL
    corr_diff <- NULL
    JSD_results <- NULL

    if (length(var_cont)>0){
      # Get Kolmogorov-Smirnov test
      # ks_results <- map_dfr(var_cont, function(var) {
      #   x <- data_i[[var]]
      #   y <- data_syn[[var]]
      #
      #   # Detect ties across combined samples; ks.test assumes continuous data
      #   has_ties <- any(duplicated(c(x, y)))
      #
      #   # Suppress the default ks.test warning about ties and emit a custom one instead
      #   ks_result <- suppressWarnings(ks.test(x, y))
      #   if (has_ties) {
      #     warning(
      #       "Kolmogorov-Smirnov test for variable '", var,
      #       "' may be approximate due to ties in the data",
      #       call. = FALSE
      #     )
      #   }
      #   p_val <- ks_result$p.value
      #
      #   # Format p-value as character to ensure consistent type for bind_rows
      #   if (p_val < 0.01) {
      #     p_val_formatted <- "<0.01"
      #   } else {
      #     p_val_formatted <- as.character(round(p_val, 4))
      #   }
      #
      #   tibble(
      #     variable = var,
      #     p.value = p_val_formatted,
      #     status = ifelse(p_val > 0.05, "similar", "different")
      #   )
      # })

      ks_results <- suppressWarnings({
        map_dfr(var_cont, function(var) {
          x <- data_i[[var]]
          y <- data_syn[[var]]

          # Detect ties across combined samples; ks.test assumes continuous data
          has_ties <- any(duplicated(c(x, y)))

          # Suppress the default ks.test warning about ties and emit a custom one instead
          ks_result <- ks.test(x, y)
          if (has_ties) {
            warning(
              "Kolmogorov-Smirnov test for variable '", var,
              "' may be approximate due to ties in the data",
              call. = FALSE
            )
          }
          p_val <- ks_result$p.value

          # Format p-value as character to ensure consistent type for bind_rows
          if (p_val < 0.01) {
            p_val_formatted <- "<0.01"
          } else {
            p_val_formatted <- as.character(round(p_val, 4))
          }

          tibble(
            variable = var,
            p.value = p_val_formatted,
            status = ifelse(p_val > 0.05, "similar", "different")
          )
        })
      })



      #Comparison of correlation matrices (requires at least 2 variables)
      if (length(var_cont) > 1) {
        corr_diff <- compare_cor_matrices(data_i, data_syn, var_cont, method = "kendall")
      }
    }

    if (length(var_cat)>0){
      n_levels <- c()
      # Jensen-Shannon (JS) divergence
      jsd_by_var <- sapply(var_cat, function(var) {

        # Align category levels
        lev <- union(
          levels(factor(data_i[[var]])),
          levels(factor(data_syn[[var]]))
        )
        # Empirical PMFs
        p <- prop.table(table(factor(data_i[[var]], levels = lev)))
        q <- prop.table(table(factor(data_syn[[var]], levels = lev)))

        # Small epsilon to avoid log(0)
        eps <- 1e-8
        p <- p + eps
        q <- q + eps
        p <- p / sum(p)
        q <- q / sum(q)

        # Jensen-Shannon divergence (obs || syn)
        as.numeric(
          distance(rbind(p, q), method = "jensen-shannon")
        )
      })

      jsd <- as.vector(jsd_by_var)
      names(jsd) <- var_cat
      n_levels <- sapply(var_cat, function(var) {
        length(levels(factor(data_i[[var]])))
      })
      names(n_levels) <- var_cat
      # Calculate weighted mean of Jensen-Shannon divergence
      JSD_results<- sum(jsd * n_levels / sum(n_levels))
    }

    # Preparation for plots
    plist_cont = NULL
    plist_cat = NULL
    plot_umap = NULL
    p_corr_heatmap = NULL

    if (diag_plots){
      rec <- recipe(~ ., data = data_i)
      if (length(var_cat)>0){
        # recipe for preprocessing
        rec <- rec %>%
          step_dummy(all_of(var_cat), one_hot = TRUE)
      }
      if (length(var_cont)>0){
        # recipe for preprocessing
        rec <- rec %>%
          step_center(all_of(var_cont)) %>%
          step_scale(all_of(var_cont)) %>%
          prep()
      }

      X_real <- bake(rec, new_data = data_i)
      X_syn  <- bake(rec, new_data = data_syn)
      # UMAP (fit on real data only)



      message("Seed for UMAP:", seed_umap, "\n")

      umap_model <- uwot::umap(
        X_real,
        n_neighbors = 15,
        min_dist = 0.1,
        metric = "euclidean",
        ret_model = TRUE,
        ret_nn = TRUE,
        seed = seed_umap
      )

      # project synthetic data
      emb_real <- umap_model$embedding
      emb_syn  <- uwot::umap_transform(X_syn, umap_model, seed = seed_umap)

      # Create combined visualization
      umap_df <- rbind(
        data.frame(X = emb_real[, 1], Y = emb_real[, 2], Source = "Original data"),
        data.frame(X = emb_syn[, 1],  Y = emb_syn[, 2],  Source = "Synthetic data")
      )

      # Create plot
      plot_umap <- ggplot(umap_df, aes(x = X, y = Y, color = Source)) +
        geom_point(alpha = 0.4, size = 1) +
        scale_color_manual(values = c("Original data" = color_p[[3]], "Synthetic data" = color_p[[4]])) +
        labs(title = paste0("UMAP: Dataset ", k),
             x = "UMAP Dimension 1", y = "UMAP Dimension 2", color = "Dataset") +
        theme_minimal() +
        theme(legend.position = "bottom")

      if (length(var_cont)>0){
        #Get Continuous histograms
        plist_cont <- var_cont %>% map(function(var){
          # Create data frame for plotting
          plot_data <- data.frame(
            value = c(data_i[[var]], data_syn[[var]]),
            source = rep(c("Original", "Synthetic"),
                         times = c(length(data_i[[var]]), length(data_syn[[var]])))
          )

          # Create ggplot histogram
          ggplot(plot_data, aes(x = value, fill = source)) +
            geom_histogram(alpha = 0.6, bins = 30, position = "identity",
                           color = "black", linewidth = 0.3) +
            scale_fill_manual(
              values = c("Original" = color_p[[1]], "Synthetic" = color_p[[2]])
            ) +
            labs(title = paste(var, "Distribution"),
                 x = var,
                 y = "Frequency",
                 fill = "Dataset") +
            theme(legend.position = "topright")
        })

        #Heat map for corr_diff R_diff (using ggplot2 for better visualization)
        # Only create heatmap if there are at least 2 continuous variables
        if (length(var_cont) > 1 && !is.null(corr_diff)) {
        corr_diff_df <- as.data.frame(as.table(corr_diff$R_diff))
        colnames(corr_diff_df) <- c("Var1", "Var2", "R_diff")

        p_corr_heatmap <- ggplot(corr_diff_df,
                                 aes(x = Var2, y = Var1, fill = R_diff)) +
          geom_tile(color = "grey80") +
          geom_text(aes(label = round(abs(R_diff), 2)),
                    color = "black", size = 3.5)  +
          scale_fill_gradient2(
            low = "blue", mid = "white", high = "red", midpoint = 0,
            limits = c(-1, 1),
            oob = scales::squish,
            name = "R_diff"
          ) +
          labs(
            title = paste0("Correlation Difference (Dataset ", k, ")"),
            x = "Variable",
            y = "Variable"
          ) +
          theme_minimal() +
          theme(
            axis.text.x = element_text(angle = 45, hjust = 1),
            legend.position = "right"
          )
        }
      }

      if (length(var_cat)>0){
        # Get barplot table for categorical data
        plist_cat <- var_cat %>% map(function(var){
          # Create tables for original and synthetic data
          orig_table <- table(data_i[[var]])
          synth_table <- table(data_syn[[var]])

          # Convert to data frame for ggplot
          plot_data <- data.frame(
            category = rep(names(orig_table), 2),
            count = c(as.numeric(orig_table), as.numeric(synth_table)),
            source = rep(c("Original", "Synthetic"), each = length(orig_table))
          )

          # Create ggplot barplot
          ggplot(plot_data, aes(x = category, y = count, fill = source)) +
            geom_col(position = "dodge", color = "black", linewidth = 0.3) +
            scale_fill_manual(values = c("Original" = color_p[[3]],
                                         "Synthetic" = color_p[[4]])) +
            labs(title = paste(var, "Distribution"),
                 x = var,
                 y = "Count",
                 fill = "Dataset") +
            theme(legend.position = "topright",
                  axis.text.x = element_text(angle = 45, hjust = 1))
        })
      }

      if (length(var_cat) == 0){message("No categorical variables to plot\n")}
      if (length(var_cont) == 0){message("No continuous variables to plot\n")}
    }

    # Store dataset and its diagnostics
    results_list[[k]] <- list(
      datagen = data_syn,
      seed = target_seed,
      exact_dupl_check = dupl_check,
      dplot_cont = plist_cont,
      dplot_cat = plist_cat,
      dplot_umap = plot_umap,
      ks_test = ks_results,
      jsd_res = JSD_results,
      corr_diff_mean = if(!is.null(corr_diff)) corr_diff$mean_abs_diff else NULL,
      corr_diff_max = if(!is.null(corr_diff)) corr_diff$max_abs_diff else NULL,
      dplot_corr_diff = p_corr_heatmap
    )

    message("Dataset", k, "diagnostics completed.\n")
  }

  # Final summary message
  if (mode_fixed_seed) {
    message("\n=== Dataset generated with fixed seed =", seed, "===\n")
  } else {
    message("\n=== All", nds, "datasets generated and diagnosed ===\n")
  }

  return(results_list)

}

## Author: Melnikova Alina
## First created: 2025-11-24
## Description: formal testing of sg-vpop-est function and its helpers
## Keywords: SimuRg, sg-vpop-est

test_that("sg-vpop-est file load works", {

  # fpath_i <- system.file("data", "data_pbc.rda", package = "SimuRg")
  # load(fpath_i)
  output <- sg_vpop_est(data_i = data_pbc, diag_plots = TRUE, id_col = "id",
                        excl_col = "years", seed = 123)

  # Output is now a list of lists
  expect_type(output, "list")
  expect_length(output, 1)  # Default generates 1 dataset

  # Check first dataset structure
  result <- output[[1]]
  expect_true(inherits(result$datagen, "data.frame"))
  expect_true(is.numeric(result$seed) || is.na(result$seed))
  expect_true(is.logical(result$exact_dupl_check))

  # Check plots
  expect_true(
    is.null(result$dplot_cont) ||
      (is.list(result$dplot_cont) &&
         all(vapply(result$dplot_cont, function(x) inherits(x, "ggplot"), logical(1))))
  )
  expect_true(
    is.null(result$dplot_cat) ||
      (is.list(result$dplot_cat) &&
         all(vapply(result$dplot_cat, function(x) inherits(x, "ggplot"), logical(1))))
  )
  expect_true(inherits(result$dplot_umap, "ggplot"))
  expect_true(
    is.null(result$ks_test) || inherits(result$ks_test, "data.frame")
  )
  expect_true(
    is.null(result$dplot_corr_diff) || inherits(result$dplot_corr_diff, "ggplot")
  )
})

test_that("sg-vpop-est does not work on empty dataset", {
  expect_error(sg_vpop_est(data.frame()))
})


# Basic functionality tests
test_that("sg_vpop_est returns correct structure with simple continuous data", {
  # Create larger test data to avoid duplicate warnings
  #skip_if(T)
  set.seed(42)
  # test_data <- data.frame(
  #   x1 = rnorm(200, mean = 10, sd = 2),
  #   x2 = rnorm(200, mean = 20, sd = 3)
  # )
  # fpath_i <- system.file("data", "data_pbc.rda", package = "SimuRg")
  # load(fpath_i)
  test_data <- data_pbc %>% select(years, age)

  output <-
    sg_vpop_est(data_i = test_data, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  # Check output structure - now a list of lists
  expect_type(output, "list")
  expect_length(output, 1)

  result <- output[[1]]
  expect_named(result, c("datagen", "seed", "exact_dupl_check", "dplot_cont",
                         "dplot_cat", "dplot_umap", "ks_test", "jsd_res",
                         "corr_diff_mean", "corr_diff_max", "dplot_corr_diff"))

  # Check datagen
  expect_true(is.data.frame(result$datagen))
  expect_equal(ncol(result$datagen), ncol(test_data))
  expect_equal(sort(colnames(result$datagen)), sort(colnames(test_data)))

  # Check seed
  expect_equal(result$seed, 123)

  # Check exact_dupl_check
  expect_true(is.logical(result$exact_dupl_check))

  # Check ks_test
  expect_true(is.data.frame(result$ks_test))
  expect_true("variable" %in% colnames(result$ks_test))
  expect_true("p.value" %in% colnames(result$ks_test))
  expect_true("status" %in% colnames(result$ks_test))

  # Check correlation diff metrics
  expect_true(is.numeric(result$corr_diff_mean))
  expect_true(is.numeric(result$corr_diff_max))

  # JSD should be NULL for continuous-only data
  expect_null(result$jsd_res)
})

test_that("sg_vpop_est works with mixed continuous and categorical data", {
  # Create larger test data with both continuous and categorical variables
  #skip_if(T)
  set.seed(43)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3),
    cat1 = factor(sample(c("A", "B", "C", "D"), 200, replace = TRUE)),
    cat2 = factor(sample(c("X", "Y", "Z"), 200, replace = TRUE))
  )

  output <-
    sg_vpop_est(data_i = test_data, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_true(is.data.frame(result$datagen))
  expect_equal(ncol(result$datagen), ncol(test_data))

  # Check that JSD is computed for categorical variables
  expect_true(is.numeric(result$jsd_res))

  # Check that correlation diff is computed for continuous variables
  expect_true(is.numeric(result$corr_diff_mean))
  expect_true(is.numeric(result$corr_diff_max))
})

test_that("sg_vpop_est respects nobj parameter", {
  #skip_if(T)
  set.seed(44)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  output <-
    sg_vpop_est(data_i = test_data, nobj = 100, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_equal(nrow(result$datagen), 100)
})

test_that("sg_vpop_est respects npop parameter", {
  #skip_if(T)
  set.seed(45)
  test_data <- data.frame(
    x1 = rnorm(150, mean = 10, sd = 2),
    x2 = rnorm(150, mean = 20, sd = 3)
  )

  output <-
    sg_vpop_est(data_i = test_data, npop = 2, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_equal(nrow(result$datagen), 300)
})

test_that("sg_vpop_est excludes idcol when specified", {
  #skip_if(T)
  set.seed(46)
  test_data <- data.frame(
    id = 1:200,
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  output <-
    sg_vpop_est(data_i = test_data, id_col = "id", diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_false("id" %in% colnames(result$datagen))
  expect_equal(ncol(result$datagen), 2)
})

test_that("sg_vpop_est excludes exclcol when specified", {
  #skip_if(T)
  set.seed(47)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3),
    exclude_me = rnorm(200, mean = 5, sd = 1)
  )

  output <-
    sg_vpop_est(data_i = test_data, excl_col = "exclude_me",
                diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_false("exclude_me" %in% colnames(result$datagen))
  expect_equal(ncol(result$datagen), 2)
})

test_that("sg_vpop_est uses seed for reproducibility", {
  #skip_if(T)
  set.seed(48)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  output1 <-
    sg_vpop_est(data_i = test_data, seed = 123, diag_plots = FALSE, remove_duplicates = TRUE)

  output2 <-
    sg_vpop_est(data_i = test_data, seed = 123, diag_plots = FALSE, remove_duplicates = TRUE)


  # Results should be identical with same seed
  expect_equal(output1[[1]]$datagen, output2[[1]]$datagen)
  expect_equal(output1[[1]]$seed, 123)
  expect_equal(output2[[1]]$seed, 123)
})

test_that("sg_vpop_est works with diag_plots = FALSE", {
  #skip_if(T)
  set.seed(49)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  output <-
    sg_vpop_est(data_i = test_data, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_null(result$dplot_cont)
  expect_null(result$dplot_cat)
  expect_null(result$dplot_umap)
  expect_null(result$dplot_corr_diff)
  expect_true(is.data.frame(result$datagen))
  expect_true(is.data.frame(result$ks_test))
})

test_that("sg_vpop_est works with diag_plots = TRUE", {
  #skip_if(T)
  test_data <- data.frame(
    x1 = rnorm(300, mean = 10, sd = 2),
    x2 = rnorm(300, mean = 20, sd = 3)
  )

  output <- sg_vpop_est(data_i = test_data, diag_plots = TRUE, seed = 123, seed_umap = 123)

  result <- output[[1]]
  expect_true(
    is.null(result$dplot_cont) ||
      (is.list(result$dplot_cont) &&
         all(vapply(result$dplot_cont, function(x) inherits(x, "ggplot"), logical(1))))
  )
  expect_true(
    is.null(result$dplot_cat) ||
      (is.list(result$dplot_cat) &&
         all(vapply(result$dplot_cat, function(x) inherits(x, "ggplot"), logical(1))))
  )
  expect_true(inherits(result$dplot_umap, "ggplot"))
  expect_true(inherits(result$dplot_corr_diff, "ggplot") || is.null(result$dplot_corr_diff))
})

test_that("sg_vpop_est converts character columns to factors", {
  #skip_if(T)
  set.seed(50)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    char_col = sample(c("A", "B", "C", "D", "E"), 200, replace = TRUE),
    stringsAsFactors = FALSE
  )

  output <-
    sg_vpop_est(data_i = test_data, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  # The synthetic data should have the same structure
  expect_true(is.data.frame(result$datagen))
  expect_true("char_col" %in% colnames(result$datagen))

  # Check that JSD is computed for categorical variable
  expect_true(is.numeric(result$jsd_res))
})

test_that("sg_vpop_est handles minnumlev parameter", {
  #skip_if(T)
  # Create data with numeric column that has few unique values
  set.seed(51)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    low_level = sample(c(1, 2, 3, 4), 200, replace = TRUE)  # Only 4 unique values
  )

  output <-
    sg_vpop_est(data_i = test_data, minnumlev = 4, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_true(is.data.frame(result$datagen))
  expect_equal(ncol(result$datagen), 2)

  # With minnumlev = 4, low_level should be treated as categorical
  # Check that JSD is computed
  expect_true(is.numeric(result$jsd_res))
})

test_that("sg_vpop_est handles NA values by removing rows", {
  #skip_if(T)
  set.seed(52)
  test_data <- data.frame(
    x1 = c(rnorm(195, mean = 10, sd = 2), rep(NA, 5)),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  # Should not error, but will remove rows with NA
  output <-
    sg_vpop_est(data_i = test_data, diag_plots = FALSE, seed = 123, remove_duplicates = TRUE)


  result <- output[[1]]
  expect_true(is.data.frame(result$datagen))
  expect_equal(nrow(result$datagen), 195)  # Should match number of non-NA rows
})

# New tests for updated features

test_that("sg_vpop_est fixed seed mode generates single dataset", {
  #skip_if(T)
  set.seed(53)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  output <-
    sg_vpop_est(data_i = test_data, seed = 456, diag_plots = FALSE, remove_duplicates = TRUE)


  # Should generate exactly 1 dataset in fixed seed mode
  expect_length(output, 1)
  expect_equal(output[[1]]$seed, 456)
})

test_that("sg_vpop_est search mode generates multiple datasets", {
  #skip_if(T)
  set.seed(54)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3),
    x3 = rnorm(200, mean = 5, sd = 1)
  )

  # Search mode with nds = 3
  output <-
    sg_vpop_est(data_i = test_data, seed = NA, nds = 3,
                tg_corrdif = 0.15, diag_plots = FALSE, remove_duplicates = TRUE)


  # Should generate 3 datasets
  expect_length(output, 3)

  # Each dataset should have different seeds
  seeds <- sapply(output, function(x) x$seed)
  expect_true(all(!is.na(seeds)))

  # Check that all datasets meet structure requirements
  for (i in 1:3) {
    expect_true(is.data.frame(output[[i]]$datagen))
    expect_true(is.numeric(output[[i]]$corr_diff_max))
  }
})

test_that("sg_vpop_est computes correlation difference metrics", {
  #skip_if(T)
  set.seed(55)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3),
    x3 = rnorm(200, mean = 5, sd = 1)
  )

  output <-
    sg_vpop_est(data_i = test_data, seed = 789, diag_plots = FALSE, remove_duplicates = TRUE)


  result <- output[[1]]

  # Check correlation metrics
  expect_true(is.numeric(result$corr_diff_mean))
  expect_true(is.numeric(result$corr_diff_max))
  expect_true(result$corr_diff_mean >= 0)
  expect_true(result$corr_diff_max >= result$corr_diff_mean)
})

test_that("sg_vpop_est computes JSD for categorical variables", {
  #skip_if(T)
  set.seed(56)
  test_data <- data.frame(
    cat1 = factor(sample(c("A", "B", "C", "D"), 200, replace = TRUE)),
    cat2 = factor(sample(c("X", "Y", "Z"), 200, replace = TRUE)),
    x1 = rnorm(200, mean = 10, sd = 2)
  )

  output <-
    sg_vpop_est(data_i = test_data, seed = 321, diag_plots = FALSE, remove_duplicates = TRUE)


  result <- output[[1]]

  # Check JSD metric
  expect_true(is.numeric(result$jsd_res))
  expect_true(result$jsd_res >= 0)
  expect_true(result$jsd_res <= 1)
})

test_that("sg_vpop_est checks for exact duplicates", {
  #skip_if(T)
  set.seed(57)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  output <-
    sg_vpop_est(data_i = test_data, seed = 111,
                diag_plots = FALSE, remove_duplicates = TRUE)


  result <- output[[1]]

  # Should have checked for duplicates
  expect_true(is.logical(result$exact_dupl_check))
})

test_that("sg_vpop_est generates correct plots with diag_plots = TRUE", {
  #skip_if(T)
  set.seed(58)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3),
    cat1 = factor(sample(c("A", "B", "C"), 200, replace = TRUE))
  )

  output <-
    sg_vpop_est(data_i = test_data, seed = 555,
                diag_plots = TRUE, seed_umap = 42, remove_duplicates = TRUE)


  result <- output[[1]]

  # Check continuous plots
  expect_true(is.list(result$dplot_cont))
  expect_length(result$dplot_cont, 2)  # 2 continuous variables
  expect_true(all(vapply(result$dplot_cont,
                         function(x) inherits(x, "ggplot"), logical(1))))

  # Check categorical plots
  expect_true(is.list(result$dplot_cat))
  expect_length(result$dplot_cat, 1)  # 1 categorical variable
  expect_true(all(vapply(result$dplot_cat,
                         function(x) inherits(x, "ggplot"), logical(1))))

  # Check UMAP plot
  expect_true(inherits(result$dplot_umap, "ggplot"))

  # Check correlation difference heatmap
  expect_true(inherits(result$dplot_corr_diff, "ggplot"))
})

test_that("sg_vpop_est respects tg_corrdif in search mode", {
  #skip_if(T)
  set.seed(59)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3),
    x3 = rnorm(200, mean = 5, sd = 1)
  )

  # Use a stricter correlation threshold
  output <-
    sg_vpop_est(data_i = test_data, seed = NA, nds = 2,
                tg_corrdif = 0.05, diag_plots = FALSE, remove_duplicates = TRUE)


  # Check that generated datasets meet (or try to meet) the threshold
  for (i in 1:length(output)) {
    result <- output[[i]]
    # If seed is not NA, it means it found a dataset
    if (!is.na(result$seed)) {
      expect_true(is.numeric(result$corr_diff_max))
    }
  }
})

test_that("sg_vpop_est handles categorical-only data", {
  #skip_if(T)
  set.seed(60)
  test_data <- data.frame(
    cat1 = factor(sample(c("A", "B", "C", "D"), 200, replace = TRUE)),
    cat2 = factor(sample(c("X", "Y", "Z"), 200, replace = TRUE)),
    cat3 = factor(sample(c("Low", "Med", "High"), 200, replace = TRUE))
  )

  output <- sg_vpop_est(data_i = test_data, seed = 999, diag_plots = FALSE, remove_duplicates = TRUE)


  result <- output[[1]]

  # Check that it works with only categorical data
  expect_true(is.data.frame(result$datagen))
  expect_equal(ncol(result$datagen), 3)

  # JSD should be computed
  expect_true(is.numeric(result$jsd_res))

  # Correlation metrics should be NULL (no continuous variables)
  expect_null(result$corr_diff_mean)
  expect_null(result$corr_diff_max)
  expect_null(result$ks_test)
})

test_that("sg_vpop_est noise_level parameter affects duplicate removal", {
  #skip_if(T)
  set.seed(61)
  test_data <- data.frame(
    x1 = rnorm(200, mean = 10, sd = 2),
    x2 = rnorm(200, mean = 20, sd = 3)
  )

  # Test with different noise levels
  output1 <-
    sg_vpop_est(data_i = test_data, seed = 123,
                remove_duplicates = TRUE, noise_level = 0.05,
                diag_plots = FALSE)


  output2 <-
    sg_vpop_est(data_i = test_data, seed = 123,
                remove_duplicates = TRUE, noise_level = 0.20,
                diag_plots = FALSE)


  # Both should complete without error
  expect_true(is.data.frame(output1[[1]]$datagen))
  expect_true(is.data.frame(output2[[1]]$datagen))
})

## Author: Mikhailova Anna, Kulesh Victoria
## First created: 2025-11-05
## Description: function for fit
## Keywords: SimuRg, fit

#' Run fit with monolix/simurg/nonmem fitter
#'
#' @inheritParams sg_dummy
#' @param model string. Path to a txt file file with model structure in Monolix syntax. This should be a valid file path pointing to a model file that defines the pharmacokinetic/pharmacodynamic model structure
#' @param theta tibble or data.frame. Should contain the following columns:
#'   * `NAME` - string, parameter name. Should match names specified in the model file
#'   * `TRANS` - string, parameter transformation type. Can be: `"normal"`, `"logNormal"`, `"logitNormal"`
#'   * `INIT` - numeric, initial value for the parameter
#'   * `LB` - numeric, lower bound for logit transformation, `NA` for other transformations
#'   * `UB` - numeric, upper bound for logit transformation,  `NA` for other transformation
#'   * `EST` - logical, whether to estimate this parameter.
#' @param max_wait_time numeric. Maximum time in seconds to wait for fit results to complete. Default is 3600 seconds (1 hour). Set to `Inf` for no timeout.
#' @returns if option `fit = TRUE`, generalized simurg output object is returned. Otherwise, the file for fit is written and no output is returned
#' @examples
#' \donttest{
#' library(tibble)
#' library(dplyr)
#' library(stringr)
#' # Specify structural model path. For fit, should be in Monolix syntax
#' model <- system.file("extdata", "models", "model_PK_1c.txt", package = "SimuRg")
#' # Specify data path. Should be ADPPK-like format
#' data  <- system.file("extdata", "datasets", "dspk-warf.csv", package = "SimuRg")
#'
#' # Specify headers. Should be a list of lists with the following elements:
#' # name - string, name of the column
#' # use - string, use of the column from Monolix documentation
#' # type - string, type of the column. for use = "covariate", should be
#' # "continuous" or "categorical", for other uses should be NULL
#'  headers <- list(list(name = "ID", use = "identifier", type = NULL),
#'                  list(name = "TIME", use = "time", type = NULL),
#'                  list(name = "DV", use = "observation", type = "continuous"),
#'                  list(name = "DVID", use = "observationtype", type = NULL),
#'                  list(name = "ADM", use = "administration", type = NULL),
#'                  list(name = "AMT", use = "amount", type = NULL),
#'                  list(name = "EVID", use = "eventidentifier", type = NULL),
#'                  list(name = "MDV", use = "missingdependentvariable", type = NULL),
#'                  list(name = "AGE", use = "covariate", type = "continuous"),
#'                  list(name = "AGE_centered", use = "covariate", type = "continuous"),
#'                  list(name = "SEX", use = "covariate", type = "categorical"),
#'                  list(name = "WEIGHT", use = "covariate", type = "continuous"),
#'                  list(name = "BMI", use = "covariate", type = "continuous"))
#'
#' # Dataset with the parameters properties. Should be a tibble with the following columns:
#' # NAME - string, name of the parameter
#' # TRANS - string, distribution of the parameter. Should be one of the following:
#' # "normal", "logNormal", "logitNormal"
#' # INIT - numeric, initial value of the parameter or its fixed value
#' # LB - numeric, lower bound of the parameter for logit transformation
#' # UB - numeric, upper bound of the parameter for logit transformation
#' # EST - logical, estimation status of the parameter. For estimation, should
#' # be TRUE, for fixed, should be FALSE
#'  theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
#'                   "Cl", "logNormal", 0.2, NA, NA, TRUE,
#'                   "V", "logNormal", 20, NA, NA, TRUE,
#'                   "ka", "logNormal", 0.2, NA, NA, TRUE
#'  )
#'  # Examples of random effect model specification
#'  # Examples of random effect model specification for fit
#'  # Single observation (legacy format):
#'  # YNAME - string, name of the observation, ususally y1, y2, ...
#'  # DVID - numeric, observation type identifier corresponding to DVID column values
#'  # TRANS - string, residual error distribution. Can be: "normal", "logNormal",
#'  # "logitNormal"
#'  # PRED - string, prediction variable name from the model
#'  # ERR - string, error model type. Options include: "constant" for additive
#'  # error, "proportional" for proportional error, "combined1" for combined
#'  # additive and proportional error
#'  # INIT - numeric vector, initial values for error parameters
#'  # (length depends on error model)
#'  # EST - logical vector, whether to estimate each error parameter
#'  # (same length as INIT)
#'  # BLQM - below limit of quantification method (can be NULL)
#'  # Single observation (legacy format):
#'  ruv <- list(YNAME = "y1", DVID = 1, TRANS = "normal", PRED = "Cc",
#'              ERR = "combined1", INIT = c(1, 1), EST = c(TRUE, TRUE), BLQM = NULL)
#'
#'  # Multiple observations (recommended format):
#' ruv <- list(
#'   list(YNAME = "y1", DVID = 1, TRANS = "normal", PRED = "Cc",
#'        ERR = "combined1", INIT = c(1, 1), EST = c(TRUE, TRUE), BLQM = NULL),
#'   list(YNAME = "y2", DVID = 2, TRANS = "normal", PRED = "EFF",
#'       ERR = "proportional", INIT = c(0.1), EST = c(TRUE), BLQM = NULL)
#' )
#'
#'  # Example of random effects (RE) specification.
#'  #
#'  # init matrix: provides the initial values for the random effects.
#'  # est matrix: controls how each random effect is handled:
#'  #   TRUE  - the random effect will be estimated,
#'  #   FALSE - the random effect will be fixed at its initial value,
#'  #   NA    - no random effect will be applied.
#'  #
#'  # To fit a model without random effects, set all entries in the
#'  # est matrix to NA.
#'  #
#'  # The same logic applies to the between-occasion variability
#'  # matrix (occ).
#'
#'  re <- list(init = tribble(~Cl, ~V, ~ka,
#'                            1, 0, 0,
#'                            0, 0, 0,
#'                            0, 0, 1) %>% as.matrix(),
#'             est = tribble(~Cl, ~V, ~ka,
#'                           TRUE, NA, NA,
#'                           NA, NA, NA,
#'                           NA, NA, TRUE) %>% as.matrix())
#' # Example of between-occasion variability (BOV) specification. The structure
#' # is the same as for RE, but for BOV
#'
#'  occ <- list(init = tribble(~Cl, ~V, ~ka,
#'                             0, 0, 0,
#'                             0, 0, 0,
#'                             0, 0, 0) %>% as.matrix(),
#'              est = tribble(~Cl, ~V, ~ka,
#'                            NA, NA, NA,
#'                            NA, NA, NA,
#'                            NA, NA, NA) %>% as.matrix())
#' # Example of covariate specification. Should be a list of lists with the following elements:
#' # PAR - string, name of the parameter to which the covariate is applied
#' # COVNAME - string, name of the covariate
#' # FUNC - string, function to apply to the covariate. Should be "linear" for
#' # continuous covariates, "categorical" for categorical covariates
#' # TRANS - string, transformation of the covariate. Should be "median" for
#' # continuous covariates, "reference" for categorical covariates
#' # INIT - numeric, initial value of the covariate
#' # EST - logical, estimation status of the covariate. For estimation, should
#' #be TRUE, for fixed, should be FALSE
#'  covs <- list(list(PAR = "V", COVNAME = "AGE", FUNC = "linear",
#'                    TRANS = "median", INIT = 1, EST = TRUE),
#'               list(PAR = "ka", COVNAME = "SEX", REF = 0, INIT = 1, EST = TRUE))
#'  output_path <- str_c(tempdir(), "/")
#'  fitter_path <- "C:/ProgramData/Lixoft/MonolixSuite2023R1/bin/monolix.bat"
#'  # Examples of task_opt parameter
#'
#'  task_opt <-  paste("populationParameters()", "individualParameters()",
#'                     "logLikelihood()", sep = "\n")
#'  task_opt_lin <-  paste("populationParameters()", "individualParameters()",
#'                         "fim(method = Linearization)",
#'                         "logLikelihood(method = Linearization)", sep = "\n")
#' # Generalized control object
#' # gco <-list(headers = headers,
#' #            data = data,
#' #            model = model,
#' #            task_opt = task_opt
#' #            covs = covs,
#' #            project_name = "test-proj",
#' #            theta = theta,
#' #            ruv = ruv,
#' #            re = re,
#' #            occ = occ,
#' #            modelText = "")
#'  result <- sg_fit(model, data, headers, theta, ruv, re, occ, covs,
#'                   project_name = "my_project", fit = FALSE, # set fit = TRUE for fit
#'                   path_to_save_output =  output_path,
#'                   path_to_fitter = fitter_path)
#' }
#' @import sys
#' @export
sg_fit <- function(model, data, headers, theta, ruv, re, occ, covs, project_name,
                   task_opt = NULL, opt_name = "Monolix", fit = FALSE,
                   path_to_save_output = NULL, path_to_fitter = NULL,
                   max_wait_time = 3600){
  sc_data <- ""
  res_fit <- NULL
  if (!file.exists(model))stop(paste("Error: model file does not exist:", model))
  if (!file.exists(data))stop(paste("Error: data file does not exist:", data))

  model <- normalizePath(model)
  data <- normalizePath(data)
  if (is.null(path_to_save_output)) stop("No path to save output was provided")
  if (is.null(path_to_fitter) & fit == TRUE) stop("No path to fitter was identified")

  if (opt_name == "Monolix") {
    # Read the data file to get column names
    data_df <- read.csv(data, nrows = 1)  # Read just the header row
    column_names <- names(data_df)

    # Create header string in the required format
    header_string <- paste0("{", paste(column_names, collapse = ", "), "}")

    # Read the full dataset to get unique categories for observation types
    data_df_full <- read.csv(data)

    # Create content section using headers
    content_lines <- character()
    for (header in headers) {
      name <- header$name
      use <- header$use
      type <- header$type

      if (use == "observation") {
        # Find the observation type column (DVID)
        obs_type_col <- headers[sapply(headers, function(x) x$use == "observationtype")][[1]]$name
        unique_categories <- unique(data_df_full[[obs_type_col]])
        yname_value <- paste0("'", paste(unique_categories, collapse = "', '"), "'")

        if (!is.null(type)) {
          content_line <- paste0(name, " = {use=", use, ", yname=", yname_value, ", type=", type, "}")
        } else {
          content_line <- paste0(name, " = {use=", use, ", yname=", yname_value, "}")
        }
      } else {
        if (!is.null(type)) {
          content_line <- paste0(name, " = {use=", use, ", type=", type, "}")
        } else {
          content_line <- paste0(name, " = {use=", use, "}")
        }
      }
      content_lines <- c(content_lines, content_line)
    }
    content_section <- paste(content_lines, collapse = "\n")

    # Create dataType section using unique categories
    obs_type_col <- headers[sapply(headers, function(x) x$use == "observationtype")][[1]]$name
    unique_categories <- unique(data_df_full[[obs_type_col]])
    dataType_entries <- paste0("'", unique_categories, "'=plasma")
    dataType_section <- paste(dataType_entries, collapse = ", ")

    # Create covariate input section
    covariate_headers <- headers[sapply(headers, function(x) x$use == "covariate")]
    covariate_names <- sapply(covariate_headers, function(x) x$name)
    covariate_input <- paste(covariate_names, collapse = ", ")

    # Create covariate section conditionally
    if (length(covariate_headers) > 0) {
      # Find categorical covariates
      categorical_covariates <- covariate_headers[sapply(covariate_headers, function(x) x$type == "categorical")]

      # Create categorical properties
      categorical_properties <- character()
      for (cov in categorical_covariates) {
        cov_name <- cov$name
        cov_type <- cov$type
        unique_cats <- unique(data_df_full[[cov_name]])
        categories_str <- paste0("'", paste(unique_cats, collapse = "', '"), "'")
        categorical_properties <- c(categorical_properties,
                                    paste0(cov_name, " = {type=", cov_type, ", categories={", categories_str, "}}"))
      }

      # Combine input and categorical properties
      covariate_content <- paste0("input = {", covariate_input, "}")
      if (length(categorical_properties) > 0) {
        covariate_content <- paste0(covariate_content, "\n\n", paste(categorical_properties, collapse = "\n"))
      }

      covariate_section <- paste0("[COVARIATE]\n", covariate_content, "\n\n")
    } else {
      covariate_section <- ""
    }

    # Parse covs: group by parameter name, generate beta parameter names
    cov_by_param <- list()
    beta_params <- list()
    all_cov_input_names <- character()
    all_beta_input_names <- character()

    if (!is.null(covs) && length(covs) > 0) {
      for (cov_entry in covs) {
        par <- cov_entry$PAR
        covname <- cov_entry$COVNAME
        if (is.null(cov_by_param[[par]])) cov_by_param[[par]] <- list()

        is_categorical <- !is.null(cov_entry$REF)

        if (is_categorical) {
          unique_cats <- sort(unique(data_df_full[[covname]]))
          non_ref_cats <- unique_cats[unique_cats != cov_entry$REF]
          n_betas <- length(non_ref_cats)
          beta_suffixes <- gsub("[^[:alnum:]_]", "_", as.character(non_ref_cats))
          beta_names <- paste0("beta_", par, "_", covname, "_", beta_suffixes)
        } else {
          n_betas <- 1
          beta_names <- paste0("beta_", par, "_", covname)
        }

        cov_by_param[[par]] <- c(cov_by_param[[par]], list(list(
          covname = covname, beta_names = beta_names
        )))
        all_cov_input_names <- c(all_cov_input_names, covname)
        all_beta_input_names <- c(all_beta_input_names, beta_names)

        for (j in seq_len(n_betas)) {
          beta_params <- c(beta_params, list(list(
            name = beta_names[j], init = cov_entry$INIT, est = cov_entry$EST
          )))
        }
      }
      all_cov_input_names <- unique(all_cov_input_names)
    }

    # Create individual parameter definitions using theta, re, and covs
    individual_definitions <- character()
    individual_input_params <- character()

    for (i in 1:nrow(theta)) {
      param_name <- theta$NAME[i]
      param_trans <- theta$TRANS[i]
      typical_name <- paste0(param_name, "_pop")

      individual_input_params <- c(individual_input_params, typical_name)

      has_variability <- re$init[i, i] != 0 && re$est[i, i] == TRUE
      if (has_variability) {
        sd_name <- paste0("omega_", param_name)
        individual_input_params <- c(individual_input_params, sd_name)
      }

      # Build covariate fragment for this parameter's DEFINITION line
      cov_fragment <- ""
      param_covs <- cov_by_param[[param_name]]
      if (!is.null(param_covs) && length(param_covs) > 0) {
        all_covnames <- character()
        all_betas <- character()
        for (pc in param_covs) {
          all_covnames <- c(all_covnames, pc$covname)
          all_betas <- c(all_betas, pc$beta_names)
        }
        if (length(all_covnames) == 1 && length(all_betas) == 1) {
          cov_fragment <- paste0(", covariate=", all_covnames,
                                 ", coefficient=", all_betas)
        } else {
          cov_fragment <- paste0(", covariate={", paste(all_covnames, collapse = ", "),
                                 "}, coefficient={", paste(all_betas, collapse = ", "), "}")
        }
      }

      if (has_variability) {
        definition_line <- paste0(param_name, " = {distribution=", param_trans,
                                  ", typical=", typical_name, cov_fragment,
                                  ", sd=", sd_name, "}")
      } else {
        definition_line <- paste0(param_name, " = {distribution=", param_trans,
                                  ", typical=", typical_name, cov_fragment,
                                  ", no-variability}")
      }

      individual_definitions <- c(individual_definitions, definition_line)
    }

    # Append beta coefficient names and covariate column names to input
    individual_input_params <- c(individual_input_params,
                                 all_beta_input_names, all_cov_input_names)
    individual_definition_section <- paste(individual_definitions, collapse = "\n")
    individual_input_section <- paste(individual_input_params, collapse = ", ")

    # Create longitudinal definition using ruv and observation information
    # Handle both single ruv object and list of ruv objects for backward compatibility
    if (!is.null(ruv$YNAME)) {
      # Single ruv object (old format) - convert to list format
      ruv <- list(ruv)
    }

    longitudinal_definitions <- character()
    longitudinal_input_params <- character()

    for (ruv_entry in ruv) {
      # Get the output name from ruv_entry
      output_name <- ruv_entry$YNAME
      pred_name <- ruv_entry$PRED
      err_model <- ruv_entry$ERR
      trans_type <- ruv_entry$TRANS

      # Create error parameters using PRED name as VARNAME
      if (grepl("constant", err_model, ignore.case = TRUE)) {
        error_params <- paste0(pred_name, "_a")
        longitudinal_input_params <- c(longitudinal_input_params, paste0(pred_name, "_a"))
      } else if (grepl("proportional", err_model, ignore.case = TRUE)) {
        error_params <- paste0(pred_name, "_b")
        longitudinal_input_params <- c(longitudinal_input_params, paste0(pred_name, "_b"))
      } else if (grepl("combined", err_model, ignore.case = TRUE)) {
        error_params <- paste0(pred_name, "_a, ", pred_name, "_b")
        longitudinal_input_params <- c(longitudinal_input_params, paste0(pred_name, "_a"), paste0(pred_name, "_b"))
      } else {
        error_params <- paste0(pred_name, "_a")  # Default to constant
        longitudinal_input_params <- c(longitudinal_input_params, paste0(pred_name, "_a"))
      }

      # Create definition line
      definition_line <- paste0(output_name, " = {distribution=", trans_type, ", prediction=", pred_name, ", errorModel=", err_model, "(", error_params, ")}")
      longitudinal_definitions <- c(longitudinal_definitions, definition_line)
    }
    longitudinal_definition_section <- paste(longitudinal_definitions, collapse = "\n")
    longitudinal_input_section <- paste(longitudinal_input_params, collapse = ", ")

    # Create FIT section using observation ynames and output names
    # Handle both single ruv object and list of ruv objects for backward compatibility
    ruv_list <- if (!is.null(ruv$YNAME)) list(ruv) else ruv

    # Extract ynames from ruv entries
    observation_ynames <- character()
    output_names <- character()

    for (ruv_entry in ruv_list) {
      # Add DVID to fit data section
      dvid <- ruv_entry$DVID
      observation_ynames <- c(observation_ynames, paste0("'", dvid, "'"))

      # Add YNAME to fit model section
      yname <- ruv_entry$YNAME
      output_names <- c(output_names, yname)
    }

    fit_data_section <- paste(observation_ynames, collapse = ", ")
    fit_model_section <- paste(output_names, collapse = ", ")

    # Create PARAMETER section using theta and ruv
    parameter_definitions <- character()

    # Add theta parameters
    for (i in 1:nrow(theta)) {
      param_name <- theta$NAME[i]
      param_init <- theta$INIT[i]
      param_est <- theta$EST[i]
      pop_param_name <- paste0(param_name, "_pop")

      method <- ifelse(param_est, "MLE", "FIXED")
      parameter_line <- paste0(pop_param_name, " = {value=", param_init, ", method=", method, "}")
      parameter_definitions <- c(parameter_definitions, parameter_line)
    }

    # Add error model parameters for multiple observation types
    # Handle both single ruv object and list of ruv objects for backward compatibility
    ruv_list <- if (is.null(ruv[[1]]$YNAME)) list(ruv) else ruv

    for (ruv_entry in ruv_list) {
      pred_name <- ruv_entry$PRED
      err_model <- ruv_entry$ERR
      init_values <- ruv_entry$INIT
      est_flags <- ruv_entry$EST

      if (grepl("constant", err_model, ignore.case = TRUE)) {
        param_name <- paste0(pred_name, "_a")
        param_value <- init_values[1]
        param_est <- est_flags[1]
        method <- ifelse(param_est, "MLE", "FIXED")
        parameter_line <- paste0(param_name, " = {value=", param_value, ", method=", method, "}")
        parameter_definitions <- c(parameter_definitions, parameter_line)
      } else if (grepl("proportional", err_model, ignore.case = TRUE)) {
        param_name <- paste0(pred_name, "_b")
        param_value <- init_values[1]
        param_est <- est_flags[1]
        method <- ifelse(param_est, "MLE", "FIXED")
        parameter_line <- paste0(param_name, " = {value=", param_value, ", method=", method, "}")
        parameter_definitions <- c(parameter_definitions, parameter_line)
      } else if (grepl("combined", err_model, ignore.case = TRUE)) {
        # Add constant parameter
        param_name_a <- paste0(pred_name, "_a")
        param_value_a <- init_values[1]
        param_est_a <- est_flags[1]
        method_a <- ifelse(param_est_a, "MLE", "FIXED")
        parameter_line_a <- paste0(param_name_a, " = {value=", param_value_a, ", method=", method_a, "}")
        parameter_definitions <- c(parameter_definitions, parameter_line_a)

        # Add proportional parameter
        param_name_b <- paste0(pred_name, "_b")
        param_value_b <- init_values[2]
        param_est_b <- est_flags[2]
        method_b <- ifelse(param_est_b, "MLE", "FIXED")
        parameter_line_b <- paste0(param_name_b, " = {value=", param_value_b, ", method=", method_b, "}")
        parameter_definitions <- c(parameter_definitions, parameter_line_b)
      }
    }

    # Add omega parameters using re$init and re$est
    for (i in 1:nrow(theta)) {
      param_name <- theta$NAME[i]
      omega_param_name <- paste0("omega_", param_name)
      omega_init_value <- re$init[i, i]
      omega_est <- re$est[i, i]

      # Only add omega parameter if there is variability (init != 0 and est == TRUE)
      if (omega_init_value != 0 && omega_est == TRUE) {
        method <- ifelse(omega_est, "MLE", "FIXED")
        omega_parameter_line <- paste0(omega_param_name, " = {value=", omega_init_value, ", method=", method, "}")
        parameter_definitions <- c(parameter_definitions, omega_parameter_line)
      }
    }

    # Add beta parameters for covariates
    for (bp in beta_params) {
      method <- ifelse(bp$est, "MLE", "FIXED")
      beta_line <- paste0(bp$name, " = {value=", bp$init, ", method=", method, "}")
      parameter_definitions <- c(parameter_definitions, beta_line)
    }

    parameter_section <- paste(parameter_definitions, collapse = "\n")

    # Add c parameters for each observation type
    c_parameters <- character()
    for (obs_yname in unique_categories) {
      c_param_name <- paste0("c", obs_yname)
      c_parameter_line <- paste0(c_param_name, " = {value=1, method=FIXED}")
      c_parameters <- c(c_parameters, c_parameter_line)
    }
    c_parameter_section <- paste(c_parameters, collapse = "\n")

    # Combine all parameter sections
    full_parameter_section <- paste(parameter_section, c_parameter_section, sep = "\n")

    # Set parameters
    if (is.null(task_opt)) {
      tasks_section <- "populationParameters()\nindividualParameters()\nfim()\nlogLikelihood()"
    } else {
      task_section <- task_opt
    }

    # Create the mlxtran string structure
    mlxtran_string <- paste0(
      "<DATAFILE>\n\n",
      "[FILEINFO]\n",
      "file='", data, "'\n",
      "delimiter = comma\n",
      "header=", header_string, "\n\n",
      "[CONTENT]\n",
      content_section, "\n\n",
      "[SETTINGS]\n",
      "dataType = {", dataType_section, "}\n\n",
      "<MODEL>\n\n",
      covariate_section,
      "[INDIVIDUAL]\n",
      "input = {", individual_input_section, "}\n\n",
      "DEFINITION:\n",
      individual_definition_section, "\n\n",
      "[LONGITUDINAL]\n",
      "input = {", longitudinal_input_section, "}\n\n",
      "file = '", model, "'\n\n",
      "DEFINITION:\n",
      longitudinal_definition_section, "\n\n",
      "<FIT>\n",
      "data = {", fit_data_section, "}\n",
      "model = {", fit_model_section, "}\n\n",
      "<PARAMETER>\n",
      full_parameter_section, "\n\n",
      "<MONOLIX>\n\n",
      "[TASKS]\n\n",
      tasks_section, "\n\n",
      "[SETTINGS]\n",
      "GLOBAL:\n",
      "exportpath = '", project_name, "'\n"
    )
    # Return the constructed string

    sc_data <- mlxtran_string
    filepath <- sprintf("%s/%s.mlxtran", path_to_save_output, project_name)

  } else if (opt_name == "Simurg"){

    simurg_cntrl_list <- list(model = model,
                              data = data,
                              headers = headers,
                              theta = theta,
                              ruv = ruv,
                              re = re,
                              occ = occ,
                              covs = covs,
                              project_name = project_name,
                              task_opt = task_opt)
    simurg_cntrl_file <- toJSON(simurg_cntrl_list, pretty = TRUE)
    sc_data <- simurg_cntrl_file

    filepath <- sprintf("%s/%s.R", path_to_save_output, project_name)
    # return(simurg_cntrl_file)
  }
  # dir.create(dirname(filepath), showWarnings = FALSE, recursive =TRUE)
  write(sc_data, filepath)
  message(sprintf("The file for fit was written %s", filepath))
  if (fit & opt_name == "Monolix") {
    dir.create(file.path(path_to_save_output, project_name), showWarnings = FALSE,
               recursive = TRUE)
    path_to_save_output1 <- normalizePath(file.path(path_to_save_output, project_name))
    filepath <- normalizePath(filepath)
    curr_dir <- getwd()
    setwd(dirname(path_to_fitter))
    on.exit(setwd(curr_dir))
    int_res <- system(sprintf('%s --no-gui -p %s -o %s -t monolix', #--mode none
                              path_to_fitter, path.expand(filepath),
                              path_to_save_output1), wait = FALSE)
    Sys.sleep(10)
    ended <- "LogLikelihood" %in% list.files(path_to_save_output1)
    start_time <- Sys.time()
    while (!ended) {
      ended <-"LogLikelihood" %in% list.files(path_to_save_output1)
      elapsed_time <- as.numeric(Sys.time() - start_time, units = "secs")
      if (elapsed_time > max_wait_time) {
        warning(sprintf("Timeout reached: Waited %d seconds for Monolix fit to complete. LogLikelihood file not found.", max_wait_time))
        break
      }
      Sys.sleep(1)  # Sleep 1 second between checks to avoid busy waiting
    }
    res_fit <- sg_converter(str_c(path_to_save_output, "/"), project_name)
    # } else if (fit & opt_name == "Simurg") {
    # int_res <- system(sprintf('%s --no-gui -p "%s" --mode none -o "%s"',
    #                           path_to_fitter, path.expand(filepath),
    #                           path.expand(path_to_save_output)))
    # res_fit <- sg_converter(path_to_save_output, project_name, opt_name = "Simurg")
  }
  return(res_fit)
}

## Author: Ugolkov Yaroslav, Anatoly Pokladyuk
## First created: 2025-09-05
## Description: sg-gof-res and its helper functions
## Keywords: SimuRg, sg-gof-res, goodness-of-fit
#' Create residual diagnostic plots
#'
#' @description
#' Generates residual diagnostic plots versus time/predictions.
#' Supports faceting, covariate coloring, quantile binning for
#' continuous covariates, and optional smoothing.
#' @inheritParams sg_dummy
#' @param vs_time Logical. If `TRUE`, plot residuals vs TIME; otherwise, plot
#'  residuals vs predictions (IPRED/PRED).
#' @param weighted Logical. If `TRUE`, use weighted residuals; otherwise, use
#'  RES/IRES
#'
#' @return A `ggplot2` object
#'
#' @examples
#' # Basic example with mock data
#' set.seed(123)  # For reproducibility
#' n_subjects <- 50
#' mock_obj <- list(
#'   SDTAB = do.call(rbind, lapply(1:n_subjects, function(id) {
#'     n_obs <- 6
#'     times <- sort(runif(n_obs, min = 0, max = 24))  # random times between 0 and 24h
#'     data.frame(
#'       ID    = id,
#'       TIME  = times,
#'       DV    = rnorm(n_obs, mean = 10, sd = 2),
#'       PRED  = rnorm(n_obs, mean = 10, sd = 1.5),
#'       IPRED = rnorm(n_obs, mean = 10, sd = 1.2),
#'       IWRES = rnorm(n_obs, mean = 0, sd = 0.8),
#'       IRES  = rnorm(n_obs, mean = 0, sd = 1.2),
#'       MDV   = 0
#'     )
#'   })),
#'   COTAB = data.frame(
#'     ID  = 1:n_subjects,
#'     AGE = sample(20:80, n_subjects, replace = TRUE)
#'   ),
#'   CATAB = data.frame(
#'     ID   = 1:n_subjects,
#'     RACE = sample(c("Hispanic", "Asian", "Caucasian"), n_subjects, replace = TRUE)
#'   )
#' )
#'
#' # Basic plot: individual weighted residuals vs TIME (weighted = TRUE, vs_time = TRUE)
#' p <- sg_gof_res(mock_obj, smooth = FALSE)
#'
#' # With covariates and faceting (use RACE as facet and AGE as color)
#' p <- sg_gof_res(
#'   mock_obj,
#'   smooth = TRUE,
#'   cov_cols = c("RACE","AGE"),
#'   col_i = "RACE",
#'   facet_i = "AGE",
#'   indiv = TRUE,
#'   weighted = TRUE,
#'   vs_time = FALSE,
#'   legend_fl = TRUE
#' )
#' p
#'
#' @import dplyr
#' @import ggplot2
#' @importFrom scales pretty_breaks pretty_breaks trans_format math_format
#' @export
sg_gof_res <- function(
    fpath_i, DVID = 1, cov_cols = NULL, indiv = TRUE, vs_time = TRUE, weighted = TRUE,
    addline = TRUE, alpha_i = 0.5, smooth = TRUE, log_x = FALSE,
    abreaks = scales::pretty_breaks(7), lab_y = NULL, lab_x = NULL,
    col_i = NULL, col_lab = NULL, facet_i = NULL, f_scales = "fixed",
    n_bins = 50, min_y = NA, max_y = NA, min_x = NA, max_x = NA, legend_fl = FALSE,
    n_quantiles = 3, levels_discrete = 10
) {

  # --- helper functions ---
  is_discrete <- function(x, max_levels = levels_discrete) {
    n_unique <- length(unique(na.omit(x)))
    n_unique <= max_levels
  }

  continuous_to_categories <- function(x, n_quant = 3) {
    cut(
      x,
      breaks = quantile(x, probs = seq(0, 1, length.out = n_quant + 1), na.rm = TRUE),
      include.lowest = TRUE,
      labels = paste0("Q", 1:n_quant)
    )
  }

  # --- data prep ---
  X <- if (vs_time) "TIME" else if (indiv) "IPRED" else "PRED"

  if (!is_null(read_smrg_obj(fpath_i)$GFO))
    smrg_obj <- read_smrg_obj(fpath_i)$GFO
  else
    smrg_obj <- read_smrg_obj(fpath_i)

  if (is.null(smrg_obj$SDTAB)) {
    stop("sg_fit object must contain SDTAB component")
  }

  sdtab <- smrg_obj$SDTAB

  if (is.data.frame(sdtab) && nrow(sdtab) == 0) {
    stop("SDTAB is empty (no rows)")
  }
  if (is.list(sdtab) && length(sdtab) == 0) {
    stop("SDTAB is empty (no elements)")
  }
  if (is.data.frame(sdtab)) {
    ds_i <- sdtab
  } else if (is.list(sdtab)) {
    ds_i <- as.data.frame(do.call(rbind, sdtab))
  } else {
    stop("SDTAB must be a data frame or a list of data frames")
  }

  ds_i <- filter_sdtab_by_DVID(ds_i, DVID)

  if (indiv) {
    if (weighted) {
      # Weighted individual residuals
      if ("IWRES" %in% colnames(ds_i)) {
        res_type <- "IWRES"
      } else {
        stop("IWRES column not found in data for individual weighted residuals.")
      }
    } else {
      # Unweighted individual residuals
      if ("IRES" %in% colnames(ds_i)) {
        res_type <- "IRES"
      } else {
        stop("IRES column not found in data for individual residuals.")
      }
    }
  } else {
    # Population plots
    if (weighted) {
      # Weighted population residuals: CWRES preferred, then WRES
      if ("CWRES" %in% colnames(ds_i)) {
        res_type <- "CWRES"
      } else if ("WRES" %in% colnames(ds_i)) {
        res_type <- "WRES"
      } else {
        stop("No weighted population residual column (CWRES or WRES) found in data.")
      }
    } else {
      # Unweighted population residuals
      if ("RES" %in% colnames(ds_i)) {
        res_type <- "RES"
      } else {
        stop("RES column not found in data for population residuals.")
      }
    }
  }

  if ("MDV" %in% colnames(ds_i)) {
    ds_i <- ds_i %>% filter(.data$MDV != 1)
  }
  ds_i <- ds_i %>%
    rename_at(vars(any_of(res_type)), ~"Y") %>%
    rename_at(vars(any_of(X)), ~"X")

  # --- covariates ---
  if (!is.null(cov_cols)) {
    suppressMessages({
      ds_covs <- left_join(smrg_obj$COTAB, smrg_obj$CATAB)
    })
    ds_covs_i <- ds_covs %>% select(ID, one_of(cov_cols)) %>% unique()
    stopifnot(n_distinct(ds_covs_i$ID) == nrow(ds_covs_i))
    suppressMessages({
      ds_i <- ds_i %>% left_join(ds_covs_i, by = "ID")
    })

    for (col in cov_cols) {
      if (!is_discrete(ds_i[[col]])) {
        ds_i[[col]] <- continuous_to_categories(ds_i[[col]], n_quant = n_quantiles)
        #message(paste(col, "was converted into", n_quantiles, "quantile-based categories."))
      } else {
        ds_i[[col]] <- factor(ds_i[[col]])
      }
    }
  }

  # Compute y-axis limits (pass NA for auto limits)
  if (!all(is.na(c(min_y, max_y)))) {
    y_limits <- c(min_y, max_y)
    y_scale <- scale_y_continuous(breaks = abreaks, limits = y_limits)
  } else {
    y_scale <- scale_y_continuous(breaks = abreaks)
  }

  if (!all(is.na(c(min_x, max_x)))) {
    x_limits <- c(min_x, max_x)
  } else {
    x_limits <- NULL
  }

  y_label <- if (is.null(lab_y)) res_type else lab_y
  x_label <- if (is.null(lab_x)) X else lab_x

  # scatter vs time/pred
  p_char <- c(list(
    labs(y = y_label, x = x_label),
    #geom_hline(yintercept = c(-2, 2), col = "firebrick", linewidth = 0.5, linetype = "dashed"),
    geom_hline(yintercept = 0, col = "black", linewidth = 0.7, linetype = "dashed"),
    y_scale,
#    scale_color_manual(values = rep(MSDcol, 20)),
    theme(legend.justification = c("left", "center"),
          legend.box.just = "left",
          legend.background = element_rect(fill = "white", linewidth = 0.15,
                                           linetype = "solid", colour = "black"),
          legend.key.size = unit(0.38, "cm"),
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 8),
          plot.title = element_text(size = 12),
          panel.grid.minor = element_blank())
    ),
  if (weighted) list(geom_hline(yintercept = c(-2, 2),
                              col = "firebrick", linewidth = 0.5, linetype = "dashed"))
  )
  if (log_x) {
    p_char <- c(p_char, scale_x_log10(
      breaks = scales::trans_breaks("log10", function(x) 10^x),
      labels = scales::trans_format("log10", scales::math_format(10^.x)),
      limits = x_limits
    ))
  } else {
    p_char <- c(p_char, scale_x_continuous(breaks = abreaks, limits = x_limits))
  }
    p_Res <- ggplot(ds_i, aes(x = X, y = Y)) + p_char + theme_bw()

  if (!is.null(col_i)) {
    if (!is.factor(ds_i[[col_i]])) {
      ds_i[[col_i]] <- factor(ds_i[[col_i]])
    }
    col_levels <- levels(ds_i[[col_i]])
    n_cols <- length(col_levels)

    col_values <- rep(MSDcol, length.out = n_cols)

    col_labels <- paste0(col_i, ": ", col_levels)

    # Add the custom color scale with formatted labels
    p_Res <- p_Res +
      scale_color_manual(values = col_values, labels = col_labels)
  }

  if (!is.null(col_i)) {
    p_Res <- p_Res +
      geom_point(aes(col = !!sym(col_i)), size = 1.5, alpha = alpha_i) +
      labs(col = col_lab) +
      guides(color = guide_legend(override.aes = list(alpha = 1)))

    if (addline) p_Res <- p_Res + geom_line(aes(col = !!sym(col_i), group = ID), linewidth = 0.4, alpha = alpha_i)
    if (smooth)  p_Res <- p_Res + geom_smooth(aes(col = !!sym(col_i), group = !!sym(col_i)), formula = "y ~ x", method = "loess", linewidth = 1.2, se = FALSE)
  } else {
    p_Res <- p_Res +
      geom_point(col = MSDcol[1], size = 1.5, alpha = alpha_i)

    if (addline) p_Res <- p_Res + geom_line(aes(group = ID), col = MSDcol[1], linewidth = 0.4, alpha = alpha_i)
    if (smooth)  p_Res <- p_Res + geom_smooth(formula = "y ~ x", method = "loess", linewidth = 1.2, se = FALSE, col = MSDcol[3])
  }

  # --- facets & legend ---
  if (!is.null(facet_i)) {
    p_Res <- p_Res + facet_wrap(as.formula(paste0("~", facet_i)), scales = f_scales, labeller = label_both)
  }
  if (!legend_fl) {
    p_Res <- p_Res + theme(legend.position = "none")
  }
  return(p_Res)
}

## Author: Ugolkov Yaroslav
## First created: 2025-09-05
## Description: formal testing of sg-gof-res function and its helpers
## Keywords: SimuRg, sg-gof-res, goodness-of-fit

test_that("sg-gof-res output is correct", {
  x <- sg_gof_res(obj1, cov_cols = 'AGE', col_i = 'AGE', log_x = T, facet_i = 'AGE', vs_time = F, indiv = F)
  expect_true(inherits(x, "ggplot"))
  expect_snapshot(ggplot2::layer_data(x))
})

test_that("sg-gof-res does not work", {
  expect_error(sg_gof_res(data.frame()))
  # expect_error(sg_gof_res(obj1, f_scales = "invalid_scale"))
})

test_that("sg-gof-res file load", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  x <- sg_gof_res(fpath_i, log_x = TRUE, addline = T, legend_fl = T,
                  cov_cols = 'AGE', col_i = 'AGE', n_quantiles = 10)
  expect_true(inherits(x, "ggplot"))
  expect_snapshot(ggplot2::layer_data(x))
})

# Дополнительные тесты для 100% покрытия
test_that("sg-gof-res works with different residual types", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")

  # Test with WRES
  x <- sg_gof_res(fpath_i, indiv = F, vs_time = TRUE)
  expect_true(inherits(x, "ggplot"))

  # Test with RES
  x <- sg_gof_res(fpath_i, vs_time = TRUE, weighted = F, f_scales = "free")
  expect_true(inherits(x, "ggplot"))
})

test_that("sg-gof-res works with different plot configurations", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")

  # Test without covariates
  x <- sg_gof_res(fpath_i, vs_time = TRUE, addline = FALSE, smooth = FALSE)
  expect_true(inherits(x, "ggplot"))

  # Test with PRED instead of IPRED
  x <- sg_gof_res(fpath_i, indiv = FALSE, vs_time = FALSE)
  expect_true(inherits(x, "ggplot"))

})

test_that("sg-gof-res works with list input", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  smrg_obj <- get(load(fpath_i))

  # Test with list input instead of file path
  x <- sg_gof_res(smrg_obj)
  expect_true(inherits(x, "ggplot"))
})

test_that("sg-gof-res handles axis limits and labels", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")

  x <- sg_gof_res(fpath_i, min_y = -0.5, max_y = 0.5, min_x = 0, max_x = 100,
                  weighted = F, lab_y = "Individual residuals, uM", lab_x = "Time, h")
  expect_true(inherits(x, "ggplot"))
})

test_that("sg-gof-res works with different facet scales", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")

  x <- sg_gof_res(fpath_i, vs_time = F, cov_cols = 'AGE', facet_i = 'AGE', f_scales = "free")
  expect_true(inherits(x, "ggplot"))

  x <- sg_gof_res(fpath_i, vs_time = F, cov_cols = 'AGE', facet_i = 'AGE', f_scales = "free_x")
  expect_true(inherits(x, "ggplot"))

  x <- sg_gof_res(fpath_i, cov_cols = 'AGE', facet_i = 'AGE', f_scales = "free_y")
  expect_true(inherits(x, "ggplot"))
})

test_that("sg-gof-res works with different quantile bins", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")

  x <- sg_gof_res(fpath_i, cov_cols = 'AGE', col_i = 'AGE', n_quantiles = 4, legend_fl = T)
  expect_true(inherits(x, "ggplot"))
})

test_that("sg-gof-res works with discrete covariates", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")

  # Test with SEX which should be discrete
  x <- sg_gof_res(fpath_i, cov_cols = 'SEX', col_i = 'SEX', legend_fl = T)
  expect_true(inherits(x, "ggplot"))
})

test_that("sg-gof-res works with multiple covariates", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")

  x <- sg_gof_res(fpath_i, cov_cols = c('AGE','WEIGHT'), col_i = c('AGE'), legend_fl = T, facet_i = 'WEIGHT')
  expect_true(inherits(x, "ggplot"))
})

## Author: Ugolkov Yaroslav
## First created: 2025-09-05
## Description: sg-gof-par-dist and its helper functions
## Keywords: SimuRg, sg-gof-par-dist, goodness-of-fit
#'
#' Plot distributions of individual parameters with optional theoretical density
#'@description
#' This function creates histograms of individual parameter estimates,
#' with optional overlay of the theoretical distribution based on the population parameters and OMEGA matrix.
#'
#' @inheritParams sg_dummy
#'
#' @param plot_type Character string specifying the type of plot to generate.
#'   Options: 'DIST' for parameter distributions (default), 'QQ' for Q-Q plots,
#'   'correlations' for correlation matrix plot.
#' @return A `ggplot` object containing histogram(s) of individual parameter distributions, optionally overlaid with theoretical densities.
#'
#' @details
#' The function visualizes the distribution of post hoc individual parameter estimates (from `$PATAB`) and optionally compares them with the expected population-level variability using `$SUMTAB$DISTRIBUTION` (see Details).
#' Shrinkage values are calculated based on `ETAshrinkage_var` from `$SUMTAB` and included in facet labels.
#'
#' Theoretical distributions are generated by sampling from a multivariate normal distribution (mean zero, covariance from `$OMEGAMAT`) and mapping to the individual-parameter scale using typical values  and the `DISTRIBUTION` column: `logNormal` uses `TV * exp(ETA)`, `normal` uses `TV + ETA`, `logitNormal` uses the inverse logit of `logit(TV) + ETA`. Parameters with missing or empty `DISTRIBUTION` do not get a theoretical curve.
#'
#'
#' @examples
#' \donttest{
#' # Basic usage: distribution plots for all parameters
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
#' sg_gof_par_dist(fpath_i = fpath_i)
#'
#' # Distribution plots for selected parameters (natural scale)
#' sg_gof_par_dist(fpath_i = fpath_i, par_seq = c("ka", "Cl"))
#'
#' # Distribution plots with theoretical densities disabled
#' sg_gof_par_dist(fpath_i = fpath_i, tdist = FALSE)
#'
#' # Distribution plots for ETA
#' sg_gof_par_dist(fpath_i = fpath_i, par_seq = c("eta_ka", "eta_Cl"), par_type = "RE")
#'
#' # Q-Q plots for all ETA parameters
#' sg_gof_par_dist(fpath_i = fpath_i, plot_type = "QQ")
#'
#' # Q-Q plot for a specific ETA parameter
#' sg_gof_par_dist(fpath_i = fpath_i, plot_type = "QQ", par_seq = "eta_ka")
#'
#' # Correlation matrix for selected parameters
#' sg_gof_par_dist(fpath_i = fpath_i, plot_type = "correlations")
#' }
#' @import dplyr
#' @importFrom MASS mvrnorm
#' @import ggplot2
#' @importFrom scales pretty_breaks
#' @importFrom grDevices recordPlot
#' @importFrom graphics abline box grid hist lines mtext par points text
#' @importFrom stats lm
#' @export

sg_gof_par_dist <- function(fpath_i, par_seq = NULL,
                            par_type = "Ind", n_bins = 30, tdist = TRUE, plot_type = 'DIST'){
  load_smrg_obj <- function(x) {
    if (inherits(x, "list")) return(x)
    if (!inherits(x, "character") || length(x) != 1L) {
      stop("fpath_i object should be either an sg_fit object (list), or a path to saved sg_fit object")
    }
    if (!file.exists(x)) stop("File does not exist: ", x)

    ext <- tolower(tools::file_ext(x))
    if (ext %in% c("rdata", "rda", "r")) {
      nm <- load(x)
      if (length(nm) == 0) stop("No objects found in file: ", x)
      return(get(nm[1]))
    }

    if (ext == "json") {
      obj <- jsonlite::read_json(x, simplifyVector = TRUE)
      if (is.list(obj) && all(c("GFO", "GCO") %in% names(obj))) obj <- obj$GFO
      return(obj)
    }

    stop("Unsupported file extension: .", ext, " (expected .RData/.rda or .json)")
  }

  smrg_obj <- load_smrg_obj(fpath_i)

  # Coerce core components after JSON import if needed
  if ("OMEGAMAT" %in% names(smrg_obj) && !is.null(smrg_obj$OMEGAMAT) && !is.matrix(smrg_obj$OMEGAMAT)) {
    smrg_obj$OMEGAMAT <- as.matrix(smrg_obj$OMEGAMAT)
  }
  if ("SIGMAMAT" %in% names(smrg_obj) && !is.null(smrg_obj$SIGMAMAT) && !is.matrix(smrg_obj$SIGMAMAT)) {
    smrg_obj$SIGMAMAT <- as.matrix(smrg_obj$SIGMAMAT)
  }
  if ("COVMAT" %in% names(smrg_obj) && !is.null(smrg_obj$COVMAT) && !is.matrix(smrg_obj$COVMAT)) {
    smrg_obj$COVMAT <- as.matrix(smrg_obj$COVMAT)
  }
  if ("CORRMAT" %in% names(smrg_obj) && !is.null(smrg_obj$CORRMAT) && !is.matrix(smrg_obj$CORRMAT)) {
    smrg_obj$CORRMAT <- as.matrix(smrg_obj$CORRMAT)
  }
  if ("PATAB" %in% names(smrg_obj) && !is.null(smrg_obj$PATAB) && !inherits(smrg_obj$PATAB, "data.frame")) {
    smrg_obj$PATAB <- as.data.frame(smrg_obj$PATAB)
  }
  if ("SUMTAB" %in% names(smrg_obj) && !is.null(smrg_obj$SUMTAB) && !inherits(smrg_obj$SUMTAB, "data.frame")) {
    smrg_obj$SUMTAB <- as.data.frame(smrg_obj$SUMTAB)
  }

  prm_i <- smrg_obj$COVMAT
  patab_i <- smrg_obj$PATAB
  omegamat_i <- smrg_obj$OMEGAMAT
  ### Distribution of individual parameters
  if(is.null(par_seq)){
    par_seq <- names(patab_i %>% select(-contains("eta"),-ID))
    # If OMEGAMAT is empty/missing, keep all parameters from PATAB
    if (!is.null(omegamat_i) && is.matrix(omegamat_i) && !is.null(colnames(omegamat_i)) && length(colnames(omegamat_i)) > 0) {
      par_seq <- par_seq[par_seq %in% sub("^omega_", "", colnames(omegamat_i))]
    }
  }

  # --- Validate par_seq ---
  missing_par <- setdiff(par_seq, names(patab_i))
  if (length(missing_par) > 0) {
    warning(paste("Parameters not found in PATAB:", paste(missing_par, collapse = ", ")))
  }

  par_seq <- intersect(par_seq, names(patab_i))
  if (length(par_seq) == 0) {
    return(invisible(NULL))
  }


  if (plot_type == 'correlations') {
    # Prepare data
    if(is.null(par_seq)){
      selected_cols <- setdiff(names(patab_i), "ID")
    } else {
      selected_cols <- par_seq
    }

    ds_eta <- patab_i %>% dplyr::select(ID, dplyr::all_of(selected_cols))
    plot_cols <- setdiff(names(ds_eta), c("ID", "eta_Vd", "eta_CL"))
    n <- length(plot_cols)

    # Save current graphics parameters
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))

    # Create layout for plots
    par(mfrow = c(n, n),
        mar = c(1.5, 1.5, 0.5, 0.5),
        oma = c(2, 2, 2, 1))

    for(i in 1:n) {
      for(j in 1:n) {
        if(i == j) {
          # DIAGONAL - histogram with density
          # First draw empty plot with histogram stats to set proper ylim
          h <- hist(ds_eta[[plot_cols[i]]], plot = FALSE, breaks = 10)

          plot(1, type = "n",
               xlim = range(ds_eta[[plot_cols[i]]]),
               ylim = c(0, max(h$density) * 1.1),
               xlab = "", ylab = "",
               axes = TRUE, cex.axis = 1, mgp = c(2, 0.5, 0))

          # Add grid
          grid(col = "grey80", lty = 1, lwd = 0.5)

          # Add histogram
          hist(ds_eta[[plot_cols[i]]],
               main = "",
               xlab = "",
               ylab = "",
               col = MSDcol[2],
               border = "grey45",
               probability = TRUE,
               breaks = 10,
               add = TRUE)

          box()

          # Density curve
          lines(density(ds_eta[[plot_cols[i]]]),
                col = "grey25",
                lwd = 1.7)

          # Vertical line at 0
          abline(v = 0, col = "grey25", lty = 3, lwd = 1)

        } else if(i < j) {
          # UPPER TRIANGLE - Pearson correlation with significance stars
          plot(1, type = "n",
               xlim = c(0, 1), ylim = c(0, 1),
               xlab = "", ylab = "",
               xaxt = "n", yaxt = "n",
               axes = FALSE)
          box()

          # Pearson correlation
          cor_test <- cor.test(ds_eta[[plot_cols[i]]], ds_eta[[plot_cols[j]]],
                               method = "pearson", use = "complete.obs")
          cor_val <- cor_test$estimate
          p_val <- cor_test$p.value

          # Significance stars
          stars <- ifelse(p_val < 0.001, "***",
                          ifelse(p_val < 0.01, "**",
                                 ifelse(p_val < 0.05, "*", "")))

          # Correlation text with stars
          text(0.5, 0.6,
               labels = paste0("Pearson correlation = ", round(cor_val, 2), stars),
               cex = 1.3,
               font = 2,
               col = "black")

        } else {
          # LOWER TRIANGLE - scatter plot with regression line
          plot(ds_eta[[plot_cols[j]]], ds_eta[[plot_cols[i]]],
               pch = 16,
               col = MSDcol[1],
               xlab = "",
               ylab = "",
               cex = 1,
               axes = TRUE,
               cex.axis = 1,
               mgp = c(2, 0.5, 0))

          # Add grid
          grid(col = "grey80", lty = 1, lwd = 0.5)

          # Redraw points on top of grid
          points(ds_eta[[plot_cols[j]]], ds_eta[[plot_cols[i]]],
                 pch = 16, col = MSDcol[1], cex = 1)

          box()

          # Linear regression line
          abline(lm(ds_eta[[plot_cols[i]]] ~ ds_eta[[plot_cols[j]]]),
                 col = MSDcol[3],
                 lwd = 1.5)
        }
      }
    }

    # Add variable labels on TOP
    for(j in 1:n) {
      mtext(plot_cols[j],
            side = 3, line = 0.2, outer = TRUE,
            at = (j - 0.5)/n, cex = 1, font = 2)
    }

    # Add variable labels on LEFT
    for(i in 1:n) {
      mtext(plot_cols[i],
            side = 2, line = 0.2, outer = TRUE,
            at = (n - i + 0.5)/n, cex = 1, font = 2)
    }

    # Capture the plot
    p_i <- recordPlot()

    # Restore graphics parameters
    par(old_par)

    return(p_i)
  }



  ### Calculate shrinkage
  if (!"ETAshrinkage_var" %in% colnames(smrg_obj$SUMTAB)) {
    message("Shrinkage was not calculated in GFO object")
    smrg_obj$SUMTAB$ETAshrinkage_var <- NA
  }
  shr_out <- smrg_obj$SUMTAB %>% filter(grepl("^omega_", PAR)) %>%
    select(any_of(c("ETAshrinkage_var","VALUE","PAR"))) %>%
    mutate(PAR = str_remove(PAR, "^omega_"), Shrinkage = signif(ETAshrinkage_var,3) )



  ind_par_dist <- patab_i %>% select(all_of(par_seq)) %>%
    gather("PAR") %>%
    left_join(shr_out, by = "PAR") %>%
    mutate(
      Shrinkage = ifelse(is.na(Shrinkage), "N/A", Shrinkage),
      LABEL = str_c(PAR, "\nShrinkage = ", Shrinkage, "%"))


  p_i <- ggplot(data = ind_par_dist, aes(x = value)) +
    geom_histogram(aes(y = after_stat(density)),
                   bins = n_bins,
                   col = "grey25",
                   fill = MSDcol[2]) +
    theme_bw()+
    facet_wrap(~PAR, scales = "free") +
    scale_y_continuous(name = "Density", breaks = scales::pretty_breaks(7), expand = c(0, 0), limits = c(0, NA)) +
    scale_x_continuous(name = "Parameter value", breaks = scales::pretty_breaks(7), expand = c(0, 0))

  suppressMessages({

    if(tdist && plot_type == 'DIST'){

      if (par_type == "Ind") {
        sumtab_i <- smrg_obj$SUMTAB
        if (dplyr::is.grouped_df(sumtab_i)) sumtab_i <- dplyr::ungroup(sumtab_i)
        if ("Distribution" %in% names(sumtab_i) && !"DISTRIBUTION" %in% names(sumtab_i)) {
          sumtab_i <- sumtab_i %>% dplyr::rename(DISTRIBUTION = Distribution)
        }

        dist_tbl <- sumtab_i %>%
          dplyr::filter(str_detect(PAR, "_pop$")) %>%
          dplyr::mutate(PAR = str_remove(PAR, "_pop$")) %>%
          dplyr::select(PAR, dplyr::any_of("DISTRIBUTION"))

        par_pop <- sumtab_i %>%
          dplyr::filter(str_detect(PAR, "_pop$")) %>%
          dplyr::mutate(PAR = str_remove(PAR, "_pop$")) %>%
          dplyr::select(PAR, TV = VALUE) %>%
          dplyr::filter(PAR %in% par_seq) %>%
          dplyr::left_join(dist_tbl, by = "PAR")

        if (!"DISTRIBUTION" %in% names(par_pop)) {
          par_pop_valid <- par_pop[FALSE, , drop = FALSE]
        } else {
          par_pop_valid <- par_pop %>%
            dplyr::filter(
              !is.na(DISTRIBUTION),
              nzchar(trimws(as.character(DISTRIBUTION)))
            )
        }

        if (nrow(par_pop_valid) > 0) {
          colnames(omegamat_i) <- sub("^omega_", "", colnames(omegamat_i))
          rownames(omegamat_i) <- sub("^omega_", "", rownames(omegamat_i))
          rownames(omegamat_i) <- colnames(omegamat_i)
          omega_subset <- omegamat_i[par_pop_valid$PAR, par_pop_valid$PAR, drop = FALSE]

          n_eta <- ncol(omega_subset)

          gen_eta <- MASS::mvrnorm(
            n = 1000,
            mu = rep(0, n_eta),
            Sigma = omega_subset
          ) %>%
            as_tibble() %>%
            setNames(par_pop_valid$PAR) %>%
            tidyr::pivot_longer(cols = dplyr::all_of(par_pop_valid$PAR), names_to = "PAR", values_to = "ETA") %>%
            dplyr::left_join(
              par_pop_valid %>% dplyr::select(PAR, TV, DISTRIBUTION),
              by = "PAR"
            ) %>%
            dplyr::mutate(
              dist_l = tolower(trimws(as.character(DISTRIBUTION))),
              TV_b = pmin(pmax(TV, .Machine$double.eps), 1 - .Machine$double.eps),
              TDIST = dplyr::case_when(
                dist_l == "lognormal" ~ TV * exp(ETA),
                dist_l == "normal" ~ TV + ETA,
                dist_l == "logitnormal" ~ plogis(qlogis(TV_b) + ETA),
                TRUE ~ NA_real_
              )
            ) %>%
            dplyr::filter(!is.na(TDIST)) %>%
            dplyr::left_join(ind_par_dist %>% dplyr::select(PAR, LABEL) %>% dplyr::distinct(), by = "PAR")

          if (nrow(gen_eta) > 0) {
            p_i <- p_i + geom_density(
              data = gen_eta,
              aes(x = TDIST),
              linewidth = 0.8,
              lty = "dashed") +
              facet_wrap(~LABEL, scales = "free")
          }
        }

      } else{
        par_pop <- smrg_obj$SUMTAB %>%
          filter(str_detect(PAR, "^omega_")) %>%
          mutate(PAR_f = str_remove(PAR, "^omega_")) %>%
          filter(PAR_f %in% str_remove(par_seq, "^eta_")) %>%
          select(PAR, TV = VALUE)

        # colnames(omegamat_i) <- sub("^omega_", "", colnames(omegamat_i))
        # rownames(omegamat_i) <- sub("^omega_", "", rownames(omegamat_i))
        rownames(omegamat_i) <- colnames(omegamat_i)
        omega_subset <- omegamat_i[par_pop$PAR, par_pop$PAR, drop = FALSE]

        n_eta <- ncol(omega_subset)

        gen_eta <- MASS::mvrnorm(
          n = 1000,
          mu = rep(0, n_eta),
          Sigma = omega_subset
        ) %>%
          as_tibble() %>%
          setNames(par_pop$PAR) %>%
          pivot_longer(cols = everything(), names_to = "PAR", values_to = "ETA") %>%
          left_join(par_pop, by = "PAR") %>%
          mutate(TDIST =  ETA)%>%
          mutate(PAR = gsub("omega_", "eta_", PAR))

        p_i <- p_i + geom_density(
          data = gen_eta,
          aes(x = TDIST),
          linewidth = 0.8,
          lty = "dashed") +
          facet_wrap(~PAR, scales = "free")
      }









    }


    if (plot_type == 'QQ') {
      eta_cols_all <- names(patab_i)[grepl("^eta_", names(patab_i))]

      if (is.null(par_seq)) {
        eta_selected <- eta_cols_all
      } else {
        # normalize to eta_*
        par_seq_eta <- ifelse(grepl("^eta_", par_seq),
                              par_seq,
                              paste0("eta_", par_seq))

        # check existence
        missing_par <- setdiff(par_seq_eta, names(patab_i))
        if (length(missing_par) > 0) {
          warning(paste("Parameters not found in PATAB:", paste(missing_par, collapse = ", ")))
        }

        eta_selected <- intersect(par_seq_eta, eta_cols_all)
      }
      eta_df <- patab_i %>%
        select(ID, all_of(eta_selected)) %>%
        pivot_longer(cols = -ID, names_to = "PAR", values_to = "ETA")

      p_i <- ggplot(eta_df, aes(sample = ETA)) +
        stat_qq(size = 1.75, color = MSDcol[1], alpha = 0.8) +
        stat_qq_line(col = MSDcol[3]) +
        labs(x = "Theoretical quantiles", y = "Sample quantiles") +
        geom_abline(linewidth = 0.5, col = "black", linetype = "dashed") +
        scale_x_continuous(breaks = scales::pretty_breaks(7)) +
        scale_y_continuous(breaks = scales::pretty_breaks(7)) +
        theme_bw() +
        facet_wrap(~PAR, scales = "free")
    }
  })
  return(p_i)
}

## Author: Ugolkov Yaroslav
## First created: 2025-09-05
## Description: formal testing of sg-gof-par-dist function and its helpers
## Keywords: SimuRg, sg-gof-par-dist, goodness-of-fit

test_that("sg-gof-par-dist output is correct", {
  x <- sg_gof_par_dist(obj1, tdist =F)
  expect_true(inherits(x, "ggplot"))
  expect_snapshot(ggplot2::layer_data(x))
})

test_that("sg-gof-par-dist does not work", {
  expect_error(sg_gof_par_dist(data.frame()))
  # expect_error(sg_gof_par_dist(obj1, plot_type = "invalid_type"))
})

test_that("sg-gof-par-dist file load", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  x <- sg_gof_par_dist(fpath_i, tdist = TRUE, plot_type = 'QQ')
  expect_true(inherits(x, "ggplot"))
  # expect_snapshot(ggplot2::layer_data(x))
})

#' Visualize global sensitivity analysis results
#'
#' @description
#' Generates visualization plots for results produced by
#' `sg_globalsens_sim()`. Supported visualizations:
#'
#' - PRCC: heatmap or barplot
#' - eFAST: barplot of first and total order indices
#'
#' @param x Result object returned by `sg_globalsens_sim()`
#' @param type Plot type for PRCC: `"heatmap"` or `"bar"`
#' @param params Optional vector of parameters to display
#' @param vars Optional vector of output variables
#' @param stats Optional vector of statistics to display
#'
#' @return ggplot object
#'
#' @examples
#' \donttest{
#' # Example model
#' library(rxode2)
#' library(tibble)
#' source(system.file("extdata", "RxODE_model", "example_rxode_model.R", package = "SimuRg")) # mod_ex
#'
#'
#' # Set up event table
#' et_base <- tribble(
#'   ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
#'   1,   0,     1,     1,    10,   2,     24,  112,   1
#' )
#'
#' # Define parameter bounds
#' inits <- rxInits(mod_ex)
#' par_bounds <- tibble::tibble(
#'   PAR = c("POPCL", "POPVC"),
#'   LB  = inits[c("POPCL", "POPVC")] * (1 - 0.9),
#'   UB  = inits[c("POPCL", "POPVC")] * (1 + 0.9)
#' )
#'
#' # Run eFAST sensitivity analysis
#' res_efast <- sg_globalsens_sim(
#'   method = c("eFAST"),
#'   model = mod_ex,
#'   params = c("POPCL"),
#'   par_bounds = par_bounds,
#'   n_sim = 100,
#'   stimes = seq(0, 168, 10),
#'   output = "Cc",
#'   cov = c("IGFR", "POPN"),
#'   et = et_base,
#'   stat_comp = c("mean")
#' )
#'
#'   efast_p <- sg_globalsens_vis(
#'     res_efast,
#'     params = c("POPCL"),
#'     vars = "Cc",
#'     stats = c("mean")
#'   )
#'
#'   efast_p
#' # Run PRCC sensitivity analysis
#' res_prcc <- sg_globalsens_sim(
#'   method = c("PRCC"),
#'   model = mod_ex,
#'   params = c("POPCL", "POPVC"),
#'   par_bounds = par_bounds,
#'   n_sim = 100,
#'   stimes = seq(0, 168, 10),
#'   output = "Cc",
#'   cov = c("IGFR", "POPN"),
#'   et = et_base,
#'   stat_comp = c("mean")
#' )
#'
#'   prcc_p <- sg_globalsens_vis(
#'     res_prcc,
#'     type = c("heatmap"),
#'     params = c("POPCL", "POPVC"),
#'     vars = "Cc",
#'     stats = c("mean")
#'   )
#'
#'   prcc_p
#' }
#'
#' @import ggplot2 dplyr tidyr
#' @export
sg_globalsens_vis <- function(x,
                              type = c("heatmap", "bar"),
                              params = NULL,
                              vars = NULL,
                              stats = NULL) {

  type <- match.arg(type)

  if (!"result" %in% names(x))
    stop("Input must be an object returned by sg_globalsens_sim()")

  df <- x$result

  # Detect method
  method <- if ("TYPE" %in% names(df) && "VALUE" %in% names(df)) {
    "eFAST"
  } else {
    "PRCC"
  }

  if (!is.null(params)) df <- df %>% dplyr::filter(PAR %in% params)
  if (!is.null(vars))   df <- df %>% dplyr::filter(VAR %in% vars)
  if (!is.null(stats))  df <- df %>% dplyr::filter(STAT %in% stats)

  # Create combined label
  if ("STAT" %in% names(df))
    df <- df %>%
    mutate(VAR_STAT = paste(VAR, STAT, sep = "_"))
  else
    df$VAR_STAT <- df$VAR


  # PRCC VISUALISATIONS
  if (method == "PRCC") {

    if (type == "heatmap") {

      p <- ggplot(df,
                  aes(x = PAR, y = VAR_STAT, fill = estimate)) +
        geom_tile(color = "grey25") +
        geom_text(aes(label = round(estimate, 2)), size = 4) +
        scale_fill_gradient2(
          low = MSDcol[3],
          mid = MSDcol[2],
          high = MSDcol[1],
          midpoint = 0,
          limits = c(-1, 1),
          name = "PRCC"
        ) +
        labs(
          x = "Parameters",
          y = "Output / statistic"
        ) +
        scale_x_discrete(expand = c(0,0)) +
        scale_y_discrete(expand = c(0,0)) +
        theme_minimal(base_size = 14)

    } else {

      p <- ggplot(df,
                  aes(x = PAR, y = estimate, fill = VAR_STAT)) +
        geom_hline(yintercept = 0, linetype = "dashed") +
        geom_bar(stat = "identity",
                 position = position_dodge()) +
        geom_text(aes(label = round(estimate,2),
                      vjust = ifelse(estimate > 0, -0.6, 1.3)),
                  position = position_dodge(0.9),
                  size = 4) +
        scale_fill_manual(values = MSDcol) +
        ylim(-1.1, 1.1) +
        labs(
          x = "Parameters",
          y = "Partial rank correlation coefficient",
          fill = "Output"
        ) +
        theme_minimal(base_size = 14)

    }

  }


  # EFAST VISUALISATION
  if (method == "eFAST") {

    p <- ggplot(df,
                aes(x = PAR, y = VALUE, fill = TYPE)) +
      geom_bar(stat = "identity",
               position = "dodge",
               color = "black") +
      geom_hline(yintercept = 0) +
      facet_grid(STAT ~ VAR) +
      scale_fill_manual(values = c(MSDcol[2],MSDcol[3])) +
      scale_y_continuous(
        limits = c(-0.1, 1),
        breaks = seq(0, 1, 0.1),
        expand = c(0, 0)
      ) +
      labs(
        x = "Parameters",
        y = "eFAST sensitivity index",
        fill = "Index type"
      ) +
      theme_minimal(base_size = 14)

  }

  return(p)
}

mod_ex <- RxODE({
  # Doses in mg
  # Time in hours

  ### Parameter values
  # Typical
  POPCL = 5;
  POPVC = 180;
  POPQ = 7;
  POPVP = 52;
  POPKTR = 6;
  FBIOPAR = 1;

  # Covariate coefficients
  POPIGFRCOV = 0.8;
  POPPATCOVCLGR1 = 0.85;
  POPPATCOVCLGR2 = 0.85;
  POPPATCOVCLGR3 = 0.85;
  POPPATCOVCLGR4 = 0.85;

  # Random effects
  PPVCL = 0;
  PPVVC = 0;
  PPVKTR = 0;

  # Residual error
  RUV = 0;

  ### Covariates
  PATCOVCL = 1
  if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
  if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
  if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
  if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
  IGFRCOV = (IGFR/112)^POPIGFRCOV

  ### Parameters
  CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
  VC = POPVC * exp(PPVVC);
  Q = POPQ;
  VP = POPVP;
  KTR = POPKTR * exp(PPVKTR);

  ### Explicit functions
  Cc = Ac/VC;                 # nmol/L
  Cp = Ap/VP;                 # nmol/L

  ### Initial conditions
  At1(0) = 0;         # mg
  At2(0) = 0;         # mg
  At3(0) = 0;         # mg
  At4(0) = 0;         # mg
  At5(0) = 0;         # mg
  At6(0) = 0;         # mg
  Ad(0) = 0;          # mg
  Ac(0) = 0;          # mg
  Ap(0) = 0;          # mg

  ### ODEs
  d/dt(At1) = - KTR*At1;
  d/dt(At2) = KTR*At1 - KTR*At2;
  d/dt(At3) = KTR*At2 - KTR*At3;
  d/dt(At4) = KTR*At3 - KTR*At4;
  d/dt(At5) = KTR*At4 - KTR*At5;
  d/dt(At6) = KTR*At5 - KTR*At6;
  d/dt(Ad) = KTR*At6 - KTR*Ad;
  d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
  d/dt(Ap) = Q*(Cc - Cp);

  FBIO = FBIOPAR
  f(At1) = FBIO*1000000/505;
  CHECKRUV = RUV;
  Cc_ResErr = Cc + RUV*Cc;
})
et_base <- tribble(
  ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
  1,   0,     1,     1,    10,   2,     24,  112,   1
)

inits <- rxInits(mod_ex)
params_ext <- c("POPCL", "POPVC", "POPQ")

par_bounds_ext <- tibble::tibble(
  PAR = params_ext,
  LB  = inits[params_ext] * 0.2,
  UB  = inits[params_ext] * 1.8
)

sim_prcc <- sg_globalsens_sim(
  method = "PRCC",
  model = mod_ex,
  params = params_ext,
  par_bounds = par_bounds_ext,
  n_sim = 120,
  stimes = seq(0, 168, 12),
  output = c("Cc", "Cp"),
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean", "max", "sd")
)

sim_efast <- sg_globalsens_sim(
  method = "eFAST",
  model = mod_ex,
  params = params_ext,
  par_bounds = par_bounds_ext,
  n_sim = 120,
  stimes = seq(0, 168, 12),
  output = c("Cc", "Cp"),
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean", "max", "sd")
)

test_that("PRCC: full combinatorial structure is preserved", {

  df <- sim_prcc$result

  expected_n <- length(unique(df$PAR)) *
    length(unique(df$VAR)) *
    length(unique(df$STAT))

  p <- sg_globalsens_vis(sim_prcc)

  expect_equal(nrow(p$data), expected_n)
})

test_that("eFAST: full combinatorial structure includes TYPE dimension", {

  df <- sim_efast$result

  expected_n <- length(unique(df$PAR)) *
    length(unique(df$VAR)) *
    length(unique(df$STAT)) *
    length(unique(df$TYPE))

  p <- sg_globalsens_vis(sim_efast)

  expect_equal(nrow(p$data), expected_n)
})

test_that("eFAST: each facet has all parameters and TYPE levels", {

  p <- sg_globalsens_vis(sim_efast)

  df <- p$data

  check <- df %>%
    group_by(STAT, VAR) %>%
    summarise(
      n_par = n_distinct(PAR),
      n_type = n_distinct(TYPE),
      .groups = "drop"
    )

  expect_true(all(check$n_par == length(unique(df$PAR))))
  expect_true(all(check$n_type == length(unique(df$TYPE))))
})

test_that("Filtering works correctly in high-dimensional PRCC", {

  p <- sg_globalsens_vis(
    sim_prcc,
    params = c("POPCL", "POPQ"),
    vars = "Cc",
    stats = c("mean", "max")
  )

  df <- p$data

  expect_setequal(unique(df$PAR), c("POPCL", "POPQ"))
  expect_setequal(unique(df$VAR), "Cc")
  expect_setequal(unique(df$STAT), c("mean", "max"))
})

test_that("PRCC heatmap result type is correct", {

  p <- sg_globalsens_vis(
    sim_prcc,
    type = c("heatmap"),
    params = c("POPCL", "POPQ"),
    vars = "Cc",
    stats = c("mean", "max", "sd")
  )

  expect_true(inherits(p, "ggplot"))
  #expect_snapshot(ggplot2::layer_data(p))
})

test_that("PRCC barplot result type is correct", {

  p <- sg_globalsens_vis(
    sim_prcc,
    type = c("bar"),
    params = c("POPCL", "POPQ", "POPVC"),
    vars = c("Cc","Cp"),
    stats = c("max", "sd")
  )

  expect_true(inherits(p, "ggplot"))
  #expect_snapshot(ggplot2::layer_data(p))
})

test_that("eFAST barplot result type is correct", {

  p <- sg_globalsens_vis(
    sim_efast,
    params = c("POPCL", "POPQ", "POPVC"),
    vars = "Cc",
    stats = c("mean", "max", "sd")
  )

  expect_true(inherits(p, "ggplot"))
  #expect_snapshot(ggplot2::layer_data(p))
})

#' @description
#' SimuRg is an R toolkit for PK/PD modeling and simulation workflows, with a focus on importing, organizing, and exploring nonlinear mixed-effects model results.
#' @keywords internal

"_PACKAGE"

rlang::on_load(local_use_cli())

# Singletons
the <- rlang::new_environment()

## Author: Victor Sokolov
## First created: 2025-11-13
## Description: Time-profile plotting with optional summary bands
## Keywords: SimuRg, time profiles, diagnostics


#' Time-profile plotting with optional summary bands
#'
#' Creates a ggplot time-profile plot from longitudinal data with optional
#' summarization (mean/median/geometric mean), variance bands (SD/SE/IQR),
#' percentile ribbons, faceting, and log scaling.
#'
#' @inheritParams sg_dummy
#' @param bands_i vector of characters. Character vector of length 2 with column names for custom ymin/ymax ribbons. Default is `NULL`
#' @param cent_i string. Central tendency measure. One of `'mean'`, `'median'`, `'geom_mean'`, or `NULL` for raw values. Default is `NULL`
#' @param vrns_i string. Variance band type. One of `'SD'`, `'SE'`, `'IQR'`, or `NULL`. Default is `NULL`
#' @param lperc_i numeric. Lower percentile for percentile ribbons (must be provided together with `uperc_i`). Default is `NULL`
#' @param uperc_i numeric. Upper percentile for percentile ribbons (must be provided together with `lperc_i`). Default is `NULL`
#' @param add_points numeric. Size of points to add to the plot. If > 0, points will be added. Default is `0`
#' @param grid_i string. Faceting formula for `facet_grid` (e.g., `'~VAR'` or `'A~B'`). Default is `NULL`
#'
#' @return A ggplot object with lines and optional ribbons/points/facets.
#'
#' @import ggplot2
#' @import dplyr
#' @importFrom rlang sym syms expr
#'
#' @examples
#' make_extended_mock_data <- function() {
#'  data.frame(
#'    TIME = rep(1:4, times = 6),
#'    VALUE = rnorm(24, mean = 10, sd = 2),
#'    VAR = rep(rep(c("A", "B"), each = 4), times = 3),
#'    Regimen = rep(c("R1", "R2", "R3"), each = 8),
#'    CAT1 = rep(c("Cat1", "Cat2"), each = 12)
#'  )
#' }
#' ds_sim <- make_extended_mock_data()
#' p <- sg_sim_tp(ds_i = ds_sim, group_i = 'VAR', col_i = 'VAR', fill_i = 'VAR',
#'                wrap_i = '~VAR', wrap_ncol = 2)
#' @export
sg_sim_tp <- function(
  ds_i,
  time_col = 'TIME',
  val_col = 'VALUE',
  group_i = 'VAR',
  bands_i = NULL,
  cent_i = NULL,
  vrns_i = NULL,
  lperc_i = NULL,
  uperc_i = NULL,
  add_points = 0,
  col_i = NULL,
  fill_i = NULL,
  lty_i = NULL,
  shp_i = NULL,
  grid_i = NULL,
  wrap_i = NULL,
  free_stat = 'free',
  wrap_ncol = NULL,
  wrap_nrow = NULL,
  min_x = NA,
  max_x = NA,
  min_y = NA,
  max_y = NA,
  log_y = FALSE,
  log_x = FALSE
) {

  # Determine all grouping columns
  group_cols <- unique(c(group_i, col_i, fill_i, lty_i, shp_i))
  group_cols <- group_cols[!is.null(group_cols)]

  # Check if summarization is needed
  needs_summary <- !is.null(cent_i) || !is.null(vrns_i) || !is.null(lperc_i) || !is.null(uperc_i)

  if (needs_summary) {
    # Prepare grouping for summarization
    group_vars <- c(time_col, group_cols)

    # Summarize data
    ds_plot <- ds_i %>%
      group_by(across(all_of(group_vars))) %>%
      summarise(
        n = n(),
        mean_val = if (!is.null(cent_i) && cent_i == "mean") mean(!!sym(val_col), na.rm = TRUE) else NA_real_,
        median_val = if (!is.null(cent_i) && cent_i == "median") median(!!sym(val_col), na.rm = TRUE) else NA_real_,
        geom_mean_val = if (!is.null(cent_i) && cent_i == "geom_mean") exp(mean(log(!!sym(val_col)), na.rm = TRUE)) else NA_real_,
        sd_val = if (!is.null(vrns_i) && vrns_i == "SD") sd(!!sym(val_col), na.rm = TRUE) else NA_real_,
        se_val = if (!is.null(vrns_i) && vrns_i == "SE") sd(!!sym(val_col), na.rm = TRUE) / sqrt(n()) else NA_real_,
        q25 = if (!is.null(vrns_i) && vrns_i == "IQR") quantile(!!sym(val_col), 0.25, na.rm = TRUE) else NA_real_,
        q75 = if (!is.null(vrns_i) && vrns_i == "IQR") quantile(!!sym(val_col), 0.75, na.rm = TRUE) else NA_real_,
        lperc = if (!is.null(lperc_i)) quantile(!!sym(val_col), lperc_i, na.rm = TRUE) else NA_real_,
        uperc = if (!is.null(uperc_i)) quantile(!!sym(val_col), uperc_i, na.rm = TRUE) else NA_real_,
        .groups = 'drop'
      )

    # Calculate central tendency value
    if (!is.null(cent_i)) {
      if (cent_i == "mean") {
        ds_plot <- ds_plot %>% mutate(y_central = mean_val)
      } else if (cent_i == "median") {
        ds_plot <- ds_plot %>% mutate(y_central = median_val)
      } else if (cent_i == "geom_mean") {
        ds_plot <- ds_plot %>% mutate(y_central = geom_mean_val)
      }
    } else {
      # If no central tendency, use original values (shouldn't happen in summary mode)
      ds_plot <- ds_plot %>% mutate(y_central = mean_val)
    }

    # Calculate variance bands if specified
    if (!is.null(vrns_i)) {
      if (vrns_i == "SD") {
        ds_plot <- ds_plot %>%
          mutate(
            y_lower = y_central - sd_val,
            y_upper = y_central + sd_val
          )
      } else if (vrns_i == "SE") {
        ds_plot <- ds_plot %>%
          mutate(
            y_lower = y_central - se_val,
            y_upper = y_central + se_val
          )
      } else if (vrns_i == "IQR") {
        ds_plot <- ds_plot %>%
          mutate(
            y_lower = q25,
            y_upper = q75
          )
      }
    }

    # Add percentile bands if specified
    if (!is.null(lperc_i) && !is.null(uperc_i)) {
      ds_plot <- ds_plot %>%
        mutate(
          y_lower_perc = lperc,
          y_upper_perc = uperc
        )
    }

  } else {
    # No summarization needed - use raw data
    ds_plot <- ds_i %>%
      rename(y_central = !!sym(val_col))
  }

  # Initialize ggplot
  p <- ggplot(ds_plot, aes(x = !!sym(time_col), y = y_central))

  # Add ribbon for bands_i if specified
  if (!is.null(bands_i) && length(bands_i) == 2) {
    ribbon_mapping <- aes(ymin = !!sym(bands_i[1]), ymax = !!sym(bands_i[2]))
    if (!is.null(fill_i)) {
      ribbon_mapping$fill <- sym(fill_i)
    }
    p <- p + geom_ribbon(ribbon_mapping, alpha = 0.3)
  }

  # Add ribbon for percentile bands
  if (!is.null(lperc_i) && !is.null(uperc_i) && needs_summary) {
    ribbon_mapping <- aes(ymin = y_lower_perc, ymax = y_upper_perc)
    if (!is.null(fill_i)) {
      ribbon_mapping$fill <- sym(fill_i)
    }
    p <- p + geom_ribbon(ribbon_mapping, alpha = 0.3)
  }

  # Add ribbon for variance bands
  if (!is.null(vrns_i) && needs_summary) {
    ribbon_mapping <- aes(ymin = y_lower, ymax = y_upper)
    if (!is.null(fill_i)) {
      ribbon_mapping$fill <- sym(fill_i)
    }
    p <- p + geom_ribbon(ribbon_mapping, alpha = 0.3)
  }

  # Build aesthetic mappings for line
  line_mapping <- aes()
  if (!is.null(col_i)) {
    line_mapping$colour <- sym(col_i)
  }
  if (!is.null(lty_i)) {
    line_mapping$linetype <- sym(lty_i)
  }
  if (length(group_cols) > 0) {
    line_mapping$group <- expr(interaction(!!!syms(group_cols)))
  }

  # Add line layer
  p <- p + geom_line(line_mapping, linewidth = 0.8)

  # Add points if requested
  if (add_points > 0) {
    point_mapping <- aes()
    if (!is.null(col_i)) {
      point_mapping$colour <- sym(col_i)
    }
    if (!is.null(shp_i)) {
      point_mapping$shape <- sym(shp_i)
    }
    p <- p + geom_point(point_mapping, size = add_points)
  }

  # Add faceting
  if (!is.null(grid_i)) {
    p <- p + facet_grid(as.formula(grid_i), scales = free_stat)
  } else if (!is.null(wrap_i)) {
    p <- p + facet_wrap(as.formula(wrap_i), scales = free_stat,
                        ncol = wrap_ncol, nrow = wrap_nrow)
  }

  # Apply log scales
  if (!is.na(log_y) && log_y) {
    p <- p + scale_y_log10(breaks = scales::trans_breaks("log10", function(x) 10^x),
                           labels = scales::trans_format("log10", scales::math_format(10^.x)))
  }

  if (!is.na(log_x) && log_x) {
    p <- p + scale_x_log10(breaks = scales::trans_breaks("log10", function(x) 10^x),
                           labels = scales::trans_format("log10", scales::math_format(10^.x)))
  }

  # Apply coordinate limits
  xlim_vec <- c(
    if (!is.na(min_x)) min_x else NA,
    if (!is.na(max_x)) max_x else NA
  )

  ylim_vec <- c(
    if (!is.na(min_y)) min_y else NA,
    if (!is.na(max_y)) max_y else NA
  )

  if (!all(is.na(xlim_vec)) || !all(is.na(ylim_vec))) {
    p <- p + coord_cartesian(
      xlim = if (!all(is.na(xlim_vec))) xlim_vec else NULL,
      ylim = if (!all(is.na(ylim_vec))) ylim_vec else NULL
    )
  }

  return(p)
}

## Author: Alina Melnikova
## Description: basic correctness checks for sg_sim_tp
## Keywords: sg_sim_tp, visualization, diagnostics

library(testthat)
theme_set(theme_bw())
theme_update(panel.grid.minor = element_blank())

# Helper: small mock dataset
make_mock_data <- function() {
  set.seed(123)  # For reproducibility
  data.frame(
    TIME = rep(rep(1:4, each = 3), times = 2),  # 3 replicates per time point
    VALUE = c(
      rnorm(12, mean = 10, sd = 1),  # VAR A: 4 time points * 3 replicates
      rnorm(12, mean = 8, sd = 1.5)  # VAR B: 4 time points * 3 replicates
    ),
    VAR = rep(c("A", "B"), each = 12),
    GRP = rep(c("X", "Y"), each = 12)
  )
}
# Helper: extended mock dataset with additional grouping variables
make_extended_mock_data <- function() {
  data.frame(
    TIME = rep(1:4, times = 6),
    VALUE = rnorm(24, mean = 10, sd = 2),
    VAR = rep(rep(c("A", "B"), each = 4), times = 3),
    Regimen = rep(c("R1", "R2", "R3"), each = 8),
    CAT1 = rep(c("Cat1", "Cat2"), each = 12)
  )
}


test_that("sg_sim_tp returns ggplot with raw data", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()
  p <- sg_sim_tp(ds_i = df, group_i = "VAR", col_i = "VAR")

  expect_s3_class(p, "ggplot")
})

test_that("sg_sim_tp handles mean + SD bands", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()
  p <- sg_sim_tp(
    ds_i = df,
    group_i = "VAR",
    cent_i = "mean",
    vrns_i = "SD",
    col_i = "VAR",
    fill_i = "VAR"
  )

  gb <- ggplot2::ggplot_build(p)
  # Expect at least one ribbon layer when variance bands requested
  has_ribbon <- any(vapply(gb$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon)
})

test_that("sg_sim_tp handles percentile ribbons", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()
  p <- sg_sim_tp(
    ds_i = df,
    group_i = "VAR",
    cent_i = "median",
    lperc_i = 0.1,
    uperc_i = 0.9,
    col_i = "VAR",
    fill_i = "VAR"
  )

  gb <- ggplot2::ggplot_build(p)
  has_ribbon <- any(vapply(gb$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon)
})



test_that("sg_sim_tp applies axis limits", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()
  p <- sg_sim_tp(ds_i = df, min_x = 0, max_x = 5, min_y = 0, max_y = 15)

  xlim <- p$coordinates$limits$x
  ylim <- p$coordinates$limits$y
  expect_equal(xlim, c(0, 5))
  expect_equal(ylim, c(0, 15))
})

test_that("sg_sim_tp supports wrap faceting", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()
  p <- sg_sim_tp(ds_i = df, wrap_i = "~VAR")

  # Check that faceting was applied by examining the plot's facet object
  # facet_wrap creates a FacetWrap object
  has_facet <- !is.null(p$facet) &&
    (inherits(p$facet, "FacetWrap") || inherits(p$facet, "FacetGrid"))

  expect_true(has_facet, "Plot should have faceting applied")

  # Alternative check: verify the built plot has multiple panels
  gb <- ggplot2::ggplot_build(p)
  n_panels <- length(gb$layout$panel_params)
  expect_true(n_panels > 1, "Faceted plot should have multiple panels")
})

test_that("sg_sim_tp handles different variance types (SD, SE, IQR)", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()

  # Test SD
  p_sd <- sg_sim_tp(ds_i = df, cent_i = 'mean', vrns_i = 'SD', col_i = 'VAR', fill_i = 'VAR')
  gb_sd <- ggplot2::ggplot_build(p_sd)
  has_ribbon_sd <- any(vapply(gb_sd$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon_sd, "SD variance should create ribbons")

  # Test SE
  p_se <- sg_sim_tp(ds_i = df, cent_i = 'mean', vrns_i = 'SE', col_i = 'VAR', fill_i = 'VAR')
  gb_se <- ggplot2::ggplot_build(p_se)
  has_ribbon_se <- any(vapply(gb_se$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon_se, "SE variance should create ribbons")

  # Test IQR
  p_iqr <- sg_sim_tp(ds_i = df, cent_i = 'median', vrns_i = 'IQR', col_i = 'VAR', fill_i = 'VAR')
  gb_iqr <- ggplot2::ggplot_build(p_iqr)
  has_ribbon_iqr <- any(vapply(gb_iqr$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon_iqr, "IQR variance should create ribbons")
})

test_that("sg_sim_tp handles different central tendency measures", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()

  # Test mean
  p_mean <- sg_sim_tp(ds_i = df, cent_i = 'mean', col_i = 'VAR')
  expect_s3_class(p_mean, "ggplot")

  # Test median
  p_median <- sg_sim_tp(ds_i = df, cent_i = 'median', col_i = 'VAR')
  expect_s3_class(p_median, "ggplot")

  # Test geometric mean
  df_pos <- df
  df_pos$VALUE <- abs(df_pos$VALUE) + 1  # Ensure positive values for geometric mean
  p_geom <- sg_sim_tp(ds_i = df_pos, cent_i = 'geom_mean', col_i = 'VAR')
  expect_s3_class(p_geom, "ggplot")
})

test_that("sg_sim_tp handles median + IQR with custom axis limits", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_extended_mock_data()
  p <- sg_sim_tp(
    ds_i = df,
    cent_i = 'median',
    vrns_i = 'IQR',
    col_i = 'CAT1',
    fill_i = 'CAT1',
    min_x = 0,
    max_x = 100,
    min_y = 0
  )

  expect_s3_class(p, "ggplot")

  # Check that axis limits were applied
  xlim <- p$coordinates$limits$x
  ylim <- p$coordinates$limits$y

  expect_false(is.null(xlim), "x limits should be set")
  expect_equal(xlim, c(0, 100), tolerance = 1e-6)

  expect_false(is.null(ylim), "min_y should be set")
  expect_equal(ylim[1], 0, tolerance = 1e-6)

  # Check that IQR ribbons exist
  gb <- ggplot2::ggplot_build(p)
  has_ribbon <- any(vapply(gb$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon, "Plot should have IQR variance ribbons")
})

test_that("sg_sim_tp handles mean + SE with points, grid faceting, and log scale", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_extended_mock_data()
  p <- sg_sim_tp(
    ds_i = df,
    cent_i = 'mean',
    vrns_i = 'SE',
    col_i = 'Regimen',
    fill_i = 'Regimen',
    add_points = 2,
    grid_i = '~VAR',
    log_y = TRUE
  )

  expect_s3_class(p, "ggplot")

  # Check that grid faceting was applied
  has_facet <- !is.null(p$facet) && inherits(p$facet, "FacetGrid")
  expect_true(has_facet, "Plot should have grid faceting")

  # Check that points were added
  gb <- ggplot2::ggplot_build(p)
  has_points <- any(vapply(gb$plot$layers, function(l) inherits(l$geom, "GeomPoint"), logical(1)))
  expect_true(has_points, "Plot should have points when add_points > 0")

  # Check that SE ribbons exist
  has_ribbon <- any(vapply(gb$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon, "Plot should have SE variance ribbons")

  # Check that log scale was applied
  y_scale <- p$scales$get_scales("y")
  expect_false(is.null(y_scale), "y scale should exist when log_y = TRUE")
})

test_that("sg_sim_tp handles median with 5th-95th percentile bands and wrap faceting", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  df <- make_mock_data()
  p <- sg_sim_tp(
    ds_i = df,
    cent_i = 'median',
    lperc_i = 0.05,
    uperc_i = 0.95,
    col_i = 'VAR',
    fill_i = 'VAR',
    wrap_i = '~VAR',
    wrap_ncol = 2
  )

  expect_s3_class(p, "ggplot")

  # Check that faceting was applied
  has_facet <- !is.null(p$facet) && inherits(p$facet, "FacetWrap")
  expect_true(has_facet, "Plot should have wrap faceting")

  # Check that percentile ribbons exist
  gb <- ggplot2::ggplot_build(p)
  has_ribbon <- any(vapply(gb$plot$layers, function(l) inherits(l$geom, "GeomRibbon"), logical(1)))
  expect_true(has_ribbon, "Plot should have percentile ribbons")

  # Check wrap_ncol parameter
  gb <- ggplot2::ggplot_build(p)
  n_panels <- length(gb$layout$panel_params)
  expect_true(n_panels > 1, "Faceted plot should have multiple panels")
})

utils::globalVariables(c(":=", ".", "..density..", ".err", ".header", ".idx", ".x",
                         "95% CI", "90%CI", "AMT", "ANOVA", "ATSLD", "BCOVVAL",
                         "BIN", "BIN_sort", "BTR", "CATDES", "CI_label", "COHORTC", "COL", "CORR_PAR", "COV",
                         "COVNAME", "COVVAL", "CV", "CV % (95% CI)", "CWRES",
                         "DISTRIBUTION", "Distribution",
                         "DT_label", "DV", "DVID", "EST", "Estimate", "ETA",
                         "ETAshrinkage_sd", "ETAshrinkage_var", "EVID", "ID",
                         "First order", "INIT", "IPRED", "IRES",
                         "IWRES", "KEY", "LAB", "LABEL", "LARGPVAL", "LCI", "LP", "MDV", "METRIC",
                         "NAME", "NAME_INIT", "NICEN", "NTILE", "NVALUE", "OBS", "OCCMAN", "OUT","P05", "P95", "P975", "PAR", "PAR1",
                         "PAR2", "PAR_NAME", "PAR_f", "Parameter", "PARVAL", "PAR_group", "PCH",
                         "PARVAL_NORM", "PARNAME", "PEARS", "PI", "PNAME", "POPN",
                         "PPRED", "PPRED_bin", "PRED", "PVAL", "Project_name",
                         "R_diff", "REF_spec", "REFVAL", "REF", "RES", "Regimen",
                         "RSE", "RUVpar_a", "RUVpar_b", "RUVpars", "Run_ID", "SE",
                         "SIGNPEARS", "Shrinkage", "Shrinkage_var", "Shrinkage (var), %",
                         "Source", "STAT", "TDIST", "TIME", "TIME_BIN", "TIME_BIN_max",
                         "TIME_BIN_max_prev", "TIME_BIN_min", "TIME_BIN_min_next", "TR",
                         "TRANS", "TSLD", "Total order", "TV", "TVALUE", "TYPE",
                         "Type", "UCI", "UP", "VALUE", "VAR", "VAR_STAT",
                         "Var1", "Var2", "Var_epsilon", "Var_mc", "Var_total",
                         "WRES", "X", "Y", "DVNAME", "aov", "as.formula", "category", "cmax",
                         "cmax_ref", "combn", "cor", "cor.test", "data", "density",
                         "distribution",
                         "dnorm", "dvid", "estimate", "errorModel", "facet_label",
                         "geom_mean_val", "id", "inside",
                         "kmeans", "ks.test", "key", "label_part", "lperc", "max_val", "mean_val",
                         "median", "median_ci_l", "median_ci_u", "median_median",
                         "median_val", "min_BIN_time", "min_val", "na.omit", "name", "p",
                         "parameter", "pi_l", "pi_l_ci_l", "pi_l_ci_u", "pi_l_median",
                         "pi_u", "pi_u_ci_l", "pi_u_ci_u", "pi_u_median", "popPred", "prediction",  "pwRes",
                         "q25", "q75", "quantile", "quantiles", "r", "read.csv", "reorder",
                         "residual_type", "rnorm", "row_id", "sd", "sd_val",
                         "se_val", "setNames", "sim.id", "theor_LCI", "theor_UCI",
                         "theor_median", "time", "toJSON",
                         "type", "use", "uperc","value", "value_at_time",
                         "value_cfb", "value_ref", "var", "x", "y", "y_central",
                         "y_lower", "y_lower_perc", "y_upper", "y_upper_perc"))
#' The function to store common parameters description
#'
#' @param abreaks A function that generates axis breaks. Default is `scales::pretty_breaks(7)`.
#' @param addcov Logical. If `TRUE`, columns with covariate values will be added
#'  to the resulting dataset. Default is `TRUE`.
#' @param addOFV Logical. If `TRUE`, information about OFV and AIC of the model
#'  will be added. Default is `TRUE`.
#' @param addline Logical. If `TRUE`, lines connecting observations of
#'  individual subjects will be added. Default is `TRUE`.
#' @param aggr A string that specifies the aggregation type. Set to `NULL` for
#'  no aggregation, `ID` for aggregation by time, population and ID, `NPOP` for
#'  aggregation by population and time, `TIME` for aggregation by time.
#'  Default is `NULL`.
#' @param alpha_i Numeric. Transparency level (from 0 to 1) for points/lines.
#'  Default is 0.5.
#' @param atol A numeric absolute tolerance used by the ODE solver to determine
#'  if a good solution has been achieved. This is also used in the solved linear
#'  model to check if prior doses do no add anything to the solution.
#'  Default is 1e-8.
#' @param cap A string that specifies the plot caption.
#' @param cat_cov_l A named list. Each element defines one categorical covariate
#'   and must itself be a list with components:
#'   \describe{
#'     \item{\code{NAME}}{Character. Column name of the covariate
#'       (e.g. \code{"SEX"}).}
#'     \item{\code{NICENAME}}{Character or \code{NULL}. Display label.}
#'     \item{\code{REF}}{Character or \code{NULL}. Reference category value.
#'       If \code{NULL}, the first factor level (alphabetically) is used.}
#'     \item{\code{par_vec}}{Character vector. Model parameter(s) affected by
#'       this covariate (e.g. \code{c("ka")}).}
#'   }
#' @param cont_cov_l A named list. Each element defines one continuous covariate
#'   and must itself be a list with components:
#'   \describe{
#'     \item{\code{NAME}}{Character. Column name of the (transformed)
#'       covariate in the dataset (e.g. \code{"LG_AGE"}).}
#'     \item{\code{UTNAME}}{Character or \code{NULL}. Column name of the
#'       untransformed (back-transformed) covariate
#'       (e.g. \code{"AGE"}).  If \code{NULL} or \code{NA}, defaults to
#'       \code{NAME}.}
#'     \item{\code{REF}}{Character or numeric. Reference value for the
#'       covariate.  Use \code{"median"} to derive from data, or a numeric
#'       value.}
#'     \item{\code{NICENAME}}{Character or \code{NULL}. Display label for
#'       plots and tables (e.g. \code{"Age, years"}).}
#'     \item{\code{par_vec}}{Character vector. Model parameter(s) affected by
#'       this covariate (e.g. \code{c("CL")}).}
#'   }
#' @param cov_cols A character vector specifying the names of the columns with covariates.
#' @param covint String. Specifies the interpolation method for time-varying
#'  covariates. When solving ODEs it often samples times outside the sampling
#'  time specified in event table. When this happens, the time varying
#'  covariates are interpolated. Currently this can be:
#'  * `"linear"` interpolation, which interpolates the covariate by solving the
#'  line between the observed covariates and extrapolating the new covariate value
#'  * `"locf"` last observation carried forward
#'  * `"NOCB"` next observation carried backward.
#'  This is the same method that NONMEM uses
#'  * `"midpoint"` last observation carried forward to midpoint;
#'  next observation carried backward to midpoint
#'
#' Default is `"locf"`
#' @param ... Other arguments that will be passed to rxSolve function.
#' @param ciLow Numeric. Lower confidence interval bound. Default is 0.025
#' @param ciUp Numeric. Upper confidence interval bound. Default is 0.975
#' @param ci_band_alpha Numeric in \eqn{[0,1]}: transparency of the shaded band.
#'   Default \code{0.2}.
#' @param ci_band_col Color for the band fill and dotted limit lines.
#'   Default \code{"firebrick"}.
#' @param ci_limits Numeric vector of length 2: lower and upper bounds of the
#'   shaded acceptance band and of the dotted horizontal guides.  Default
#'   \code{c(0.8, 1.25)} is a common bioequivalence-style window on the ratio
#'   scale.
#' @param ci_quantiles Character vector of length 2: names of the lower and
#'   upper uncertainty columns in the sensitivity table, in that order.
#'   Defaults \code{c("P025", "P975")} to match the default percentiles in
#'   \code{sg_covsens_sim}.  Use other names (e.g. \code{c("P05", "P95")}) if
#'   you changed \code{quantiles} in the simulation and the columns exist.
#' @param col_i String. Column name for color
#' @param col_lab String. Label for color legend
#' @param covs A list of covariate structures. Each element should be a list containing covariate relationship definitions with the following structure:
#'   * `PAR` - string, name of the parameter to which covariate effect is applied
#'   * `COVNAME` - string, name of the covariate column in the dataset
#'   * `FUNC` - string, functional form of the covariate effect
#'   * `TRANS` - string, transformation applied to covariate
#'   * `REF` - numeric, reference value for categorical covariates
#'   * `INIT` - numeric, initial value for the covariate effect parameter
#'   * `EST` - logical, whether to estimate the covariate effect
#'   Can be `NULL`, if no covariates are used. Default is `NULL`
#' @param covsens_res Named list as returned by \code{sg_covsens_sim()}.  Must
#' contain the element selected by \code{type} (\code{PARSENS} and/or
#' \code{EXPSENS} data.frames with columns \code{LAB}, \code{VAR},
#' \code{mean}, \code{Type}, and the interval columns named by
#' \code{ci_quantiles}).
#' @param data String. Path to the dataset used to fit a model. Should be a CSV file containing the pharmacokinetic/pharmacodynamic data with appropriate column structure matching the headers specification
#' @param dens Logical. If `TRUE`, plot histogram/density of residuals instead of scatter
#' @param ds_covs Data.frame. The dataframe with covariates
#' @param ds_i Data.frame. The data frame with source data.
#' @param ds_parest Data.frame. Parameter estimates table with columns
#'   \code{parameter} and \code{value}.
#'   Required when \code{fpath_i} is \code{NULL}; must be provided together
#'   with \code{ds_covs}.  Default is \code{NULL}.
#' @param dt_obs_fl Logical. Show observed data points. Default is `FALSE`
#' @param DVID Restrict \code{SDTAB} to one observation type. Numeric values
#'   select the \code{DVID} column (default \code{1}). If \code{SDTAB} has
#'   \code{DVNAME}, a character or factor is matched to \code{DVNAME} first;
#'   otherwise a digit-only string is coerced to numeric \code{DVID}.
#' @param dv_col Character. Name of DV column in data_i. Default is `DV`
#' @param emp_perc Logical. Show empirical percentiles. Default is `TRUE`
#' @param errorbar_width Numeric. Width argument for \code{geom_errorbar}.  Default
#'   \code{0.2}.
#' @param est_covmat Data.frame. Parameter estimation covariance matrix.  The
#'   first column (\code{X1}) must list parameter names; remaining columns
#'   (named identically) form the symmetric variance–covariance matrix.
#' @param et Data.frame. Event table
#' @param eta_seq Vector of strings. Character vector of parameter names to be plotted. If `NULL`, all parameters be included. Default is `NULL`
#' @param par_seq Vector of strings. Character vector of parameter names to be plotted. If `NULL`, all parameters be included. Default is `NULL`
#' @param par_type String. A character string specifying the type of parameters used for
#'   theoretical distribution overlay (only relevant when `plot_type = 'DIST'`
#'   and `tdist = TRUE`). If 'Ind' - Individual (default), distributions are shown on the
#'   natural parameter scale assuming log-normal variability. If 'RE' - Random Effect, distributions are shown on the ETA scale,
#'   assuming a normal distribution with mean zero and covariance defined
#'   by `$OMEGAMAT`, without transformation.
#' @param excl_col Character vector. Contains column names to exclude from synthesis. Default: \code{NULL}
#' @param exclude_vars Character vector. Contains \code{VAR} levels to omit (e.g.
#'   \code{"Cc_Cmin"}).  \code{NULL} keeps all rows.
#' @param f_scales String, one of `"fixed"`, `"free"`, `"free_x"`, `"free_y"`. User can specify whether the scales (x and y axes) should be fixed across all panels (`"fixed"`), free for each panel (`"free"`), or free only in one dimension (`"free_x"` or `"free_y"`). Default is `"fixed"`
#' @param facet_i String. Column name for facet
#' @param fill_i String. Column name for fill aesthetic. Default is `NULL`
#' @param filt String. Provide a filter to apply. Default is `"T"`
#' @param fit Logical. If `TRUE`, the model fitting will be executed immediately
#'  using the specified fitter. If `FALSE`, only the fit configuration file will
#'  be generated without running the fit. Set to `FALSE` for file preparation only,
#'  or `TRUE` to run the complete fitting process. Default is `FALSE`.
#' @param fpath_i String or sg-fit object. If the string is given, the path to
#'  `.Rdata` or `.json` file with sg-fit object is expected.
#' @param free_stat String. Facet scaling option. One of `"free"`, `"free_x"`,
#'  `"free_y"`, or `"fixed"`. Default is `'free'`.
#' @param group_i String. Primary grouping variable for lines. Default is `'VAR'`.
#' @param headers List. A list specifying the data frame column headers.
#' Each element should be a list containing column information with the following structure:
#'   * `name` - string, column name in the dataset
#'   * `use` - string, column usage type. Valid values include:
#'     - "identifier" for subject ID columns
#'     - "time" for time columns
#'     - "observation" for dependent variable columns
#'     - "observationtype" for observation type identifier
#'     - "administration" for administration route
#'     - "amount" for dose amount
#'     - "eventidentifier" for event ID
#'     - "missingdependentvariable" for missing DV flag
#'     - "covariate" for covariate columns
#'   * `type` - string or NULL, data type specification. For observations use
#'    "continuous", "count/categorical" or "event", depending on the nature of
#'    observations. For covariates use "continuous" or "categorical". Can be NULL for non-covariate columns.
#' @param ncores Integer. Number of cores used for calculations. Default is 1.
#' @param id_col Character string. Specify the name of the identifier column
#'  to exclude from synthesis. Default: \code{NULL}.
#' @param indiv Logical. If `TRUE`, use individual predictions (`"IPRED"`);
#'  otherwise use population predictions (`"PRED"`). Default is `TRUE`.
#' @param inits A named vector specifying initial conditions for model variables; Default is `NULL`.
#' @param keep Vector of strings. Columns of event table to keep in the output data frame. Default is `NULL`.
#' @param lab_x String. X-axis label.
#' @param lab_y String. Y-axis label.
#' @param legend_fl Logical. Show legend. Default is `FALSE`.
#' @param levels_discrete Integer. Maximum unique values to consider a variable discrete. Default is 10.
#' @param log_x Logical. If `TRUE`, a logarithmic scale is applied to x-axis. Default is `FALSE`.
#' @param log_y Logical. If `TRUE`, a logarithmic scale is applied to y-axis. Default is `FALSE`.
#' @param log_axes Logical. If `TRUE`, a logarithmic scale is applied to all axes. Default is `FALSE`.
#' @param lty_i String. Column name for linetype aesthetic. Default is `NULL`.
#' @param max_x Numeric. X-axis maximum limit. Default is `NA`.
#' @param max_y Numeric. Y-axis maximum limit. Default is `NA`.
#' @param method One of c("liblsoda", "lsoda", "dop853", "indLin"). Method for solving ODE.
#' @param min_x Numeric. X-axis minimum limit. Default is `NA`.
#' @param min_y Numeric. Y-axis minimum limit. Default is `NA`.
#' @param model RxODE model. The model to simulate from.
#' @param n_bins Integer. Number of bins to use in the histogram. Default is 30.
#' @param n_quantiles Integer. Number of quantile groups for continuous variables in `col_i`. Default is 3.
#' @param no_leg Logical. If `TRUE`, the legend is not shown. Default is `FALSE`
#' @param npop Integer. Number of population replicates. Default is 1.
#' @param nsub Integer. Number of subjects sampled per population (omega/sigma matrices per ID). Default is 1.
#' @param occ A list specifying interoccasion variability properties, containing:
#'   * `init` - matrix, initial values for the interoccasion variance-covariance matrix.
#'    Same structure as `re$init` but for occasion-to-occasion variability. Use 0 for no interoccasion variability
#'   * `est` - matrix, logical matrix specifying which interoccasion variance-covariance
#'    elements to estimate. Use `TRUE` to estimate, `FALSE` to fix, `NA` for
#'    elements not applicable. Typically all elements are NA when no interoccasion variability is modeled.
#' @param omega A named matrix or vector.
#' @param opt_name String. Specify the optimizer/fitter to use for model fitting.
#'  Currently supported options:
#'   * `"Monolix"` - uses Monolix Suite for population pharmacokinetic modeling (generates .mlxtran files)
#'   * `"Simurg"` - uses SimuRg internal fitter (generates .R files with JSON control structure)
#'   Default is `"Monolix"`.
#' @param outputs Vector of strings. Names of the model variables to output.
#'  If `NULL`, all variables returned. Default is `NULL`.
#' @param path_to_save_output String. Path to save fit output files.
#'  Should be a valid directory path where the fit results and project files
#'  will be saved. If `NULL`, current working directory will be used. Default is `NULL`.
#' @param path_to_fitter String. The path to the program fitter executable.
#'  For Monolix, this should point to the monolix.bat file.
#'  If `NULL`, "C:/ProgramData/Lixoft/MonolixSuite2023R1/bin/monolix.bat" will be used as default. Default is `NULL`.
#' @param piLow Numeric. Lower prediction interval bound. Default is 0.10.
#' @param piUp Numeric. Upper prediction interval bound Default is 0.90.
#' @param plot_type Character. Type of plot to produce:
#'   * `"DIST"` (default) - histogram of individual parameters,
#'   * `"QQ"` - QQ-plot of individual parameters.
#' @param point_size Numeric. Point size for \code{geom_point}.  Default \code{2.5}.
#' @param pred.corr Logical. Apply prediction correction. Default is `FALSE`.
#' @param project_name String. The name of the Monolix project without file extension.
#'  This will be used as the base name for output files and directories.
#' @param re List. A list specifying options for random effects used in model fit, containing:
#'   * `init` - matrix, initial values for the variance-covariance matrix of random
#'    effects. Rows and columns should correspond to parameters defined in theta.
#'    Diagonal elements represent variances, off-diagonal elements represent covariances. Use 0 for no variability
#'   * `est` - matrix, logical matrix of same dimensions as `init` specifying
#'    which variance-covariance elements to estimate. Use `TRUE` to estimate,
#'    `FALSE` to fix, `NA` to not use this random effect.
#' @param ref_line_col Color for the dashed horizontal line at \code{y = 1}
#'   (no change from reference).  Default \code{"grey25"}.
#' @param rtol Numeric. A numeric relative tolerance used by the ODE solver to determine
#'  if a good solution has been achieved. This is also used in the solved linear
#'  model to check if prior doses do not add anything to the solution. Default is 1e-6.
#' @param run_id Integer. Tested model ID. Default is 1.
#' @param ruv A list specifying options for residual error used in model fit, containing:
#'   * `YNAME` - string, output name. Typically `"y1"`, `"y2"`,...
#'   * `DVID` - numeric, observation type identifier corresponding to DVID column values
#'   * `TRANS` - string, residual error distribution. Can be: `"normal"`, `"logNormal"`, `"logitNormal"`
#'   * `PRED` - string, prediction variable name from the model
#'   * `ERR` - string, error model type. Options include:
#'     - "constant" for additive error
#'     - "proportional" for proportional error
#'     - "combined1" for combined additive and proportional error
#'   * `INIT` - numeric vector, initial values for error parameters (length depends on error model)
#'   * `EST` - logical vector, whether to estimate each error parameter (same length as INIT)
#'   * `BLQM` - below limit of quantification method (can be NULL)
#'  For several observation types list of lists should be provided:
#'   one residual error (ruv) object for each observations.
#' @param sc_factor Numeric. Scaling factor for DV/PRED/IPRED values. Default is 1 (no scaling).
#' @param scale A numeric named vector. Scaling for ODE parameters of the system.
#'  The names must correspond to the parameter identifiers in the ODE specification.
#'  Each of the ODE variables will be divided by the scaling factor. Default is `NULL`.
#' @param seed Integer. Random seed for synthetic data generation reproducibility. Default is \code{123}.
#' @param shp_i String. Column name for shape aesthetic. Default is `NULL`.
#' @param sigma A named matrix representing a sigma covariance matrix or its
#'  Cholesky decomposition; Default is `NULL`.
#' @param smooth Logical. Add LOESS smooth line. Default is `TRUE`.
#' @param stimes Vector of numeric. Sampling time points. Default is `NULL`.
#' @param task_opt String. Additional task options to be passed to the fitting software.
#'  For Monolix, this can include specific task configurations or optimization settings.
#'  When `NULL`, default tasks (populationParameters, individualParameters, fim, logLikelihood)
#'  will be used.  Default is `NULL`.
#' @param tdist Logical. If `TRUE`, overlay theoretical parameter distributions
#'  based on population mean and OMEGA matrix. Default is `TRUE`.
#' @param time_col String. The column to use as a time column. Currently, can be only `TIME`. Default is `TIME`.
#' @param theor_perc Logical. Show theoretical percentiles. Default is `TRUE`.
#' @param theor_percCI Logical. Show CI around theoretical percentiles. Default is `TRUE`.
#' @param theta A named vector or data frame. Values of population parameters to simulate with. Default is `NULL`.
#' @param thetamat A named variance-covariance matrix (for parameters brought to normal distribution).
#'  Default is `NULL`.
#' @param tsld Logical. If `TRUE`, uses time since last dose instead of time from first dose. Default is `FALSE`.
#' @param quantiles A numeric vector of length 2. Lower and upper quantiles of
#'   the continuous covariate distribution to test.
#'   Default is \code{c(0.1, 0.9)}.
#' @param val_col String. Name of value column. Default is `VALUE`.
#' @param wrap_i String. Faceting formula for `facet_wrap`. Default is `NULL`.
#' @param wrap_ncol Integer. Number of columns for `facet_wrap`. Default is `NULL`.
#' @param wrap_nrow Integer. Number of rows for `facet_wrap`. Default is `NULL`.
#' @param method Character string. `"PRCC"` or `"eFAST"`.
#' @param model A model object passed to `sg_sim()`.
#' @param params Character vector of parameter names to vary.

#' @param par_bounds A tibble or data frame with columns `PAR`, `LB`, `UB`.
#' @param point_size Point size for \code{geom_point}.  Default \code{2.5}.
#' @param n_sim Integer. Number of samples (LHS size for PRCC, base frequency size for eFAST).
#' @param stimes A numeric vector of simulation times.
#' @param output A character vector of outputs to keep. Passed to `sg_sim()`.
#' @param stat_comp A character vector of summary statistics to compute.
#'        Supported internally: `"mean","median","min","max","sd","cmax","SS"`.
#' @param et An event table passed to `sg_sim()`.
#' @param theta A named numeric vector of baseline parameters. Default is `NULL`.
#'        Parameters listed in `params` are replaced by sampled values.
#' @param cov Covariates passed to `sg_sim()`. Default is `NULL`.
sg_dummy <- function(
    ...,
  abreaks = scales::pretty_breaks(7),
  addcov = TRUE,
  addOFV = TRUE,
  addline = TRUE,
  aggr = NULL,
  alpha_i = 0.5,
  atol = 1e-8,
  cap,
  cat_cov_l,
  ci_band_alpha = 0.2,
  ci_band_col = "firebrick",
  ci_limits = c(0.8, 1.25),
  ci_quantiles = c("P025", "P975"),
  cont_cov_l,
  cov_cols,
  cov,
  covint = "locf",
  covsens_res,
  ciLow = 0.025,
  ciUp = 0.975,
  col_i,
  col_lab,
  covs,
  data,
  dens,
  ds_covs,
  ds_i,
  ds_parest = NULL,
  dt_obs_fl = FALSE,
  DVID = 1,
  dv_col = "DV",
  emp_perc = TRUE,
  errorbar_width = 0.2,
  est_covmat,
  et,
  eta_seq = NULL,
  excl_col = NULL,
  exclude_vars = NULL,
  f_scales = "fixed",
  facet_i,
  fill_i = NULL,
  filt = "T",
  fit = FALSE,
  fpath_i,
  free_stat = "free",
  group_i = "VAR",
  headers,
  ncores = 1,
  id_col = NULL,
  indiv = TRUE,
  inits = NULL,
  keep = NULL,
  lab_x,
  lab_y,
  legend_fl = FALSE,
  levels_discrete = 10,
  log_x = FALSE,
  log_y = FALSE,
  log_axes = FALSE,
  lty_i = NULL,
  max_x = NULL,
  max_y = NULL,
  method = "lsoda",
  min_x = NULL,
  min_y = NULL,
  model,
  n_bins = 30,
  n_quantiles = 3,
  n_sim,
  no_leg = FALSE,
  npop = 1,
  nsub = 1,
  occ,
  opt_name = "Monolix",
  omega,
  outputs = NULL,
  output = NULL,
  path_to_fitter = NULL,
  path_to_save_output = NULL,
  par_bounds,
  par_seq = NULL,
  par_type = "Ind",
  params =NULL,
  piLow = 0.10,
  piUp = 0.90,
  plot_type = "DIST",
  point_size = 2.5,
  pred.corr = FALSE,
  project_name,
  quantiles = c(0.1, 0.9),
  re,
  ref_line_col = "grey25",
  rtol = 1e-6,
  run_id = 1,
  ruv,
  sc_factor = 1,
  scale = NULL,
  seed = 123,
  sigma = NULL,
  shp_i = NULL,
  smooth = TRUE,
  stat_comp = NULL,
  stimes = NULL,
  task_opt = NULL,
  tdist = TRUE,
  time_col = "TIME",
  theor_perc = TRUE,
  theor_percCI = TRUE,
  theta = NULL,
  thetamat = NULL,
  tsld = FALSE,
  val_col = "VALUE",
  wrap_i = NULL,
  wrap_ncol = NULL,
  wrap_nrow = NULL
) {}

# Ensure SimuRg object table components are single data frames
#
# Converts SDTAB, EVTAB, COTAB, CATAB, SUMTAB from list-of-rows to one data frame
# when needed. Idempotent if already data frames.

smrg_ensure_tables_df <- function(obj) {
  table_names <- c("SDTAB", "EVTAB", "COTAB", "CATAB", "SUMTAB")
  for (nm in table_names) {
    x <- obj[[nm]]
    if (is.null(x)) next
    if (is.data.frame(x)) next
    if (!is.list(x) || length(x) == 0) next
    # List of rows (each element is a list or atomic vector)
    if (is.list(x[[1]]) && !is.data.frame(x[[1]])) {
      obj[[nm]] <- as.data.frame(do.call(rbind, lapply(x, as.data.frame)))
    } else {
      obj[[nm]] <- as.data.frame(x)
    }
  }
  obj
}

# Subset SDTAB-like data to one endpoint: numeric DVID, or DVNAME when present.
filter_sdtab_by_DVID <- function(ds, DVID = 1) {
  if (missing(DVID) || is.null(DVID)) {
    DVID <- 1
  }
  if (!"DVID" %in% names(ds)) {
    ep_num <- suppressWarnings(as.numeric(DVID))
    if (length(ep_num) == 1L && !is.na(ep_num) && ep_num == 1) {
      return(ds)
    }
    stop("SDTAB has no DVID column; cannot filter by DVID.")
  }

  if ((is.character(DVID) || is.factor(DVID)) && "DVNAME" %in% names(ds)) {
    ep_chr <- as.character(DVID)
    dvname_vals <- unique(stats::na.omit(as.character(ds$DVNAME)))
    if (ep_chr %in% dvname_vals) {
      out <- dplyr::filter(ds, as.character(.data$DVNAME) == ep_chr)
      if (nrow(out) == 0L) {
        stop("No SDTAB rows for DVNAME = '", ep_chr, "'.")
      }
      return(out)
    }
  }

  dvid_target <- suppressWarnings(as.numeric(DVID))
  if (length(dvid_target) != 1L || is.na(dvid_target)) {
    stop("DVID must be a single DVID (numeric) or DVNAME (string present in SDTAB$DVNAME).")
  }
  out <- dplyr::filter(ds, .data$DVID == dvid_target)
  if (nrow(out) == 0L) {
    stop("No SDTAB rows for DVID = ", dvid_target, ".")
  }
  out
}

read_smrg_ctrl <- function(ctrl) {
  if (inherits(ctrl, "character")) {
    if (!file.exists(ctrl)) {
      stop("File does not exist: ", ctrl)
    }

    ext <- tools::file_ext(ctrl)

    if (tolower(ext) == "rdata") {
      result <- get(load(ctrl))
    } else if (tolower(ext) == "json") {
      if (!requireNamespace("jsonlite", quietly = TRUE)) {
        stop("Package 'jsonlite' is required for reading JSON files.")
      }
      result <- jsonlite::fromJSON(ctrl, simplifyVector = TRUE, simplifyDataFrame = TRUE)
    } else {
      stop("Unsupported file type: ", ext, ". Supported: .RData, .json")
    }

    return(result)
  } else if (inherits(ctrl, "list")) {
    return(ctrl)
  } else if (inherits(ctrl, "data.frame")) {
    stop("ctrl cannot be a data.frame. Provide either a file path (character) or a GCO list object")
  } else {
    stop("ctrl should be either a file path (character) or a GCO list object. Got: ", class(ctrl)[1])
  }
}

#' Read or validate a SimuRg fit object
#'
#' Loads a SimuRg fit object from an `.RData` or `.json` file, or accepts an
#' already-loaded list and ensures all table components (`SDTAB`, `EVTAB`,
#' `COTAB`, `CATAB`, `SUMTAB`) are proper data frames.
#'
#' @inheritParams sg_dummy
#'
#' @return List. A SimuRg fit object with all table components coerced to
#'   data frames.
#'
#' @keywords internal
#'
#' @examples
#' # Pass an already-loaded sg-fit object
#' obj <- read_smrg_obj(gfo4cov)
#' class(obj$SDTAB)
#'
#' @export
read_smrg_obj <- function(fpath_i) {
  if (inherits(fpath_i, "character")) {
    if (!file.exists(fpath_i)) {
      stop("File does not exist: ", fpath_i)
    }

    ext <- tools::file_ext(fpath_i)

    if (tolower(ext) == "rdata") {
      result <- get(load(fpath_i))
    } else if (tolower(ext) == "json") {
      if (!requireNamespace("jsonlite", quietly = TRUE)) {
        stop("Package 'jsonlite' is required for reading JSON files.")
      }
      # simplifyVector = TRUE, simplifyDataFrame = TRUE so SDTAB and other
      # array-of-objects become single data frames
      result <- jsonlite::fromJSON(fpath_i, simplifyVector = TRUE, simplifyDataFrame = TRUE)
      # Ensure known table components are data frames (in case of list of rows)
      result <- smrg_ensure_tables_df(result)
    } else {
      stop("Unsupported file type: ", ext, ". Supported: .RData, .json")
    }

    return(result)
  } else if (inherits(fpath_i, "list")) {
    # fpath_i is already an object (sg_fit object); ensure tables are data frames
    return(smrg_ensure_tables_df(fpath_i))
  } else if (inherits(fpath_i, "data.frame")) {
    stop("fpath_i cannot be a data.frame. Provide either a file path (character) or an sg_fit object (list with SDTAB)")
  } else {
    stop("fpath_i should be either a file path (character) or an sg_fit object (list). Got: ", class(fpath_i)[1])
  }
}
gmo_converter <- function(gco_path, output_path = NULL) {
  gco <- read_smrg_ctrl(gco_path)
  theta <- gco$theta
  covs <- gco$covs
  re <- gco$re
  ruv <- gco$ruv
  model_file <- gco$model
  data_path <- gco$data
  data <- utils::read.csv(data_path, check.names = FALSE)
  model_lines <- readLines(model_file)
  input_idx <- which(grepl("^# \\[INPUT\\]", model_lines))[1]
  model_idx <- which(grepl("^# \\[MODEL\\]", model_lines))[1]
  pre_input <- model_lines[seq_len(input_idx)]
  post_input <- model_lines[seq(model_idx, length(model_lines))]
  input_lines <- character()
  if (!is.null(theta) && nrow(theta) > 0) {
    for (i in seq_len(nrow(theta))) {
      name <- theta$NAME[i]
      trans <- tolower(theta$TRANS[i])
      init <- theta$INIT[i]
      if (is.na(init)) init <- 0
      if (trans == "lognormal") {
        input_lines <- c(input_lines, paste0(name, "_pop = log(", init, ");"))
      } else if (trans == "logitnormal") {
        input_lines <- c(input_lines, paste0(name, "_pop = logit(", init, ");"))
      } else {
        input_lines <- c(input_lines, paste0(name, "_pop = ", init, ";"))
      }
    }
    input_lines <- c(input_lines, "")
  }
  cov_stat_lines <- character()
  cov_par_lines <- character()
  if (!is.null(covs) && length(covs) > 0) {
    for (j in seq_along(covs)) {
      covj <- covs[[j]]
      par <- covj$PAR
      covname <- covj$COVNAME
      func <- covj$FUNC
      cov_trans <- covj$TRANS
      init <- if (is.null(covj$INIT)) 1 else covj$INIT
      if (!is.null(func) && func == "linear" && !is.null(cov_trans) && tolower(cov_trans) == "median") {
        med_name <- paste0(covname, "_med")
        med_val <- stats::median(data[[covname]], na.rm = TRUE)
        cov_stat_lines <- c(cov_stat_lines, paste0(med_name, " = ", med_val, ";"))
        beta_name <- paste0("beta_", par, "_", covname)
        cov_par_lines <- c(cov_par_lines, paste0(beta_name, " = ", init, ";"))
      } else if (!is.null(covj$REF)) {
        # Categorical covariate: create one beta per observed non-reference category value
        ref <- covj$REF
        cov_values <- unique(stats::na.omit(data[[covname]]))
        cov_values <- sort(cov_values)
        non_ref_vals <- cov_values[cov_values != ref]
        for (val in non_ref_vals) {
          beta_name <- paste0("beta_", par, "_", val)
          cov_par_lines <- c(cov_par_lines, paste0(beta_name, " = ", init, ";"))
        }
      }
    }
  }
  if (length(cov_stat_lines) > 0) {
    input_lines <- c(input_lines, cov_stat_lines, "")
  }
  if (length(cov_par_lines) > 0) {
    input_lines <- c(input_lines, cov_par_lines, "")
  }
  omega_names <- character()
  if (!is.null(re) && !is.null(re$est)) {
    est_mat <- re$est
    if (is.matrix(est_mat)) {
      pars <- colnames(est_mat)
      for (k in seq_along(pars)) {
        par_name <- pars[k]
        est_col <- est_mat[, k]
        if (any(!is.na(est_col) & est_col)) {
          oname <- paste0("omega_", par_name)
          omega_names <- c(omega_names, par_name)
          input_lines <- c(input_lines, paste0(oname, " = 1;"))
        }
      }
    }
  }
  if (length(omega_names) > 0) {
    input_lines <- c(input_lines, "")
  }
  if (!is.null(ruv) && !is.null(ruv$ERR)) {
    ruv_list <- list(ruv)
  } else if (is.list(ruv) && length(ruv) > 0 && !is.null(ruv[[1]]$ERR)) {
    ruv_list <- ruv
  } else {
    ruv_list <- list()
  }
  err_par_lines <- character()
  for (r in seq_along(ruv_list)) {
    ruvr <- ruv_list[[r]]
    pred <- ruvr$PRED
    err_type <- ruvr$ERR
    init <- ruvr$INIT
    if (is.null(err_type) || is.null(pred) || is.null(init)) next
    if (tolower(err_type) == "combined1" && length(init) >= 2) {
      a_name <- paste0(pred, "_a")
      b_name <- paste0(pred, "_b")
      err_par_lines <- c(err_par_lines, paste0(a_name, " = ", init[1], ";"))
      err_par_lines <- c(err_par_lines, paste0(b_name, " = ", init[2], ";"))
    } else if (tolower(err_type) == "proportional" && length(init) >= 1) {
      b_name <- paste0(pred, "_b")
      err_par_lines <- c(err_par_lines, paste0(b_name, " = ", init[1], ";"))
    }
  }
  if (length(err_par_lines) > 0) {
    input_lines <- c(input_lines, err_par_lines, "")
  }
  tv_lines <- character()
  if (!is.null(theta) && nrow(theta) > 0) {
    for (i in seq_len(nrow(theta))) {
      name <- theta$NAME[i]
      trans <- tolower(theta$TRANS[i])
      if (trans == "lognormal") {
        tv_lines <- c(tv_lines, paste0(name, "_tv = exp(", name, "_pop);"))
      } else if (trans == "logitnormal") {
        tv_lines <- c(tv_lines, paste0(name, "_tv = expit(", name, "_pop);"))
      } else {
        tv_lines <- c(tv_lines, paste0(name, "_tv = ", name, "_pop;"))
      }
    }
  }
  if (length(tv_lines) > 0) {
    input_lines <- c(input_lines, tv_lines, "")
  }
  par_def_lines <- character()
  if (!is.null(theta) && nrow(theta) > 0) {
    for (i in seq_len(nrow(theta))) {
      name <- theta$NAME[i]
      par_trans <- tolower(theta$TRANS[i])
      has_omega <- name %in% omega_names
      base_expr <- paste0(name, "_tv")
      if (has_omega) {
        oname <- paste0("omega_", name)
        if (par_trans == "lognormal") {
          base_expr <- paste0(base_expr, " * exp(", oname, ")")
        } else if (par_trans == "logitnormal") {
          base_expr <- paste0(base_expr, " * expit(", oname, ")")
        } else {
          base_expr <- paste0(base_expr, " + ", oname)
        }
      }
      if (!is.null(covs) && length(covs) > 0) {
        for (j in seq_along(covs)) {
          covj <- covs[[j]]
          if (!identical(covj$PAR, name)) next
          covname <- covj$COVNAME
          func <- covj$FUNC
          cov_trans <- covj$TRANS
          if (!is.null(func) && func == "linear" && !is.null(cov_trans) && tolower(cov_trans) == "median") {
            med_name <- paste0(covname, "_med")
            beta_name <- paste0("beta_", name, "_", covname)
            base_expr <- paste0(base_expr, " * (", covname, "/", med_name, ")^", beta_name)
          } else if (!is.null(covj$REF)) {
            # Categorical covariate
            ref <- covj$REF
            cov_values <- unique(stats::na.omit(data[[covname]]))
            cov_values <- sort(cov_values)
            non_ref_vals <- cov_values[cov_values != ref]
            if (length(non_ref_vals) == 0) next
            cat_terms <- character()
            for (val in non_ref_vals) {
              beta_name <- paste0("beta_", name, "_", val)
              cat_terms <- c(cat_terms, paste0(beta_name, " * (", covname, " == ", val, ")"))
            }
            if (length(cat_terms) > 0) {
              if (par_trans == "lognormal") {
                # Multiplicative effect on typical value: tv * exp(sum(beta * indicator))
                base_expr <- paste0(base_expr, " * exp(", paste(cat_terms, collapse = " + "), ")")
              } else {
                # Additive effect on natural scale for non-lognormal
                base_expr <- paste0(base_expr, " + ", paste(cat_terms, collapse = " + "))
              }
            }
          }
        }
      }
      par_def_lines <- c(par_def_lines, paste0(name, " = ", base_expr))
    }
  }
  all_input <- c(pre_input, input_lines, "", par_def_lines, "", post_input)
  res_err_lines <- character()
  for (r in seq_along(ruv_list)) {
    ruvr <- ruv_list[[r]]
    pred <- ruvr$PRED
    err_type <- ruvr$ERR
    if (is.null(err_type) || is.null(pred)) next
    if (tolower(err_type) == "combined1") {
      res_err_lines <- c(res_err_lines, paste0(pred, "_Res_err = ", pred, " * (1 + ", pred, "_b ) + ", pred, "_a;"))
    } else if (tolower(err_type) == "proportional") {
      res_err_lines <- c(res_err_lines, paste0(pred, "_Res_err = ", pred, " * (1 + ", pred, "_b );"))
    }
  }
  if (length(res_err_lines) > 0) {
    all_input <- c(all_input, res_err_lines)
  }
  if (is_null(output_path)) {
    return(all_input)
  } else {
    writeLines(all_input, con = output_path)
    return(all_input)
  }
}

#' Visualisation of covariate sensitivity analysis results
#'
#' Draw a forest-style graphic (points with uncertainty intervals) from the
#' sensitivity tables produced by \code{\link{sg_covsens_sim}}.  Each facet row
#' corresponds to a model output or exposure metric (\code{VAR}); each point is a
#' covariate scenario (\code{LAB}), coloured by covariate type (\code{Type}).
#'
#' Two views are available, matching the named elements of the simulation
#' output:
#' \itemize{
#'   \item \strong{\code{PARSENS}} — sensitivity of individual parameters
#'     (simulation at time zero, no ODE time course).
#'   \item \strong{\code{EXPSENS}} — sensitivity of exposure summaries
#'     (e.g. Cmin, Cmax, Cavg) after full simulation over \code{stimes} in
#'     \code{sg_covsens_sim}.
#' }
#' @inheritParams sg_dummy
#' @param plot_type Character scalar: \code{"PARSENS"} (default) or \code{"EXPSENS"}.
#' @param palette Colors for the \code{Type} scale (continuous vs
#'   categorical covariates, etc.).  Recycled if there are more levels than
#'   colors.  Default \code{MSDcol[c(1, 3, 4, 5, 6, 7)]}.
#' @param lab_y Axis label for the numeric scale (this becomes the horizontal
#'   axis after \code{coord_flip()}).  Default mentions a 95\% interval; change
#'   if you use different \code{ci_quantiles}.
#' @param cap Optional figure caption, passed to
#'   \code{ggplot2::labs(caption = ...)} (e.g. text describing reference
#'   covariate values).  Default \code{NULL}.
#'
#' @details
#' Values on the y-axis are ratios relative to the reference scenario: \code{1}
#' means no change.  In \code{sg_covsens_sim}, percent change relative to
#' reference is transformed to this scale before tabulation.  The shaded region
#' between \code{ci_limits} highlights a target interval; points whose
#' intervals lie largely inside can be read as scenarios consistent with that
#' criterion, subject to study-specific rules.
#'
#' The plot layers are drawn in order: reference band, error bars, points,
#' reference line at 1, dotted lines at \code{ci_limits}, then faceting by
#' \code{VAR} and flipped coordinates so labels read along the vertical axis.
#'
#' @returns A \code{ggplot2} object (inactive until printed or saved).  You can
#'   add further layers or themes with the usual \pkg{ggplot2} API.
#'
#' @examples
#' \donttest{
#' library(tibble)
#' library(dplyr)
#' library(rxode2)
#' # Typical workflow: run the simulation (see examples in ?sg_covsens_sim),
#' # then visualise parameter and exposure sensitivity.
#'
#' cont_cov_l <- list(
#'   LG_AGE = list(NAME = "LG_AGE", UTNAME = "AGE",
#'                 REF = "median", NICENAME = "Age, years",
#'                 par_vec = c("CL")),
#'   LG_WEIGHT = list(NAME = "LG_WEIGHT", UTNAME = "WEIGHT",
#'                 REF = "median", NICENAME = "Weight, kg",
#'                 par_vec = c("Vd"))
#' )
#'
#' cat_cov_l <- list(
#'   SEX = list(NAME = "SEX", NICENAME = "Sex",
#'              REF = "0", par_vec = c("ka")),
#'   CYP2C9 = list(NAME = "CYP2C9", NICENAME = "CYP2C9 genotype",
#'                 REF = NULL, par_vec = c("CL"))
#' )
#'
#' # --- Dosing ---
#' ev_t_input <- tribble(
#'  ~id, ~time, ~ii, ~amt, ~addl, ~dur, ~evid, ~Regimen,        ~Dose,
#'   1,   0,     336, 10,   21,    0.5,  1,     "0.3 mg/kg Q2W", 0.3
#' )
#' # --- Model ---
#' model <- RxODE({
#'  # Doses in mg
#'   # Time in hours
#'
#'  ### Parameter values
#'   # Typical values
#'  ka_pop = 0.073;
#'   Vd_pop = 14.8;
#'   CL_pop = 0.347;
#'
#'   # Random effects
#'   omega_ka = 0;
#'   omega_Vd = 0;
#'   omega_CL = 0;
#'
#'   # Covariate effect
#'   # Continuous
#'   beta_CL_LG_AGE = 0.49990114;
#'   beta_Vd_LG_WEIGHT = 0.60529433;
#'
#'   # Categorical
#'   beta_CL_CYP2C9_1_2 = -0.339;
#'   beta_CL_CYP2C9_1_3 = -0.574;
#'   beta_CL_CYP2C9_2_2 = -1.079;
#'   beta_CL_CYP2C9_2_3 = -0.745;
#'   beta_CL_CYP2C9_3_3 = -2.13;
#'
#'   beta_ka_SEX_1 = -0.12198035;
#'
#'   # Residual error
#'   Cc_b = 0;
#'
#'  # Transformations
#'   ka_tv = exp(ka_pop);
#'   Vd_tv = exp(Vd_pop);
#'   CL_tv = exp(CL_pop);
#'
#'   CL_multiplier = 1.0;  # Default/reference
#'   ka_multiplier = 1.0;
#'
#'   if (SEX == "1") {ka_multiplier = exp(beta_ka_SEX_1)}
#'
#'   if (CYP2C9 == "1") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_1_2);
#'   } else if (CYP2C9 == "2") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_1_3);
#'   } else if (CYP2C9 == "3") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_2_2);
#'   } else if (CYP2C9 == "4") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_2_3);
#'   } else if (CYP2C9 == "5") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_3_3);
#'   }
#'
#'   ka = ka_tv*ka_multiplier*exp(omega_ka);
#'   Vd = Vd_tv*exp(beta_Vd_LG_WEIGHT * LG_WEIGHT + omega_Vd); #Vd_tv*exp(omega_Vd);
#'   CL = CL_tv*CL_multiplier*exp(beta_CL_LG_AGE * LG_AGE + omega_CL);
#'
#'
#'   ### Explicit functions
#'   Cc = Ac/Vd;
#'
#'   ### Initial conditions
#'   Ad(0) = 0;
#'   Ac(0) = 0;
#'
#'   ### Differential equations
#'   d/dt(Ad) = - ka*Ad;
#'   d/dt(Ac) = ka*Ad - CL*Cc;
#'
#'   Cc_ResErr = Cc*(1 + Cc_b);
#' })
#'
#' # --- Estimation covariance (mock) ---
#' pnames     <- parest$parameter
#' npar       <- length(pnames)
#' set.seed(1)
#' m_cov      <- matrix(0.02, npar, npar)
#' diag(m_cov) <- 0.05 + runif(npar, 0, 0.05)
#' m_cov      <- (m_cov + t(m_cov)) / 2
#' est_covmat <- as_tibble(cbind(X1 = pnames, as.data.frame(m_cov)))
#' names(est_covmat)[-1] <- pnames
#'
#' # --- Simulation times (steady-state cycle 10) ---
#' ss_cycle <- 10
#' stimes_ss <- c(
#'   ss_cycle * 4 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1)),
#'   ss_cycle * 4 * 7 * 24 + 2 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1))
#' )
#' result <- sg_covsens_sim(
#'   fpath_i = NULL, ds_parest = parest, ds_covs = ds_covval,
#'   model = model, stimes = stimes_ss, et = ev_t_input,
#'   est_covmat = est_covmat, npop = 10,
#'   cont_cov_l = cont_cov_l, cat_cov_l = cat_cov_l,
#'   quantiles = c(0.1, 0.9), aggr = c("min", "max", "mean"),
#'   outputs = "Cc"
#' )
#'
#' p_par <- sg_covsens_vis(result, plot_type = "PARSENS")
#' p_exp <- sg_covsens_vis(result, plot_type = "EXPSENS")
#' print(p_par)
#' print(p_exp)
#'
#' # Alternate interval columns (must exist in the sensitivity tables)
#' sg_covsens_vis(result, ci_quantiles = c("P05", "P95"))
#'
#' # Drop selected metrics from the exposure panel
#' sg_covsens_vis(result, plot_type = "EXPSENS", exclude_vars = c("Cc_Cmin"))
#' }
#'
#' @seealso \code{\link{sg_covsens_sim}}
#'
#' @export
sg_covsens_vis <- function(
    covsens_res,
    plot_type      = c("PARSENS", "EXPSENS"),
    exclude_vars   = NULL,
    ci_quantiles   = c("P025", "P975"),
    ci_limits      = c(0.8, 1.25),
    ci_band_alpha  = 0.2,
    ci_band_col    = "firebrick",
    ref_line_col   = "grey25",
    palette    = MSDcol[c(1, 3, 4, 5, 6, 7)],
    point_size     = 2.5,
    errorbar_width = 0.2,
    lab_y           = "Mean (95% CI)\nchange from reference",
    cap        = NULL
) {
  plot_type <- match.arg(plot_type)

  if (!plot_type %in% names(covsens_res)) {
    stop("'covsens_res' does not contain an element named '", plot_type, "'.")
  }
  if (length(ci_quantiles) != 2) {
    stop("'ci_quantiles' must be a character vector of length 2.")
  }
  if (length(ci_limits) != 2) {
    stop("'ci_limits' must be a numeric vector of length 2.")
  }

  ds <- covsens_res[[plot_type]]

  missing_q <- setdiff(ci_quantiles, colnames(ds))
  if (length(missing_q) > 0) {
    stop("Column(s) not found in data: ", paste(missing_q, collapse = ", "),
         ". Adjust 'ci_quantiles'.")
  }

  if (!is.null(exclude_vars)) {
    ds <- dplyr::filter(ds, !VAR %in% exclude_vars)
  }

  p <- ggplot2::ggplot(
    data = ds,
    ggplot2::aes(
      x    = LAB,
      y    = mean,
      ymin = .data[[ci_quantiles[1]]],
      ymax = .data[[ci_quantiles[2]]],
      col  = Type
    )
  ) +
    ggplot2::annotate(
      "rect",
      xmin  = -Inf, xmax = Inf,
      ymin  = ci_limits[1], ymax = ci_limits[2],
      fill  = ci_band_col,
      alpha = ci_band_alpha
    ) +
    ggplot2::geom_errorbar(width = errorbar_width) +
    ggplot2::geom_point(size = point_size) +
    ggplot2::geom_hline(
      yintercept = 1,
      col = ref_line_col, lwd = 0.8, lty = "dashed"
    ) +
    ggplot2::geom_hline(
      yintercept = ci_limits,
      col = ci_band_col, lwd = 0.8, lty = "dotted"
    ) +
    ggplot2::scale_color_manual(values = palette) +
    ggplot2::scale_y_continuous(breaks = scales::pretty_breaks(7)) +
    ggplot2::labs(x = NULL, y = lab_y, caption = cap) +
    ggplot2::facet_grid(VAR ~ ., scales = "free") +
    ggplot2::coord_flip() +
    ggplot2::theme_bw() +
    ggplot2::theme(
      panel.grid.minor  = ggplot2::element_blank(),
      legend.position   = "top",
      legend.background = ggplot2::element_rect(
        fill      = "white",
        linewidth = 0.15,
        linetype  = "solid",
        colour    = "black"
      )
    )

  p
}

## Author: Melnikova Alina
## First created: 2026-04-03
## Description: formal testing of sg_covsens_vis function and its helpers
## Keywords: SimuRg, sg-covsens-vis

# ---- Shared fixtures -------------------------------------------------------
# Defined at file scope so all test_that() blocks can reuse them.

.cont_cov_l <- list(
  LG_AGE = list(
    NAME     = "LG_AGE",
    UTNAME   = "AGE",
    REF      = "median",
    NICENAME = "Age, years",
    par_vec  = c("CL")
  ),
  LG_WEIGHT = list(
    NAME     = "LG_WEIGHT",
    UTNAME   = "WEIGHT",
    REF      = "median",
    NICENAME = "Weight, kg",
    par_vec  = c("Vd")
  )
)

.cat_cov_l <- list(
  SEX = list(
    NAME     = "SEX",
    NICENAME = "Sex",
    REF      = "0",
    par_vec  = c("ka")
  ),
  CYP2C9 = list(
    NAME     = "CYP2C9",
    NICENAME = "CYP2C9 genotype",
    REF      = NULL,
    par_vec  = c("CL")
  )
)

.ev_t_input <- tribble(
  ~id, ~time, ~ii, ~amt, ~addl, ~dur, ~evid, ~Regimen,         ~Dose,
  1,   0,     336, 10,   21,    0.5,  1,     "0.3 mg/kg Q2W",  0.3
)

.ss_cycle <- 10
.stimes_ss <- c(
  .ss_cycle * 4 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1)),
  .ss_cycle * 4 * 7 * 24 + 2 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1))
)

.mod_fin <- RxODE({
  ka_pop = 0.073;
  Vd_pop = 14.8;
  CL_pop = 0.347;

  omega_ka = 0;
  omega_Vd = 0;
  omega_CL = 0;

  beta_CL_LG_AGE     = 0.49990114;
  beta_Vd_LG_WEIGHT  = 0.60529433;

  beta_CL_CYP2C9_1_2 = -0.339;
  beta_CL_CYP2C9_1_3 = -0.574;
  beta_CL_CYP2C9_2_2 = -1.079;
  beta_CL_CYP2C9_2_3 = -0.745;
  beta_CL_CYP2C9_3_3 = -2.13;

  beta_ka_SEX_1 = -0.12198035;

  Cc_b = 0;

  ka_tv = exp(ka_pop);
  Vd_tv = exp(Vd_pop);
  CL_tv = exp(CL_pop);

  CL_multiplier = 1.0;
  ka_multiplier = 1.0;

  if (SEX == "1") {ka_multiplier = exp(beta_ka_SEX_1)}

  if (CYP2C9 == "1") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_2);
  } else if (CYP2C9 == "2") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_3);
  } else if (CYP2C9 == "3") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_2);
  } else if (CYP2C9 == "4") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_3);
  } else if (CYP2C9 == "5") {
    CL_multiplier = exp(beta_CL_CYP2C9_3_3);
  }

  ka = ka_tv * ka_multiplier * exp(omega_ka);
  Vd = Vd_tv * exp(beta_Vd_LG_WEIGHT * LG_WEIGHT + omega_Vd);
  CL = CL_tv * CL_multiplier * exp(beta_CL_LG_AGE * LG_AGE + omega_CL);

  Cc = Ac / Vd;

  Ad(0) = 0;
  Ac(0) = 0;

  d/dt(Ad) = -ka * Ad;
  d/dt(Ac) =  ka * Ad - CL * Cc;

  Cc_ResErr = Cc * (1 + Cc_b);
})

# Mock estimation covariance matrix built from parest (symmetric, positive-definite)
.pnames <- parest$parameter
.npar   <- length(.pnames)
set.seed(1)
.m_cov  <- matrix(0.02, .npar, .npar)
diag(.m_cov) <- 0.05 + runif(.npar, 0, 0.05)
.m_cov  <- (.m_cov + t(.m_cov)) / 2
.est_covmat <- as_tibble(cbind(X1 = .pnames, as.data.frame(.m_cov)))
names(.est_covmat)[-1] <- .pnames

# ---- Cached sg_covsens_sim output (table mode) -----------------------------
.covsens_res <- sg_covsens_sim(
  fpath_i    = NULL,
  ds_parest  = parest,
  ds_covs    = ds_covval,
  model      = .mod_fin,
  stimes     = .stimes_ss,
  et         = .ev_t_input,
  est_covmat = .est_covmat,
  npop       = 10,
  cont_cov_l = .cont_cov_l,
  cat_cov_l  = .cat_cov_l,
  quantiles  = c(0.2, 0.8),
  aggr       = c("max"),
  outputs    = "Cc"
)


# ============================================================
# 1. Return plot_type and successful build
# ============================================================

test_that("sg_covsens_vis returns a ggplot object", {
  p <- sg_covsens_vis(.covsens_res)
  expect_s3_class(p, "gg")
})

test_that("sg_covsens_vis default type builds PARSENS with ggplot_build", {
  p <- sg_covsens_vis(.covsens_res)
  pb <- ggplot2::ggplot_build(p)
  expect_s3_class(pb, "ggplot_built")
})

test_that("sg_covsens_vis plot_type EXPSENS builds with ggplot_build", {
  p <- sg_covsens_vis(.covsens_res, plot_type = "EXPSENS")
  pb <- ggplot2::ggplot_build(p)
  expect_s3_class(pb, "ggplot_built")
})


# ============================================================
# 2. Labels and plotting options
# ============================================================

test_that("sg_covsens_vis applies lab_y and cap to plot labels", {
  ylab_txt <- "Custom y label"
  cap_txt <- "Ref: custom caption"
  p <- sg_covsens_vis(.covsens_res, lab_y = ylab_txt, cap = cap_txt)
  expect_equal(p$labels$y, ylab_txt)
  expect_equal(p$labels$caption, cap_txt)
})

test_that("sg_covsens_vis accepts alternate ci_quantiles when columns exist", {
  p <- sg_covsens_vis(.covsens_res, ci_quantiles = c("P05", "P95"))
  pb <- ggplot2::ggplot_build(p)
  expect_s3_class(pb, "ggplot_built")
})


# ============================================================
# 3. exclude_vars
# ============================================================

test_that("sg_covsens_vis exclude_vars reduces facet rows to remaining VAR levels", {
  v_excl <- "CL"
  ds_kept <- dplyr::filter(.covsens_res$PARSENS, !.data$VAR %in% v_excl)
  n_var_kept <- dplyr::n_distinct(ds_kept$VAR)

  p <- sg_covsens_vis(.covsens_res, exclude_vars = v_excl)
  pb <- ggplot2::ggplot_build(p)
  layout <- pb$layout$layout

  # One row in layout per facet panel (facet_grid VAR ~ .)
  expect_equal(nrow(layout), n_var_kept)
})


# ============================================================
# 4. Input validation — error handling
# ============================================================

test_that("sg_covsens_vis errors when covsens_res lacks requested plot_type", {
  expect_error(
    sg_covsens_vis(list(PARSENS = .covsens_res$PARSENS), plot_type = "EXPSENS"),
    regexp = "does not contain an element named"
  )
})

test_that("sg_covsens_vis errors when ci_quantiles length is not 2", {
  expect_error(
    sg_covsens_vis(.covsens_res, ci_quantiles = "P025"),
    regexp = "length 2"
  )
})

test_that("sg_covsens_vis errors when ci_limits length is not 2", {
  expect_error(
    sg_covsens_vis(.covsens_res, ci_limits = 0.8),
    regexp = "length 2"
  )
})

test_that("sg_covsens_vis errors when ci_quantiles columns are missing", {
  expect_error(
    sg_covsens_vis(.covsens_res, ci_quantiles = c("P025", "NOT_A_COL")),
    regexp = "Column\\(s\\) not found"
  )
})

## Author: Ugolkov Yaroslav, Victoria Kulesh
## First created: 2025-10-24
## Description: Convert Monolix project results to SimuRg objects
## Keywords: SimuRg, monolix, converter

#' Convert Monolix project output to R objects
#'
#' @description
#' Reads and parses Monolix project output files (.mlxtran and associated data files)
#' into a structured list of SimuRg objects including parameter estimates, individual predictions,
#' residuals, and diagnostic information.
#'
#' @param folder_path Character string. Path to the directory containing Monolix project files.
#' @param proj_name Character string. Name of the Monolix project (without file extension).
#' @param save_file Logical. If \code{TRUE}, saves \code{GCO} and \code{GFO} JSON files
#'   to \code{folder_path} with names \code{<proj_name>_GCO.json} and \code{<proj_name>_GFO.json},
#'   and also saves two RData files: \code{<proj_name>_GCO.RData} (object \code{gco})
#'   and \code{<proj_name>_GFO.RData} (object \code{gfo}).
#'
#' @return
#' Returns a list with the following components:
#' \itemize{
#'   \item \code{GFO}: SimuRg generalized fit object output object with:
#'   \itemize{
#'     \item \code{SDTAB}: A tibble containing data used for fitting
#'     \item \code{SUMTAB}: A tibble with parameter summary statistics containing
#'     \item \code{SIGMAMAT}: Residual variability matrix
#'     \item \code{OMEGAMAT}: Inter-individual variability matrix
#'     \item \code{OCCMAT}: Inter-occasion variability matrix (NA if not present)
#'     \item \code{EVTAB}: A tibble with event information
#'     \item \code{PATAB}: A tibble with individual parameter estimates
#'     \item \code{COTAB}: A tibble with continuous covariates
#'     \item \code{CATAB}: A tibble with categorical covariates
#'     \item \code{REGTAB}: Regression parameters (empty data.frame if not present)
#'     \item \code{OFV}: A tibble with objective function values
#'     \item \code{COVMAT}: Variance-covariance matrix of parameter estimates
#'     \item \code{CORRMAT}: Correlation matrix of parameter estimates
#'     \item \code{OPTIONS}: Model options (NULL if not present)
#'     \item \code{PROJNAME}: Project name
#'   }
#'   \item \code{GCO}: SimuRg generalized control object parsed from the mlxtran project with:
#'   \itemize{
#'     \item \code{headers}: List of dataset column descriptors (\code{name}, \code{use}, \code{type})
#'     \item \code{data}: Path to source data file
#'     \item \code{model}: Path to model file
#'     \item \code{task_opt}: Task options placeholder (empty object)
#'     \item \code{covs}: Covariate names detected
#'     \item \code{project_name}: Project name
#'     \item \code{theta}: List of population parameter definitions (\code{NAME}, \code{INIT}, \code{EST}, \code{TRANS})
#'     \item \code{ruv}: Residual error model definition (\code{YNAME}, \code{DVID}, \code{TRANS}, \code{PRED}, \code{ERR}, \code{INIT}, \code{EST})
#'     \item \code{re}: Between-subject variability matrices (\code{init}, \code{est})
#'     \item \code{occ}: Between-occasion variability matrices (\code{init}, \code{est})
#'     \item \code{modelText}: Text content of the model file
#'   }
#' }
#'
#' @details
#' This function serves as a bridge between Monolix output and R by parsing the .mlxtran file
#' and associated data files to create a comprehensive R object containing all relevant model
#' outputs. This facilitates further analysis, visualization, and reporting in R.
#'
#' The function automatically detects and imports various components of Monolix output
#' including population parameters, individual parameters, covariates, and diagnostic
#' metrics.
#'
#' If \code{save_file = TRUE}, the function additionally writes \code{GCO} and \code{GFO}
#' JSON files and \code{.RData} files to \code{folder_path}.
#'
#' @examples
#' \donttest{
#' library(stringr)
#' # Convert Monolix project results
#' test_folder <- system.file("extdata", "Monolix_objects", package = "SimuRg")
#' if (substr(test_folder, nchar(test_folder), nchar(test_folder)) != "/")
#'   test_folder <- str_c(test_folder, "/")
#' pro_name <- "proj-solo"
#' result <- sg_converter(folder_path = test_folder, proj_name = pro_name)
#' # save(results, file = "./models/simurg_object/Warfarin_PK.RData")
#' # Access individual predictions
#' head(result$GFO$SDTAB)
#'
#' # View parameter estimates
#' print(result$GFO$SUMTAB)
#'
#' # Check objective function value
#' print(result$GFO$OFV)
#'}
#' @importFrom readr read_csv read_tsv cols parse_number
#' @importFrom stringr str_c
#' @import tibble
#' @import dplyr
#' @export

sg_converter <- function(folder_path, proj_name, save_file = FALSE){
  #####--------------- Helper function for WRES calculation ---------------#####

  # Function to calculate WRES using FO approximation with partial derivatives
  # Based on Phoenix documentation: https://onlinehelp.certara.com/phoenix/8.5/Phoenix_UserDocs/Pop_weighted_residuals/Population_weighted_residuals__44__NPD__44__and_NPDE.htm
  #
  # WRES (Population Weighted Residuals) are calculated using first-order (FO) approximation:
  # WRES = (y_ij - f_ij) / sqrt(Var(y_ij))
  #
  # Where:
  # - y_ij is the observation for subject i at time j
  # - f_ij is the population prediction
  # - Var(y_ij) = Var(f_ij) + Var(epsilon_ij)
  #
  # For FO approximation:
  # - Var(f_ij) = sum of squared partial derivatives * omega
  # - Var(epsilon_ij) = sigma_a^2 + (sigma_b * f_ij)^2
  #
  # This implementation calculates actual partial derivatives for accurate FO approximation.

  # Function to detect model structure from mlxtran file
  detect_model_structure <- function(contr_obj) {
    # Look for model definition in the mlxtran file
    model_section <- contr_obj[grep("\\[LONGITUDINAL\\]", contr_obj):length(contr_obj)]

    # Check for compartment indicators
    if (any(str_detect(model_section, "cmt\\s*="))) {
      # Check number of compartments
      cmt_lines <- model_section[str_detect(model_section, "cmt\\s*=")]
      if (length(cmt_lines) > 1) {
        return("2comp")
      } else {
        return("1comp")
      }
    }

    # Check for absorption parameters (ka)
    if (any(str_detect(model_section, "ka\\s*=")) || any(str_detect(model_section, "absorption"))) {
      return("1comp_ka")
    }

    # Check for bioavailability parameter (F)
    if (any(str_detect(model_section, "F\\s*=")) || any(str_detect(model_section, "bioavailability"))) {
      return("1comp_ka")
    }

    # Check for two-compartment indicators
    if (any(str_detect(model_section, "Q\\s*=")) || any(str_detect(model_section, "V2\\s*="))) {
      return("2comp")
    }

    # Default to 1-compartment if no clear indication
    return("1comp")
  }

  # Function to calculate partial derivatives for common PK model structures
  # This function uses generalized parameter names and automatically detects parameter types
  # based on parameter name patterns, making it flexible for any model structure.
  #
  # Parameter type detection:
  # - Elimination parameters: contain "cl", "ke", "elim" (affect rate, proportional to TIME)
  # - Volume parameters: contain "v", "vol" (affect concentration directly)
  # - Absorption parameters: contain "ka", "abs" (affect absorption rate)
  # - Bioavailability parameters: contain "f", "bio" (affect bioavailability)
  # - Inter-compartmental parameters: contain "q", "inter" (for 2-compartment models)
  # - Other parameters: proportional to prediction
  calculate_partial_derivatives <- function(pred_data, omega_params, sum_dt_i, model_structure = "1comp") {

    # Get typical parameter values for partial derivative calculation
    pop_params <- sum_dt_i$parameter[str_detect(sum_dt_i$parameter, "_pop$")]
    params <- str_replace(pop_params, "_pop$", "")

    # Get typical parameter values
    typical_values <- sapply(params, function(par) {
      sum_dt_i$value[sum_dt_i$parameter == str_c(par, "_pop")]
    })

    # Get the parameter names that have random effects (omega parameters)
    random_effect_params <- str_replace(omega_params, "omega_", "")

    # Calculate partial derivatives based on model structure
    if (model_structure == "1comp") {
      # One-compartment model: C = (D/V) * exp(-CL/V * t)
      # For any parameter with random effects, calculate proportional derivatives

      pred_data <- pred_data %>%
        mutate(
          # Initialize all partial derivatives to 0
          across(starts_with("d"), ~0, .unpack_fail = "ignore")
        )

      # Calculate partial derivatives for each parameter with random effects
      for (param in random_effect_params) {
        deriv_col <- str_c("d", toupper(param))

        if (param %in% names(typical_values)) {
          # Calculate proportional partial derivative based on parameter type
          # For elimination parameters (affect rate): proportional to TIME
          # For volume parameters (affect concentration): proportional to 1/PRED
          # For other parameters: proportional to PRED

          if (str_detect(tolower(param), "cl|ke|elim")) {
            # Elimination parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED * TIME / typical_values[param])
          } else if (str_detect(tolower(param), "v|vol")) {
            # Volume parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED / typical_values[param])
          } else {
            # Other parameters - proportional to prediction
            pred_data <- pred_data %>%
              mutate(!!deriv_col := PRED / typical_values[param])
          }
        }
      }

    } else if (model_structure == "2comp") {
      # Two-compartment model: C = A*exp(-alpha*t) + B*exp(-beta*t)

      pred_data <- pred_data %>%
        mutate(
          # Initialize all partial derivatives to 0
          across(starts_with("d"), ~0, .unpack_fail = "ignore")
        )

      # Calculate partial derivatives for each parameter with random effects
      for (param in random_effect_params) {
        deriv_col <- str_c("d", toupper(param))

        if (param %in% names(typical_values)) {
          # Calculate proportional partial derivative based on parameter type
          if (str_detect(tolower(param), "cl|ke|elim")) {
            # Elimination parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED * TIME / typical_values[param])
          } else if (str_detect(tolower(param), "v|vol")) {
            # Volume parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED / typical_values[param])
          } else if (str_detect(tolower(param), "q|inter")) {
            # Inter-compartmental clearance
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED * TIME * 0.5)
          } else {
            # Other parameters - proportional to prediction
            pred_data <- pred_data %>%
              mutate(!!deriv_col := PRED / typical_values[param])
          }
        }
      }

    } else if (model_structure == "1comp_ka") {
      # One-compartment with absorption

      pred_data <- pred_data %>%
        mutate(
          # Initialize all partial derivatives to 0
          across(starts_with("d"), ~0, .unpack_fail = "ignore")
        )

      # Calculate partial derivatives for each parameter with random effects
      for (param in random_effect_params) {
        deriv_col <- str_c("d", toupper(param))

        if (param %in% names(typical_values)) {
          # Calculate proportional partial derivative based on parameter type
          if (str_detect(tolower(param), "cl|ke|elim")) {
            # Elimination parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED * TIME / typical_values[param])
          } else if (str_detect(tolower(param), "v|vol")) {
            # Volume parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED / typical_values[param])
          } else if (str_detect(tolower(param), "ka|abs")) {
            # Absorption parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED * TIME * 0.3)
          } else if (str_detect(tolower(param), "f|bio")) {
            # Bioavailability parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := PRED / typical_values[param])
          } else {
            # Other parameters - proportional to prediction
            pred_data <- pred_data %>%
              mutate(!!deriv_col := PRED / typical_values[param])
          }
        }
      }

    } else {
      # Generic approach for any model structure

      pred_data <- pred_data %>%
        mutate(
          # Initialize all partial derivatives to 0
          across(starts_with("d"), ~0, .unpack_fail = "ignore")
        )

      # Calculate partial derivatives for each parameter with random effects
      for (param in random_effect_params) {
        deriv_col <- str_c("d", toupper(param))

        if (param %in% names(typical_values)) {
          # Generic proportional derivatives based on parameter characteristics
          if (str_detect(tolower(param), "cl|ke|elim")) {
            # Elimination parameters - affect rate
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED * TIME / 100)
          } else if (str_detect(tolower(param), "v|vol")) {
            # Volume parameters - affect concentration directly
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED / 100)
          } else if (str_detect(tolower(param), "ka|abs")) {
            # Absorption parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := -PRED * TIME * 0.1)
          } else if (str_detect(tolower(param), "f|bio")) {
            # Bioavailability parameters
            pred_data <- pred_data %>%
              mutate(!!deriv_col := PRED / 10)
          } else {
            # Other parameters - proportional to prediction
            pred_data <- pred_data %>%
              mutate(!!deriv_col := PRED / typical_values[param])
          }
        }
      }
    }

    return(pred_data)
  }

  # Function to extract parameter distributions from mlxtran file
  extract_parameter_distributions <- function(contr_obj) {
    # Find the [INDIVIDUAL] section
    start_idx <- which(str_detect(contr_obj, fixed("[INDIVIDUAL]")))[1]
    if (is.na(start_idx)) {
      return(list())
    }

    # Find the end of the [INDIVIDUAL] section
    end_idx <- which((str_detect(contr_obj, "\\[.*\\]") | str_detect(contr_obj, "\\<.*\\>")) &
                       seq_along(contr_obj) > start_idx)[1]
    if (is.na(end_idx)) {
      end_idx <- length(contr_obj) + 1
    }

    # Extract the [INDIVIDUAL] section
    individual_section <- contr_obj[(start_idx + 1):(end_idx - 1)]
    def_idx <- which(str_detect(individual_section, "^\\s*DEFINITION:\\s*$"))[1]
    if (is.na(def_idx)) {
      return(list())
    }
    definition_lines <- individual_section[(def_idx + 1):length(individual_section)]
    definition_lines <- definition_lines[str_detect(definition_lines, "=") & str_detect(definition_lines, "\\{")]

    # Parse parameter definitions
    param_defs <- list()

    for (line in definition_lines) {
      line_clean <- str_squish(line)
      # Extract parameter name
      param_name <- str_extract(line_clean, "^[^=]+") %>% str_trim()

      # Extract the definition part
      def_part <- str_extract(line_clean, "\\{.*\\}")

      if (!is.na(def_part) && !is.na(param_name)) {
        # Parse the definition
        def_clean <- str_remove_all(def_part, "[\\{\\}]")
        def_parts <- str_split(def_clean, ",") %>% unlist() %>% str_trim()

        # Extract distribution, typical value, and sd
        distribution <- NA_character_
        typical <- NA_character_
        sd <- NA_character_

        for (part in def_parts) {
          if (str_detect(part, "distribution=")) {
            distribution <- str_remove(part, "distribution=") %>% str_trim()
          } else if (str_detect(part, "typical=")) {
            typical <- str_remove(part, "typical=") %>% str_trim()
          } else if (str_detect(part, "sd=")) {
            sd <- str_remove(part, "sd=") %>% str_trim()
          }
        }

        param_defs[[param_name]] <- list(
          distribution = distribution,
          typical = typical,
          sd = sd
        )
      }
    }

    return(param_defs)
  }

  # Function to map residual error parameter distributions from [LONGITUDINAL]
  extract_residual_param_distributions <- function(contr_obj, dt_ruv_map) {
    start_idx <- which(str_detect(contr_obj, fixed("[LONGITUDINAL]")))[1]
    if (is.na(start_idx)) return(setNames(character(0), character(0)))

    end_idx <- which((str_detect(contr_obj, "\\[.*\\]") | str_detect(contr_obj, "\\<.*\\>")) &
                       seq_along(contr_obj) > start_idx)[1]
    if (is.na(end_idx)) end_idx <- length(contr_obj) + 1
    long_section <- contr_obj[(start_idx + 1):(end_idx - 1)]

    def_idx <- which(str_detect(long_section, "^\\s*DEFINITION:\\s*$"))[1]
    if (is.na(def_idx)) return(setNames(character(0), character(0)))

    def_lines <- long_section[(def_idx + 1):length(long_section)]
    def_lines <- def_lines[str_detect(def_lines, "=") & str_detect(def_lines, "\\{")]

    y_dist_map <- tibble(y = character(0), distribution = character(0))
    for (line in def_lines) {
      line_clean <- str_squish(line)
      y_name <- str_extract(line_clean, "^[^=]+") %>% str_trim()
      dist <- str_match(line_clean, "distribution\\s*=\\s*([[:alnum:]_]+)")[, 2]
      if (!is.na(y_name) && !is.na(dist)) {
        y_dist_map <- bind_rows(y_dist_map, tibble(y = y_name, distribution = dist))
      }
    }
    if (nrow(y_dist_map) == 0) return(setNames(character(0), character(0)))

    dist_lookup <- setNames(y_dist_map$distribution, y_dist_map$y)
    ruv_dist <- dt_ruv_map %>%
      mutate(distribution = unname(dist_lookup[COL])) %>%
      select(RUVpar_a, RUVpar_b, distribution) %>%
      pivot_longer(cols = c(RUVpar_a, RUVpar_b), values_to = "PAR") %>%
      filter(!is.na(PAR) & !is.na(distribution)) %>%
      distinct(PAR, distribution)

    if (nrow(ruv_dist) == 0) return(setNames(character(0), character(0)))
    setNames(ruv_dist$distribution, ruv_dist$PAR)
  }

  # Function to calculate WRES using Monte Carlo simulation
  # Based on Phoenix documentation and PMC article: https://pmc.ncbi.nlm.nih.gov/articles/PMC5321813/
  #
  # This approach uses Monte Carlo simulation to estimate the variance-covariance matrix
  # of observations, making it completely independent of parameter names and model structure.
  #
  # Key improvements in this implementation:
  # 1. Uses proper parameter distributions (logNormal, Normal, logitNormal for PK parameters)
  # 2. Applies random effects correctly: P_i = P_pop * exp(eta_i) for logNormal
  # 3. Simulates predictions using perturbed parameters
  # 4. No assumptions about parameter names or model structure
  # 5. Handles any model complexity automatically
  # 6. Provides accurate variance estimates for nonlinear models
  # 7. Robust to model misspecification
  # 8. Can handle correlated random effects naturally
  #
  # WRES = (y_ij - f_ij) / sqrt(Var(y_ij))
  # Where Var(y_ij) is estimated using Monte Carlo simulation

  calculate_wres_mc <- function(pred_data, omega_params, resid_err_params, sum_dt_i, ruv_info, param_distributions, n_sim = 1000) {

    # Get residual error parameters
    sigma_a <- ifelse(!is.na(ruv_info$RUVpar_a),
                      sum_dt_i$value[sum_dt_i$parameter == ruv_info$RUVpar_a], 0)
    sigma_b <- ifelse(!is.na(ruv_info$RUVpar_b),
                      sum_dt_i$value[sum_dt_i$parameter == ruv_info$RUVpar_b], 0)

    # Get typical parameter values for random effects
    pop_params <- sum_dt_i$parameter[str_detect(sum_dt_i$parameter, "_pop$")]
    params <- str_replace(pop_params, "_pop$", "")

    # Get typical parameter values
    typical_values <- sapply(params, function(par) {
      sum_dt_i$value[sum_dt_i$parameter == str_c(par, "_pop")]
    })

    # Get omega values for random effects
    omega_values <- sapply(omega_params, function(par) {
      sum_dt_i$value[sum_dt_i$parameter == par]
    })

    # Get parameter names that have random effects
    random_effect_params <- str_replace(omega_params, "omega_", "")

    # Monte Carlo simulation to estimate variance-covariance matrix
    pred_data <- pred_data %>%
      mutate(
        # Residual error variance: Var(epsilon_ij) = sigma_a^2 + (sigma_b * f_ij)^2
        Var_epsilon = sigma_a^2 + (sigma_b * PRED)^2
      )

    if (length(omega_params) > 0) {
      # Perform Monte Carlo simulation using a simplified approach
      # Calculate variance for each observation individually to avoid list() issues

      # Initialize variance columns
      pred_data <- pred_data %>%
        mutate(
          Var_mc = 0,
          Var_total = Var_epsilon
        )

      # For each observation, calculate Monte Carlo variance
      for (row_idx in 1:nrow(pred_data)) {
        # Generate random effects for this observation
        eta_sim <- rnorm(n_sim * length(random_effect_params), 0, 1)
        eta_sim_matrix <- matrix(eta_sim, nrow = n_sim, ncol = length(random_effect_params))

        # Scale by omega values
        eta_scaled <- t(t(eta_sim_matrix) * omega_values)

        # Simulate individual parameters
        sim_params <- matrix(0, nrow = n_sim, ncol = length(random_effect_params))

        for (i in seq_along(random_effect_params)) {
          param <- random_effect_params[i]
          if (param %in% names(typical_values)) {
            # Get the distribution information for this parameter
            param_dist <- param_distributions[[param]]

            if (!is.null(param_dist)) {
              # Use the actual distribution from the mlxtran file
              if (param_dist$distribution == "logNormal") {
                # Log-normal distribution: P_i = P_pop * exp(eta_i)
                sim_params[, i] <- typical_values[param] * exp(eta_scaled[, i])
              } else if (param_dist$distribution == "Normal") {
                # Normal distribution: P_i = P_pop + eta_i
                sim_params[, i] <- typical_values[param] + eta_scaled[, i]
              } else if (param_dist$distribution == "logitNormal") {
                # Logit-normal distribution: P_i = 1 / (1 + exp(-(logit(P_pop) + eta_i)))
                logit_pop <- log(typical_values[param] / (1 - typical_values[param]))
                logit_sim <- logit_pop + eta_scaled[, i]
                sim_params[, i] <- 1 / (1 + exp(-logit_sim))
              } else {
                # Default to log-normal for PK parameters
                sim_params[, i] <- typical_values[param] * exp(eta_scaled[, i])
              }
            } else {
              # Fallback to log-normal if distribution not found
              sim_params[, i] <- typical_values[param] * exp(eta_scaled[, i])
            }
          }
        }

        # Simulate predictions using perturbed parameters
        sim_pred <- rep(pred_data$PRED[row_idx], n_sim)

        # Apply parameter perturbations to predictions
        for (i in seq_along(random_effect_params)) {
          param <- random_effect_params[i]
          if (param %in% names(typical_values)) {
            # Calculate the proportional change in the parameter
            param_ratio <- sim_params[, i] / typical_values[param]
            # Apply the parameter change to the prediction
            sim_pred <- sim_pred * param_ratio
          }
        }

        # Add residual error
        sim_pred <- sim_pred + rnorm(n_sim, 0, sqrt(pred_data$Var_epsilon[row_idx]))

        # Calculate variance for this observation
        pred_data$Var_mc[row_idx] <- var(sim_pred)
        pred_data$Var_total[row_idx] <- pred_data$Var_epsilon[row_idx] + pred_data$Var_mc[row_idx]
      }

      # Calculate WRES using Monte Carlo variance
      pred_data <- pred_data %>%
        mutate(
          WRES = RES / sqrt(Var_total)
        ) %>%
        select(-Var_mc)  # Remove intermediate variance column

    } else {
      # No random effects, only residual error
      pred_data <- pred_data %>%
        mutate(
          Var_total = Var_epsilon,
          WRES = RES / sqrt(Var_total)
        )
    }

    return(pred_data)
  }
  #### main function ####
  input_path <- normalizePath(str_c(folder_path, proj_name, ".mlxtran"), mustWork = FALSE)
  if(!file.exists(input_path)) {
    stop("Project file does not exist. Check file existance or try to use absolute path")
  }

  contr_obj <- readLines(input_path)

  normalize_mlx_file_path <- function(path) {
    if (length(path) != 1L || is.na(path) || !nzchar(str_trim(path))) return(path)
    path <- str_trim(path)
    path <- str_replace_all(path, "\\\\", "/")
    path
  }

  ## info about datafile
  start_idx_data <- which(str_detect(contr_obj, fixed("<DATAFILE>")))
  end_idx_data <- which(str_detect(contr_obj, "\\<.*\\>") & seq_along(contr_obj) > start_idx_data)[1]
  data_path_raw <- contr_obj[(start_idx_data + 1):(end_idx_data - 1)] %>% str_squish() %>%
    str_subset(., "file=", negate = FALSE) %>% str_remove(., "^[^=]+=\\s*") %>%
    str_remove("^\\{path=") %>% str_remove("\\}\\s*$") %>%        # Monolix 2024: file={path='...'} -> bare path
    str_replace_all("'", "") %>%
    normalize_mlx_file_path()
  data_path <- data_path_raw
  if (!file.exists(data_path)) data_path <- file.path(folder_path, data_path)
  peek <- readLines(data_path, n = 1L, warn = FALSE, encoding = "UTF-8")
  if (!length(peek) || !nzchar(str_trim(peek[1]))) {
    stop("Data file is empty or unreadable: ", data_path)
  }
  first_line <- peek[1]
  # Monolix datasets are often tab-separated; read_csv would collapse the row into one column.
  data_file <- suppressMessages(
    if (grepl("\t", first_line, fixed = TRUE)) {
      read_tsv(data_path)
    } else {
      read_csv(data_path)
    }
  )
  colnames(data_file) <- gsub("[^[:alnum:]]+", "_", colnames(data_file))

  ## info about columns mapping
  start_idx_col_map <- which(str_detect(contr_obj, fixed("[CONTENT]")))
  end_idx_col_map <- which((str_detect(contr_obj, "\\[.*\\]") | str_detect(contr_obj, "\\<.*\\>")) &
                             seq_along(contr_obj) > start_idx_col_map)[1]
  dt_col_map <- contr_obj[(start_idx_col_map + 1):(end_idx_col_map - 1)] %>% str_squish() %>% str_subset(., "^$", negate = TRUE)


  col_map_df <- tibble(raw = dt_col_map) %>%
    mutate(COL = str_extract(raw, "^[^=]+") %>% str_trim(),
           inside = str_extract(raw, "\\{.*\\}") %>% str_remove_all("[\\{\\}]")) %>%
    separate_rows(inside, sep = ",\\s*") %>%
    separate(inside, into = c("key", "value"), sep = "=", fill = "right") %>%
    pivot_wider(names_from = key, values_from = value, values_fn = function(x) x[[1]]) %>%
    select(COL, everything(), -raw)

  ## Check for duplicate 'use' mappings
  # covariate / regressor: many columns may share the same use (legitimate in Monolix).
  use_counts <- col_map_df %>%
    filter(!is.na(use) & !use %in% c("covariate", "regressor")) %>%
    count(use) %>%
    filter(n > 1)

  if (nrow(use_counts) > 0) {
    for (duplicate_use in use_counts$use) {
      duplicate_cols <- col_map_df$COL[col_map_df$use == duplicate_use & !is.na(col_map_df$use)]

      # Check if columns exist in the dataset
      existing_cols <- duplicate_cols[duplicate_cols %in% names(data_file)]

      # Compare content of duplicate columns
      col_data <- data_file[existing_cols]

      # Check if all columns are identical
      all_identical <- TRUE
      for (i in 2:length(existing_cols)) {
        if (!identical(col_data[[1]], col_data[[i]])) {
          all_identical <- FALSE
          break
        }
      }

      if (all_identical) {
        # Keep only the first column, remove duplicates from col_map_df
        keep_col <- existing_cols[1]
        remove_indices <- which(col_map_df$use == duplicate_use & col_map_df$COL != keep_col)
        col_map_df <- col_map_df[-remove_indices, ]

        message(sprintf("Warning: Multiple columns found for use='%s' (%s). All columns have identical content. Using column '%s'.",
                        duplicate_use, paste(duplicate_cols, collapse = ", "), keep_col))
      } else {
        stop(sprintf("Error: Multiple columns found for use='%s' (%s) with different content. Please resolve the mapping conflict in the control file.",
                     duplicate_use, paste(duplicate_cols, collapse = ", ")))
      }

    }
  }

  ## Check for existing target column names and handle conflicts
  rename_mapping <- list(
    "ID" = col_map_df$COL[col_map_df$use == "identifier"],
    "TIME" = col_map_df$COL[col_map_df$use == "time"],
    "DV" = col_map_df$COL[col_map_df$use == "observation"],
    "DVID" = col_map_df$COL[col_map_df$use == "observationtype"],
    "AMT" = col_map_df$COL[col_map_df$use == "amount"],
    "EVID" = col_map_df$COL[col_map_df$use == "eventidentifier"],
    "MDV" = col_map_df$COL[col_map_df$use == "missingdependentvariable"],
    "CENS" = col_map_df$COL[col_map_df$use == "censored"],
    "OCC" = col_map_df$COL[col_map_df$use == "occasion"],
    "LIMIT" = col_map_df$COL[col_map_df$use == "limit"],
    "ADDL" = col_map_df$COL[col_map_df$use == "additionaldoses"],
    "II" = col_map_df$COL[col_map_df$use == "interdoseinterval"],
    "TINF" = col_map_df$COL[col_map_df$use == "infusiontime"],
    "SS" = col_map_df$COL[col_map_df$use == "steadystate"]
  )

  # Remove NULL values (where no mapping exists)
  rename_mapping <- rename_mapping[!sapply(rename_mapping, is_empty)]

  for (new_name in names(rename_mapping)) {
    old_name <- rename_mapping[[new_name]]

    # Check if old and new names are different
    if (!is.na(old_name) && old_name != new_name) {
      # Check if the target column name already exists in the dataset
      if (new_name %in% names(data_file)) {
        # Rename existing column to avoid conflict
        old_col_name <- paste0(new_name, "_old")
        data_file <- data_file %>% rename(!!old_col_name := !!new_name)
        message(sprintf("Warning: Column '%s' already exists in dataset. Renamed to '%s' to avoid conflict with mapping from '%s'.",
                        new_name, old_col_name, old_name))
      }
    }
  }

  ## dataset column renaming according to the mapping
  data_file_mod <- data_file %>%
    rename(ID = col_map_df$COL[col_map_df$use == "identifier"],
           TIME = col_map_df$COL[col_map_df$use == "time"],
           DV = col_map_df$COL[col_map_df$use == "observation"],
           DVID = col_map_df$COL[col_map_df$use == "observationtype"],
           AMT = col_map_df$COL[col_map_df$use == "amount"],
           EVID = col_map_df$COL[col_map_df$use == "eventidentifier"],
           MDV = col_map_df$COL[col_map_df$use == "missingdependentvariable"],
           CENS = col_map_df$COL[col_map_df$use == "censored"],
           OCC = col_map_df$COL[col_map_df$use == "occasion"],
           LIMIT = col_map_df$COL[col_map_df$use == "limit"],
           ADDL = col_map_df$COL[col_map_df$use == "additionaldoses"], # !!! check
           II = col_map_df$COL[col_map_df$use == "interdoseinterval"], # !!! check
           TINF = col_map_df$COL[col_map_df$use == "infusiontime"],
           SS = col_map_df$COL[col_map_df$use == "steadystate"]
           #  and expand
    )


  ## dvid and residual error mapping (Monolix <FIT>: maps observation-type indices to longitudinal outputs y1, y2, ...)
  start_idx_dvid_map <- which(str_detect(contr_obj, fixed("<FIT>")))
  end_idx_dvid_map <- which(str_detect(contr_obj, "\\<.*\\>") & seq_along(contr_obj) > start_idx_dvid_map)[1]

  dt_dvid_map <- contr_obj[(start_idx_dvid_map + 1):(end_idx_dvid_map - 1)] %>% str_squish() %>% str_subset(., "^$", negate = TRUE)

  extract_values <- function(line) {
    val <- str_remove(line, "^[^=]+=\\s*")
    val_clean <- val %>%
      str_remove_all("[\\{\\}]") %>%
      str_replace_all("['\"]", "") %>%
      str_trim()

    if (str_detect(val_clean, ",")) {
      str_split(val_clean, ",\\s*")[[1]]
    } else {
      val_clean
    }
  }

  parse_fit_endpoint_map <- function(fit_lines) {
    data_line <- fit_lines[str_detect(fit_lines, "^data\\s*=")][1]
    model_line <- fit_lines[str_detect(fit_lines, "^model\\s*=")][1]
    if (is.na(data_line) || !nzchar(as.character(data_line))) {
      stop("Monolix project: <FIT> block must contain a line 'data = {...}' (endpoint / observation-type indices).")
    }
    if (is.na(model_line) || !nzchar(as.character(model_line))) {
      stop("Monolix project: <FIT> block must contain a line 'model = {...}' (longitudinal output names, e.g. y1, y2).")
    }
    data_vals <- extract_values(data_line)
    model_vals <- extract_values(model_line)
    data_vals <- as.character(unlist(data_vals, use.names = FALSE))
    model_vals <- as.character(unlist(model_vals, use.names = FALSE))
    if (length(data_vals) != length(model_vals)) {
      stop(sprintf(
        "Monolix <FIT>: length of data list (%d) and model list (%d) must be equal (one longitudinal output per observation-type index).",
        length(data_vals), length(model_vals)
      ))
    }
    tibble(data = data_vals, model = model_vals)
  }

  extract_values_ruv <- function(x) {
    matches <- regmatches(x, gregexpr("\\(([^)]+)\\)", x))
    values <- lapply(matches, function(m) strsplit(gsub("[()]", "", m), "; |;")) %>% unlist()
    return(values)
  }

  parse_error_model <- function(error_model) {
    if (is.na(error_model) || is.null(error_model)) {
      return(list(err = NA_character_, pars = character(0)))
    }
    err <- str_extract(error_model, "^[^\\(]+") %>% str_trim()
    in_par <- str_match(error_model, "\\(([^\\)]*)\\)")[, 2]
    in_par <- ifelse(is.na(in_par), "", str_trim(in_par))
    par_names <- if (in_par == "") character(0) else str_split(in_par, ";|,")[[1]] %>% str_trim()
    list(err = err, pars = par_names)
  }

  extract_model_text <- function(contr_obj, folder_path) {
    model_path <- extract_model_path(contr_obj)
    if (is.na(model_path)) return(NA_character_)
    model_path_full <- normalizePath(file.path(folder_path, model_path), mustWork = FALSE)
    if (!file.exists(model_path_full)) return(NA_character_)
    str_c(readLines(model_path_full), collapse = "\n")
  }

  extract_model_path <- function(contr_obj) {
    start_idx <- which(str_detect(contr_obj, fixed("[LONGITUDINAL]")))[1]
    if (is.na(start_idx)) return(NA_character_)
    end_idx <- which((str_detect(contr_obj, "\\[.*\\]") | str_detect(contr_obj, "\\<.*\\>")) &
                       seq_along(contr_obj) > start_idx)[1]
    if (is.na(end_idx)) end_idx <- length(contr_obj) + 1
    long_section <- contr_obj[(start_idx + 1):(end_idx - 1)]
    model_line <- long_section %>% str_squish() %>% str_subset("^file\\s*=") %>% .[1]
    if (is.na(model_line) || length(model_line) == 0) return(NA_character_)
    model_line %>%
      str_remove("^[^=]+=\\s*") %>%
      str_replace_all("['\"]", "") %>%
      str_trim() %>%
      normalize_mlx_file_path()
  }

  extract_longitudinal_ruv_map <- function(contr_obj) {
    start_idx <- which(str_detect(contr_obj, fixed("[LONGITUDINAL]")))[1]
    if (is.na(start_idx)) return(tibble(COL = character(), distribution = character(), prediction = character()))
    end_idx <- which((str_detect(contr_obj, "\\[.*\\]") | str_detect(contr_obj, "\\<.*\\>")) &
                       seq_along(contr_obj) > start_idx)[1]
    if (is.na(end_idx)) end_idx <- length(contr_obj) + 1
    long_section <- contr_obj[(start_idx + 1):(end_idx - 1)]
    def_idx <- which(str_detect(long_section, "^\\s*DEFINITION:\\s*$"))[1]
    if (is.na(def_idx)) return(tibble(COL = character(), distribution = character(), prediction = character()))

    def_lines <- long_section[(def_idx + 1):length(long_section)] %>%
      str_squish() %>%
      str_subset("^$", negate = TRUE) %>%
      str_subset("=")

    tibble(raw = def_lines) %>%
      mutate(
        COL = str_extract(raw, "^[^=]+") %>% str_trim(),
        distribution = str_match(raw, "distribution\\s*=\\s*([^,\\}]+)")[, 2] %>% str_trim(),
        prediction = str_match(raw, "prediction\\s*=\\s*([^,\\}]+)")[, 2] %>% str_trim()
      ) %>%
      select(COL, distribution, prediction)
  }

  # Parse transformed covariates from [COVARIATE] DEFINITION:/EQUATION:
  extract_covariate_extensions <- function(contr_obj, data_dt) {
    start_idx_cov <- which(str_detect(contr_obj, fixed("[COVARIATE]")))[1]
    if (is.na(start_idx_cov)) {
      return(list(data = data_dt, added_cat = character(), added_cont = character()))
    }

    end_idx_cov <- which((str_detect(contr_obj, "\\[.*\\]") | str_detect(contr_obj, "\\<.*\\>")) &
                           seq_along(contr_obj) > start_idx_cov)[1]
    if (is.na(end_idx_cov)) end_idx_cov <- length(contr_obj) + 1
    cov_section <- contr_obj[(start_idx_cov + 1):(end_idx_cov - 1)]

    idx_def <- which(str_detect(cov_section, "^\\s*DEFINITION:\\s*$"))[1]
    idx_eq <- which(str_detect(cov_section, "^\\s*EQUATION:\\s*$"))[1]

    out_dt <- data_dt
    added_cat <- character()
    added_cont <- character()

    # Parse transformed categorical covariates in DEFINITION:
    if (!is.na(idx_def)) {
      def_end <- ifelse(!is.na(idx_eq) && idx_eq > idx_def, idx_eq - 1, length(cov_section))
      def_lines <- cov_section[(idx_def + 1):def_end]
      def_lines <- def_lines[!str_detect(def_lines, "^\\s*$")]

      i <- 1
      while (i <= length(def_lines)) {
        target_match <- str_match(def_lines[i], "^\\s*([[:alnum:]_]+)\\s*=\\s*$")
        if (is.na(target_match[1, 2])) {
          i <- i + 1
          next
        }

        target_name <- target_match[1, 2]
        if ((i + 1) > length(def_lines) || !str_detect(def_lines[i + 1], "\\{")) {
          i <- i + 1
          next
        }

        j <- i + 1
        brace_depth <- str_count(def_lines[j], "\\{") - str_count(def_lines[j], "\\}")
        block_lines <- def_lines[j]
        while (j < length(def_lines) && brace_depth > 0) {
          j <- j + 1
          block_lines <- c(block_lines, def_lines[j])
          brace_depth <- brace_depth + str_count(def_lines[j], "\\{") - str_count(def_lines[j], "\\}")
        }

        block_text <- str_c(block_lines, collapse = " ")
        source_match <- str_match(block_text, "transform\\s*=\\s*([[:alnum:]_]+)")
        source_name <- source_match[1, 2]

        if (!is.na(source_name) && source_name %in% names(out_dt)) {
          out_dt[[target_name]] <- NA_character_
          map_matches <- str_match_all(block_text, "'([^']+)'\\s*=\\s*\\{([^\\}]*)\\}")[[1]]
          if (nrow(map_matches) > 0) {
            for (k in seq_len(nrow(map_matches))) {
              tgt_level <- map_matches[k, 2]
              src_levels <- str_match_all(map_matches[k, 3], "'([^']+)'")[[1]][, 2]
              out_dt[[target_name]][out_dt[[source_name]] %in% src_levels] <- tgt_level
            }
          }
          added_cat <- c(added_cat, target_name)
        }

        i <- j + 1
      }
    }

    # Parse transformed/derived covariates in EQUATION:
    if (!is.na(idx_eq) && idx_eq < length(cov_section)) {
      eq_lines <- cov_section[(idx_eq + 1):length(cov_section)]
      eq_lines <- eq_lines[!str_detect(eq_lines, "^\\s*$")]
      eq_lines <- eq_lines[str_detect(eq_lines, "=")]

      for (eq_line in eq_lines) {
        lhs <- str_extract(eq_line, "^[^=]+") %>% str_trim()
        rhs <- str_remove(eq_line, "^[^=]+=") %>% str_trim()
        if (is.na(lhs) || lhs == "" || is.na(rhs) || rhs == "") next

        eval_env <- new.env(parent = baseenv())
        list2env(as.list(out_dt), envir = eval_env)
        out_dt[[lhs]] <- tryCatch(
          eval(parse(text = rhs), envir = eval_env),
          error = function(e) rep(NA_real_, nrow(out_dt))
        )
        added_cont <- c(added_cont, lhs)
      }
    }

    return(list(
      data = out_dt,
      added_cat = unique(added_cat),
      added_cont = unique(added_cont)
    ))
  }


  replace_commas_in_parentheses <- function(input_string) {
    matches <- gregexpr("\\(([^()]*)\\)", input_string)

    modified_string <- input_string
    for (match in regmatches(input_string, matches)) {
      modified_substring <- gsub(",", ";", match)
      modified_string <- sub(match, modified_substring, modified_string)
    }

    return(modified_string)
  }


  dvid_map_df <- parse_fit_endpoint_map(dt_dvid_map)

  start_idx_ruv_map <- which(str_detect(contr_obj, fixed("[LONGITUDINAL]")))
  end_idx_ruv_map <- which(str_detect(contr_obj, "\\<.*\\>") & seq_along(contr_obj) > start_idx_ruv_map)[1]

  dt_ruv_map <- contr_obj[(start_idx_ruv_map + 1):(end_idx_ruv_map - 1)] %>%
    str_squish() %>% str_subset(., "^$", negate = TRUE)
  dt_ruv_map <- grep(str_c(str_c(dvid_map_df$model, " ="), collapse = "|"), dt_ruv_map, value = TRUE) %>%
    tibble(raw = .) %>%
    mutate(COL = str_extract(raw, "^[^=]+") %>% str_trim()) %>%
    group_by(COL) %>%
    mutate(inside = str_extract(raw, "\\{.*\\}") %>% replace_commas_in_parentheses(.) %>% str_remove_all("[\\{\\}]")) %>%
    ungroup() %>%
    separate_rows(inside, sep = ",\\s*") %>%
    separate(inside, into = c("key", "value"), sep = "=", fill = "right") %>%
    pivot_wider(names_from = key, values_from = value, values_fn = function(x) x[[1]]) %>%
    group_by(COL) %>%
    mutate(RUVpars = list(extract_values_ruv(errorModel)),
           RUVpar_a = ifelse(grepl("constant|combined", errorModel), unlist(RUVpars)[1], NA) ,
           RUVpar_b = ifelse(grepl("proportional", errorModel), unlist(RUVpars)[1],
                             ifelse(grepl("combined", errorModel),unlist(RUVpars)[2], NA))) %>%
    ungroup() %>%
    select(COL, everything(), -c("raw", "RUVpars")) %>%
    left_join(dvid_map_df %>% rename(COL = model, dvid = data), "COL")


  ## cotab, catab, regtab compiling
  cov_ext <- extract_covariate_extensions(contr_obj, data_file_mod)
  data_file_mod <- cov_ext$data

  cotab_cols <- col_map_df %>% filter(use == "identifier" | (use == "covariate" & type == "continuous")) %>% select(COL) %>% pull()
  catab_cols <- col_map_df %>% filter(use == "identifier" | (use == "covariate" & type == "categorical")) %>% select(COL) %>% pull()
  cotab_cols <- unique(c(cotab_cols, cov_ext$added_cont))
  catab_cols <- unique(c(catab_cols, cov_ext$added_cat))
  cotab_cols <- cotab_cols[cotab_cols %in% names(data_file_mod)]
  catab_cols <- catab_cols[catab_cols %in% names(data_file_mod)]
  regtab_cols <- col_map_df %>% filter(use %in% c("identifier", "time", "regressor")) %>% select(COL) %>% pull()

  if (length(cotab_cols) > 1) {cotab <- data_file_mod %>% select(all_of(cotab_cols)) %>% unique()} else {cotab <- data.frame()}
  if (length(catab_cols) > 1) {catab <- data_file_mod %>% select(all_of(catab_cols)) %>% unique()} else {catab <- data.frame()}
  if (length(regtab_cols) > 2) {regtab <- data_file_mod %>% select(all_of(regtab_cols)) %>% unique()} else {regtab <- data.frame()}


  ## patab and sumtab compiling

  sum_dt_i <- readr::read_csv(str_c(folder_path, proj_name, "/populationParameters", ".txt"), col_types = cols())

  pop_params <- sum_dt_i$parameter[str_detect(sum_dt_i$parameter, "_pop$")]
  params <- str_replace(pop_params, "_pop$", "")
  omega_params <- sum_dt_i$parameter[sum_dt_i$parameter %in% str_c("omega_", params)]
  beta_params <- sum_dt_i$parameter[str_detect(sum_dt_i$parameter, "^beta_")]
  corr_params <- sum_dt_i$parameter[str_detect(sum_dt_i$parameter, "^corr_")]
  resid_err_params <- sum_dt_i$parameter[!sum_dt_i$parameter %in% c(pop_params, omega_params, beta_params, corr_params)]
  eta_params <- str_replace(omega_params, "omega_", "eta_")

  long_ruv_map <- extract_longitudinal_ruv_map(contr_obj)

  ## sdtab compiling
  sdtab <- unique(dvid_map_df$model) %>% map_dfr(function(y_name) {

    if (length(dvid_map_df$model) == 1) {y_name_i <- ""; dvid_i <- 1} else {y_name_i <- str_c("_", y_name); dvid_i <- dvid_map_df$data[dvid_map_df$model == y_name] %>% as.numeric()}
    recode_vector <- c(
      "ID" = "id",
      "TIME" = "time"
    )
    pred_dt_i <- readr::read_csv(str_c(folder_path, proj_name, "/predictions",y_name_i, ".txt"), col_types = cols()) %>%
      rename(any_of(recode_vector))

    # if there is no DVID column?
    obs_data_i <- data_file_mod %>% filter(DVID == dvid_i) %>%
      filter(EVID != 1) %>%
      select(any_of(c("ID", "TIME", "MDV", "OCC", "BLQ", "CENS", "LIMIT"))) %>% unique()

    if (any(grepl("_mode$", colnames(pred_dt_i)))){suffix <- "_mode"} else {suffix <- "_SAEM"}

    # Get residual error parameters for this DVID
    ruv_info <- dt_ruv_map %>% filter(dvid == dvid_i)

    # Detect model structure
    model_structure <- detect_model_structure(contr_obj)

    # Debug information
    message(sprintf("Detected model structure:%s\n", model_structure))
    message(sprintf("Omega parameters: %s\n", paste(omega_params, collapse = ", ")))

    # Get parameter names that have random effects
    random_effect_params <- str_replace(omega_params, "omega_", "")
    message(sprintf("Random effect parameters:%s\n", paste(random_effect_params, collapse = ", ")))
    message("Using Monte Carlo simulation with n_sim = 1000\n")

    # Extract parameter distributions
    param_distributions <- extract_parameter_distributions(contr_obj)
    message(sprintf("Extracted parameter distributions: %.0f parameters\n", length(param_distributions)))
    for (param in names(param_distributions)) {
      dist_info <- param_distributions[[param]]
      message("  %s: %s (typical=%.2f, sd=%.2f)\n", param,  dist_info$distribution, dist_info$typical, dist_info$sd)
    }

    # Prepare sdtab data
    sdtab_i <- left_join(pred_dt_i, obs_data_i, by = c("ID", "TIME")) %>%
      rename(PRED = popPred,
             IPRED = str_c("indivPred", suffix), #indivPred_mode,
             IWRES = str_c("indWRes", suffix), #indWRes_mode,
             DV = all_of(y_name)) %>%
      mutate(RES = PRED - DV, IRES = IPRED - DV,
             DVID = dvid_i,
             DVNAME = long_ruv_map$prediction[match(y_name, long_ruv_map$COL)])

    residuals_file <- str_c(folder_path, proj_name, "/ChartsData/ScatterPlotOfTheResiduals/y1_residuals", y_name_i, ".txt")

    if (file.exists(residuals_file)) {
      # Read the residuals file
      residuals_dt_i <- readr::read_csv(residuals_file, col_types = cols()) %>%
        rename(any_of(recode_vector))

      # Check if the required column 'pwRes' exists
      if ("pwRes" %in% names(residuals_dt_i)) {
        # Select and rename to WRES
        residuals_dt_i <- residuals_dt_i %>%
          select(ID, TIME, WRES = pwRes)

        # Join to the main sdtab_i
        sdtab_i <- sdtab_i %>%
          left_join(residuals_dt_i, by = c("ID", "TIME"))
      }
    }
    else {

      # Calculate WRES using Monte Carlo simulation
      if (nrow(ruv_info) > 0) {
        # Use the Monte Carlo simulation function to calculate WRES
        sdtab_i <- calculate_wres_mc(sdtab_i, omega_params, resid_err_params, sum_dt_i, ruv_info, param_distributions)
      } else {
        # If no residual error model info, set WRES to NA
        sdtab_i <- sdtab_i %>% mutate(WRES = NA_real_)
      }

    }

    if (!"MDV" %in% names(sdtab_i)) {
      sdtab_i <- sdtab_i %>% mutate(MDV = 0L)
    }

    sdtab_i %>%
      select(any_of(c("ID", "TIME", "DV", "DVID", "DVNAME", "PRED", "IPRED", "RES", "IRES", "WRES", "CWRES",
                      "IWRES", "EVID", "MDV", "OCC", "BLQ", "CENS", "LIMIT")))

  })

  ## reverse naming
  #sdtab <- sdtab %>%
  #  rename(!!!setNames(col_map_df$COL[col_map_df$use %in% c("identifier", "time")], c("ID", "TIME")))


  ## patab and sumtab compiling


  eta_i <- readr::read_csv(str_c(folder_path, proj_name, "/IndividualParameters/estimatedRandomEffects.txt"), col_types = cols())
  indpar_i <- readr::read_csv(str_c(folder_path, proj_name, "/IndividualParameters/estimatedIndividualParameters.txt"), col_types = cols())

  if (any(grepl("_mode$", colnames(indpar_i)))){suffix <- "_mode"} else {suffix <- "_SAEM"}
  eta_clnms <- c("id", str_c(eta_params, suffix))
  patab <- left_join(eta_i %>% select(any_of(eta_clnms)), #select_at(vars("id", ends_with(suffix))),
                     indpar_i %>% select_at(vars("id", ends_with(suffix)))) %>%
    rename_with(~ str_replace(., str_c(suffix, "$"), ""), ends_with(suffix)) %>% rename(ID = id)


  ## sumtab compiling
  param_distributions <- extract_parameter_distributions(contr_obj)
  resid_dist_map <- extract_residual_param_distributions(contr_obj, dt_ruv_map)

  if (sum(grepl("sa$", colnames(sum_dt_i))) > 0 &
      sum(grepl("lin$", colnames(sum_dt_i))) > 0) sum_dt_i <-sum_dt_i %>%
    select(-ends_with("lin"))
  sumtab <- sum_dt_i %>%
    rename(PAR = parameter, VALUE = value) %>%
    rename(!!!setNames(c(colnames(sum_dt_i)[grepl("^se_", colnames(sum_dt_i))],
                         colnames(sum_dt_i)[grepl("^rse_", colnames(sum_dt_i))]),
                       c("SE", "RSE"))) %>%
    mutate(LCI = VALUE - 1.96*SE, UCI = VALUE + 1.96*SE) %>%
    group_by(PAR) %>%
    mutate(TYPE =case_when(PAR %in% pop_params ~ "Typical values",
                           PAR %in% omega_params ~ "Random effects",
                           PAR %in% beta_params ~ "Covariate coefficients",
                           PAR %in% corr_params ~ "Correlation coefficients",
                           T ~ "Residual error model"),
           PAR_NAME = str_replace(PAR, "_pop|omega_", ""),
           OMEGA = ifelse(str_detect(PAR, "_pop"), sum_dt_i$value[sum_dt_i$parameter == str_c("omega_", PAR_NAME)], NA),
           ETAshrinkage_var = ifelse(PAR %in% omega_params, 100*(1 - sd(patab[[str_c("eta_", PAR_NAME)]])^2/VALUE^2), NA),
           ETAshrinkage_sd = ifelse(PAR %in% omega_params, 100*(1 - sd(patab[[str_c("eta_", PAR_NAME)]])/VALUE), NA),
           EPSshrinkage_sd = ifelse(PAR %in% resid_err_params, (1 - sd(sdtab$IWRES[sdtab$DVID == na.omit(dt_ruv_map$dvid[dt_ruv_map$RUVpar_a == PAR | dt_ruv_map$RUVpar_b == PAR]) %>% as.numeric()]))*100, NA),
           DISTRIBUTION = case_when(
             PAR %in% pop_params ~ sapply(PAR_NAME, function(x) {
               dist_i <- param_distributions[[x]]$distribution
               if (is.null(dist_i) || is.na(dist_i)) NA_character_ else dist_i
             }),
             PAR %in% omega_params ~ NA_character_,
             PAR %in% beta_params ~ NA_character_,
             PAR %in% resid_err_params ~ unname(resid_dist_map[PAR]),
             TRUE ~ NA_character_
           ),
           EST = ifelse(is.na(SE), "FIXED", "ESTIMATED")
    ) %>%
    select(any_of(c("PAR", "VALUE", "TYPE", "DISTRIBUTION", "EST", "SE", "RSE", "CV", "LCI", "UCI", "ETAshrinkage_sd", "ETAshrinkage_var", "EPSshrinkage_sd")))


  ## covmat and corrmat compiling

  fi_files <- list.files(str_c(folder_path, proj_name, "/FisherInformation"))
  corr_path <- str_c(folder_path, proj_name, "/FisherInformation/", fi_files[str_detect(fi_files, "correlation")])
  cov_path <- str_c(folder_path, proj_name, "/FisherInformation/", fi_files[str_detect(fi_files, "covariance")])

  covmat_dt <- readr::read_csv(cov_path, col_types = cols(), col_names = FALSE)
  colnames(covmat_dt) <- c("PAR", covmat_dt$X1)
  covmat <- covmat_dt %>% select(-PAR) %>% as.matrix()

  corrmat_dt <- readr::read_csv(corr_path, col_types = cols(), col_names = FALSE)
  colnames(corrmat_dt) <- c("PAR", corrmat_dt$X1)
  corrmat <- corrmat_dt %>% select(-PAR) %>% as.matrix()

  ## evtab compiling

  evtab <- data_file_mod %>% filter(EVID == 1) %>%
    select(any_of(c("ID", "TIME", "OCC", "EVID", "CMT", "ADM", "AMT", "ADDL", "II", "DUR", "TINF", "RATE", "SS")))


  ## omegamat compiling

  omegamat <- setNames(data.frame(matrix(ncol = length(omega_params), nrow = length(omega_params))), omega_params)
  if (!is_empty(omega_params)) {
    for (i in 1:length(omega_params)){

      omega_par <- omega_params[i]
      omegamat[i,i] <-  sumtab$VALUE[sumtab$PAR == omega_par]^2

    }

    # if correlations exists
    if (!is_empty(corr_params)){

      corr_params_dt <- data.frame(CORR_PAR = corr_params) %>%
        group_by(CORR_PAR) %>%
        mutate(PAR1 = na.omit(str_extract(CORR_PAR, str_remove(omega_params, "omega_")))[1],
               PAR2 = na.omit(str_extract(CORR_PAR, str_remove(omega_params, "omega_")))[2],
               PAR_N1 = which(PAR1 == str_remove(omega_params, "omega_")),
               PAR_N2 = which(PAR2 == str_remove(omega_params, "omega_"))) %>%
        ungroup()

      for (i in 1:length(corr_params)){

        corr_params_dt_i <- corr_params_dt %>% filter(CORR_PAR == corr_params[i])
        corr_value <- sumtab$VALUE[sumtab$PAR == corr_params[i]]
        var1_value <- omegamat[corr_params_dt_i$PAR_N1, corr_params_dt_i$PAR_N1]
        var2_value <- omegamat[corr_params_dt_i$PAR_N2, corr_params_dt_i$PAR_N2]

        omegamat[corr_params_dt_i$PAR_N1, corr_params_dt_i$PAR_N2] <- corr_value*var1_value*var2_value
        omegamat[corr_params_dt_i$PAR_N2, corr_params_dt_i$PAR_N1] <- corr_value*var1_value*var2_value

      }
    }

    omegamat[is.na(omegamat)] <- 0
    omegamat <- omegamat %>% as.matrix()
  } else {omegamat <- matrix()}

  ## sigmamat compiling

  sigmamat <- setNames(data.frame(matrix(ncol = length(resid_err_params), nrow = length(resid_err_params))), resid_err_params)
  for (i in 1:length(resid_err_params)){

    resid_par <- resid_err_params[i]
    sigmamat[i,i] <-  sumtab$VALUE[sumtab$PAR == resid_par]
  }
  sigmamat <- sigmamat %>% as.matrix()

  ## occmat compiling - !!! re-write
  occmat <- matrix()

  ## ofv compiling

  summ_fl <- readLines(str_c(folder_path, proj_name, "/summary.txt"))

  ofv <- tibble(LL = grep("log-likelihood", summ_fl, fixed = TRUE, value = TRUE) %>%
                  gsub("^.*?:", "", .) %>% parse_number(),
                AIC = grep("AIC", summ_fl, fixed = TRUE, value = TRUE) %>% parse_number(),
                BIC = grep("(BIC)", summ_fl, fixed = TRUE, value = TRUE) %>% parse_number(),
                BICc = grep("(BICc)", summ_fl, fixed = TRUE, value = TRUE) %>% parse_number()) #%>% unlist()

  ## options compiling - !!! re-write
  opt <- NULL

  ## gco compiling
  headers_gco <- col_map_df %>%
    transmute(
      name = COL,
      use = use,
      type = ifelse(is.na(type) | type == "", NA_character_, type)
    ) %>%
    rowwise() %>%
    mutate(.header = list(list(
      name = name,
      use = use,
      type = if (is.na(type)) setNames(list(), character(0)) else type
    ))) %>%
    ungroup() %>%
    pull(.header)

  start_idx_param <- which(str_detect(contr_obj, fixed("<PARAMETER>")))[1]
  end_idx_param <- which(str_detect(contr_obj, "\\<.*\\>") & seq_along(contr_obj) > start_idx_param)[1]
  if (is.na(end_idx_param)) end_idx_param <- length(contr_obj) + 1
  param_lines <- contr_obj[(start_idx_param + 1):(end_idx_param - 1)] %>%
    str_squish() %>%
    str_subset("^$", negate = TRUE)
  param_map <- tibble(raw = param_lines) %>%
    mutate(PAR = str_extract(raw, "^[^=]+") %>% str_trim(),
           value = str_match(raw, "value\\s*=\\s*([^,\\}]+)")[, 2] %>% as.numeric(),
           method = str_match(raw, "method\\s*=\\s*([^,\\}]+)")[, 2] %>% str_trim())

  theta_gco <- tibble(NAME = str_remove(pop_params, "_pop$")) %>%
    mutate(
      INIT = param_map$value[match(str_c(NAME, "_pop"), param_map$PAR)],
      EST = !toupper(param_map$method[match(str_c(NAME, "_pop"), param_map$PAR)]) %in% c("FIXED"),
      TRANS = sapply(NAME, function(x) {
        d <- param_distributions[[x]]$distribution
        if (is.null(d) || is.na(d)) NA_character_ else unname(d)
      })
    )
  theta_gco <- lapply(seq_len(nrow(theta_gco)), function(i) {
    list(
      NAME = theta_gco$NAME[i],
      INIT = theta_gco$INIT[i],
      EST = theta_gco$EST[i],
      TRANS = theta_gco$TRANS[i]
    )
  })

  ruv_trans <- long_ruv_map$distribution[match(dt_ruv_map$COL, long_ruv_map$COL)]
  ruv_pred <- long_ruv_map$prediction[match(dt_ruv_map$COL, long_ruv_map$COL)]
  if ("distribution" %in% names(dt_ruv_map)) {
    ruv_trans[is.na(ruv_trans)] <- dt_ruv_map$distribution[is.na(ruv_trans)]
  }
  if ("prediction" %in% names(dt_ruv_map)) {
    ruv_pred[is.na(ruv_pred)] <- dt_ruv_map$prediction[is.na(ruv_pred)]
  }
  ruv_gco <- dt_ruv_map %>%
    mutate(.idx = row_number()) %>%
    rowwise() %>%
    mutate(
      .err = list(parse_error_model(errorModel)),
      YNAME = COL,
      DVID = as.character(dvid),
      TRANS = ruv_trans[.idx],
      PRED = ruv_pred[.idx],
      ERR = .err$err,
      INIT = list(if (length(.err$pars) > 0) param_map$value[match(.err$pars, param_map$PAR)] else NA_real_),
      EST = list(if (length(.err$pars) > 0) (!toupper(param_map$method[match(.err$pars, param_map$PAR)]) %in% c("FIXED")) else NA)
    ) %>%
    ungroup() %>%
    select(-.idx)
  ruv_gco <- lapply(seq_len(nrow(ruv_gco)), function(i) {
    list(
      YNAME = ruv_gco$YNAME[i],
      DVID = ruv_gco$DVID[i],
      TRANS = ruv_gco$TRANS[i],
      PRED = ruv_gco$PRED[i],
      ERR = ruv_gco$ERR[i],
      INIT = ruv_gco$INIT[[i]],
      EST = ruv_gco$EST[[i]]
    )
  })
  if (length(ruv_gco) == 1) ruv_gco <- ruv_gco[[1]]

  re_names <- str_remove(pop_params, "_pop$")
  re_init <- matrix(0, nrow = length(re_names), ncol = length(re_names))
  re_est <- matrix(NA, nrow = length(re_names), ncol = length(re_names))
  colnames(re_init) <- rownames(re_init) <- re_names
  colnames(re_est) <- rownames(re_est) <- re_names
  if (length(omega_params) > 0) {
    for (op in omega_params) {
      p_name <- str_remove(op, "omega_")
      p_idx <- match(p_name, re_names)
      if (!is.na(p_idx)) {
        re_init[p_idx, p_idx] <- param_map$value[match(op, param_map$PAR)]
        re_est[p_idx, p_idx] <- !toupper(param_map$method[match(op, param_map$PAR)]) %in% c("FIXED")
      }
    }
  }
  re_gco <- list(init = re_init, est = re_est)
  occ_gco <- list(
    init = matrix(0, nrow = length(re_names), ncol = length(re_names), dimnames = list(re_names, re_names)),
    est = matrix(NA, nrow = length(re_names), ncol = length(re_names), dimnames = list(re_names, re_names))
  )

  covs_gco <- col_map_df %>% filter(use == "covariate") %>% pull(COL)
  model_path_raw <- extract_model_path(contr_obj)
  gco <- list(
    headers = headers_gco,
    data = c(data_path_raw),
    model = c(model_path_raw),
    task_opt = setNames(list(), character(0)),
    covs = covs_gco,
    project_name = proj_name,
    theta = theta_gco,
    ruv = ruv_gco,
    re = re_gco,
    occ = occ_gco,
    modelText = extract_model_text(contr_obj, folder_path)
  )

  ## final
  gfo <- list(SDTAB = sdtab,
              SUMTAB = sumtab,
              SIGMAMAT = sigmamat,
              OMEGAMAT = omegamat,
              OCCMAT = occmat,
              EVTAB = evtab,
              PATAB = patab,
              COTAB = cotab,
              CATAB = catab,
              REGTAB = regtab,
              OFV = ofv,
              COVMAT = covmat,
              CORRMAT = corrmat,
              OPTIONS = opt,
              PROJNAME = proj_name)
  sg_object <- list(GFO = gfo, GCO = gco)

  if (isTRUE(save_file)) {
    output_dir <- normalizePath(folder_path, mustWork = FALSE)
    if (!dir.exists(output_dir)) {
      dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)
    }

    gco_path <- file.path(output_dir, str_c(proj_name, "_GCO.json"))
    gfo_path <- file.path(output_dir, str_c(proj_name, "_GFO.json"))
    writeLines(jsonlite::toJSON(gco, pretty = TRUE, auto_unbox = FALSE, null = "null"), gco_path)
    writeLines(jsonlite::toJSON(gfo, pretty = TRUE, auto_unbox = FALSE, null = "null"), gfo_path)

    gco_rdata_path <- file.path(output_dir, str_c(proj_name, "_GCO.RData"))
    gfo_rdata_path <- file.path(output_dir, str_c(proj_name, "_GFO.RData"))
    save(gco, file = gco_rdata_path)
    save(gfo, file = gfo_rdata_path)
  }

  return(sg_object)


}

## Author: Yaroslav Ugolkov
## First created: 2025-10-17
## Description: Basic testing of sg-converter function
## Keywords: SimuRg, sg-converter

test_that("sg-converter works and contains all elements", {
  test_folder <- system.file("extdata", "Monolix_objects", package = "SimuRg")
  if (substr(test_folder, nchar(test_folder), nchar(test_folder)) != "/")
    test_folder <- str_c(test_folder, "/")
  pro_name <- "proj-solo"

  # Simple function call
  result <- sg_converter(folder_path = test_folder, proj_name = pro_name)
  expect_type(result, "list")

  required_elements <- c("SDTAB", "SUMTAB", "SIGMAMAT", "OMEGAMAT", "OCCMAT",
                         "EVTAB", "PATAB", "COTAB", "CATAB", "REGTAB",
                         "OFV", "COVMAT", "CORRMAT", "OPTIONS", "PROJNAME")
  expect_true(all(required_elements %in% names(result$GFO)))

  expect_gt(nrow(result$GFO$SDTAB), 0)
  expect_gt(nrow(result$GFO$SUMTAB), 0)
  expect_gt(nrow(result$GFO$PATAB), 0)
})

test_that("sg-converter fails gracefully", {
  expect_error(
    suppressWarnings(sg_converter("invalid_path", "invalid_project")),
    "Project file does not exist. Check file existance or try to use absolute path"
  )
})

## Author: Mikhailova Anna
## First created: 2025-10-27
## Description: functions for vpc calculations
## Keywords: SimuRg, vpc, diagnostics

#' Perform simulations for VPC plot
#'
#' @inheritParams sg_dummy
#' @param fpath_i Either a character string specifying the file path to a saved `sg_fit` object (R data file), or a list object containing the `sg_fit` results directly. The object must contain: `SDTAB`, `EVTAB`, `SUMTAB`, `OMEGAMAT`, `SIGMAMAT`, and optionally `COTAB` and `CATAB`
#' @param gco Generalized control object. Either an R list or path to Rdata/json file. Either `gco` or `model` file should be specified for simulations model specification. Defauls is `NULL`
#' @param npop Integer specifying the number of virtual subjects to simulate per original individual. Higher values provide more robust percentile estimates but increase computation time. Default is `100`
#' @returns A dataset with simulation results
#' @examples
#' \donttest{
#' library(rxode2)
#' library(tibble)
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
#' # Simulate VPC from with generalized model object
#' mod <- rxode2::rxode2({
#'   ka_pop = 0.1;
#'   Vd_pop = 10;
#'   Cl_pop = 0.5;
#'
#'   omega_ka = 0;
#'   omega_Vd = 0;
#'   omega_Cl = 0;
#'
#'   Cc_b = 0;
#'   ka_tv = exp(ka_pop);
#'   Vd_tv = exp(Vd_pop);
#'   Cl_tv = exp(Cl_pop);
#'
#'   ka = ka_tv * exp(omega_ka);
#'   Vd = Vd_tv * exp(omega_Vd);
#'   Cl = Cl_tv * exp(omega_Cl);
#'
#'   Cc = Ac / Vd;
#'
#'   Ad(0) = 0;
#'   Ac(0) = 0;
#'
#'   d/dt(Ad) = -ka * Ad;
#'   d/dt(Ac) = ka * Ad - Cl * Cc;
#'
#'   Cc_ResErr = Cc * (1 + Cc_b);
#' })
#'
#' sg_vpc_sim(fpath_i, model=mod, outputs = "Cc_ResErr")
#' # Make VPC plot with the generalized control object
#' model <- system.file("extdata", "models", "rxode", "model_PK_1c.txt", package = "SimuRg")
#' data  <- system.file("extdata", "datasets", "dspk-warf.csv", package = "SimuRg")
#' headers <- list(list(name = "ID", use = "identifier", type = NULL),
#'              list(name = "TIME", use = "time", type = NULL),
#'                           list(name = "DV", use = "observation", type = "continuous"),
#'                           list(name = "DVID", use = "observationtype", type = NULL),
#'                           list(name = "ADM", use = "administration", type = NULL),
#'                           list(name = "AMT", use = "amount", type = NULL),
#'                           list(name = "EVID", use = "eventidentifier", type = NULL),
#'                           list(name = "MDV", use = "missingdependentvariable", type = NULL),
#'                           list(name = "AGE", use = "covariate", type = "continuous"),
#'                           list(name = "AGE_centered", use = "covariate", type = "continuous"),
#'                           list(name = "SEX", use = "covariate", type = "categorical"),
#'                           list(name = "WEIGHT", use = "covariate", type = "continuous"),
#'                           list(name = "BMI", use = "covariate", type = "continuous"))
#'
#' theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
#'                   "Cl", "logNormal", 0.2, NA, NA, TRUE,
#'                 "V", "logNormal", 20, NA, NA, TRUE,
#'                 "ka", "logNormal", 0.2, NA, NA, TRUE
#')
#'
#' ruv <- list(YNAME = "y1", DVID = 1, TRANS = "normal", PRED = "Cc",
#'            ERR = "combined1", INIT = c(1, 1), EST = c(TRUE, TRUE), BLQM = NULL)
#'
#' re <- list(init = tribble(~Cl, ~V, ~ka,
#'                             1, 0, 0,
#'                             0, 1, 0,
#'                             0, 0, 1) %>% as.matrix(),
#'             est = tribble(~Cl, ~V, ~ka,
#'                          TRUE, NA, NA,
#'                            NA, TRUE, NA,
#'                           NA, NA, TRUE) %>% as.matrix())
#' occ <- list(init = tribble(~Cl, ~V, ~ka,
#'                         0, 0, 0,
#'                           0, 0, 0,
#'                           0, 0, 0) %>% as.matrix(),
#'            est = tribble(~Cl, ~V, ~ka,
#'                          NA, NA, NA,
#'                          NA, NA, NA,
#'                          NA, NA, NA) %>% as.matrix())
#' covs <- list(list(PAR = "V", COVNAME = "AGE", FUNC = "linear",
#'                  TRANS = "median", INIT = 1, EST = TRUE),
#'             list(PAR = "ka", COVNAME = "SEX", REF = 0, INIT = 1, EST = TRUE))
#'
#' gco <-list(headers = headers, data = data, model = model, task_opt = "",
#'           covs = covs, project_name = "test-proj", theta = theta,
#'           ruv = ruv, re = re, occ = occ, modelText = "")
#'
#' res <- sg_vpc_sim(fpath_i, gco = gco, output = "Cc")
#' }
#' @import rxode2
#' @importFrom purrr map_dfr
#' @import dplyr
#' @importFrom stringr str_remove
#' @importFrom rlang .data
#' @export
sg_vpc_sim <- function(fpath_i, gco = NULL, model = NULL, time_col = "TIME", outputs = NULL, npop = 100){
  obj <- read_smrg_obj(fpath_i)
  if (is_null(model) & is_null(gco)) {
    stop("Specify either a generalized control object (gco) or model to simulate from")
  } else if (is_null(model) & !is_null(gco)) {
    model <- rxode2::rxode2(gmo_converter(gco, output_path = NULL))
  } else if (!is_null(model) & !is_null(gco)) {
    message("Both gco and model specified. Model from gco is used for simulations")
    model <- rxode2::rxode2(gmo_converter(gco))
  }
  data_fin.noex <-  obj$SDTAB %>% filter(MDV != 1) %>% select(-MDV)
  data_fin.noex$time  <- data_fin.noex[[time_col]]
  ev_tab <- obj$EVTAB

  if (!is.null(obj$COTAB)) ev_tab <-  merge(ev_tab, obj$COTAB, by = "ID", all.x = TRUE)
  if (!is.null(obj$CATAB)) ev_tab <-  merge(ev_tab, obj$CATAB, by = "ID", all.x = TRUE)
  if (!is_empty(obj$REGTAB)) ev_tab <-  merge(ev_tab, obj$REGTAB, by = c("ID", time_col), all.x = TRUE)
  covs_i <- c(colnames(obj$COTAB), colnames(obj$CATAB))
  covs_i <- covs_i[covs_i != "ID"]
  id_seq <- unique(data_fin.noex$ID)
  par_fin_tv <- obj$SUMTAB %>% filter(TYPE == "Typical values") %>%
    select(PAR, VALUE) %>% #mutate(PAR = str_remove(PAR, "_pop")) %>%
    deframe()
  sim_vpc_full <- id_seq %>% map_dfr(function(id_seq.i){
    data_fin.noex.i <- data_fin.noex %>% filter(ID == id_seq.i) %>% pull(time)
    ev_tab.i <- ev_tab %>% filter(ID == id_seq.i) %>%
      rename(DEFID = ID) %>% mutate(id = 1)
    sim.i <- sg_sim(model = model, et = ev_tab.i, stimes = data_fin.noex.i,
                    outputs = outputs, theta = par_fin_tv, omega = obj$OMEGAMAT,
                    sigma = obj$SIGMAMAT, covs = covs_i, nsub = npop, byID = TRUE,
                    addcov = FALSE, ncores = parallel::detectCores()-1) %>%
      mutate(ID = id_seq.i)
    return(sim.i)
  })
  return(sim_vpc_full)
}

mod_fin <- RxODE({
  # Doses in mg
  # Time in hours

  ### Parameter values
  # Typical
  Cl_pop = 5;
  V_pop = 180;

  ka_pop = 6;


  # Random effects
  omega_Cl = 0;
  omega_V = 0;
  omega_ka = 0;

  # Residual error
  Cc_b = 0;

  ### Parameters
  Cl = Cl_pop * exp(omega_Cl);
  V = V_pop * exp(omega_V);
  ka = ka_pop * exp(omega_ka);

  ### Explicit functions
  Cc = Ac/V;                 # nmol/L

  ### Initial conditions
  Ad(0) = 0;          # mg
  Ac(0) = 0;          # mg

  ### ODEs
  d/dt(Ad) = - ka*Ad;
  d/dt(Ac) = ka*Ad - Cl*Cc ;

  CHECKRUV = Cc_b;
  Cc_ResErr = Cc + Cc_b*Cc;
})

model <-  system.file("extdata", "models", "rxode", "model_PK_1c.txt", package = "SimuRg")
data  <- system.file("extdata", "datasets", "dspk-warf.csv", package = "SimuRg")

headers <- list(list(name = "ID", use = "identifier", type = NULL),
                list(name = "TIME", use = "time", type = NULL),
                list(name = "DV", use = "observation", type = "continuous"),
                list(name = "DVID", use = "observationtype", type = NULL),
                list(name = "ADM", use = "administration", type = NULL),
                list(name = "AMT", use = "amount", type = NULL),
                list(name = "EVID", use = "eventidentifier", type = NULL),
                list(name = "MDV", use = "missingdependentvariable", type = NULL),
                list(name = "AGE", use = "covariate", type = "continuous"),
                list(name = "AGE_centered", use = "covariate", type = "continuous"),
                list(name = "SEX", use = "covariate", type = "categorical"),
                list(name = "WEIGHT", use = "covariate", type = "continuous"),
                list(name = "BMI", use = "covariate", type = "continuous"))

theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
                 "Cl", "logNormal", 0.2, NA, NA, TRUE,
                 "V", "logNormal", 20, NA, NA, TRUE,
                 "ka", "logNormal", 0.2, NA, NA, TRUE
)
# Examples of random effect model specification
# Single observation (legacy format):
ruv <- list(YNAME = "y1", DVID = 1, TRANS = "normal", PRED = "Cc",
            ERR = "combined1", INIT = c(1, 1), EST = c(TRUE, TRUE), BLQM = NULL)

# Example of random effects (RE) specification.
#
# init matrix: provides the initial values for the random effects.
# est matrix: controls how each random effect is handled:
#   TRUE  - the random effect will be estimated,
#   FALSE - the random effect will be fixed at its initial value,
#   NA    - no random effect will be applied.
#
# To fit a model without random effects, set all entries in the
# est matrix to NA.
#
# The same logic applies to the between-occasion variability
# matrix (occ).

re <- list(init = tribble(~Cl, ~V, ~ka,
                          1, 0, 0,
                          0, 1, 0,
                          0, 0, 1) %>% as.matrix(),
           est = tribble(~Cl, ~V, ~ka,
                         TRUE, NA, NA,
                         NA, TRUE, NA,
                         NA, NA, TRUE) %>% as.matrix())

occ <- list(init = tribble(~Cl, ~V, ~ka,
                           0, 0, 0,
                           0, 0, 0,
                           0, 0, 0) %>% as.matrix(),
            est = tribble(~Cl, ~V, ~ka,
                          NA, NA, NA,
                          NA, NA, NA,
                          NA, NA, NA) %>% as.matrix())
covs <- list(list(PAR = "V", COVNAME = "AGE", FUNC = "linear",
                  TRANS = "median", INIT = 1, EST = TRUE),
             list(PAR = "ka", COVNAME = "SEX", REF = 0, INIT = 1, EST = TRUE))

gco <-list(headers = headers,
           data = data,
           model = model,
           task_opt = "",
           covs = covs,
           project_name = "test-proj",
           theta = theta,
           ruv = ruv,
           re = re,
           occ = occ,
           modelText = "")

test_that("sg_sim_vps works ", {
  res <- sg_vpc_sim(obj1, model = mod_fin, output = "Cc")
  expect_equal(res %>% pull(ID) %>% unique() %>% length(), 100)
  expect_equal(res %>% pull(TIME) %>% unique() %>% length(),
               obj1$SDTAB$TIME %>% unique() %>% length())
})
test_that("sg_sim_vps works ", {
  res <- sg_vpc_sim(obj1, gco = gco, output = "Cc")
  expect_equal(res %>% pull(ID) %>% unique() %>% length(), 100)
  expect_equal(res %>% pull(TIME) %>% unique() %>% length(),
               obj1$SDTAB$TIME %>% unique() %>% length())
})

# Helper function to create mock sg_fit object
create_mock_sg_fit <- function() {
  list(
    SDTAB = data.frame(
      ID = rep(1:2, each = 5),
      TIME = c(0, 1, 2, 4, 8, 0, 1, 2, 4, 8),
      DV = rnorm(10, mean = 10, sd = 2),
      MDV = c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    ),
    EVTAB = data.frame(
      ID = c(1, 2),
      time = c(0, 0),
      amt = c(100, 100),
      cmt = c(1, 1),
      evid = c(1, 1)
    ),
    SUMTAB = data.frame(
      PAR = c("ka_pop", "Cl_pop", "V_pop"),
      VALUE = c(1.5, 2.0, 50.0),
      TYPE = "Typical values"
    ),
    OMEGAMAT = matrix(c(0.1, 0, 0, 0, 0.15, 0, 0, 0, 0.2), nrow = 3, ncol = 3),
    SIGMAMAT = matrix(c(0.05), nrow = 1, ncol = 1),
    COTAB = NULL,
    CATAB = NULL
  )
}

# Error handling tests
test_that("sg_vpc_sim errors on invalid fpath_i type", {
  skip_if_not_installed("rxode2")

  expect_error(
    sg_vpc_sim(fpath_i = 123, model = mod_fin)
  )
})

test_that("sg_vpc_sim errors on non-existent file path", {
  skip_if_not_installed("rxode2")



  expect_error(
    sg_vpc_sim(fpath_i = "nonexistent_file.rda", model = mod_fin)
  )
})


# Basic functionality tests
test_that("sg_vpc_sim accepts list object as fpath_i", {
  skip_if_not_installed("rxode2")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("purrr")

  # Check if sg_sim function exists
  if (!exists("sg_sim")) {
    skip("sg_sim function not available")
  }


  mock_obj <- create_mock_sg_fit()

  # This test may fail if sg_sim has additional requirements
  # but it validates the input handling
  result <- tryCatch({
    sg_vpc_sim(fpath_i = mock_obj, model = mod_fin, npop = 10)
  }, error = function(e) {
    # If sg_sim fails due to model complexity, that's okay for this test
    # We're mainly testing that the function accepts the list input
    NULL
  })

  # If successful, check output structure
  if (!is.null(result)) {
    expect_true(is.data.frame(result))
    expect_true("ID" %in% colnames(result))
  }
})

test_that("sg_vpc_sim handles custom time_col parameter", {
  skip_if_not_installed("rxode2")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("purrr")

  if (!exists("sg_sim")) {
    skip("sg_sim function not available")
  }


  mock_obj <- create_mock_sg_fit()
  # Rename TIME column
  mock_obj$SDTAB$TAD <- mock_obj$SDTAB$TIME
  mock_obj$SDTAB$TIME <- NULL

  result <- tryCatch({
    sg_vpc_sim(fpath_i = mock_obj, model = mod_fin, time_col = "TAD", npop = 10)
  }, error = function(e) NULL)

  if (!is.null(result)) {
    expect_true(is.data.frame(result))
  }
})

test_that("sg_vpc_sim handles output parameter", {
  skip_if_not_installed("rxode2")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("purrr")

  if (!exists("sg_sim")) {
    skip("sg_sim function not available")
  }

  mock_obj <- create_mock_sg_fit()

  result <- tryCatch({
    sg_vpc_sim(fpath_i = mock_obj, model = mod_fin, output = "Cc", npop = 10)
  }, error = function(e) NULL)

  if (!is.null(result)) {
    expect_true(is.data.frame(result))
  }
})

test_that("sg_vpc_sim handles npop parameter", {
  skip_if_not_installed("rxode2")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("purrr")

  if (!exists("sg_sim")) {
    skip("sg_sim function not available")
  }

  mock_obj <- create_mock_sg_fit()

  result <- tryCatch({
    sg_vpc_sim(fpath_i = mock_obj, model = mod_fin, npop = 5)
  }, error = function(e) NULL)

  if (!is.null(result)) {
    expect_true(is.data.frame(result))
    # With npop=5 and 2 individuals, we expect at least some rows
    expect_true(nrow(result) > 0)
  }
})

test_that("sg_vpc_sim handles optional COTAB and CATAB", {
  skip_if_not_installed("rxode2")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("purrr")

  if (!exists("sg_sim")) {
    skip("sg_sim function not available")
  }


  mock_obj <- create_mock_sg_fit()
  # Add optional covariates
  mock_obj$COTAB <- data.frame(
    ID = c(1, 2),
    WT = c(70, 80),
    AGE = c(30, 40)
  )
  mock_obj$CATAB <- data.frame(
    ID = c(1, 2),
    SEX = c("M", "F")
  )

  result <- tryCatch({
    sg_vpc_sim(fpath_i = mock_obj, model = mod_fin, npop = 10)
  }, error = function(e) NULL)

  if (!is.null(result)) {
    expect_true(is.data.frame(result))
  }
})

test_that("sg_vpc_sim filters MDV correctly", {
  skip_if_not_installed("rxode2")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("purrr")

  if (!exists("sg_sim")) {
    skip("sg_sim function not available")
  }


  mock_obj <- create_mock_sg_fit()
  # Add some MDV=1 rows (should be filtered out)
  mock_obj$SDTAB <- rbind(
    mock_obj$SDTAB,
    data.frame(
      ID = c(1, 2),
      TIME = c(0, 0),
      DV = c(NA, NA),
      MDV = c(1, 1)
    )
  )

  result <- tryCatch({
    sg_vpc_sim(fpath_i = mock_obj, model = mod_fin, npop = 10)
  }, error = function(e) NULL)

  if (!is.null(result)) {
    expect_true(is.data.frame(result))
    # MDV column should be removed from output
    expect_false("MDV" %in% colnames(result))
  }
})

test_that("sg_vpc_sim returns data frame with ID column", {
  skip_if_not_installed("rxode2")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("purrr")

  if (!exists("sg_sim")) {
    skip("sg_sim function not available")
  }


  mock_obj <- create_mock_sg_fit()

  result <- tryCatch({
    sg_vpc_sim(fpath_i = mock_obj, model = mod_fin, npop = 10)
  }, error = function(e) NULL)

  if (!is.null(result)) {
    expect_true(is.data.frame(result))
    expect_true("ID" %in% colnames(result))
    # ID values should match original IDs
    expect_true(all(unique(result$ID) %in% unique(mock_obj$SDTAB$ID)))
  }
})

#' Visualize Local Sensitivity Analysis Results
#'
#' @description
#' This function creates plots to visualize the results of a local sensitivity analysis:
#' (1) a family of curves plot showing how model outputs vary over time for different parameter perturbations,
#' and (2) a tornado plot summarizing the relative influence of each parameter on selected summary metrics
#' (e.g., output value at a chosen time or Cmax).
#'
#' @param sens_data A data frame containing sensitivity analysis results from sg_localsens_sim()
#' @param tornado_time Numeric value specifying the time point (in the same units as `TIME`)
#' at which to evaluate the model output for the tornado plot. Defaults to the maximum time in `sens_data` if not specified.
#' @param metrics Character vector specifying which metrics to include in the tornado plot.
#'   Supported options are `"value"` (value at `tornado_time`) and `"cmax"` (maximum output). Defaults to `"value"`.
#' @param ref_data Optional data frame providing reference (baseline) output values,
#'   formatted similarly to `sens_data`. If `NULL`, the function uses the mid-range parameter value as the baseline.
#' @param log_scale Logical. If `TRUE`, the y-axis of the family-of-curves plot is displayed on a log scale.
#' @param color_low Character. Color used for the lower parameter bound (default: "#1f77b4").
#' @param color_high Character. Color used for the upper parameter bound (default: "#ff7f0e").
#' @param facet_scales Character string passed to `facet_grid()`, defining how y-axis scales behave across facets (default = `"free"`).
#'
#' @return A named list containing:
#'   \describe{
#'     \item{`family_of_curves`}{A `ggplot2` object showing parameter sensitivity curves over time.}
#'     \item{`tornado`}{A `ggplot2` object showing the tornado plot of relative sensitivity.}
#'   }
#'
#' @examples
#' \donttest{
#' library(rxode2)
#' library(tibble)
#' mod_ex <- RxODE({
#'   # Doses in mg
#'   # Time in hours
#'   ### Parameter values
#'   # Typical
#'   POPCL = 5;
#'   POPVC = 180;
#'   POPQ = 7;
#'   POPVP = 52;
#'   POPKTR = 6;
#'   FBIOPAR = 1;

#'   # Covariate coefficients
#'   POPIGFRCOV = 0.8;
#'   POPPATCOVCLGR1 = 0.85;
#'   POPPATCOVCLGR2 = 0.85;
#'   POPPATCOVCLGR3 = 0.85;
#'   POPPATCOVCLGR4 = 0.85;

#'   # Random effects
#'   PPVCL = 0;
#'   PPVVC = 0;
#'   PPVKTR = 0;

#'   # Residual error
#'   RUV = 0;

#'   ### Covariates
#'   PATCOVCL = 1
#'   if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
#'   if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
#'   if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
#'   if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
#'   IGFRCOV = (IGFR/112)^POPIGFRCOV

#'   ## Parameters
#'   CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
#'   VC = POPVC * exp(PPVVC);
#'   Q = POPQ;
#'   VP = POPVP;
#'   KTR = POPKTR * exp(PPVKTR);

#'   ### Explicit functions
#'   Cc = Ac/VC;                 # nmol/L
#'   Cp = Ap/VP;                 # nmol/L

#'   ### Initial conditions
#'   At1(0) = 0;         # mg
#'   At2(0) = 0;         # mg
#'   At3(0) = 0;         # mg
#'   At4(0) = 0;         # mg
#'   At5(0) = 0;         # mg
#'   At6(0) = 0;         # mg
#'   Ad(0) = 0;          # mg
#'   Ac(0) = 0;          # mg
#'   Ap(0) = 0;          # mg

#'   ### ODEs
#'   d/dt(At1) = - KTR*At1;
#'   d/dt(At2) = KTR*At1 - KTR*At2;
#'   d/dt(At3) = KTR*At2 - KTR*At3;
#'   d/dt(At4) = KTR*At3 - KTR*At4;
#'   d/dt(At5) = KTR*At4 - KTR*At5;
#'   d/dt(At6) = KTR*At5 - KTR*At6;
#'   d/dt(Ad) = KTR*At6 - KTR*Ad;
#'   d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
#'   d/dt(Ap) = Q*(Cc - Cp);

#'   FBIO = FBIOPAR
#'   f(At1) = FBIO*1000000/505;
#'   CHECKRUV = RUV;
#'   Cc_ResErr = Cc + RUV*Cc;
#' })
#' et_base <- tribble(
#'   ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
#'   1,   0,     1,     1,    10,   2,     24,  112,   1
#' )
#' sens_loc <- sg_localsens_sim(
#'   model = mod_ex,
#'   params = c("POPCL", "POPVC"),
#'   stimes = seq(0, 168, 0.1),
#'   output = "Cc",
#'   perc = 0.9,
#'   cov = c("IGFR", "POPN"),
#'   et = et_base
#' )
#' plots <- sg_localsens_vis(
#'   sens_data = sens_loc,
#'   tornado_time = 168,
#'   metrics = c("value", "cmax"),
#'   log_scale = TRUE,
#'   color_low = "#4575b4",
#'   color_high = "#d73027",
#'   facet_scales = "free"
#' )
#'
#' # Display plots
#' print(plots$family_of_curves)
#' print(plots$tornado)
#' }
#' @import ggplot2
#' @import dplyr
#' @import tidyr
#' @importFrom grid unit
#' @export

sg_localsens_vis <- function(sens_data,
                             tornado_time = NULL,
                             metrics = "value",
                             ref_data = NULL,
                             log_scale = FALSE,
                             color_low = "#1f77b4",
                             color_high = "#ff7f0e",
                             facet_scales = "free") {

  output_var <- unique(sens_data$VAR)

  if(is.null(tornado_time)) {
    tornado_time <- max(sens_data$TIME)
  }

  foc_plot <- ggplot(data = sens_data) +
    geom_line(aes(x = TIME, y = VALUE, group = interaction(PARVAL, ID),
                  color = PARVAL_NORM), alpha = 0.7, linewidth = 0.6) +
    facet_grid(PARNAME ~ ., scales = facet_scales) +
    theme_bw() +
    theme(
      strip.background = element_blank(),
      axis.text.x = element_text(size = 11),
      axis.text.y = element_text(size = 11),
      axis.title.x = element_text(size = 13),
      axis.title.y = element_text(size = 13),
      legend.text = element_text(size = 10),
      strip.text = element_text(size = 11),
      plot.title = element_text(size = 14, hjust = 0.5),
      legend.key.size = unit(0.3, "cm")
    ) +
    labs(
      x = "Time",
      y = output_var,
      color = "Parameter\nvalue"
    ) +
    scale_color_gradient(
      low = color_low,
      high = color_high,
      breaks = c(0, 1),
      labels = c("Lower bound", "Upper bound"),
      guide = guide_colorbar(
        title.theme = element_text(size = 10),
        label.theme = element_text(size = 9),
        barwidth = unit(0.5, "cm"),
        barheight = unit(2, "cm")
      )
    )

  if(log_scale) {
    foc_plot <- foc_plot + scale_y_log10()
  }


  # Tornado plot
  metrics_data <- sens_data %>%
    group_by(PARNAME, PARVAL, PARVAL_NORM) %>%
    summarise(
      value_at_time = VALUE[which.min(abs(TIME - tornado_time))],
      cmax = max(VALUE, na.rm = TRUE),
      .groups = 'drop'
    )

  # Get reference values (either from ref_data or middle parameter value)
  if(!is.null(ref_data)) {
    ref_metrics <- ref_data %>%
      summarise(
        value_ref = VALUE[which.min(abs(TIME - tornado_time))],
        cmax_ref = max(VALUE, na.rm = TRUE)
      )
  } else {
    ref_metrics <- metrics_data %>%
      group_by(PARNAME) %>%
      filter(abs(PARVAL_NORM - 0.5) == min(abs(PARVAL_NORM - 0.5))) %>%
      summarise(
        value_ref = first(value_at_time),
        cmax_ref = first(cmax),
        .groups = 'drop'
      )
  }

  tornado_data <- metrics_data %>%
    group_by(PARNAME) %>%
    filter(PARVAL_NORM %in% c(0, 1)) %>%
    mutate(
      PAR_group = if_else(PARVAL_NORM == 0, "L", "U")
    ) %>%
    select(PARNAME, PARVAL, PAR_group, value_at_time, cmax) %>%
    pivot_longer(
      cols = c(value_at_time, cmax),
      names_to = "METRIC",
      values_to = "VALUE"
    ) %>%
    # Join with reference values
    left_join(ref_metrics, by = "PARNAME") %>%
    mutate(
      value_cfb = if_else(METRIC == "value_at_time",
                          (VALUE - value_ref) * 100 / value_ref,
                          (VALUE - cmax_ref) * 100 / cmax_ref),
      METRIC = case_when(
        METRIC == "value_at_time" ~ paste0("Value at t=", tornado_time),
        METRIC == "cmax" ~ "Cmax"
      )
    )

  metric_labels <- c(
    "value" = paste0("Value at t=", tornado_time),
    "cmax" = "Cmax"
  )

  selected_metrics <- metric_labels[metrics]
  tornado_data <- tornado_data %>%
    filter(METRIC %in% selected_metrics)

  tornado_plot <- ggplot(tornado_data,
                         aes(x = reorder(PARNAME, abs(value_cfb)),
                             y = value_cfb,
                             fill = PAR_group)) +
    geom_bar(stat = "identity", position = "identity", alpha = 0.8) +
    facet_grid(METRIC ~ ., scales = facet_scales) +
    coord_flip() +
    labs(
      y = "Change from baseline, %",
      x = "Parameter"
    ) +
    scale_fill_manual(
      values = c("U" = color_high, "L" = color_low),
      labels = c("U" = "Upper bound", "L" = "Lower bound"),
      name = "Parameter\nvalue"
    ) +
    geom_hline(yintercept = 0, linetype = "longdash", alpha = 0.7) +
    theme_minimal() +
    theme(
      axis.title.y = element_blank(),
      strip.text = element_text(size = 11),
      axis.text.x = element_text(size = 10),
      axis.text.y = element_text(size = 10),
      legend.position = "bottom"
    )

   return(list(
     family_of_curves = foc_plot,
     tornado = tornado_plot
   ))
}

## Author: Mikhailova Anna
## First created: 2025-09-11
## Description: functions for simulations from the model
## Keywords: SimuRg, simulations

#' Perform simulations from an rxode2 model
#'
#' Runs simulations using an rxode2 model object with explicit R objects for
#' parameters (theta), variance-covariance (thetamat), omega, and sigma.
#'
#' @inheritParams sg_dummy
#' @param byID logical. If \code{TRUE}, replicate populations/subjects per event-table ID.
#'   Default \code{NULL} is treated as \code{TRUE}.
#' @param byPOP logical. When both thetamat and omega are used: if \code{TRUE},
#'   replicate subjects by population. Default \code{NULL} is treated as \code{TRUE}.
#' @param shared logical. If \code{TRUE},
#'   one shared population draw for all IDs. Default \code{NULL} is treated as \code{TRUE}.
#' @param ... optional arguments passed to \code{rxode2::rxSolve} (e.g. \code{method}, \code{maxsteps}).
#' @returns A dataset with simulation results (long format).
#' @details
#'  Index columns \code{ID}, \code{POPN}, and \code{sim.id} are included depending on what is used:
#'  \describe{
#'     \item{No uncertainty, no variability}{Only \code{ID} (and \code{TIME}, \code{VAR}, \code{VALUE})}
#'     \item{Only uncertainty (\code{thetamat})}{ \code{POPN} and \code{ID}}
#'     \item{Only variability (\code{omega})}{\code{sim.id} and \code{ID}}
#'     \item{Uncertainty and variability}{\code{POPN}, \code{sim.id}, and \code{ID}}
#'  }
#'  Other columns:
#'  \describe{
#'     \item{TIME}{Timepoint of the simulations}
#'     \item{VAR}{Names of the output variables}
#'     \item{VALUE}{Optional. Simulated value. Returned when no aggregation applied}
#'     \item{mean, median, ...}{Optional. Aggregated statistic of simulated values. Returned when aggregation is applied}
#'     \item{COV1, ...COVn}{Optional. Covariates value. Returned when \code{addcov} is \code{TRUE}}
#'     \item{KEEP1, ...KEEPn}{Optional. Columns that were specified in the \code{keep} argument}
#'  }
#'  Event table can use either \code{id} or \code{ID}. Parameters can be fixed per ID by
#'  including parameter names (e.g. \code{*_pop}, \code{omega_*}) as columns in the event table.
#' @examples
#' \donttest{
#' library(rxode2)
#' library(dplyr)
#' library(tibble)
#' # Base 1-compartment PK model (no covariate)
#' mod <- rxode2::rxode2({
#'   ka_pop = 0.1;
#'   Vd_pop = 10;
#'   CL_pop = 0.5;
#'
#'   omega_ka = 0;
#'   omega_Vd = 0;
#'   omega_CL = 0;
#'
#'   Cc_b = 0;
#'   ka_tv = exp(ka_pop);
#'   Vd_tv = exp(Vd_pop);
#'   CL_tv = exp(CL_pop);
#'
#'   ka = ka_tv * exp(omega_ka);
#'   Vd = Vd_tv * exp(omega_Vd);
#'   CL = CL_tv * exp(omega_CL);
#'
#'   Cc = Ac / Vd;
#'
#'   Ad(0) = 0;
#'   Ac(0) = 0;
#'
#'   d/dt(Ad) = -ka * Ad;
#'   d/dt(Ac) = ka * Ad - CL * Cc;
#'
#'   Cc_ResErr = Cc * (1 + Cc_b);
#' })
#'
#' # Model with covariate: WTBL on Vd (Vd_tv = exp(Vd_pop) * (WTBL/70)^beta_WTBL_Vd_pop)
#'
#' mod_cov <- rxode2::rxode2({
#'   ka_pop = 0.1;
#'   Vd_pop = 10;
#'   CL_pop = 0.5;
#'
#'   beta_WTBL_Vd_pop = 0;
#'
#'   omega_ka = 0;
#'   omega_Vd = 0;
#'   omega_CL = 0;
#'
#'   Cc_b = 0;
#'
#'   ka_tv = exp(ka_pop);
#'   Vd_tv = exp(Vd_pop) * (WTBL/70)^beta_WTBL_Vd_pop;
#'   CL_tv = exp(CL_pop);
#'
#'   ka = ka_tv * exp(omega_ka);
#'   Vd = Vd_tv * exp(omega_Vd);
#'   CL = CL_tv * exp(omega_CL);
#'
#'   Cc = Ac / Vd;
#'
#'   Ad(0) = 0;
#'   Ac(0) = 0;
#'
#'   d/dt(Ad) = -ka * Ad;
#'   d/dt(Ac) = ka * Ad - CL * Cc;
#'
#'   Cc_ResErr = Cc * (1 + Cc_b);
#' })
#'
#' et_test <- tibble::tribble(
#'   ~ID, ~TIME, ~EVID, ~CMT, ~AMT,
#'   1, 0, 1, 1, 10,
#'   2, 0, 1, 1, 50
#' )
#'
#' stimes_test <- seq(0, 24, 0.1)
#' output_test <- c("Ac", "Ad", "Cc", "Cc_ResErr")
#' theta_test <- c(ka_pop = log(0.1), Vd_pop = log(10), CL_pop = log(0.5))
#'
#' thetamat_test <- matrix(c(0.05, 0.01, 0, 0.01, 0.05, 0, 0, 0, 0.05), nrow = 3, byrow = TRUE)
#' rownames(thetamat_test) <- colnames(thetamat_test) <- c("ka_pop", "Vd_pop", "CL_pop")
#'
#' omega_test <- matrix(c(0.2, 0.05, 0, 0.05, 0.2, 0, 0, 0, 0.2), nrow = 3, byrow = TRUE)
#' rownames(omega_test) <- colnames(omega_test) <- c("omega_ka", "omega_Vd", "omega_CL")
#'
#' sigma_test <- matrix(0.1); rownames(sigma_test) <- colnames(sigma_test) <- "Cc_b"
#'
#' # No variability
#' sim1 <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                outputs = output_test)
#'
#' # Population uncertainty, single scenario (ID)
#' sim2a <- sg_sim(model = mod, et = dplyr::filter(et_test, ID == 1),
#'                 stimes = stimes_test, theta = theta_test, thetamat = thetamat_test,
#'                 npop = 10)
#'
#' # Population uncertainty, multiple IDs, byID = TRUE (populations are replicated
#' # for each scenario (ID)),
#' # shared = TRUE (the same populations are used for each scenario (ID))
#' sim2b <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                thetamat = thetamat_test, npop = 10, byID = TRUE, shared = TRUE)
#'
#' # Population uncertainty, multiple IDs, byID = FALSE (one solve over full event table;
#' # npop must equal n(ID); ID - virtual subject)
#' sim2c <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                thetamat = thetamat_test, npop = nrow(et_test), byID = FALSE)
#'
#'
#' # Between-subject variability (BSV), multiple IDs, byID = TRUE (subjects are
#' # replicated for each scenario (ID)), shared = FALSE (separate subjects per ID)
#' sim3a <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                 omega = omega_test, nsub = 10, byID = TRUE, shared = FALSE)
#'
#'
#' # Between-subject variability (BSV), byID = FALSE (one solve over full event table;
#' # nsub must equal n(ID); ID - virtual subject)
#' sim3b <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                 omega = omega_test, nsub = nrow(et_test), byID = FALSE)
#'
#'
#' # BSV + uncertainty: byID = TRUE (populations/subjects are replicated for each
#' # scenario (ID)), byPOP = TRUE (subjects are replicated for each population),
#' # shared = FALSE (separate populations/subjects per ID)
#' sim4a <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                 thetamat = thetamat_test, npop = 10, omega = omega_test, nsub = 5,
#'                 byID = TRUE, byPOP = TRUE, shared = FALSE)
#'
#' # BSV + uncertainty: byID = TRUE (populations/subjects are replicated for each
#' # scenario (ID)), byPOP = FALSE (subjects are not replicated between population;
#' # npop = nsub)
#' sim4b <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                 thetamat = thetamat_test, npop = nrow(et_test), omega = omega_test,
#'                 nsub = nrow(et_test), byID = TRUE, byPOP = FALSE)
#'
#' # BSV + uncertainty: byID = FALSE (one solve over full event table; nsub and/or
#' # npop must equal n(ID); ID - virtual subject), byPOP = TRUE (subjects are
#' # replicated for each population), shared = FALSE (separate populations/subjects
#' # per ID)
#' sim4c <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                 thetamat = thetamat_test, npop = 10, omega = omega_test, nsub = 5,
#'                 byID = FALSE, byPOP = TRUE, shared = FALSE)
#'
#' # BSV + uncertainty: byID = FALSE (one solve over full event table; nsub and/or
#' # npop must equal n(ID); ID - virtual subject), byPOP = FALSE (subjects are not
#' # replicated between population; npop = nsub)
#' sim4d <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
#'                 thetamat = thetamat_test, npop = nrow(et_test), omega = omega_test,
#'                 nsub = nrow(et_test), byID = FALSE, byPOP = FALSE)
#'
#' # Parameter override in event table
#' sim4 <- sg_sim(model = mod, et = dplyr::filter(et_test, ID == 1) %>%
#'                                  dplyr::mutate(Vd_pop = 2),
#'                stimes = stimes_test, theta = theta_test, thetamat = thetamat_test,
#'                npop = 1)
#'
#'
#' # Covariate (WTBL): pass covs and keep so WTBL is used and returned
#' theta_cov <- c(theta_test, beta_WTBL_Vd_pop = 0.5)
#' sim5 <- sg_sim(model = mod_cov, et = dplyr::filter(et_test, ID == 1) %>%
#'                                      dplyr::mutate(WTBL = 70),
#'                stimes = stimes_test, theta = theta_cov, covs = c("WTBL"),
#'                keep = c("WTBL"),
#'                thetamat = thetamat_test, npop = 5)
#' }
#' @import rxode2
#' @importFrom purrr map_dfr
#' @import dplyr
#' @import tidyr
#' @export
sg_sim <- function(model, et, stimes = NULL, outputs = NULL, theta = NULL,
                   omega = NULL, sigma = NULL, thetamat = NULL, covs = NULL,
                   npop = 1, nsub = 1, aggr = NULL, addcov = TRUE, keep = NULL,
                   scale = NULL, covint = "locf", inits = NULL,
                   byID = NULL, byPOP = NULL, shared = NULL,
                   ncores = 1, atol = 1e-8, rtol = 1e-6, ...){

  # ── id column: et may use "id" or "ID" ─────────────────────────────────
  et_id_col <- intersect(c("id", "ID"), colnames(et))[1]

  # ── auto-detect per-ID parameters from model + et columns ────────────
  et_standard <- c("id", "ID", "time", "TIME", "evid", "EVID", "cmt", "CMT",
                   "amt", "AMT", "addl", "ADDL", "ii", "II", "dur", "DUR",
                   "rate", "RATE", "ss", "SS", "mdv", "MDV", "tinf", "TINF",
                   "adm", "ADM")
  mod_pars    <- rxode2::rxModelVars(model)$params
  et_par_cols <- setdiff(
    intersect(mod_pars, colnames(et)),
    c(et_standard, covs, keep)
  )
  has_et_pars <- length(et_par_cols) > 0
  et_pars     <- NULL
  if (has_et_pars) {
    et_pars <- et %>%
      dplyr::select(dplyr::any_of(c("ID", "id")), dplyr::all_of(et_par_cols)) %>%
      dplyr::distinct()
    if ("ID" %in% colnames(et_pars) && !"id" %in% colnames(et_pars)) {
      et_pars <- et_pars %>% dplyr::rename(id = ID)
    }
  }

  # ── drop et-fixed parameters from variability matrices ─────────────
  if (has_et_pars) {
    if (!is.null(thetamat)) {
      drop <- intersect(et_par_cols, colnames(thetamat))
      if (length(drop) > 0) {
        idx <- !colnames(thetamat) %in% drop
        thetamat <- thetamat[idx, idx, drop = FALSE]
        if (ncol(thetamat) == 0) thetamat <- NULL
      }
    }
    if (!is.null(omega)) {
      drop <- intersect(et_par_cols, colnames(omega))
      if (length(drop) > 0) {
        idx <- !colnames(omega) %in% drop
        omega <- omega[idx, idx, drop = FALSE]
        if (ncol(omega) == 0) omega <- NULL
      }
    }
    if (!is.null(sigma)) {
      drop <- intersect(et_par_cols, colnames(sigma))
      if (length(drop) > 0) {
        idx <- !colnames(sigma) %in% drop
        sigma <- sigma[idx, idx, drop = FALSE]
        if (ncol(sigma) == 0) sigma <- NULL
      }
    }
  }

  # ── event table compilation ──────────────────────────────────────────
  et_i_m <- et %>% rxode2::et()
  if (!is.null(stimes)) et_i_m <- et_i_m %>% rxode2::add.sampling(stimes)

  et_i_cov <- NULL
  if (!is.null(covs) && !is.na(et_id_col)) {
    if (!is.null(stimes)) {
      et_i_cov <- et %>%
        dplyr::select(dplyr::all_of(et_id_col), dplyr::all_of(covs)) %>%
        dplyr::distinct()
      if (et_id_col == "ID") et_i_cov <- et_i_cov %>% dplyr::rename(id = ID)
      et_i_m   <- et_i_m %>%
        dplyr::as_tibble() %>%
        dplyr::left_join(et_i_cov, by = "id")
    } else {
      et_i_cov <- et %>%
        dplyr::select(dplyr::all_of(et_id_col), dplyr::all_of(covs))
      if (et_id_col == "ID") et_i_cov <- et_i_cov %>% dplyr::rename(id = ID)
      et_i_m   <- et_i_m %>%
        dplyr::bind_cols(et_i_cov %>% dplyr::select(-dplyr::any_of("id")))
    }
  }

  # ── internal solve helpers ───────────────────────────────────────────

  get_theta <- function(k = NULL) {
    if (!has_et_pars || is.null(k)) return(theta)
    theta_k <- et_pars %>%
      dplyr::filter(id == k) %>%
      dplyr::select(-dplyr::any_of("id")) %>%
      as.list() %>%
      unlist()
    if (!is.null(theta)) {
      theta_k <- c(theta_k, theta[!names(theta) %in% names(theta_k)])
    }
    theta_k
  }

  run_solve <- function(events, nStud, nSub, params = theta, ...) {
    rxode2::rxSolve(model, events = events, params = params, omega = omega,
                    inits = inits, sigma = sigma, thetaMat = thetamat,
                    nStud = nStud, nSub = nSub,
                    covsInterpolation = covint, cores = ncores,
                    simVariability = TRUE, atol = atol, rtol = rtol, ...)
  }

  solve_by_id <- function(nStud, nSub, post_fn = NULL, verbose = FALSE, ...) {
    et_tbl <- et_i_m %>% dplyr::as_tibble()
    id_col_et <- intersect(c("id", "ID"), colnames(et_tbl))[1]
    if (is.na(id_col_et)) {
      id_col_et <- "id"
      if (!id_col_et %in% colnames(et_tbl)) et_tbl <- dplyr::mutate(et_tbl, id = 1L)
    }
    ids <- unique(et_tbl[[id_col_et]])
    purrr::map_dfr(ids, function(k) {
      if (verbose) message("id = ", k, " (out of ", max(et_tbl[[id_col_et]]), ")")
      et_k <- et_tbl %>%
        dplyr::filter(.data[[id_col_et]] == k) %>%
        dplyr::mutate(!!id_col_et := 1L)
      theta_k <- get_theta(k)
      sim_k <- run_solve(et_k, nStud, nSub, params = theta_k, ...)
      if ("ID" %in% colnames(sim_k) && !"id" %in% colnames(sim_k)) {
        sim_k <- dplyr::rename(sim_k, id = ID)
      }
      sim_k <- dplyr::mutate(sim_k, id = k)
      if (!is.null(post_fn)) sim_k <- post_fn(sim_k, k)
      sim_k
    })
  }

  do_solve <- function(nStud, nSub, ...) {
    if (has_et_pars) solve_by_id(nStud, nSub, ...)
    else run_solve(et_i_m, nStud, nSub, ...)
  }

  # ── resolve strategy ─────────────────────────────────────────────────
  has_thetamat <- !is.null(thetamat)
  has_omega    <- !is.null(omega)
  by_id        <- is.null(byID)  || isTRUE(byID)
  by_pop       <- is.null(byPOP) || isTRUE(byPOP)
  is_shared    <- is.null(shared) || isTRUE(shared)
  n_ids        <- if (!is.na(et_id_col)) length(unique(et[[et_id_col]])) else 1L

  add_popn_ceiling <- FALSE
  sim_i <- NULL

  if (!has_thetamat && !has_omega) {
    sim_i <- do_solve(npop, nsub, ...)

  } else if (has_thetamat && !has_omega) {
    add_popn_ceiling <- TRUE
    if (by_id && is_shared) {
      sim_i <- do_solve(npop, nsub, ...)
    } else if (by_id && !is_shared) {
      sim_i <- solve_by_id(npop, nsub, ...)
    } else {
      sim_i <- run_solve(et_i_m, npop, nsub, ...)
    }

  } else if (!has_thetamat && has_omega) {
    if (by_id) {
      sim_i <- solve_by_id(npop, nsub, ...)
    } else {
      if (n_ids != nsub) {
        message("!Critical condition: nsub = n(ID); otherwise byID = TRUE")
        return(NULL)
      }
      sim_i <- do_solve(npop, nsub, ...)
    }

  } else {
    if (by_id) {
      if (by_pop) {
        add_popn_ceiling <- TRUE
        sim_i <- solve_by_id(npop, nsub, ...)
      } else {
        if (npop != nsub) {
          message("!Critical condition: nsub = npop; otherwise byPOP = TRUE")
          return(NULL)
        }
        sim_i <- do_solve(npop, nsub, ...)
      }
    } else {
      if (npop != n_ids && nsub != n_ids) {
        message("!Critical condition: nsub = n(ID) OR npop = n(ID); otherwise byID = TRUE")
        return(NULL)
      }
      if (by_pop) {
        if (is_shared) {
          sim_i <- solve_by_id(npop, nsub, ...)
        } else if (npop != n_ids) {
          sim_i <- solve_by_id(npop, 1, post_fn = function(sim_k, k) {
            sim_k %>% dplyr::mutate(sim.id = k) %>%
              dplyr::group_by(time) %>%
              dplyr::mutate(POPN = seq_len(npop)) %>%
              dplyr::ungroup()
          }, ...)
        } else {
          sim_i <- solve_by_id(1, nsub, post_fn = function(sim_k, k) {
            sim_k %>% dplyr::mutate(POPN = k) %>%
              dplyr::group_by(time) %>%
              dplyr::mutate(sim.id = seq_len(nsub)) %>%
              dplyr::ungroup()
          }, ...)
        }
      } else {
        if (npop != nsub) {
          message("!Critical condition: nsub = npop; otherwise byPOP = TRUE")
          return(NULL)
        }
        sim_i <- solve_by_id(1, 1, post_fn = function(sim_k, k) {
          sim_k %>% dplyr::mutate(POPN = k, sim.id = k)
        }, ...)
      }
    }
  }

  if (add_popn_ceiling && "sim.id" %in% colnames(sim_i)) {
    sim_i <- sim_i %>% dplyr::mutate(POPN = ceiling(sim.id / nsub))
  }

  # ── post-processing: ID / POPN / sim.id by scenario ───────────────────
  # No uncertainty & no variability → only ID
  # Only uncertainty (thetamat)     → POPN and ID
  # Only variability (omega)        → sim.id and ID
  # Uncertainty + variability      → POPN, sim.id, ID
  sim_i_ind <- sim_i %>% dplyr::as_tibble()
  if ("ID" %in% colnames(sim_i_ind) && !"id" %in% colnames(sim_i_ind)) {
    sim_i_ind <- dplyr::rename(sim_i_ind, id = ID)
  }
  if (!"id" %in% colnames(sim_i_ind)) sim_i_ind <- dplyr::mutate(sim_i_ind, id = 1)
  if (has_thetamat && !"POPN" %in% colnames(sim_i_ind)) {
    sim_i_ind <- dplyr::mutate(sim_i_ind, POPN = 1)
  }
  if (has_omega && !"sim.id" %in% colnames(sim_i_ind)) {
    sim_i_ind <- dplyr::mutate(sim_i_ind, sim.id = 1)
  }
  if (!has_thetamat && "POPN" %in% colnames(sim_i_ind)) {
    sim_i_ind <- dplyr::select(sim_i_ind, -POPN)
  }
  if (!has_omega && "sim.id" %in% colnames(sim_i_ind)) {
    sim_i_ind <- dplyr::select(sim_i_ind, -sim.id)
  }
  id_cols <- c("id", "time", if (has_thetamat) "POPN", if (has_omega) "sim.id")
  sim_i_ind <- sim_i_ind %>%
    dplyr::mutate(dplyr::across(-dplyr::any_of(id_cols), as.numeric)) %>%
    tidyr::gather("VAR", "VALUE", -dplyr::any_of(id_cols))
  if (!is.null(outputs)) sim_i_ind <- sim_i_ind %>% dplyr::filter(VAR %in% outputs)

  sim_i_out <- sim_i_ind

  if (is.null(aggr)) {
    #if (interactive()) message("No aggregation applied;")
  } else if (aggr == "ID") {
    sim_i_out <- sim_i_ind %>%
      dplyr::group_by(id, time, VAR) %>%
      dplyr::summarise_at(dplyr::vars(VALUE), funSum_sim) %>%
      dplyr::ungroup()
  } else if (aggr == "NPOP") {
    if ("POPN" %in% colnames(sim_i_ind)) {
      sim_i_out <- sim_i_ind %>%
        dplyr::group_by(POPN, time, VAR) %>%
        dplyr::summarise_at(dplyr::vars(VALUE), funSum_sim) %>%
        dplyr::ungroup()
    } else {
      sim_i_out <- sim_i_ind %>%
        dplyr::group_by(time, VAR) %>%
        dplyr::summarise_at(dplyr::vars(VALUE), funSum_sim) %>%
        dplyr::ungroup()
    }
  } else if (aggr == "TIME") {
    sim_i_out <- sim_i_ind %>%
      dplyr::group_by(time, VAR) %>%
      dplyr::summarise_at(dplyr::vars(VALUE), funSum_sim) %>%
      dplyr::ungroup()
  }

  if (!is.null(keep) && !is.null(sim_i_out) && "id" %in% colnames(sim_i_out) && !is.na(et_id_col)) {
    keep_et <- et %>%
      dplyr::select(dplyr::all_of(et_id_col), dplyr::all_of(keep)) %>%
      dplyr::distinct()
    if (et_id_col == "ID") keep_et <- keep_et %>% dplyr::rename(id = ID)
    suppressMessages(
      sim_i_out <- dplyr::left_join(sim_i_out, keep_et, by = "id", relationship = "many-to-many")
    )
  }

  if (addcov && !is.null(covs) && !is.null(sim_i_out) && "id" %in% colnames(sim_i_out) && !is.null(et_i_cov)) {
    suppressMessages(
      sim_i_out <- dplyr::left_join(
        sim_i_out,
        dplyr::distinct(dplyr::select(et_i_cov, id, dplyr::all_of(covs))),
        relationship = "many-to-many"
      )
    )
  }

  lookup <- c(TIME = "time", ID = "id")
  req_cols <- c(
    if (has_thetamat) "POPN",
    if (has_omega) "sim.id",
    "ID", "TIME", "VAR", "VALUE",
    "mean", "median", "min", "max", "sd",
    "P025", "P05", "P10", "P15", "P25", "P75", "P85", "P90", "P95", "P975",
    "geom_mean", "CV", covs, keep
  )
  sim_i_out <- sim_i_out %>%
    dplyr::rename(dplyr::any_of(lookup)) %>%
    dplyr::select(dplyr::any_of(req_cols))
  sim_i_out
}

## Scenarios aligned with roxygen @examples in R/sg-sim.R
## All inputs (model, theta, thetamat, omega, sigma) are explicit R objects — no file uploads.
##
## Test structure:
##   (1) Base 1-compartment PK, no covariate — sim1, sim2a–c, sim3a–b, sim4a–d, parameter override.
##   (2) 1-compartment PK with WTBL covariate on Vd — covs/keep and covariate + variability (sim5).

test_that("sg_sim runs with 1-compartment model and explicit parameters", {
  # ---- Model: 1-compartment PK, no covariate ----
  mod <- rxode2::rxode2({
    ka_pop = 0.1;
    Vd_pop = 10;
    CL_pop = 0.5;
    omega_ka = 0;
    omega_Vd = 0;
    omega_CL = 0;
    Cc_b = 0;
    ka_tv = ka_pop;
    Vd_tv = Vd_pop;
    CL_tv = CL_pop;
    ka <- ka_tv * exp(omega_ka);
    Vd <- Vd_tv * exp(omega_Vd);
    CL <- CL_tv * exp(omega_CL);
    Cc = Ac / Vd;
    Ad(0) = 0;
    Ac(0) = 0;
    d/dt(Ad) = -ka * Ad;
    d/dt(Ac) = ka * Ad - CL * Cc;
    Cc_ResErr = Cc * (1 + Cc_b);
  })

  et_ex <- tibble::tribble(
    ~ID, ~TIME, ~EVID, ~CMT, ~AMT,
    1, 0, 1, 1, 10,
    2, 0, 1, 1, 50
  )
  stimes_ex <- seq(0, 24, 0.1)
  output_ex <- c("Ac", "Ad", "Cc", "Cc_ResErr")
  theta_ex <- c(ka_pop = log(0.1), Vd_pop = log(10), CL_pop = log(0.5))
  thetamat_ex <- matrix(c(0.05, 0.01, 0, 0.01, 0.05, 0, 0, 0, 0.05), nrow = 3, byrow = TRUE)
  rownames(thetamat_ex) <- colnames(thetamat_ex) <- c("ka_pop", "Vd_pop", "CL_pop")
  omega_ex <- matrix(c(0.2, 0.05, 0, 0.05, 0.2, 0, 0, 0, 0.2), nrow = 3, byrow = TRUE)
  rownames(omega_ex) <- colnames(omega_ex) <- c("omega_ka", "omega_Vd", "omega_CL")
  sigma_ex <- matrix(0.1)
  rownames(sigma_ex) <- colnames(sigma_ex) <- "Cc_b"

  # ---- sim1: No variability (theta only); single solve, no thetamat/omega ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex, outputs = output_ex)
  )

  # ---- sim2a: Population uncertainty, single scenario (ID) ----
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1), stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 1)
  )
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1), stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 10)
  )

  # ---- sim2b: Population uncertainty, multiple IDs, byID=TRUE (populations replicated per ID), shared=TRUE (same populations for each ID) ----
  expect_warning(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = nrow(et_ex), byID = TRUE, shared = TRUE)
  )
  expect_warning(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 10, byID = TRUE, shared = TRUE)
  )

  # ---- Population uncertainty, multiple IDs, byID=TRUE shared=FALSE (separate pop per ID) ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = nrow(et_ex), byID = TRUE, shared = FALSE)
  )

  # ---- sim2c: Population uncertainty, byID=FALSE (one solve over full event table; npop must equal n(ID); ID = virtual subject) ----
  expect_warning(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = nrow(et_ex), byID = FALSE)
  )
  expect_warning(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 10, byID = FALSE)
  )

  # ---- Between-subject variability (BSV), single ID ----
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1), stimes = stimes_ex, theta = theta_ex,
           omega = omega_ex, nsub = 1)
  )
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1), stimes = stimes_ex, theta = theta_ex,
           omega = omega_ex, nsub = 10)
  )

  # ---- BSV, multiple IDs, byID=TRUE shared=TRUE ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           omega = omega_ex, nsub = 10, byID = TRUE, shared = TRUE)
  )

  # ---- sim3a: BSV, multiple IDs, byID=TRUE (subjects replicated per ID), shared=FALSE (separate subjects per ID) ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           omega = omega_ex, nsub = 10, byID = TRUE, shared = FALSE)
  )
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           omega = omega_ex, nsub = 5, byID = TRUE, shared = FALSE)
  )

  # ---- sim3b: BSV, byID=FALSE (one solve over full event table; nsub must equal n(ID); ID = virtual subject) ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           omega = omega_ex, nsub = nrow(et_ex), byID = FALSE)
  )

  # ---- Both thetamat and omega, byID=FALSE (nsub = n(ID)) ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, omega = omega_ex, nsub = nrow(et_ex), byID = FALSE)
  )

  # ---- sim4a: BSV + uncertainty, byID=TRUE (pop/subjects replicated per ID), byPOP=TRUE (subjects replicated per population), shared=FALSE ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 10, omega = omega_ex, nsub = 5,
           byID = TRUE, byPOP = TRUE, shared = FALSE)
  )

  # ---- sim4b: BSV + uncertainty, byID=TRUE, byPOP=FALSE (subjects not replicated between populations; npop = nsub) ----
  # expect_no_error(
  #   sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
  #          thetamat = thetamat_ex, npop = 5, omega = omega_ex, nsub = 5,
  #          byID = TRUE, byPOP = FALSE)
  # )

  # ---- sim4c: BSV + uncertainty, byID=FALSE (one solve; nsub/npop vs n(ID)), byPOP=TRUE, shared=FALSE ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 10, omega = omega_ex, nsub = 5,
           byID = FALSE, byPOP = TRUE, shared = FALSE)
  )
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 10, omega = omega_ex, nsub = 2,
           byID = FALSE, byPOP = TRUE, shared = FALSE)
  )
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 2, omega = omega_ex, nsub = 10,
           byID = FALSE, byPOP = TRUE, shared = FALSE)
  )

  # ---- sim4d: BSV + uncertainty, byID=FALSE (one solve; nsub and npop equal n(ID)), byPOP=FALSE (npop = nsub) ----
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = nrow(et_ex), omega = omega_ex, nsub = nrow(et_ex),
           byID = FALSE, byPOP = FALSE)
  )
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 10, omega = omega_ex, nsub = 5,
           byID = FALSE, byPOP = FALSE)
  )
  expect_no_error(
    sg_sim(model = mod, et = et_ex, stimes = stimes_ex, theta = theta_ex,
           thetamat = thetamat_ex, npop = 2, omega = omega_ex, nsub = 2,
           byID = FALSE, byPOP = FALSE)
  )

  # ---- Parameter override in event table (Vd_pop, omega_Vd) ----
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(Vd_pop = 2),
           stimes = stimes_ex, theta = theta_ex, thetamat = thetamat_ex, npop = 1)
  )
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(Vd_pop = 2),
           stimes = stimes_ex, theta = theta_ex, thetamat = thetamat_ex, npop = 10)
  )
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(Vd_pop = 2),
           stimes = stimes_ex, theta = theta_ex, omega = omega_ex, nsub = 1)
  )
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(Vd_pop = 2),
           stimes = stimes_ex, theta = theta_ex, omega = omega_ex, nsub = 10)
  )
  expect_no_error(
    sg_sim(model = mod, et = dplyr::filter(et_ex, ID == 1) %>%
             dplyr::mutate(Vd_pop = 2, omega_Vd = 0.5),
           stimes = stimes_ex, theta = theta_ex, omega = omega_ex, nsub = 10)
  )
})

test_that("sg_sim runs with covariate model and explicit parameters (sim5)", {
  # ---- Model: 1-compartment PK with WTBL covariate on Vd (Vd_tv = exp(Vd_pop) * (WTBL/70)^beta_WTBL_Vd_pop) ----
  mod_cov <- rxode2::rxode2({
    ka_pop = 0.1;
    Vd_pop = 10;
    CL_pop = 0.5;
    beta_WTBL_Vd_pop = 0;
    omega_ka = 0;
    omega_Vd = 0;
    omega_CL = 0;
    Cc_b = 0;
    ka_tv = exp(ka_pop);
    Vd_tv = exp(Vd_pop) * (WTBL / 70)^beta_WTBL_Vd_pop;
    CL_tv = exp(CL_pop);
    ka <- ka_tv * exp(omega_ka);
    Vd <- Vd_tv * exp(omega_Vd);
    CL <- CL_tv * exp(omega_CL);
    Cc = Ac / Vd;
    Ad(0) = 0;
    Ac(0) = 0;
    d/dt(Ad) = -ka * Ad;
    d/dt(Ac) = ka * Ad - CL * Cc;
    Cc_ResErr = Cc * (1 + Cc_b);
  })

  et_ex <- tibble::tribble(
    ~ID, ~TIME, ~EVID, ~CMT, ~AMT,
    1, 0, 1, 1, 10,
    2, 0, 1, 1, 50
  )
  stimes_ex <- seq(0, 24, 0.1)
  theta_ex <- c(ka_pop = log(0.1), Vd_pop = log(10), CL_pop = log(0.5))
  thetamat_ex <- matrix(c(0.05, 0.01, 0, 0.01, 0.05, 0, 0, 0, 0.05), nrow = 3, byrow = TRUE)
  rownames(thetamat_ex) <- colnames(thetamat_ex) <- c("ka_pop", "Vd_pop", "CL_pop")
  omega_ex <- matrix(c(0.2, 0.05, 0, 0.05, 0.2, 0, 0, 0, 0.2), nrow = 3, byrow = TRUE)
  rownames(omega_ex) <- colnames(omega_ex) <- c("omega_ka", "omega_Vd", "omega_CL")
  theta_cov <- c(theta_ex, beta_WTBL_Vd_pop = 0.5)

  # ---- sim5: Covariate (WTBL); pass covs and keep so WTBL is used and returned ----
  expect_no_error(
    sg_sim(model = mod_cov, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(WTBL = 70),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           thetamat = thetamat_ex, npop = 1)
  )
  expect_no_error(
    sg_sim(model = mod_cov, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(WTBL = 70),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           thetamat = thetamat_ex, npop = 5)
  )
  expect_no_error(
    sg_sim(model = mod_cov, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(WTBL = 70),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           thetamat = thetamat_ex, npop = 10)
  )

  # ---- Covariate, multiple IDs with different WTBL, population uncertainty ----
  expect_warning(
    sg_sim(model = mod_cov, et = dplyr::mutate(et_ex, WTBL = c(70, 80)),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           thetamat = thetamat_ex, npop = 10, byID = TRUE, shared = TRUE)
  )

  # ---- Covariate, single ID, BSV only ----
  expect_no_error(
    sg_sim(model = mod_cov, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(WTBL = 70),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           omega = omega_ex, nsub = 1)
  )
  expect_no_error(
    sg_sim(model = mod_cov, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(WTBL = 70),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           omega = omega_ex, nsub = 10)
  )

  # ---- Covariate + uncertainty + BSV, byID=TRUE byPOP=TRUE shared=FALSE ----
  expect_no_error(
    sg_sim(model = mod_cov, et = dplyr::filter(et_ex, ID == 1) %>% dplyr::mutate(WTBL = 70),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           thetamat = thetamat_ex, npop = 5, omega = omega_ex, nsub = 3,
           byID = TRUE, byPOP = TRUE, shared = FALSE)
  )

  # ---- Covariate, multiple IDs, byID=TRUE shared=FALSE with omega ----
  expect_no_error(
    sg_sim(model = mod_cov, et = dplyr::mutate(et_ex, WTBL = c(70, 80)),
           stimes = stimes_ex, theta = theta_cov, covs = c("WTBL"), keep = c("WTBL"),
           omega = omega_ex, nsub = 5, byID = TRUE, shared = FALSE)
  )
})

#' Plot random effects or individual parameters vs covariates
#'
#' Generates ggplot2 visualizations for either random effects (RE) or individual parameters (IndPar)
#' versus continuous or categorical covariates from a Simurg object.
#'
#' @inheritParams sg_dummy
#' @param ptype Character. Type of plot: `"REvsCov"` for random effects or `"IndParvsCov"` for individual parameters.
#' @param cat_cov Optional tibble with categorical covariates. Must have columns `COV` and optionally `COVNAME` for labels.
#' @param cont_cov Optional tibble with continuous covariates. Must have columns `COV` and optionally `COVNAME` for labels.
#'
#' @return A list of ggplot objects. Returns versus continious covariates
#' `vs_contcov` and versus categorical covariates `vs_catcov` plots.
#'
#' @examples
#' library(tibble)
#' cont_cov <- tibble(
#'   COV = c("AGE", "WEIGHT"),
#'   COVNAME = c("Age, years", "Body weight, kg")
#' )
#' cat_cov <- tibble(
#'   COV = c("SEX", "VKORC1_gentyp"),
#'   COVNAME = c("Sex, M/F", "VKORC1 genotype")
#' )
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
#' p <- sg_gof_par_cov(
#'   fpath_i = fpath_i,
#'   ptype = "IndParvsCov",
#'   cont_cov = cont_cov,
#'   cat_cov = cat_cov
#' )
#' p$vs_contcov
#' p$vs_catcov
#'
#' @import dplyr
#' @import tidyr
#' @import ggplot2
#' @importFrom scales pretty_breaks
#' @importFrom grid unit
#' @export
sg_gof_par_cov <- function(fpath_i,
                           ptype = "REvsCov",
                           cat_cov = NULL,
                           cont_cov = NULL) {

  if (!is_null(read_smrg_obj(fpath_i)$GFO))
    smrg_obj <- read_smrg_obj(fpath_i)$GFO
  else
    smrg_obj <- read_smrg_obj(fpath_i)

  if (is.null(smrg_obj$PATAB) | is.null(smrg_obj$COTAB) | is.null(smrg_obj$CATAB) |
      is.null(smrg_obj$SUMTAB)) {
    stop("sg_fit object must contain PATAB, COTAB, CATAB and SUMTAB components")
  }
  patab <- smrg_obj$PATAB
  cols_to_convert <- c("ID", cont_cov$COV)
  cotab <- smrg_obj$COTAB %>% mutate(across(any_of(cols_to_convert), as.numeric))
  catab <- smrg_obj$CATAB
  sumtab <- smrg_obj$SUMTAB

  if (is.data.frame(patab) && nrow(patab) == 0) {
    stop("PATAB is empty (no rows)")
  } else if (is.data.frame(cotab) && nrow(cotab) == 0) {
    stop("COTAB is empty (no rows)")
  } else if (is.data.frame(catab) && nrow(catab) == 0) {
    stop("CATAB is empty (no rows)")
  } else if (is.data.frame(sumtab) && nrow(sumtab) == 0) {
    stop("SUMTAB is empty (no rows)")
  }

  if (is.list(patab) && length(patab) == 0) {
    stop("PATAB is empty (no elements)")
  } else if (is.list(cotab) && length(cotab) == 0) {
    stop("COTAB is empty (no elements)")
  } else if (is.list(catab) && length(catab) == 0) {
    stop("CATAB is empty (no elements)")
  } else if (is.list(sumtab) && length(sumtab) == 0) {
    stop("SUMTAB is empty (no elements)")
  }

  if (is.data.frame(patab)) { patab <- patab
  } else if (is.list(patab)) { patab <- as.data.frame(do.call(rbind, patab))
  } else stop("PATAB must be a data frame or a list of data frames")

  if (is.data.frame(cotab)) { cotab <- cotab
  } else if (is.list(cotab)) {cotab <- as.data.frame(do.call(rbind, cotab))
  } else stop("COTAB must be a data frame or a list of data frames")

  if (is.data.frame(catab)) { catab <- catab
  } else if (is.list(catab)) {catab <- as.data.frame(do.call(rbind, catab))
  } else stop("CATAB must be a data frame or a list of data frames")

  if (is.data.frame(sumtab)) { sumtab <- sumtab
  } else if (is.list(sumtab)) {sumtab <- as.data.frame(do.call(rbind, sumtab))
  } else stop("SUMTAB must be a data frame or a list of data frames")



  # IndPar columns
  typical_pars <- sumtab$PAR[sumtab$TYPE == "Typical values"]
  indpar_cols <- intersect(names(patab), gsub("_pop$", "", typical_pars, ignore.case = TRUE))
  if (is_empty(indpar_cols)) {
    indpar_cols <- intersect(names(patab), str_c("psi_", gsub("_pop$", "", typical_pars, ignore.case = TRUE)))

  }
  # RE columns
  re_pars <- sumtab$PAR[grepl("Random effects", sumtab$TYPE, ignore.case = TRUE)]
  eta_cols <- intersect(names(patab), paste0("eta_", gsub("omega_|gamma_", "", re_pars)))

  # Covariate columns
  contcov_cols <- if (!is.null(cont_cov)) intersect(cont_cov$COV, setdiff(names(cotab), "ID")) else setdiff(names(cotab), "ID")
  catcov_cols <- if (!is.null(cat_cov)) intersect(cat_cov$COV, setdiff(names(catab), "ID")) else setdiff(names(catab), "ID")

  # Merge covariates
  cov_df <- catab %>%
    left_join(cotab, by = "ID")

  # Merge with parameters
  df <- patab %>%
    left_join(cov_df, by = "ID") %>%
    mutate(COHORTC = "All subjects")

  # Handle covariate labels
  if (!is.null(suppressWarnings(cont_cov$COVNAME))) {
    contcov_labels <- cont_cov
  } else {
    contcov_labels <- tibble(COV = contcov_cols, COVNAME = contcov_cols)
  }

  if (!is.null(suppressWarnings(cat_cov$COVNAME))) {
    catcov_labels <- cat_cov
  } else {
    catcov_labels <- tibble(COV = catcov_cols, COVNAME = catcov_cols)
  }

  # Helper functions: correlation + labeling
  add_corr_labels <- function(data) {
    data %>%
      group_by(PNAME, COV) %>%
      summarise(
        r = suppressWarnings(cor(COVVAL, VALUE, use = "complete.obs")),
        p = suppressWarnings(cor.test(COVVAL, VALUE)$p.value),
        .groups = "drop"
      ) %>%
      mutate(
        PEARS = sprintf("r = %.2f\np = %.3f", r, p),
        SIGNPEARS = if_else(p < 0.05, "firebrick", "black")
      )
  }

  add_anova_labels <- function(data) {
    data %>%
      group_by(PNAME, COV) %>%
      mutate(
        ANOVA = suppressWarnings(summary(aov(VALUE ~ as.factor(COVVAL))) [[1]][["Pr(>F)"]][1])
      ) %>%
      ungroup() %>%
      mutate(
        PVAL = ifelse(ANOVA < 0.001, "p < 0.001", paste0("p = ", round(ANOVA, 2))),
        LARGPVAL = ifelse(ANOVA < 0.05, "2", "1") # "2" significant, "1" not
      )
  }

  # select RE or IndPar columns
  if (ptype == "REvsCov") {
    # Random effects vs Continuous Covariates
    if (length(contcov_cols) > 0) {
    re_long <- df %>%
      select(ID, COHORTC, all_of(eta_cols), all_of(contcov_cols)) %>%
      pivot_longer(cols = all_of(eta_cols), names_to = "PNAME", values_to = "VALUE") %>%
      pivot_longer(cols = all_of(contcov_cols), names_to = "COV", values_to = "COVVAL")

    corr_stats <- add_corr_labels(re_long) %>%
      left_join(contcov_labels, by = "COV")

    plot_data <- re_long %>%
      left_join(corr_stats %>% select(PNAME, COV, COVNAME, PEARS, SIGNPEARS), by = c("PNAME", "COV"))

    re_vs_cont_p <- unique(select(corr_stats, PNAME, COV, COVNAME, PEARS, SIGNPEARS))
    # Plot REvsContCov
    p_cont <- ggplot(data = plot_data, aes(x = COVVAL, y = VALUE)) +
      geom_label(
        data = re_vs_cont_p,
        aes(x = -Inf, y = Inf, label = PEARS), colour = re_vs_cont_p$SIGNPEARS,
        hjust = -0.1, vjust = 1.1, show.legend = FALSE, size = 2.5
      ) +
      geom_point(aes(color = COHORTC), size = 1.5, alpha = 0.7, show.legend = FALSE) +
      geom_hline(yintercept = 0, col = "grey25", lty = "dotted", linewidth = 0.5) +
      geom_smooth(method = "lm", formula = y ~ x, color = MSDcol[3], se = FALSE) +
      facet_grid(PNAME ~ COVNAME, scales = "free", switch = "y") +
      scale_x_continuous(breaks = pretty_breaks(7), name = "Covariate value") +
      scale_y_continuous(breaks = pretty_breaks(7), name = "Random effect", position = "right") +
      scale_colour_manual(values = MSDcol) +
      theme_bw(base_size = 11) +
      theme(
        legend.position = "top",
        strip.text.y = element_text(angle = 0),
        panel.spacing = unit(0.4, "lines"),
        panel.grid.minor = element_blank()
      ) +
      guides(color = guide_legend(title = "Cohort")) +
      labs(title = "Random Effects vs Continious Covariates")
  } else p_cont <- NULL
    # Random effects vs Categorical Covariates
    if (length(catcov_cols) > 0) {
      re_cat <- df %>%
        select(ID, COHORTC, all_of(eta_cols), all_of(catcov_cols)) %>%
        mutate(across(all_of(catcov_cols), as.character))  %>%
        pivot_longer(cols = all_of(eta_cols), names_to = "PNAME", values_to = "VALUE") %>%
        pivot_longer(cols = all_of(catcov_cols), names_to = "COV", values_to = "COVVAL")

      re_cat <- add_anova_labels(re_cat) %>%
        left_join(catcov_labels, by = "COV")

      # Plot REvsCatCov
      p_cat <- ggplot(re_cat, aes(x = as.factor(COVVAL), y = VALUE)) +
        geom_boxplot(fill = MSDcol[1], alpha = 0.5, outlier.colour = MSDcol[3], outlier.shape = 3) +
        geom_hline(yintercept = 0, col = "grey25", lty = "dotted", linewidth = 0.5) +
        geom_label(data = unique(select(re_cat, PNAME, COV, COVNAME, PVAL, LARGPVAL)),
                   aes(x = -Inf, y = Inf, label = PVAL, col = LARGPVAL),
                   hjust = -0.1, vjust = 1.1, show.legend = FALSE, size = 2.5) +
        facet_grid(PNAME ~ COVNAME, scales = "free", switch = "y") +
        scale_y_continuous(breaks = pretty_breaks(7), name = "Random effect", position = "right") +
        scale_color_manual(values = c("1" = "black", "2" = "firebrick")) +
        labs(x = "Category", title = "Random Effects vs Categorical Covariates") +
        theme_bw(base_size = 11) +
        theme(axis.text.x = element_text(angle = 25, hjust = 1, vjust = 1),
              panel.grid.minor = element_blank())
    } else p_cat <- NULL

    return(list(vs_contcov = p_cont, vs_catcov = p_cat))

  } else if (ptype == "IndParvsCov") {
    indpar_cols_ <- indpar_cols
    indpar_cols <- intersect(indpar_cols, gsub("^eta_", "", eta_cols))
    if (is_empty(indpar_cols)){
      indpar_cols <- str_c("psi_", intersect(gsub("^psi_", "", indpar_cols_),
                                             gsub("^eta_", "", eta_cols)))
    }

    # Individual parameter vs Continuous Covariates
    if ((length(contcov_cols) > 0) && (length(indpar_cols) > 0)) {
    ind_long <- df %>%
      select(ID, COHORTC, all_of(indpar_cols), all_of(contcov_cols)) %>%
      pivot_longer(cols = all_of(indpar_cols), names_to = "PNAME", values_to = "VALUE") %>%
      pivot_longer(cols = all_of(contcov_cols), names_to = "COV", values_to = "COVVAL")

    corr_stats <- add_corr_labels(ind_long) %>%
      left_join(contcov_labels, by = "COV")

    plot_data <- ind_long %>%
      left_join(corr_stats %>% select(PNAME, COV, COVNAME, PEARS, SIGNPEARS), by = c("PNAME", "COV"))

    ip_vs_cont_p <- unique(select(corr_stats, PNAME, COV, COVNAME, PEARS, SIGNPEARS))
    # Plot IndParvsContCov
    p_cont <- ggplot(plot_data, aes(x = COVVAL, y = VALUE)) +
      geom_label(
        data = ip_vs_cont_p,
        aes(x = -Inf, y = Inf, label = PEARS), colour = ip_vs_cont_p$SIGNPEARS,
        hjust = -0.1, vjust = 1.1, show.legend = FALSE, size = 2.5
      ) +
      geom_point(aes(color = COHORTC), size = 1.5, alpha = 0.7, show.legend = FALSE) +
      geom_smooth(method = "lm", formula = y ~ x, color = MSDcol[3], se = FALSE) +
      facet_grid(PNAME ~ COVNAME, scales = "free", switch = "y") +
      scale_x_continuous(breaks = pretty_breaks(7), name = "Covariate value") +
      scale_y_continuous(breaks = pretty_breaks(7), name = "Individual parameter", position = "right") +
      scale_colour_manual(values = MSDcol) +
      theme_bw(base_size = 11) +
      theme(
        legend.position = "top",
        strip.text.y = element_text(angle = 0),
        panel.spacing = unit(0.4, "lines")
      ) +
      guides(color = guide_legend(title = "Cohort")) +
      labs(title = "Individual Parameters vs Continious Covariates")
  } else p_cont <- NULL
  # IndPar vs Categorical Covariates
  if ((length(catcov_cols) > 0) && (length(indpar_cols) > 0)) {
    ind_cat <- df %>%
      select(ID, COHORTC, all_of(indpar_cols), all_of(catcov_cols)) %>%
      mutate(across(all_of(catcov_cols), as.character))  %>%
      pivot_longer(cols = all_of(indpar_cols), names_to = "PNAME", values_to = "VALUE") %>%
      pivot_longer(cols = all_of(catcov_cols), names_to = "COV", values_to = "COVVAL")

    ind_cat <- add_anova_labels(ind_cat) %>%
      left_join(catcov_labels, by = "COV")

    # Plot IndParvsCatCov
    p_cat <- ggplot(ind_cat, aes(x = as.factor(COVVAL), y = VALUE)) +
      geom_boxplot(fill = MSDcol[1], alpha = 0.5, outlier.colour = MSDcol[3], outlier.shape = 3) +
      geom_label(data = unique(select(ind_cat, PNAME, COV, COVNAME, PVAL, LARGPVAL)),
                 aes(x = -Inf, y = Inf, label = PVAL, col = LARGPVAL),
                 hjust = -0.1, vjust = 1.1, show.legend = FALSE, size = 2.5) +
      facet_grid(PNAME ~ COVNAME, scales = "free", switch = "y") +
      scale_y_continuous(breaks = pretty_breaks(7), name = "Individual parameter", position = "right") +
      scale_color_manual(values = c("1" = "black", "2" = "firebrick")) +
      labs(x = "Category", title = "Individual Parameters vs Categorical Covariates") +
      theme_bw(base_size = 11) +
      theme(axis.text.x = element_text(angle = 25, hjust = 1, vjust = 1))
  } else p_cat <- NULL

  return(list(vs_contcov = p_cont, vs_catcov = p_cat))

  }
}

## Author: Anatoly Pokladyuk
## First created: 2025-12-17
## Description: formal testing of sg-gof-par-cov function and its helpers
## Keywords: SimuRg, sg-gof-par-cov, goodness-of-fit
test_that("sg_gof_par_cov output is correct", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  cont_cov <- tibble(
    COV = c("AGE", "WEIGHT"),
    COVNAME = c("Age, years", "Body weight, kg")
  )
  cat_cov <- tibble(
    COV = c("SEX", "VKORC1_gentyp"),
    COVNAME = c("Sex, M/F", "VKORC1 genotype")
  )

  x <- sg_gof_par_cov(fpath_i = fpath_i, ptype = "IndParvsCov", cont_cov = cont_cov)
  expect_true(inherits(x$vs_contcov, "ggplot"))
  expect_snapshot(ggplot2::layer_data(x$vs_contcov))
})

test_that("sg_gof_par_cov does not work", {
  expect_error(sg_gof_par_cov(data.frame()))
})

test_that("sg_gof_par_cov file load", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  cat_cov <- tibble(
    COV = c("SEX", "VKORC1_gentyp"),
    COVNAME = c("Sex, M/F", "VKORC1 genotype")
  )
  x <- sg_gof_par_cov(fpath_i, ptype = "REvsCov", cat_cov = cat_cov)
  expect_true(inherits(x$vs_catcov, "ggplot"))
  expect_snapshot(ggplot2::layer_data(x$vs_catcov))
})

## Author: Melnikova Alina,  Mikhailova Anna
## First created: 2025-09-05
## Description: sg-modcomp and its helper functions
## Keywords: SimuRg, sg-modcomp, model building, comparison

#' Returns dataframe with summary statistics for model comparison
#'
#' @inheritParams sg_dummy
#' @returns A data.frame with model summary metrics for futher comparison
#' @examples
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
#' sum_tab <- sg_modcomp(fpath_i, 3)
#' print(sum_tab)
#' @import dplyr
#' @import stringr
#' @import stringr
#' @export
sg_modcomp <- function(fpath_i, run_id = 1){

  # Load the object from the specified path
  if (inherits(fpath_i, "character")) {
    obj <- get(load(fpath_i))
  } else if (inherits(fpath_i, "list")) {
    obj <- fpath_i
  } else {
    stop("fpath_i object should be either an sg_fit object, or a path to saved sg_fit object")
  }


  # Initialize empty dataframe for model comparisons
  modcomp <- data.frame()

  # Extract Project name
  project_name <- obj$PROJNAME

  # Extract parameters from parsum
  parameters <- obj$SUMTAB %>% filter(TYPE=="Typical values") %>% select(PAR) %>% pull()
  parameters <- parameters %>% str_remove("_pop")

  # Extract ETAs - get column names starting with "eta_" from PATAB
  eta_cols <- grep("^eta_", names(obj$PATAB), value = TRUE)
  eta_names <- gsub("^eta_", "", eta_cols)  # Remove "eta_" prefix

  # Extract Covariates - process SUMTAB$PAR for beta_ terms

  covar_vec <- c()
  cont_cov <- names(obj$CATAB)
  cat_cov <- names(obj$COTAB)

  if ("SUMTAB" %in% names(obj)) {
    beta_terms <- grep("^beta_", obj$SUMTAB$PAR, value = TRUE)

    cov_pairs <- gsub("^beta_", "", beta_terms)
    for (term in cov_pairs) {
      # Split beta_[covariate1]_[covariate2] into components
      cont_cov <- names(obj$CATAB)
      cat_cov <- names(obj$COTAB)

      for (par in parameters){
        if (str_detect(term,fixed(par))){
          term1 <- str_remove(term, fixed(par))
          par_i <- par
          break} else {term1 <- term}
      }
      for (cont in cont_cov){
        if (str_detect(term1,fixed(cont))){
          term2 <- str_remove(term1, fixed(cont))
          cov_i <- cont
          break} else {term2 <- term1}
      }

      for (cat in cat_cov){
        if (str_detect(term2,fixed(cat))){
          term3 <- str_remove(term2, fixed(cat))
          cov_i <- cat
          break}
        else {term3 <- term2}
      }

      covar_vec <- c(covar_vec, str_c( cov_i,par_i, sep = "~"))
    }
  }

  # Count identical elements and rename if n>1
  if (length(covar_vec) > 0) {
    # Count occurrences of each element
    covar_counts <- table(covar_vec)

    # Create new vector with renamed elements
    covar_vec_renamed <- covar_vec

    for (i in seq_along(covar_vec)) {
      element <- covar_vec[i]
      count <- covar_counts[element]

      if (count > 1) {
        # Extract cov_i and par_i from the element (format: "cov_i~par_i")
        parts <- str_split(element, "~", simplify = TRUE)
        if (length(parts) == 2) {
          cov_i <- parts[1]
          par_i <- parts[2]
          # Rename with count: "cov_i(n levels)~par_i"
          covar_vec_renamed[i] <- str_c(cov_i, "(", count, " levels)", "~", par_i)
        }
      }
    }

    # Replace original vector with renamed version
    covar_vec <- covar_vec_renamed
  }

  # Remove identical elements (duplicates) from covar_vec
  if (length(covar_vec) > 0) {
    covar_vec <- unique(covar_vec)
  }

  # Create dataframe with covariate information
  modcomp <- data.frame(
    Run_ID = run_id,
    `Project_name` = project_name,
    `ETAs` = paste(eta_names, collapse = ", "),
    `Covariates` = paste(unique(covar_vec), collapse = ", "),
    stringsAsFactors = FALSE
  )

  # Create dataframe with parameter Identifiability inforamtion
  ds_out_i <- sg_parsum(fpath_i, addOFV = TRUE)

  ds_idetn_i <- ds_out_i %>% filter(RSE > 50)
  ident_stat <- NULL
  if(nrow(ds_idetn_i) > 0){
    ident_stat <- str_c(str_remove(ds_idetn_i$Parameter, "[,].*?$"), collapse = "; ")
  }

  ds_shr_i <- ds_out_i %>% filter(`Shrinkage (var), %` > 50)
  shr_stat <- NULL
  if(nrow(ds_shr_i) > 0){
    shr_stat <- str_c(str_remove(ds_shr_i$Parameter, "[,].*?$"), collapse = "; ")
  }


  ds_out <- modcomp %>% mutate(Identifiability = ifelse(nrow(ds_idetn_i) == 0, "Yes", "No"),
                               Shrinkage = ifelse(nrow(ds_shr_i) == 0, "Yes", "No"),
                               `Pars. with RSE > 50%` = ident_stat, `Pars. with shr. > 50%` = shr_stat,
                               `Max RSE, %` = max(ds_out_i$RSE, na.rm = TRUE),
                               `Max shrinkage, %` = max(ds_out_i$`Shrinkage (var), %`, na.rm = TRUE),
                               OFV = unique(ds_out_i$OFV), AIC = unique(ds_out_i$AIC)) %>%
    rename(`Run ID` = Run_ID, `Project name` = Project_name)
  return(ds_out)
}

## Author: Mikhailova Anna
## First created: 2025-09-05
## Description: formal testing of sg-modcomp function and its helpers
## Keywords: SimuRg, sg-modcomp, model building, comparison

test_that("sg-modcomp works", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK_cov.RData", package = "SimuRg")
  sum_tab <- sg_modcomp(fpath_i, 3)
  expect_true(inherits(sum_tab, "data.frame"))
  expect_snapshot(sum_tab)
  sum_tab <- sg_modcomp(obj1, 4)
  expect_true(inherits(sum_tab, "data.frame"))
  expect_snapshot(sum_tab)
})
test_that("sg-modcomp does not work", {
  expect_error(sg_modcomp(data.frame()))
})

#' Perform simulations for prediction distribution plots
#'
#' @inheritParams sg_dummy
#' @returns A dataset with simulation results
#' @examples
#' \donttest{
#' library(rxode2)
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
#' mod_fin <- rxode2({
#'   # Differential equations
#'   d/dt(Ad) = -ka * Ad
#'   d/dt(Ac) = ka * Ad - Cl/V * Ac
#'   # Concentration calculations
#'   Cc = Ac / V
#' })
#' sg_predist_sim(fpath_i, mod_fin, outputs = "Cc")
#' }
#' @import rxode2
#' @importFrom purrr map_dfr
#' @import dplyr
#' @export
sg_predist_sim <- function(fpath_i, model, time_col = "TIME", outputs = NULL, npop = 500){
  if (inherits(fpath_i, "character")) {
    if (file.exists(fpath_i)) {
      obj <- get(load(fpath_i))
    } else {
      stop("File specified by fpath_i does not exist")
    }
  } else if (inherits(fpath_i, "list")) {
    obj <- fpath_i
  } else {
    stop("fpath_i object should be either an sg_fit object, or a path to saved sg_fit object")
  }

  data_fin.noex <- obj$SDTAB %>% filter(MDV != 1) %>% select(-MDV)
  data_fin.noex$time <- data_fin.noex[[time_col]]
  ev_tab <- obj$EVTAB

  if (!is.null(obj$COTAB)) ev_tab <- merge(ev_tab, obj$COTAB, by = "ID", all.x = TRUE)
  if (!is.null(obj$CATAB)) ev_tab <- merge(ev_tab, obj$CATAB, by = "ID", all.x = TRUE)

  covs_i <- c(colnames(obj$COTAB), colnames(obj$CATAB))
  covs_i <- covs_i[covs_i != "ID"]
  id_seq <- unique(data_fin.noex$ID)

  par_fin_tv <- obj$SUMTAB %>%
    filter(TYPE == "Typical values") %>%
    select(PAR, VALUE) %>%
    mutate(PAR = str_remove(PAR, "_pop")) %>%
    deframe()

  sim_predist_full <- id_seq %>% map_dfr(function(id_seq.i){
    data_fin.noex.i <- data_fin.noex %>%
      filter(ID == id_seq.i) %>%
      pull(time)

  ev_tab.i <- ev_tab %>%
      filter(ID == id_seq.i) %>%
      rename(DEFID = ID) %>%
      mutate(id = 1)

  sim.i <- sg_sim(model = model,
                    et = ev_tab.i,
                    stimes = data_fin.noex.i,
                    outputs = outputs,
                    theta = par_fin_tv,
                    omega = obj$OMEGAMAT,
                    sigma = NULL,
                    covs = covs_i,
                    npop = npop,
                    addcov = FALSE,
                    ncores = parallel::detectCores()-1) %>%
                    mutate(ID = id_seq.i)
    return(sim.i)
  })

  return(sim_predist_full)
}

test_that("sg_predist_sim works ", {
  mod_fin <- RxODE({
    # Doses in mg
    # Time in hours

    ### Parameter values
    # Typical
    Cl_pop = 5;
    V_pop = 180;

    ka_pop = 6;


    # Random effects
    omega_Cl = 0;
    omega_V = 0;
    omega_ka = 0;

    # Residual error
    b = 0;

    ### Parameters
    Cl = Cl_pop * exp(omega_Cl);
    V = V_pop * exp(omega_V);
    ka = ka_pop * exp(omega_ka);

    ### Explicit functions
    Cc = Ac/V;                 # nmol/L

    ### Initial conditions
    Ad(0) = 0;          # mg
    Ac(0) = 0;          # mg

    ### ODEs
    d/dt(Ad) = - ka*Ad;
    d/dt(Ac) = ka*Ad - Cl*Cc ;

    CHECKRUV = b;
    Cc_ResErr = Cc + b*Cc;
  })
  res <- sg_predist_sim(obj1, mod_fin, output = "Cc", npop=100)
  expect_equal(res %>% pull(ID) %>% unique() %>% length(), 100)
  expect_equal(res %>% pull(TIME) %>% unique() %>% length(),
               obj1$SDTAB$TIME %>% unique() %>% length())
})

## Author: Ugolkov Yaroslav
## First created: 2025-09-05
## Description: sg-gof-obpr and its helper functions
## Keywords: SimuRg, sg-gof-obpr, goodness-of-fit
#'
#'Observed vs predicted plot
#'
#'
#' @description
#' Function generates observed versus predicted (OBS vs PRED/IPRED) scatter plots,
#' a fundamental goodness-of-fit diagnostic tool in pharmacometric modeling.
#' This visualization assesses model adequacy by comparing observed clinical measurements against model-predicted values,
#' enabling identification of systematic bias, heteroscedasticity, and model misspecification patterns.
#' Function contains options for faceting, coloring by covariates, and trend lines.
#'
#' @inheritParams sg_dummy
#' @param lab_x X-axis label. Default "Model-predicted values"
#' @param lab_y Y-axis label. Default "Observed values"
#'
#' @return A ggplot2 object
#'

#' @examples
#' \donttest{
#' # Basic example with mock data
#' set.seed(123)  # For reproducibility
#' mock_obj <- list(
#'  SDTAB = data.frame(
#'    ID = rep(1:3, each = 5),
#'    TIME = rep(c(0, 1, 2, 4, 8), 3),
#'    DV = rnorm(15, mean = 10, sd = 2),
#'    PRED = rnorm(15, mean = 10, sd = 1.5),
#'    IPRED = rnorm(15, mean = 10, sd = 1.2),
#'    MDV = rep(0, 15)
#'  ),
#'  COTAB = data.frame(ID = 1:3, AGE = c(30, 45, 60)),
#'  CATAB = data.frame(ID = 1:3, RACE = c("Hispanic", "Hispanic", "Asian"))
#')
#'
#' # Basic plot
#' p <- sg_gof_obpr(mock_obj)
#'
#' # With covariates and faceting
#' p <- sg_gof_obpr(
#'   mock_obj,
#'   cov_cols = "RACE",
#'   col_i = "RACE",
#'   facet_i = "RACE"
#' )
#' }
#'
#' @import dplyr
#' @import ggplot2
#' @importFrom jsonlite fromJSON
#' @importFrom scales pretty_breaks trans_breaks trans_format math_format number_format
#' @export

sg_gof_obpr <- function(
    fpath_i, DVID = 1, cov_cols = NULL, indiv = TRUE, addline = TRUE, alpha_i = 0.5,
    smooth = TRUE, log_axes = FALSE, sc_factor = 1, abreaks = scales::pretty_breaks(7),
    lab_x = "Model-predicted values", lab_y = "Observed values", col_i = NULL, col_lab = NULL,
    facet_i = NULL, f_scales = "fixed",
    no_leg = FALSE, n_quantiles = 3, levels_discrete = 10
){


  is_discrete <- function(x, max_levels = levels_discrete) {
    n_unique <- length(unique(na.omit(x)))
    n_unique <= max_levels
  }
  continuous_to_categories <- function(x, n_quant = 3) {
    cut(
      x,
      breaks = quantile(x, probs = seq(0, 1, length.out = n_quant + 1), na.rm = TRUE),
      include.lowest = TRUE,
      labels = paste0("Q", 1:n_quant)
    )
  }

  X <- ifelse(indiv, "IPRED", "PRED")
  smrg_obj <- read_smrg_obj(fpath_i) #get(load(fpath_i))

  # Validate that smrg_obj has SDTAB
  if (is.null(smrg_obj$SDTAB)) {
    stop("sg_fit object must contain SDTAB component")
  }

  sdtab <- smrg_obj$SDTAB

  # Check if SDTAB is empty
  if (is.data.frame(sdtab) && nrow(sdtab) == 0) {
    stop("SDTAB is empty (no rows)")
  }
  if (is.list(sdtab) && length(sdtab) == 0) {
    stop("SDTAB is empty (no elements)")
  }



  # Handle SDTAB as either a data frame or a list of data frames
  if (is.data.frame(sdtab)) {
    ds_i <- sdtab
  } else if (is.list(sdtab)) {
    ds_i <- as.data.frame(do.call(rbind, sdtab))
  } else {
    stop("SDTAB must be a data frame or a list of data frames")
  }

  ds_i <- filter_sdtab_by_DVID(ds_i, DVID)

  # Check for required columns and convert to numeric
  required_cols <- c("IPRED", "PRED", "TIME", "DV")
  missing_cols <- setdiff(required_cols, colnames(ds_i))
  if (length(missing_cols) > 0) {
    stop("SDTAB is missing required columns: ", paste(missing_cols, collapse = ", "))
  }

  # Convert columns to numeric (only if they exist)
  cn_ds <- colnames(ds_i)
  ds_i <- ds_i %>%
    mutate(
      IPRED = if ("IPRED" %in% cn_ds) as.numeric(IPRED) else NA_real_,
      PRED = if ("PRED" %in% cn_ds) as.numeric(PRED) else NA_real_,
      TIME = as.numeric(TIME),
      DV = as.numeric(DV),
      MDV = if ("MDV" %in% cn_ds) as.numeric(MDV) else 0
    )

  # Add optional columns if they exist
  if ("WRES" %in% colnames(ds_i)) {
    ds_i <- ds_i %>% mutate(WRES = as.numeric(WRES))
  }
  if ("IWRES" %in% colnames(ds_i)) {
    ds_i <- ds_i %>% mutate(IWRES = as.numeric(IWRES))
  }

  ds_i <- ds_i %>% filter(.data$MDV != 1)

  # Apply scaling factor to IPRED, PRED, and DV if they exist
  scale_cols <- intersect(c("IPRED", "PRED", "DV"), colnames(ds_i))
  if (length(scale_cols) > 0) {
    ds_i <- ds_i %>%
      mutate(across(all_of(scale_cols), ~ .x / sc_factor))
  }

  # Rename the selected prediction column to "X"
  if (X %in% colnames(ds_i)) {
    ds_i <- ds_i %>% rename(X = all_of(X))
  } else {
    stop("Column '", X, "' not found in SDTAB. Available columns: ", paste(colnames(ds_i), collapse = ", "))
  }

  if(!is.null(cov_cols)){
    suppressMessages({
      ds_covs <- left_join(smrg_obj$COTAB,smrg_obj$CATAB)})
    ds_covs_i <- ds_covs %>% select(ID, one_of(cov_cols)) %>% unique()
    stopifnot(n_distinct(ds_covs_i$ID) == nrow(ds_covs_i))
    suppressMessages({
      ds_i <- ds_i %>% left_join(ds_covs_i, by = "ID")})
    for (col in cov_cols) {
      if (!is_discrete(ds_i[[col]])) {
        ds_i[[col]] <- continuous_to_categories(ds_i[[col]], n_quant = n_quantiles)
        #message(paste(col, "was converted into", n_quantiles, "quantile-based categories."))
      } else {
        ds_i[[col]] <- factor(ds_i[[col]])
      }
    }
  }

  lim_obpr <- c(min(ds_i$X, ds_i$DV), max(ds_i$X, ds_i$DV))

  p_char <- list(
    labs(y = lab_y, x = lab_x),
    geom_abline(linewidth = 0.5, col = "black", linetype = "dashed"),
    scale_color_manual(values = rep(MSDcol[c(2, 3, 4, 5, 7, 9, 6, 8, 1)], 30)),
    theme(legend.justification = c("left", "center"),
          legend.box.just = "left",
          legend.background = element_rect(fill = "white", linewidth = 0.15,
                                           linetype = "solid", colour = "black"),
          legend.key.size = unit(0.38, "cm"),
          legend.title = element_text(size = 8),
          legend.text = element_text(size = 8),
          plot.title = element_text(size = 12),
          panel.grid.minor = element_blank())
  )

  if(log_axes){
    p_char <- c(p_char,
                scale_x_log10(
                  breaks = scales::trans_breaks("log10", function(x) 10^x),
                  labels = scales::trans_format("log10", scales::math_format(10^.x))),
                scale_y_log10(
                  breaks = scales::trans_breaks("log10", function(x) 10^x),
                  labels = scales::trans_format("log10", scales::math_format(10^.x))))
  } else {
    p_char <- c(p_char,
                scale_x_continuous(breaks = abreaks,
                                   labels = scales::number_format()),
                scale_y_continuous(breaks = abreaks,
                                   labels = scales::number_format()))
  }

  p_ObPr <- ggplot(data = ds_i, aes(x = X, y = DV)) +
    theme_bw()
  if (is.null(facet_i) || f_scales == "fixed") {
    p_ObPr <- p_ObPr +
      coord_cartesian(xlim = lim_obpr, ylim = lim_obpr)
  }
  if(!is.null(col_i)){
    p_ObPr <- p_ObPr +
      geom_point(aes(col = !!sym(col_i)), size = 1.5, alpha = alpha_i) +
      guides(color = guide_legend(override.aes = list(alpha = 1))) +
      labs(col = col_lab)

    if(addline){ p_ObPr <- p_ObPr + geom_line(aes(col = !!sym(col_i), group = ID), lwd = 0.4, alpha = alpha_i) }
    if(smooth){ p_ObPr <- p_ObPr + geom_smooth(aes(col = !!sym(col_i), group = !!sym(col_i)),
                                               formula = "y ~ x", method = "loess",
                                               linewidth = 1.2, se = FALSE) }

  } else {
    p_ObPr <- p_ObPr +
      geom_point(size = 1.5, alpha = alpha_i) +
      guides(color = guide_legend(override.aes = list(alpha = 1))) +
      labs(col = col_lab)

    if(addline){ p_ObPr <- p_ObPr + geom_line(aes(group = ID), col = MSDcol[1], lwd = 0.4, alpha = alpha_i) }
    if(smooth){ p_ObPr <- p_ObPr + geom_smooth(formula = "y ~ x", method = "loess",
                                               linewidth = 1.2, se = FALSE, col = MSDcol[3]) }
  }

  if (!is.null(facet_i)) {

    p_ObPr <- p_ObPr +
      facet_wrap(
        stats::as.formula(paste("~", facet_i)),
        scales = f_scales,
        labeller = label_both
      )
  }

  if(no_leg){
    p_ObPr <- p_ObPr + theme(legend.position = "none")
  }

  return(p_ObPr + p_char)
}

## Author: Ugolkov Yaroslav
## First created: 2025-09-05
## Description: formal testing of sg-gof-obpr function and its helpers
## Keywords: SimuRg, sg-gof-obpr, goodness-of-fit

##Updated tests

test_that("sg-gof-obpr output is correct", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  if (exists("obj1")) {
    x <- sg_gof_obpr(obj1, col_i = 'AGE', cov_cols = c('AGE'), facet_i = 'AGE')
    expect_true(inherits(x, "ggplot"))
  } else {
    skip("obj1 not available in test environment")
  }
})

test_that("sg-gof-obpr does not work", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  expect_error(sg_gof_obpr(data.frame()))
})

test_that("sg-gof-obpr file load", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  if (file.exists(fpath_i)) {
    x <- sg_gof_obpr(fpath_i, log_axes = TRUE, no_leg = TRUE)
    expect_true(inherits(x, "ggplot"))
  } else {
    skip("Test data file not available")
  }
})

##New tests
# Helper function to create mock sg_fit object for testing
create_mock_sg_fit_gof <- function() {
  set.seed(123)
  list(
    SDTAB = data.frame(
      ID = rep(1:3, each = 10),
      TIME = rep(c(0, 1, 2, 4, 8, 10, 13, 24, 48, 72), 3),
      DV = rnorm(30, mean = 10, sd = 2),
      PRED = rnorm(30, mean = 10, sd = 1.5),
      IPRED = rnorm(30, mean = 10, sd = 1.2),
      MDV = rep(0, 30),
      WRES = rnorm(30, mean = 0, sd = 1),
      IWRES = rnorm(30, mean = 0, sd = 1)
    ),
    COTAB = data.frame(
      ID = 1:3,
      AGE = c(30, 45, 60),
      WT = c(70, 80, 90)
    ),
    CATAB = data.frame(
      ID = 1:3,
      SEX = c("M", "F", "M")
    )
  )
}

# Error handling tests
test_that("sg_gof_obpr errors on data.frame input", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  expect_error(
    sg_gof_obpr(data.frame()),
    "fpath_i cannot be a data.frame"
  )
})

test_that("sg_gof_obpr errors on missing SDTAB", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  obj_no_sdtab <- list(COTAB = data.frame(), CATAB = data.frame())

  expect_error(
    sg_gof_obpr(obj_no_sdtab),
    "SDTAB"
  )
})

test_that("sg_gof_obpr errors on empty SDTAB", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  obj_empty <- list(SDTAB = data.frame())

  expect_error(
    sg_gof_obpr(obj_empty),
    "SDTAB is empty"
  )
})

test_that("sg_gof_obpr errors on non-existent file", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  expect_error(
    sg_gof_obpr("nonexistent_file.RData"),
    "File does not exist"
  )
})

test_that("sg_gof_obpr errors on missing required columns in SDTAB", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  obj_missing_cols <- list(
    SDTAB = data.frame(
      ID = 1:5,
      TIME = 1:5
      # Missing DV, PRED, IPRED, MDV
    )
  )

  expect_error(
    sg_gof_obpr(obj_missing_cols),
    "SDTAB is missing required columns"
  )
})

# Basic functionality tests
test_that("sg_gof_obpr returns ggplot object", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  p <- sg_gof_obpr(mock_obj)

  expect_s3_class(p, "ggplot")
})

test_that("sg_gof_obpr handles individual predictions (IPRED)", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  p <- sg_gof_obpr(mock_obj, indiv = TRUE)

  expect_s3_class(p, "ggplot")

  # Check that plot was created successfully
  gb <- ggplot2::ggplot_build(p)
  expect_true(nrow(gb$data[[1]]) > 0)
})

test_that("sg_gof_obpr handles population predictions (PRED)", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  p <- sg_gof_obpr(mock_obj, indiv = FALSE)

  expect_s3_class(p, "ggplot")

  gb <- ggplot2::ggplot_build(p)
  expect_true(nrow(gb$data[[1]]) > 0)
})

test_that("sg_gof_obpr handles addline parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  # With lines
  p_with_lines <- sg_gof_obpr(mock_obj, addline = TRUE)
  gb_with <- ggplot2::ggplot_build(p_with_lines)
  has_lines <- any(vapply(gb_with$plot$layers, function(l) inherits(l$geom, "GeomLine"), logical(1)))
  expect_true(has_lines || nrow(gb_with$data[[1]]) > 0)

  # Without lines
  p_without_lines <- sg_gof_obpr(mock_obj, addline = FALSE)
  expect_s3_class(p_without_lines, "ggplot")
})

test_that("sg_gof_obpr handles smooth parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  # With smooth
  p_smooth <- sg_gof_obpr(mock_obj, smooth = TRUE)
  gb_smooth <- ggplot2::ggplot_build(p_smooth)
  has_smooth <- any(vapply(gb_smooth$plot$layers, function(l) inherits(l$geom, "GeomSmooth"), logical(1)))
  expect_true(has_smooth || nrow(gb_smooth$data[[1]]) > 0)

  # Without smooth
  p_no_smooth <- sg_gof_obpr(mock_obj, smooth = FALSE)
  expect_s3_class(p_no_smooth, "ggplot")
})

test_that("sg_gof_obpr handles log_axes parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  # Linear axes
  p_linear <- sg_gof_obpr(mock_obj, log_axes = FALSE)
  expect_s3_class(p_linear, "ggplot")

  # Log axes
  p_log <- sg_gof_obpr(mock_obj, log_axes = TRUE)
  expect_s3_class(p_log, "ggplot")

  # Check that log scales are applied
  x_scale_log <- p_log$scales$get_scales("x")
  y_scale_log <- p_log$scales$get_scales("y")
  expect_false(is.null(x_scale_log))
  expect_false(is.null(y_scale_log))
})

test_that("sg_gof_obpr handles sc_factor parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  # With scaling factor
  p_scaled <- sg_gof_obpr(mock_obj, sc_factor = 2)
  expect_s3_class(p_scaled, "ggplot")

  # Without scaling (default)
  p_unscaled <- sg_gof_obpr(mock_obj, sc_factor = 1)
  expect_s3_class(p_unscaled, "ggplot")
})

test_that("sg_gof_obpr handles covariate coloring", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  p <- sg_gof_obpr(mock_obj, cov_cols = "SEX", col_i = "SEX")

  expect_s3_class(p, "ggplot")

  gb <- ggplot2::ggplot_build(p)
  expect_true(nrow(gb$data[[1]]) > 0)
})

test_that("sg_gof_obpr handles continuous covariate categorization", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  p <- sg_gof_obpr(mock_obj, cov_cols = "AGE", col_i = "AGE", n_quantiles = 3)

  expect_s3_class(p, "ggplot")

  # AGE should be categorized into quantiles
  gb <- ggplot2::ggplot_build(p)
  expect_true(nrow(gb$data[[1]]) > 0)
})

test_that("sg_gof_obpr handles faceting", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  p <- sg_gof_obpr(mock_obj, cov_cols = "SEX", facet_i = "SEX")

  expect_s3_class(p, "ggplot")

  # Check that faceting was applied
  has_facet <- !is.null(p$facet) && inherits(p$facet, "FacetWrap")
  expect_true(has_facet, "Plot should have faceting applied")

  gb <- ggplot2::ggplot_build(p)
  n_panels <- length(gb$layout$panel_params)
  expect_true(n_panels > 1, "Faceted plot should have multiple panels")
})

test_that("sg_gof_obpr handles different facet scales", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  # Fixed scales
  p_fixed <- sg_gof_obpr(mock_obj, cov_cols = "SEX", facet_i = "SEX", f_scales = "fixed")
  expect_s3_class(p_fixed, "ggplot")

  # Free scales
  p_free <- sg_gof_obpr(mock_obj, cov_cols = "SEX", facet_i = "SEX", f_scales = "free")
  expect_s3_class(p_free, "ggplot")

  # Free x
  p_free_x <- sg_gof_obpr(mock_obj, cov_cols = "SEX", facet_i = "SEX", f_scales = "free_x")
  expect_s3_class(p_free_x, "ggplot")

  # Free y
  p_free_y <- sg_gof_obpr(mock_obj, cov_cols = "SEX", facet_i = "SEX", f_scales = "free_y")
  expect_s3_class(p_free_y, "ggplot")
})

test_that("sg_gof_obpr handles alpha_i parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  p_alpha <- sg_gof_obpr(mock_obj, alpha_i = 0.3)
  expect_s3_class(p_alpha, "ggplot")

  p_alpha2 <- sg_gof_obpr(mock_obj, alpha_i = 0.8)
  expect_s3_class(p_alpha2, "ggplot")
})

test_that("sg_gof_obpr handles custom axis labels", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  p <- sg_gof_obpr(mock_obj, lab_x = "Predicted (ng/mL)", lab_y = "Observed (ng/mL)")

  expect_s3_class(p, "ggplot")

  # Check that labels are set
  expect_equal(p$labels$x, "Predicted (ng/mL)")
  expect_equal(p$labels$y, "Observed (ng/mL)")
})

test_that("sg_gof_obpr handles no_leg parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  # With legend
  p_legend <- sg_gof_obpr(mock_obj, cov_cols = "SEX", col_i = "SEX", no_leg = FALSE)
  expect_s3_class(p_legend, "ggplot")

  # Without legend
  p_no_legend <- sg_gof_obpr(mock_obj, cov_cols = "SEX", col_i = "SEX", no_leg = TRUE)
  expect_s3_class(p_no_legend, "ggplot")

  # Check legend position
  expect_equal(p_no_legend$theme$legend.position, "none")
})

test_that("sg_gof_obpr handles n_quantiles parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  p_q3 <- sg_gof_obpr(mock_obj, cov_cols = "AGE", col_i = "AGE", n_quantiles = 3)
  expect_s3_class(p_q3, "ggplot")

  p_q5 <- sg_gof_obpr(mock_obj, cov_cols = "AGE", col_i = "AGE", n_quantiles = 5)
  expect_s3_class(p_q5, "ggplot")
})

test_that("sg_gof_obpr handles levels_discrete parameter", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()

  p_disc <- sg_gof_obpr(mock_obj, cov_cols = "AGE", col_i = "AGE", levels_discrete = 5)
  expect_s3_class(p_disc, "ggplot")

  p_disc2 <- sg_gof_obpr(mock_obj, cov_cols = "AGE", col_i = "AGE", levels_discrete = 20)
  expect_s3_class(p_disc2, "ggplot")
})

test_that("sg_gof_obpr filters MDV correctly", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  mock_obj <- create_mock_sg_fit_gof()
  # Add some MDV=1 rows
  mock_obj$SDTAB <- rbind(
    mock_obj$SDTAB,
    data.frame(
      ID = c(1, 2),
      TIME = c(0, 0),
      DV = c(NA, NA),
      PRED = c(100, 100),
      IPRED = c(100, 100),
      MDV = c(1, 1),
      WRES = c(NA, NA),
      IWRES = c(NA, NA)
    )
  )

  p <- sg_gof_obpr(mock_obj)
  expect_s3_class(p, "ggplot")

  # MDV=1 rows should be filtered out
  gb <- ggplot2::ggplot_build(p)
  # Should have fewer points than total rows (MDV=1 filtered)
  expect_true(nrow(gb$data[[1]]) <= nrow(mock_obj$SDTAB))
})

test_that("sg_gof_obpr handles SDTAB as list of data frames", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("dplyr")

  # Create SDTAB as a list (simulating some data structures)
  mock_obj <- list(
    SDTAB = list(
      data.frame(
        ID = 1:3,
        TIME = c(0, 1, 2),
        DV = rnorm(3, 10, 2),
        PRED = rnorm(3, 10, 1.5),
        IPRED = rnorm(3, 10, 1.2),
        MDV = c(0, 0, 0),
        WRES = rnorm(3),
        IWRES = rnorm(3)
      )
    ),
    COTAB = NULL,
    CATAB = NULL
  )

  p <- sg_gof_obpr(mock_obj)
  expect_s3_class(p, "ggplot")
})

## Author: Alina Melnikova
## First created: 2026-01-28
## Description: Simulation for covariate sensitivity analysis
## Keywords: covariates, simulations, sensitivity

###-----Functions----####
#' @noRd
funSum_av <- list(mean   = ~mean(.),
                  median   = ~median(., na.rm = TRUE))

#' @noRd
funSum_exp <- list(`Cavg`   = ~mean(., na.rm = TRUE),
                   `Cmin`    = ~min(., na.rm = TRUE),
                   `Cmax`    = ~max(., na.rm = TRUE))

#' @noRd
fun_EtCC <- function(et_base_i, cc_ds_i, cat = FALSE){
  et_scov_i <- unique(cc_ds_i$COV) %>% map(function(n){
    cc_ds_n <- cc_ds_i %>% filter(COV == n)

    et_scov_n <- seq(nrow(cc_ds_n)) %>% map_dfr(function(m){
      row_m <- cc_ds_n %>% filter(row_number() == m)

      et_scov_m <- et_base_i %>%
        mutate_at(vars(all_of(row_m$COV)), function(k){k = row_m$COVVAL})
      if(!cat){
        et_scov_m <- et_scov_m %>% bind_cols(select(row_m, COV, PAR, BTR, KEY, COVVAL, BCOVVAL))
      } else {
        et_scov_m <- et_scov_m %>% bind_cols(select(row_m, COV, PAR, KEY, COVVAL, CATDES))
      }

      return(et_scov_m)

    }) %>% mutate(id = row_number())

    return(et_scov_n)
  })
  names(et_scov_i) <- unique(cc_ds_i$COV)
  return(et_scov_i)
}

#' @noRd
fun_CovSens <- function(et_sim_i, cat = FALSE, expos = FALSE, covs_i = NULL, nsim = 100, stime_exp = NULL,
                        mod_fin_i,
                        theta_i, omega_i, thetamat_i, nice_names_i,
                        quantiles = c(0.1,0.9),
                        var_exp = "Cc", aggr = c("min", "max", "mean")) #nsim=1000
{


  #Test
  # et_sim_i = ets_cc
  # covs_i = nice_names$COV
  # expos = T
  # stime_exp = stimes
  # cat = F; nsim = 5;
  # var_exp = "Cc"
  # theta_i = par_fin_tv
  # thetamat_i = m_theta_norm_pop


  keep_i <- c("Regimen", "KEY", "COV", "COVVAL")
  if(!cat){
    keep_i <- c(keep_i, "BTR", "BCOVVAL")
  } else {
    keep_i <- c(keep_i, "CATDES")
  }

  sens_i <- et_sim_i %>% map_dfr(function(et_i){
    #Test
    #et_i <- et_sim_i[[1]]

    if(!expos){
      par_i <- unique(et_i$PAR)
      if(is.list(par_i)){ par_i <- par_i[[1]] }
      sim_i <- sg_sim(mod_fin_i, et_i, 0, outputs = par_i, theta = theta_i, omega = omega_i,
                      thetamat = thetamat_i, covs = covs_i, npop = nsim, keep = keep_i)
    } else {
      sim_raw <- sg_sim(mod_fin_i, et_i, stime_exp, outputs = var_exp, theta = theta_i, omega = omega_i,
                        thetamat = thetamat_i, covs = covs_i, npop = nsim,
                        keep = keep_i, ncores = max(1, parallel::detectCores()-2))

      message(sprintf("NA Cc rows: %d / %d (%.1f%%)",
                      sum(is.na(sim_raw$VALUE)), nrow(sim_raw),
                      100 * mean(is.na(sim_raw$VALUE))))

      aggr_map <- c("mean" = "Cavg", "min" = "Cmin", "max" = "Cmax")
      stopifnot("aggr must only contain 'mean', 'min', 'max'" = all(aggr %in% names(aggr_map)))
      funSum_exp_i <- funSum_exp[unname(aggr_map[aggr])]

      sim_i <- sim_raw %>% mutate(VALUE = as.numeric(VALUE))

      sim_i <- sim_i %>%
        # group_by_at(vars(all_of(c("POPN", keep_i, covs_i)))) %>% summarise_at(vars(VALUE), funSum_exp_i) %>% ungroup() %>%
        # gather("VAR", "VALUE", -all_of(c("POPN", keep_i, covs_i)))
      group_by_at(vars(all_of(c("POPN", "VAR", keep_i, covs_i)))) %>% summarise_at(vars(VALUE), funSum_exp_i) %>% ungroup() %>%
        gather("METRIC", "VALUE", -all_of(c("POPN", "VAR", keep_i, covs_i))) %>%
        mutate(VAR = paste(VAR, METRIC, sep = "_")) %>%
        select(-METRIC)

    }

    sim_i <- sim_i %>% mutate(VALUE = as.numeric(VALUE))

    sim_i_ref <- sim_i %>% filter(KEY == "REF" | KEY == 1) %>% select(POPN, Regimen, VAR, REFVAL = VALUE)
    sim_i_ch <- sim_i %>% filter(KEY != "REF" & KEY != 1) %>% left_join(sim_i_ref, by = c("POPN", "Regimen", "VAR")) %>%
      mutate(PCH = 100*(VALUE - REFVAL)/REFVAL)


    out_i <- sim_i_ch %>% group_by_at(vars(all_of(c("VAR", keep_i, covs_i)))) %>% summarise_at(vars(PCH), funSum_sim) %>% ungroup()

    # Ensure BCOVVAL is added from et_i if not already present (for continuous covariates)
    if(!cat && "BCOVVAL" %in% names(et_i)){
      if(!"BCOVVAL" %in% names(out_i)){
        bcovval_lookup <- et_i %>% select(KEY, COV, COVVAL, BCOVVAL) %>% unique()
        out_i <- out_i %>% left_join(bcovval_lookup, by = c("KEY", "COV", "COVVAL"))
      }
    }
    out_i

  })  %>% left_join(nice_names_i, by = "COV")

  if(!cat){
    sens_out <- sens_i %>% select(NICEN, VAR, KEY, mean:P975, Regimen, COVVAL:BCOVVAL) %>%
      mutate(KEY = ifelse(KEY == "LP", paste0(quantiles[[1]]*100, "th perc."), paste0(quantiles[[2]]*100, "th perc.")), LAB = str_c(NICEN, "\n", KEY, " (", round(BCOVVAL, 1), ")"))
  } else {
    sens_out <- sens_i %>% select(NICEN, VAR, CATDES, mean:P975, Regimen) %>% mutate(LAB = str_c(NICEN, " (", CATDES, ")"))
  }
  sens_out <- sens_out %>% mutate_at(vars(mean:P975), function(p){p/100+1})
  return(sens_out)
}
#####

#' Covariate sensitivity analysis via simulation
#'
#' Evaluate the impact of continuous and categorical covariates on model
#' parameters and exposure metrics.  For each covariate the function perturbs
#' its value to selected quantiles (continuous) or observed categories
#' (categorical) while holding the remaining covariates at their reference
#' values, simulates from the pharmacometric model under parameter uncertainty,
#' and summarises the resulting percent change relative to the reference.
#'
#' Two data-source modes are supported (mutually exclusive):
#' \itemize{
#'   \item \strong{File mode} — supply \code{fpath_i} (path to a SimuRg JSON /
#'     RData object that contains SUMTAB, CATAB and COTAB).
#'   \item \strong{Table mode} — supply both \code{ds_parest} (parameter
#'     estimates) and \code{ds_covs} (covariate dataset).
#' }
#'
#' @inheritParams sg_dummy
#' @param ds_covs data.frame. Subject-level covariate dataset (one row per
#'   subject) containing both continuous and categorical covariate columns.
#'   Required when \code{fpath_i} is \code{NULL}; must be provided together
#'   with \code{ds_parest}.  Default is \code{NULL}.
#' @param stimes numeric vector. Sampling time points for steady-state
#'   simulation (e.g. generated by a cycling function over dosing intervals).
#' @param et data.frame. Event (dosing) table.  Must contain at least
#'   columns \code{id}, \code{time}, \code{amt}, \code{evid} and
#'   \code{Regimen}.  Additional columns such as \code{ii}, \code{addl},
#'   \code{dur} and \code{Dose} are used when present.
#' @param npop integer. Number of population-level simulation replicates drawn
#'   from the parameter uncertainty distribution.  Default is \code{5}.
#' @param outputs character vector. Name(s) of the model output variable(s)
#'   to evaluate (e.g. \code{"Cc"} or \code{c("Cc", "Effect")}).
#'   Default is \code{"Cc"}.
#' @param aggr character vector. Exposure aggregation metric(s) applied over
#'   the simulation time grid.
#'   Allowed values: \code{"min"} (Cmin), \code{"max"} (Cmax),
#'   \code{"mean"} (Cavg).  Default is \code{c("min", "max", "mean")}.
#'
#'@returns A named list of three elements:
#' \describe{
#'   \item{\code{$PARSENS}}{data.frame.
#'     Full sensitivity results for model parameters: percent change
#'     statistics (mean, median, percentiles) for each covariate–parameter
#'     combination, with columns \code{NICEN}, \code{VAR}, \code{KEY},
#'     \code{LAB}, \code{Type} and summary statistics
#'     (\code{mean} through \code{P975}).}
#'   \item{\code{$SUMPARSENS}}{data.frame.
#'     Compact summary table with columns \code{Parameter},
#'     \code{Covariate}, \code{Cov. percentile}, \code{Cov. value},
#'     \code{Mean} and \code{90\%CI}.}
#'   \item{\code{$EXPSENS}}{data.frame.
#'     Sensitivity of exposure metrics (Cmin, Cmax, Cavg) to covariates,
#'     structured identically to \code{$PARSENS}.}
#' }
#'
#' @details
#' The analysis proceeds in two stages for each covariate type (continuous and
#' categorical):
#' \enumerate{
#'   \item \strong{Parameter sensitivity} — the covariate is set to its
#'     quantile or category value and the model is evaluated at time zero (no
#'     ODE integration) to quantify the direct effect on each parameter.
#'   \item \strong{Exposure sensitivity} — the full time-course is simulated
#'     over \code{stimes} and exposure metrics (\code{aggr}) are computed.
#' }
#'
#' Uncertainty is propagated by sampling \code{npop} parameter vectors from a
#' multivariate normal distribution parameterised by the population estimates
#' and the estimation covariance matrix (\code{est_covmat}).
#'
#' @examples
#' \donttest{
#' library(dplyr)
#' library(rxode2)
#'
#' # --- Covariate definitions ---
#' cont_cov_l <- list(
#'   LG_AGE = list(NAME = "LG_AGE", UTNAME = "AGE",
#'                 REF = "median", NICENAME = "Age, years",
#'                 par_vec = c("CL")),
#'   LG_WEIGHT = list(NAME = "LG_WEIGHT", UTNAME = "WEIGHT",
#'                    REF = "median", NICENAME = "Weight, kg",
#'                    par_vec = c("Vd"))
#' )
#'
#' cat_cov_l <- list(
#'   SEX = list(NAME = "SEX", NICENAME = "Sex",
#'              REF = "0", par_vec = c("ka")),
#'   CYP2C9 = list(NAME = "CYP2C9", NICENAME = "CYP2C9 genotype",
#'                 REF = NULL, par_vec = c("CL"))
#' )
#'
#' # --- Dosing ---
#' ev_t_input <- tribble(
#'   ~id, ~time, ~ii, ~amt, ~addl, ~dur, ~evid, ~Regimen,        ~Dose,
#'   1,   0,     336, 10,   21,    0.5,  1,     "0.3 mg/kg Q2W", 0.3
#' )
#' # --- Model ---
#' model <- RxODE({
#'   # Doses in mg
#'   # Time in hours
#'
#'   ### Parameter values
#'   # Typical values
#'   ka_pop = 0.073;
#'   Vd_pop = 14.8;
#'   CL_pop = 0.347;
#'
#'   # Random effects
#'   omega_ka = 0;
#'   omega_Vd = 0;
#'   omega_CL = 0;
#'
#'   # Covariate effect
#'   # Continuous
#'   beta_CL_LG_AGE = 0.49990114;
#'   beta_Vd_LG_WEIGHT = 0.60529433;
#'
#'   # Categorical
#'   beta_CL_CYP2C9_1_2 = -0.339;
#'   beta_CL_CYP2C9_1_3 = -0.574;
#'   beta_CL_CYP2C9_2_2 = -1.079;
#'   beta_CL_CYP2C9_2_3 = -0.745;
#'   beta_CL_CYP2C9_3_3 = -2.13;
#'
#'   beta_ka_SEX_1 = -0.12198035;
#'
#'   # Residual error
#'   Cc_b = 0;
#'
#'   # Transformations
#'   ka_tv = exp(ka_pop);
#'   Vd_tv = exp(Vd_pop);
#'   CL_tv = exp(CL_pop);
#'
#'   CL_multiplier = 1.0;  # Default/reference
#'   ka_multiplier = 1.0;
#'
#'   if (SEX == "1") {ka_multiplier = exp(beta_ka_SEX_1)}
#'
#'   if (CYP2C9 == "1") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_1_2);
#'   } else if (CYP2C9 == "2") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_1_3);
#'   } else if (CYP2C9 == "3") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_2_2);
#'   } else if (CYP2C9 == "4") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_2_3);
#'   } else if (CYP2C9 == "5") {
#'     CL_multiplier = exp(beta_CL_CYP2C9_3_3);
#'   }
#'
#'   ka = ka_tv*ka_multiplier*exp(omega_ka);
#'   Vd = Vd_tv*exp(beta_Vd_LG_WEIGHT * LG_WEIGHT + omega_Vd); #Vd_tv*exp(omega_Vd);
#'   CL = CL_tv*CL_multiplier*exp(beta_CL_LG_AGE * LG_AGE + omega_CL);
#'
#'
#'   ### Explicit functions
#'   Cc = Ac/Vd;
#'
#'   ### Initial conditions
#'   Ad(0) = 0;
#'   Ac(0) = 0;
#'
#'   ### Differential equations
#'   d/dt(Ad) = - ka*Ad;
#'   d/dt(Ac) = ka*Ad - CL*Cc;
#'
#'   Cc_ResErr = Cc*(1 + Cc_b);
#' })
#'
#' # --- Estimation covariance (mock) ---
#' pnames     <- parest$parameter
#' npar       <- length(pnames)
#' set.seed(1)
#' m_cov      <- matrix(0.02, npar, npar)
#' diag(m_cov) <- 0.05 + runif(npar, 0, 0.05)
#' m_cov      <- (m_cov + t(m_cov)) / 2
#' est_covmat <- as_tibble(cbind(X1 = pnames, as.data.frame(m_cov)))
#' names(est_covmat)[-1] <- pnames
#'
#' # --- Simulation times (steady-state cycle 10) ---
#' ss_cycle <- 10
#' stimes_ss <- c(
#'   ss_cycle * 4 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1)),
#'   ss_cycle * 4 * 7 * 24 + 2 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1))
#' )
#'
#' # --- Run ---
#' result <- sg_covsens_sim(
#'   fpath_i=NULL, ds_parest = parest, ds_covs = ds_covval,
#'   model = model, stimes = stimes_ss, et = ev_t_input,
#'   est_covmat = est_covmat, npop = 10,
#'   cont_cov_l = cont_cov_l, cat_cov_l = cat_cov_l,
#'   quantiles = c(0.2, 0.8), aggr = c("max"),
#'   outputs = "Cc"
#' )
#' print(result[["PARSENS"]])
#' print(result[["SUMPARSENS"]])
#' print(result[["EXPSENS"]])
#'
#' }
#'
#' @seealso \code{\link{sg_sim}}, \code{\link{read_smrg_obj}}
#'
#' @import dplyr
#' @importFrom purrr map map_dfr map_chr
#' @importFrom tidyr gather separate
#' @importFrom stringr str_detect str_c
#' @importFrom forcats fct_inorder
#' @export
sg_covsens_sim <- function(fpath_i = NULL, ds_parest = NULL, ds_covs = NULL,
                           model, stimes, et,
                           est_covmat,
                           npop = 5,
                           cont_cov_l, cat_cov_l,  quantiles = c(0.1, 0.9),
                           outputs  = "Cc", aggr = c("min", "max", "mean")){

  # --- Input validation ---
  # Data source: must provide either fpath_i alone, or both ds_parest and ds_covs
  has_fpath   <- !is.null(fpath_i)
  has_ds      <- !is.null(ds_parest) && !is.null(ds_covs)
  has_ds_part <- !is.null(ds_parest) || !is.null(ds_covs)

  if (!has_fpath && !has_ds) {
    stop(
      "No data source provided. Supply either:\n",
      "  - 'fpath_i': path to a Simurg output object, OR\n",
      "  - both 'ds_parest' (parameter estimates) and 'ds_covs' (covariate dataset)."
    )
  }
  if (has_fpath && has_ds_part) {
    stop(
      "'fpath_i' and 'ds_parest'/'ds_covs' are mutually exclusive. ",
      "Supply one data source only."
    )
  }
  if (!has_fpath && has_ds_part && !has_ds) {
    missing_ds <- if (is.null(ds_parest)) "'ds_parest'" else "'ds_covs'"
    stop(
      "Incomplete data source: ", missing_ds, " is missing. ",
      "Both 'ds_parest' and 'ds_covs' must be provided together."
    )
  }

  if (missing(model))    stop("'model' is required: provide the compiled rxode2/nlmixr model object.")
  if (missing(stimes))  stop("'stimes' is required: provide the steady-state simulation time points.")
  if (missing(et)) stop("'et' is required: provide the event table (dosing schedule).")
  if (missing(est_covmat)) stop("'est_covmat' is required: provide the parameter estimation covariance matrix.")
  if (missing(cont_cov_l)) stop("'cont_cov_l' is required: provide the continuous covariate definition list.")
  if (missing(cat_cov_l))  stop("'cat_cov_l' is required: provide the categorical covariate definition list.")

  # Warn about non-default aggregation choices to alert on typos
  valid_aggr <- c("min", "max", "mean")
  bad_aggr   <- setdiff(aggr, valid_aggr)
  if (length(bad_aggr) > 0) {
    warning("Unrecognised aggregation function(s) in 'aggr': ",
            paste(bad_aggr, collapse = ", "),
            ". Valid options are: ", paste(valid_aggr, collapse = ", "), ".")
    aggr <- valid_aggr
  }
  # -------------------------
  ds_catcov <- NULL
  if((!is.null(fpath_i))&(is.null(ds_parest))){
  obj_data <- read_smrg_obj(fpath_i)
  #ds_mod <-  obj_data$SDTAB
  par_sum <- obj_data$SUMTAB
  ds_catcov <- obj_data$CATAB
  ds_ccov <- obj_data$COTAB
  data_fin <- ds_ccov %>% left_join(ds_catcov, by = "ID")
  ### Population parameters
  par_fin <- par_sum %>% rename(parameter = PAR, value = VALUE)
  } else if((is.null(fpath_i))&(!is.null(ds_parest))&(!is.null(ds_covs))){
    par_fin <- ds_parest
    data_fin <- ds_covs
  }

  par_pop <- par_fin %>% filter(str_detect(parameter, "_pop")) %>% select(parameter, value) %>% deframe()
  par_fin_tv <- par_fin %>% filter(str_detect(parameter, "_pop$") | str_detect(parameter, "^beta_")) %>% select(parameter, value) %>%
  #mutate(value = ifelse(str_detect(parameter, "_pop$") & !str_detect(parameter, "^beta_"), log(value), value)) %>% deframe()
  mutate(value = {
    cond <- str_detect(parameter, "_pop$") & !str_detect(parameter, "^beta_")
    ifelse(cond, log(ifelse(cond, value, 1)), value)
  }) %>% deframe()

  ### Reconstruct omega matrix (random effects on K_a and V_pop)
  d_omega <- par_fin %>% filter(str_detect(parameter, "omega_"))
  m_omega <- diag(d_omega$value, ncol = length(d_omega$value))
  colnames(m_omega) <- d_omega$parameter; rownames(m_omega) <- d_omega$parameter
  m_omega_full <- m_omega

  corr_set <- par_fin %>% filter(str_detect(parameter, "corr_"))
  if(nrow(corr_set) > 0){
    corr_set <- corr_set %>%
      separate(parameter, c("TYPE", "PAR1", "PAR2"), "_") %>%
      select(PAR1, PAR2, value) %>%
      mutate_at(vars(PAR1, PAR2), function(x){str_c("omega_", x)})

    omega_corr <- m_omega
    diag(omega_corr) <- rep(1, ncol(m_omega))
    for(i in 1:nrow(corr_set)){
      cs_i <- corr_set[i, ]
      omega_corr[cs_i$PAR1, cs_i$PAR2] <- cs_i$value
      omega_corr[cs_i$PAR2, cs_i$PAR1] <- cs_i$value
    }

    m_omega_full <- m_omega %*% omega_corr %*% m_omega
  }

  # ### Reconstruct residual error model matrix
  # d_reserr <- par_fin %>% filter(!str_detect(parameter, "_pop|omega_|corr_|beta_"))
  # m_reserr <- diag(d_reserr$value, ncol = length(d_reserr$value))
  # colnames(m_reserr) <- d_reserr$parameter; rownames(m_reserr) <- d_reserr$parameter

  m_theta_norm <- est_covmat %>% select_if(is.numeric) %>% as.matrix()
  colnames(m_theta_norm) <- est_covmat$X1; rownames(m_theta_norm) <- est_covmat$X1
  m_theta_norm_pop <- m_theta_norm[str_detect(rownames(m_theta_norm), "_pop|beta_"), str_detect(colnames(m_theta_norm), "_pop|beta_")]

  #####----Covariate tables----####
  # Validate and filter cont_cov_l against data_fin columns
  .missing_cont_name <- map_chr(cont_cov_l, function(x) x$NAME) %>%
    setdiff(names(data_fin))
  if (length(.missing_cont_name) > 0) {
    warning(
      "The following continuous covariate NAME(s) from 'cont_cov_l' were not found ",
      "as columns in the covariate data and will be skipped: ",
      paste(.missing_cont_name, collapse = ", "), "."
    )
    cont_cov_l <- Filter(function(x) !(x$NAME %in% .missing_cont_name), cont_cov_l)
    if (length(cont_cov_l) == 0) {
      stop(
        "All continuous covariates in 'cont_cov_l' were removed because their NAME columns ",
        "are absent from the covariate data. Check 'cont_cov_l' and the dataset."
      )
    }
  }

  .missing_cont_utname <- unlist(Filter(Negate(is.null), map(cont_cov_l, function(x) {
    ut <- x$UTNAME
    if (!is.null(ut) && !isTRUE(is.na(ut)) && ut != x$NAME) ut else NULL
  }))) %>% setdiff(names(data_fin))
  if (length(.missing_cont_utname) > 0) {
    warning(
      "The following continuous covariate UTNAME(s) from 'cont_cov_l' were not found ",
      "as columns in the covariate data; affected covariates will be skipped: ",
      paste(.missing_cont_utname, collapse = ", "), "."
    )
    cont_cov_l <- Filter(function(x) {
      ut <- x$UTNAME
      if (!is.null(ut) && !isTRUE(is.na(ut)) && ut != x$NAME) !(ut %in% .missing_cont_utname) else TRUE
    }, cont_cov_l)
    if (length(cont_cov_l) == 0) {
      stop(
        "All continuous covariates in 'cont_cov_l' were removed because their UTNAME columns ",
        "are absent from the covariate data. Check 'cont_cov_l' and the dataset."
      )
    }
  }

  # Validate and filter cat_cov_l against data_fin columns
  .missing_cat_name <- map_chr(cat_cov_l, function(x) x$NAME) %>%
    setdiff(names(data_fin))
  if (length(.missing_cat_name) > 0) {
    warning(
      "The following categorical covariate NAME(s) from 'cat_cov_l' were not found ",
      "as columns in the covariate data and will be skipped: ",
      paste(.missing_cat_name, collapse = ", "), "."
    )
    cat_cov_l <- Filter(function(x) !(x$NAME %in% .missing_cat_name), cat_cov_l)
    if (length(cat_cov_l) == 0) {
      stop(
        "All categorical covariates in 'cat_cov_l' were removed because their NAME columns ",
        "are absent from the covariate data. Check 'cat_cov_l' and the dataset."
      )
    }
  }

  # Continuous covariates table
  cont_cov_vec <- map_chr(cont_cov_l, function(x) x$NAME)

  cont_cov <- map_dfr(cont_cov_l, function(x) {
    #tibble(TR = x$NAME, BTR = x$UTNAME, PAR = list(x$par_vec))
    btr <- if (is.null(x$UTNAME) || isTRUE(is.na(x$UTNAME))) x$NAME else x$UTNAME
    tibble(TR = x$NAME, BTR = btr, PAR = list(x$par_vec))
  })

  # Categorical covariates table
  cat_cov_vec <- map_chr(cat_cov_l, function(x) x$NAME)

  if (is.null(ds_catcov)){
    ds_catcov <- ds_covs %>% select(all_of(c("ID",cat_cov_vec)))
  }

  cat_unique <- map(cat_cov_vec, function(x){
    ds_catcov[[x]] %>% unique()
  })
  names(cat_unique) <- cat_cov_vec

  # Build cat_cov automatically from cat_cov_l, cat_cov_vec and cat_unique
  cat_cov <- map_dfr(cat_cov_vec, function(x) {
    vals <- as.character(cat_unique[[x]])
    #ref  <- cat_cov_l[[x]]$REF
    ref <- if (is.null(cat_cov_l[[x]]$REF)) {
      as.character(levels(factor(data_fin[[cat_cov_l[[x]]$NAME]]))[1])
    } else {
      cat_cov_l[[x]]$REF
    }
    tibble(
      COV    = x,
      COVVAL = vals,
      CATDES = vals,
      KEY    = as.integer(vals == ref),
      PAR    = list(cat_cov_l[[x]]$par_vec)
    )
  })

  nice_names_cont <- map_dfr(cont_cov_vec, function(x) {
    tibble(COV = x, NICEN = cont_cov_l[[x]][["NICENAME"]])
  })
  nice_names_cat <- map_dfr(cat_cov_vec, function(x) {
    tibble(COV = x, NICEN = cat_cov_l[[x]][["NICENAME"]])
  })
  nice_names <- rbind(nice_names_cont,nice_names_cat)

  ## Derive Covariate distributions
  ### Continuous covariates
  ID <- "ID"
  CCov_vec <- cont_cov$TR
  CatCov_vec <- cat_cov$COV %>% unique()

  data_fin <- data_fin %>%
    mutate(across(all_of(CatCov_vec), as.character))

  ds_cc_tr <- data_fin %>% select(all_of(c(ID)), all_of(cont_cov$TR)) %>% unique() %>%
    gather("TR", "TVALUE", -all_of(c(ID))) %>% left_join(cont_cov, by = "TR")

  ds_cc_btr <- data_fin %>% select(all_of(c(ID)), all_of(cont_cov$BTR)) %>% unique() %>%
    gather("BTR", "NVALUE", -all_of(c(ID)))
  ds_cc_btr_av <- ds_cc_btr %>% group_by(BTR) %>% summarise_at(vars(NVALUE), funSum_av)

  ds_cc <- ds_cc_tr %>%
    left_join(ds_cc_btr, by = c(ID, "BTR")) %>%
    left_join(ds_cc_btr_av, by = "BTR") %>%
    group_by(TR) %>%
    mutate(
      LP = quantile(TVALUE, quantiles[[1]]),
      UP = quantile(TVALUE, quantiles[[2]]))



  # Build per-covariate REF lookup from cont_cov_l
  ref_lookup <- map_dfr(cont_cov_l, function(cov) {
    tibble(TR = cov$NAME, REF_spec = cov$REF)
  })

  # Add REF column: use median(TVALUE) when REF_spec == "median", else use the numeric value
  ds_cc <- ds_cc %>%
    left_join(ref_lookup, by = "TR") %>%
    group_by(TR) %>%
    #mutate(REF = if_else(REF_spec == "median", median(TVALUE), as.numeric(REF_spec))) %>%
    mutate(REF = if_else(REF_spec == "median", median(TVALUE), suppressWarnings(as.numeric(REF_spec)))) %>%
    select(-REF_spec) %>%
    ungroup()


  #ds_cc_reflab <- select(ds_cc, COV = TR, median) %>% unique() %>% left_join(nice_names, by = "COV") %>% summarise(OUT = str_c(str_c(NICEN, " = ", round(median, 1)), collapse = "\n")) %>% pull(OUT)
  ds_cc_reflab <- select(ds_cc, COV = TR, median, LP, UP) %>% unique() %>% left_join(nice_names, by = "COV") %>% summarise(OUT = str_c(str_c(NICEN, " = ", round(median, 1), " (median)\n", "[",quantiles[[1]]*100,"th percentile: ", round(LP, 1), "; ",quantiles[[2]]*100,"th percentile: ", round(UP, 1), "]"), collapse = "\n")) %>% pull(OUT)

  # ds_cc_reflab <- select(ds_cc, COV = TR, median) %>% unique() %>% left_join(nice_names, by = "COV") %>% summarise(OUT = str_c(str_c(NICEN, " = ", round(median, 1)), collapse = "\n")) %>% pull(OUT) %>% str_c(., "\nResults shown for a dose calculated for the median (reference) patient")

  # Calculate low and upper quantile values for Back transformed covariates
  #Take into account that there are several continuous covariates!!!
  map_ccont <- function(target_ccont, data) {
    idx <- which.min(abs(data$TVALUE - target_ccont))
    data$NVALUE[idx]
  }
  ccont_lab_list <- list()
  for (i in seq_along(cont_cov_l)){
    ds_cc_i <- ds_cc %>% filter(TR == cont_cov_l[[i]]$NAME)
    q_ccont <- c(unique(ds_cc_i$LP), unique(ds_cc_i$UP), unique(ds_cc_i$REF))

    ccont_labels <- sapply(
      q_ccont,
      map_ccont,
      data = ds_cc_i
    )

    names(ccont_labels) <- c("LP_BTR", "UP_BTR", "REF_BTR")
    ccont_lab_list[[cont_cov_l[[i]]$NAME]] <- ccont_labels
  }

  # Convert ccont_lab_list to a long data frame for joining: COV + KEY -> BCOVVAL
  ccont_lab_df <- map_dfr(names(ccont_lab_list), function(cov_name) {
    vals <- ccont_lab_list[[cov_name]]
    tibble(
      COV     = cov_name,
      KEY     = c("LP",       "UP",       "REF"),
      BCOVVAL = as.numeric(vals[c("LP_BTR", "UP_BTR", "REF_BTR")])
    )
  })

  cc_to_test <- ds_cc %>%
    select(COV = TR, BTR, PAR, mean, median, LP, UP, REF) %>%
    unique() %>%
    mutate(across(c(LP, UP, REF), as.numeric)) %>%
    gather("KEY", "COVVAL", -COV:-median) %>%
    left_join(ccont_lab_df, by = c("COV", "KEY"))

  ### Categorical covariates
  ds_catc <- data_fin %>% select(all_of(c(ID)), all_of(cat_cov$COV)) %>% unique()

  catc_to_test <- ds_catc %>% select(-all_of(c(ID))) %>% gather("COV", "COVVAL") %>% unique() %>% left_join(cat_cov, by = c("COV", "COVVAL"))

  # Reference values for continuous covariates: log-scale (NAME) and back-transformed (UTNAME)
  cont_cov_ref <- do.call(c, unname(map(cont_cov_l, function(cov) {
    ref_row <- cc_to_test %>% filter(COV == cov$NAME, KEY == "REF")
    ut_name <- if (is.null(cov$UTNAME) || is.na(cov$UTNAME)) cov$NAME else cov$UTNAME
    setNames(
      list(ref_row %>% pull(COVVAL), ref_row %>% pull(BCOVVAL)),
      c(cov$NAME, ut_name)
    )
  })))
  cont_cov_ref <- cont_cov_ref[!duplicated(names(cont_cov_ref))]


  # Reference values for categorical covariates: use REF if defined, else first factor level
  cat_cov_ref <- setNames(
    map(cat_cov_l, function(cov) {
      if (is.null(cov$REF)) {
        as.character(levels(factor(data_fin[[cov$NAME]]))[1])
      } else {
        cov$REF
      }
    }),
    map_chr(cat_cov_l, ~ .x$NAME)
  )

  all_cov_ref <- c(cont_cov_ref, cat_cov_ref)

  # Combine with event table
  ev_t_base <- et %>%
    mutate(!!!all_cov_ref)

  ets_cc <- fun_EtCC(ev_t_base, cc_to_test)
  ets_catc <- fun_EtCC(ev_t_base, catc_to_test, T)

   # Sensitivity of model parameters to covariate values
  out_cov_exp_sens = NULL
  t_cov_sens_par = NULL
  out_cov_par_sens = NULL

  out_cov_par_sens <- bind_rows(
    fun_CovSens(ets_cc, covs_i = nice_names$COV, nsim = npop,
                mod_fin_i = model,
                theta_i = par_fin_tv, omega_i = m_omega_full,
                thetamat_i = m_theta_norm_pop,
                nice_names_i = nice_names,quantiles = quantiles,
                var_exp = outputs, aggr = aggr
                ) %>% mutate(Type = "Continuous"),
    fun_CovSens(ets_catc, cat = TRUE, covs_i = nice_names$COV, nsim = npop,
                mod_fin_i = model,
                theta_i = par_fin_tv, omega_i = m_omega_full,
                thetamat_i = m_theta_norm_pop,
                nice_names_i = nice_names,quantiles = quantiles,
                var_exp = outputs, aggr = aggr
                ) %>% mutate(Type = "Categorical")
    ) %>% mutate(LAB = fct_inorder(LAB), Type = fct_inorder(Type))

  ## Summary of sensitivity of model parameters to covariate values
  t_cov_sens_par <- out_cov_par_sens %>% mutate_at(vars(mean:P975), signif, 3) %>%
    mutate(`90%CI` = str_c(P05, ", ", P95), KEY = ifelse(is.na(KEY), "Category", KEY), BCOVVAL = as.character(BCOVVAL), BCOVVAL = ifelse(is.na(BCOVVAL), CATDES, BCOVVAL)) %>%
    select(Parameter = VAR, Covariate = NICEN, `Cov. percentile` = KEY, `Cov. value` = BCOVVAL, Mean = mean, `90%CI`)

  # Sensitivity of exposure parameters to covariate values
  out_cov_exp_sens <- bind_rows(
    fun_CovSens(ets_cc, expos = TRUE, covs_i = nice_names$COV, nsim = npop,
                stime_exp = stimes,
                mod_fin_i = model,
                theta_i = par_fin_tv, omega_i = m_omega_full,
                thetamat_i = m_theta_norm_pop,
                nice_names_i = nice_names, quantiles = quantiles,
                var_exp = outputs, aggr = aggr
                ) %>% mutate(Type = "Continuous"),
    fun_CovSens(ets_catc, cat = TRUE, expos = TRUE, covs_i = nice_names$COV, nsim = npop,
                stime_exp = stimes,
                mod_fin_i = model,
                theta_i = par_fin_tv, omega_i = m_omega_full,
                thetamat_i = m_theta_norm_pop,
                nice_names_i = nice_names, quantiles = quantiles,
                var_exp = outputs, aggr = aggr
                ) %>% mutate(Type = "Categorical")
  ) %>% mutate(LAB = fct_inorder(LAB), Type = fct_inorder(Type))

  covsens_res = list(PARSENS = out_cov_par_sens,
                     SUMPARSENS = t_cov_sens_par,
                     EXPSENS = out_cov_exp_sens)

  return(covsens_res)

}

## Author: Melnikova Alina
## First created: 2026-03-19
## Description: formal testing of sg_covsens_sim function and its helpers
## Keywords: SimuRg, sg-covsens-sim

# ---- Shared fixtures -------------------------------------------------------
# Defined at file scope so all test_that() blocks can reuse them.

.cont_cov_l <- list(
  LG_AGE = list(
    NAME     = "LG_AGE",
    UTNAME   = "AGE",
    REF      = "median",
    NICENAME = "Age, years",
    par_vec  = c("CL")
  ),
  LG_WEIGHT = list(
    NAME     = "LG_WEIGHT",
    UTNAME   = "WEIGHT",
    REF      = "median",
    NICENAME = "Weight, kg",
    par_vec  = c("Vd")
  )
)

.cat_cov_l <- list(
  SEX = list(
    NAME     = "SEX",
    NICENAME = "Sex",
    REF      = "0",
    par_vec  = c("ka")
  ),
  CYP2C9 = list(
    NAME     = "CYP2C9",
    NICENAME = "CYP2C9 genotype",
    REF      = NULL,
    par_vec  = c("CL")
  )
)

.ev_t_input <- tribble(
  ~id, ~time, ~ii, ~amt, ~addl, ~dur, ~evid, ~Regimen,         ~Dose,
  1,   0,     336, 10,   21,    0.5,  1,     "0.3 mg/kg Q2W",  0.3
)

.ss_cycle <- 10
.stimes_ss <- c(
  .ss_cycle * 4 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1)),
  .ss_cycle * 4 * 7 * 24 + 2 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1))
)

.mod_fin <- RxODE({
  ka_pop = 0.073;
  Vd_pop = 14.8;
  CL_pop = 0.347;

  omega_ka = 0;
  omega_Vd = 0;
  omega_CL = 0;

  beta_CL_LG_AGE     = 0.49990114;
  beta_Vd_LG_WEIGHT  = 0.60529433;

  beta_CL_CYP2C9_1_2 = -0.339;
  beta_CL_CYP2C9_1_3 = -0.574;
  beta_CL_CYP2C9_2_2 = -1.079;
  beta_CL_CYP2C9_2_3 = -0.745;
  beta_CL_CYP2C9_3_3 = -2.13;

  beta_ka_SEX_1 = -0.12198035;

  Cc_b = 0;

  ka_tv = exp(ka_pop);
  Vd_tv = exp(Vd_pop);
  CL_tv = exp(CL_pop);

  CL_multiplier = 1.0;
  ka_multiplier = 1.0;

  if (SEX == "1") {ka_multiplier = exp(beta_ka_SEX_1)}

  if (CYP2C9 == "1") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_2);
  } else if (CYP2C9 == "2") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_3);
  } else if (CYP2C9 == "3") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_2);
  } else if (CYP2C9 == "4") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_3);
  } else if (CYP2C9 == "5") {
    CL_multiplier = exp(beta_CL_CYP2C9_3_3);
  }

  ka = ka_tv * ka_multiplier * exp(omega_ka);
  Vd = Vd_tv * exp(beta_Vd_LG_WEIGHT * LG_WEIGHT + omega_Vd);
  CL = CL_tv * CL_multiplier * exp(beta_CL_LG_AGE * LG_AGE + omega_CL);

  Cc = Ac / Vd;

  Ad(0) = 0;
  Ac(0) = 0;

  d/dt(Ad) = -ka * Ad;
  d/dt(Ac) =  ka * Ad - CL * Cc;

  Cc_ResErr = Cc * (1 + Cc_b);
})

# Mock estimation covariance matrix built from parest (symmetric, positive-definite)
.pnames <- parest$parameter
.npar   <- length(.pnames)
set.seed(1)
.m_cov  <- matrix(0.02, .npar, .npar)
diag(.m_cov) <- 0.05 + runif(.npar, 0, 0.05)
.m_cov  <- (.m_cov + t(.m_cov)) / 2
.est_covmat <- as_tibble(cbind(X1 = .pnames, as.data.frame(.m_cov)))
names(.est_covmat)[-1] <- .pnames

# ---- Helper: run once with file mode (gfo4cov) and cache result ------------
.output_01 <- sg_covsens_sim(
  fpath_i    = gfo4cov,
  ds_parest  = NULL,
  ds_covs     = NULL,
  model      = .mod_fin,
  stimes  = .stimes_ss,
  et         = .ev_t_input,
  est_covmat = .est_covmat,
  npop       = 10,
  cont_cov_l = .cont_cov_l,
  cat_cov_l  = .cat_cov_l,
  quantiles  = c(0.2, 0.8),
  aggr       = c("max"),
  outputs    = "Cc"
)

# ---- Helper: run once with table mode (parest + ds_covval) -----------------
.output_02 <- sg_covsens_sim(
  fpath_i    = NULL,
  ds_parest  = parest,
  ds_covs     = ds_covval,
  model      = .mod_fin,
  stimes  = .stimes_ss,
  et         = .ev_t_input,
  est_covmat = .est_covmat,
  npop       = 10,
  cont_cov_l = .cont_cov_l,
  cat_cov_l  = .cat_cov_l,
  quantiles  = c(0.2, 0.8),
  aggr       = c("max"),
  outputs    = "Cc"
)


# ============================================================
# 1. Top-level output structure
# ============================================================

test_that("sg_covsens_sim returns a named list of length 3", {
  expect_type(.output_01, "list")
  expect_length(.output_01, 3)
  expect_named(.output_01, c("PARSENS", "SUMPARSENS", "EXPSENS"))
})

test_that("sg_covsens_sim all three list elements are data frames", {
  expect_true(is.data.frame(.output_01$PARSENS))
  expect_true(is.data.frame(.output_01$SUMPARSENS))
  expect_true(is.data.frame(.output_01$EXPSENS))
  expect_true(nrow(.output_01$PARSENS)   > 0)
  expect_true(nrow(.output_01$SUMPARSENS) > 0)
  expect_true(nrow(.output_01$EXPSENS)   > 0)
})


# ============================================================
# 2. PARSENS — parameter sensitivity
# ============================================================

test_that("PARSENS has required columns", {
  required_cols <- c("NICEN", "VAR", "LAB", "Type",
                     "mean", "median", "min", "max", "sd",
                     "P025", "P05", "P95", "P975")
  expect_true(all(required_cols %in% names(.output_01$PARSENS)))
})

test_that("PARSENS VAR contains expected model parameters", {
  vars <- unique(.output_01$PARSENS$VAR)
  expect_true("CL" %in% vars)
  expect_true("Vd" %in% vars)
  expect_true("ka" %in% vars)
})

test_that("PARSENS NICEN contains expected covariate display names", {
  nicen <- unique(.output_01$PARSENS$NICEN)
  expect_true("Age, years"        %in% nicen)
  expect_true("Weight, kg"        %in% nicen)
  expect_true("Sex"               %in% nicen)
  expect_true("CYP2C9 genotype"   %in% nicen)
})

test_that("PARSENS Type contains both Continuous and Categorical", {
  types <- unique(as.character(.output_01$PARSENS$Type))
  expect_true("Continuous"  %in% types)
  expect_true("Categorical" %in% types)
})

test_that("PARSENS LAB is a factor", {
  expect_true(is.factor(.output_01$PARSENS$LAB))
})

test_that("PARSENS Type is a factor", {
  expect_true(is.factor(.output_01$PARSENS$Type))
})

test_that("PARSENS summary statistics are numeric and non-negative", {
  stat_cols <- c("mean", "median", "P05", "P95", "P025", "P975")
  for (col in stat_cols) {
    vals <- .output_01$PARSENS[[col]]
    expect_true(is.numeric(vals),
                label = paste("PARSENS", col, "is numeric"))
    expect_false(all(is.na(vals)),
                 label = paste("PARSENS", col, "is not all NA"))
  }
})

test_that("PARSENS ratio values are centred around 1 (stored as ratio, not percent)", {
  # Values are transformed: ratio = pct_change/100 + 1, so reference = 1.
  # Non-reference rows should have values reasonably bounded away from zero.
  expect_true(all(.output_01$PARSENS$mean > 0, na.rm = TRUE))
  expect_true(all(.output_01$PARSENS$P05  > 0, na.rm = TRUE))
})

test_that("PARSENS continuous rows have KEY labels with percentile text", {
  cont_rows <- .output_01$PARSENS[.output_01$PARSENS$Type == "Continuous", ]
  key_labels <- unique(as.character(cont_rows$KEY))
  expect_true(any(grepl("perc\\.", key_labels)))
})

test_that("PARSENS BCOVVAL is present and numeric for continuous rows", {
  expect_true("BCOVVAL" %in% names(.output_01$PARSENS))
  cont_rows <- .output_01$PARSENS[.output_01$PARSENS$Type == "Continuous", ]
  expect_true(is.numeric(cont_rows$BCOVVAL) || is.character(cont_rows$BCOVVAL))
})

test_that("PARSENS CATDES is present for categorical rows", {
  expect_true("CATDES" %in% names(.output_01$PARSENS))
  cat_rows <- .output_01$PARSENS[.output_01$PARSENS$Type == "Categorical", ]
  expect_false(all(is.na(cat_rows$CATDES)))
})


# ============================================================
# 3. SUMPARSENS — compact summary table
# ============================================================

test_that("SUMPARSENS has required columns", {
  required_cols <- c("Parameter", "Covariate", "Cov. percentile",
                     "Cov. value", "Mean", "90%CI")
  expect_true(all(required_cols %in% names(.output_01$SUMPARSENS)))
})

test_that("SUMPARSENS Parameter contains expected model parameters", {
  params <- unique(.output_01$SUMPARSENS$Parameter)
  expect_true("CL" %in% params)
  expect_true("Vd" %in% params)
  expect_true("ka" %in% params)
})

test_that("SUMPARSENS 90%CI column is a character with comma-separated values", {
  ci_vals <- .output_01$SUMPARSENS$`90%CI`
  expect_true(is.character(ci_vals))
  expect_true(all(grepl(",", ci_vals)))
})

test_that("SUMPARSENS Mean is numeric", {
  expect_true(is.numeric(.output_01$SUMPARSENS$Mean))
})

test_that("SUMPARSENS row count matches PARSENS non-duplicated covariate-parameter combinations", {
  # Each unique NICEN x VAR x KEY/CATDES combination should yield one SUMPARSENS row
  expect_equal(nrow(.output_01$SUMPARSENS), nrow(.output_01$PARSENS))
})


# ============================================================
# 4. EXPSENS — exposure sensitivity
# ============================================================

test_that("EXPSENS has required columns", {
  required_cols <- c("NICEN", "VAR", "LAB", "Type",
                     "mean", "median", "P05", "P95")
  expect_true(all(required_cols %in% names(.output_01$EXPSENS)))
})

test_that("EXPSENS VAR reflects output and aggr (Cc + max -> Cc_Cmax)", {
  vars <- unique(.output_01$EXPSENS$VAR)
  expect_true("Cc_Cmax" %in% vars)
})

test_that("EXPSENS Type contains both Continuous and Categorical", {
  types <- unique(as.character(.output_01$EXPSENS$Type))
  expect_true("Continuous"  %in% types)
  expect_true("Categorical" %in% types)
})

test_that("EXPSENS summary statistics are numeric and positive", {
  expect_true(is.numeric(.output_01$EXPSENS$mean))
  expect_true(all(.output_01$EXPSENS$mean > 0, na.rm = TRUE))
})

test_that("EXPSENS LAB is a factor", {
  expect_true(is.factor(.output_01$EXPSENS$LAB))
})


# ============================================================
# 5. Table mode (ds_parest + ds_covs) produces consistent output
# ============================================================

test_that("sg_covsens_sim table mode returns a named list of length 3", {
  expect_type(.output_02, "list")
  expect_length(.output_02, 3)
  expect_named(.output_02, c("PARSENS", "SUMPARSENS", "EXPSENS"))
})

test_that("sg_covsens_sim table mode PARSENS has same columns as file mode", {
  expect_equal(sort(names(.output_02$PARSENS)), sort(names(.output_01$PARSENS)))
})

test_that("sg_covsens_sim table mode PARSENS contains same VAR set as file mode", {
  expect_equal(sort(unique(.output_02$PARSENS$VAR)),
               sort(unique(.output_01$PARSENS$VAR)))
})

test_that("sg_covsens_sim table mode EXPSENS VAR matches file mode", {
  expect_equal(sort(unique(.output_02$EXPSENS$VAR)),
               sort(unique(.output_01$EXPSENS$VAR)))
})


# ============================================================
# 6. Input validation — error handling
# ============================================================

test_that("sg_covsens_sim errors when no data source is provided", {
  expect_error(
    sg_covsens_sim(
      fpath_i    = NULL,
      ds_parest  = NULL,
      ds_covs     = NULL,
      model      = .mod_fin,
      stimes  = .stimes_ss,
      et         = .ev_t_input,
      est_covmat = .est_covmat,
      cont_cov_l = .cont_cov_l,
      cat_cov_l  = .cat_cov_l
    ),
    regexp = "No data source provided"
  )
})

test_that("sg_covsens_sim errors when fpath_i and ds_parest are both supplied", {
  expect_error(
    sg_covsens_sim(
      fpath_i    = gfo4cov,
      ds_parest  = parest,
      ds_covs     = NULL,
      model      = .mod_fin,
      stimes  = .stimes_ss,
      et         = .ev_t_input,
      est_covmat = .est_covmat,
      cont_cov_l = .cont_cov_l,
      cat_cov_l  = .cat_cov_l
    ),
    regexp = "mutually exclusive"
  )
})

test_that("sg_covsens_sim errors when ds_parest is provided without ds_covs", {
  expect_error(
    sg_covsens_sim(
      fpath_i    = NULL,
      ds_parest  = parest,
      ds_covs     = NULL,
      model      = .mod_fin,
      stimes  = .stimes_ss,
      et         = .ev_t_input,
      est_covmat = .est_covmat,
      cont_cov_l = .cont_cov_l,
      cat_cov_l  = .cat_cov_l
    ),
    regexp = "ds_covs"
  )
})

test_that("sg_covsens_sim errors when ds_covs is provided without ds_parest", {
  expect_error(
    sg_covsens_sim(
      fpath_i    = NULL,
      ds_parest  = NULL,
      ds_covs     = ds_covval,
      model      = .mod_fin,
      stimes  = .stimes_ss,
      et         = .ev_t_input,
      est_covmat = .est_covmat,
      cont_cov_l = .cont_cov_l,
      cat_cov_l  = .cat_cov_l
    ),
    regexp = "ds_parest"
  )
})


# ============================================================
# 7. aggr argument validation
# ============================================================

test_that("sg_covsens_sim warns on unrecognised aggr value", {
  expect_warning(
    sg_covsens_sim(
      fpath_i    = NULL,
      ds_parest  = parest,
      ds_covs     = ds_covval,
      model      = .mod_fin,
      stimes  = .stimes_ss,
      et         = .ev_t_input,
      est_covmat = .est_covmat,
      npop       = 5,
      cont_cov_l = .cont_cov_l,
      cat_cov_l  = .cat_cov_l,
      quantiles  = c(0.2, 0.8),
      aggr       = c("max", "invalid_metric"),
      outputs    = "Cc"
    ),
    regexp = "Unrecognised aggregation"
  )
})



# ============================================================
# 8. Multiple aggr and multiple outputs
# ============================================================

test_that("sg_covsens_sim EXPSENS VAR reflects multiple aggr metrics", {
  out <- sg_covsens_sim(
    fpath_i    = NULL,
    ds_parest  = parest,
    ds_covs     = ds_covval,
    model      = .mod_fin,
    stimes  = .stimes_ss,
    et         = .ev_t_input,
    est_covmat = .est_covmat,
    npop       = 5,
    cont_cov_l = .cont_cov_l,
    cat_cov_l  = .cat_cov_l,
    quantiles  = c(0.2, 0.8),
    aggr       = c("max", "mean"),
    outputs    = "Cc"
  )
  vars <- unique(out$EXPSENS$VAR)
  expect_true("Cc_Cmax" %in% vars)
  expect_true("Cc_Cavg" %in% vars)
})

## Author: Ugolkov Yaroslav
## First created: 2025-09-05
## Description: sg-gof-res-dist and its helper functions
## Keywords: SimuRg, sg-gof-res-dist, goodness-of-fit
#'
#' Plot distribution or QQ-plot of residuals from NONMEM/Simurg run
#'@description
#' This function visualizes residuals from estimation results
#' stored in a Simurg output object. The residuals can be plotted either as
#' histograms with overlaid normal density curves, or as QQ-plots to assess
#' normality.
#'
#' @inheritParams sg_dummy
#' @param res_type Character vector. One or several types of residuals to plot.
#'   Values must correspond to column name(s) in `smrg_obj$SDTAB`, e.g.
#'   `"RES"`, `"IWRES"`, `"IRES"`. If multiple residual types are provided,
#'   plots will be generated for each of them.
#' @param ndist Logical. If `TRUE`, overlays the corresponding normal density
#'   curve on the histogram (default: TRUE).

#'
#' @return A `ggplot` object.
#' @examples
#' # Plot the distribution of individual weighted residuals (IWRES)
#' # (default \code{DVID = 1}; pass \code{DVID} or a \code{DVNAME} string to select another endpoint)
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData",
#'                         package = "SimuRg")
#' sg_gof_res_dist(fpath_i = fpath_i, res_type = "IWRES")
#'
#' # QQ-plots for the same default endpoint
#' sg_gof_res_dist(fpath_i = fpath_i, res_type = "RES", plot_type = "QQ")
#'
#' # Several residual columns at once (still one \code{DVID} unless you change it)
#' sg_gof_res_dist(fpath_i = fpath_i, res_type = c("IWRES", "IRES"))
#' @import dplyr
#' @import tidyr
#' @import ggplot2
#' @importFrom scales pretty_breaks
#' @export


sg_gof_res_dist <- function(fpath_i, res_type = 'RES', DVID = 1, n_bins = 30,
                            ndist = TRUE, plot_type = 'DIST'){
  smrg_obj <- read_smrg_obj(fpath_i)
  if (is.null(smrg_obj$SDTAB)) {
    stop("sg_fit object must contain SDTAB component")
  }

  sdtab <- smrg_obj$SDTAB

  if (is.data.frame(sdtab) && nrow(sdtab) == 0) {
    stop("SDTAB is empty (no rows)")
  }
  if (is.list(sdtab) && length(sdtab) == 0) {
    stop("SDTAB is empty (no elements)")
  }
  if (is.data.frame(sdtab)) {
    sdtab_i <- sdtab
  } else if (is.list(sdtab)) {
    sdtab_i <- as.data.frame(do.call(rbind, sdtab))
  } else {
    stop("SDTAB must be a data frame or a list of data frames")
  }

  sdtab_i <- filter_sdtab_by_DVID(sdtab_i, DVID)

  # Check for required columns and convert to numeric
  required_cols <- c("DVID", res_type)
  required_cols <- unique(required_cols)
  missing_cols <- setdiff(required_cols, colnames(sdtab_i))
  if (length(missing_cols) > 0) {
    stop("SDTAB is missing required columns: ", paste(missing_cols, collapse = ", "))
  }

  # Convert columns to numeric (only if they exist)
  sdtab_i <- sdtab_i %>%
    mutate(
      DVID = if ("DVID" %in% colnames(.)) as.numeric(DVID) else NA_real_
    )

  # Convert residual columns to numeric
  for (res_col in res_type) {
    if (res_col %in% colnames(sdtab_i)) {
      sdtab_i <- sdtab_i %>% mutate(across(all_of(res_col), as.numeric))
    }
  }

  # Add optional columns if they exist
  if ("WRES" %in% colnames(sdtab_i) && !("WRES" %in% res_type)) {
    sdtab_i <- sdtab_i %>% mutate(WRES = as.numeric(WRES))
  }
  if ("IWRES" %in% colnames(sdtab_i) && !("IWRES" %in% res_type)) {
    sdtab_i <- sdtab_i %>% mutate(IWRES = as.numeric(IWRES))
  }
  if ("CWRES" %in% colnames(sdtab_i) && !("CWRES" %in% res_type)) {
    sdtab_i <- sdtab_i %>% mutate(CWRES = as.numeric(CWRES))
  }
  if ("RES" %in% colnames(sdtab_i) && !("RES" %in% res_type)) {
    sdtab_i <- sdtab_i %>% mutate(RES = as.numeric(RES))
  }
  if ("IRES" %in% colnames(sdtab_i) && !("IRES" %in% res_type)) {
    sdtab_i <- sdtab_i %>% mutate(IRES = as.numeric(IRES))
  }

  res_for_plot <- sdtab_i %>% select(DVID,all_of(res_type))
  res_for_plot2 <- res_for_plot %>%
    gather(key = "residual_type", value = "value", -DVID)

  p_i <- ggplot(data = res_for_plot2, aes(x = value, y = ..density..)) +
    geom_histogram(bins = n_bins, col = "grey25", fill = MSDcol[2]) +
    facet_wrap(DVID~residual_type, scales = "free") +
    scale_y_continuous(name = "Density", breaks = scales::pretty_breaks(7), expand = c(0, 0), limits = c(0, NA)) +
    scale_x_continuous(name = 'Residuals', breaks = scales::pretty_breaks(7), expand = c(0, 0))+
    theme(panel.grid.minor = element_blank()) +
    theme_bw()
  if (ndist) {

    # mean_val <- mean(res_for_plot[[res_type]], na.rm = TRUE)
    # sd_val <- sd(res_for_plot[[res_type]], na.rm = TRUE)
    #
    #
    # x_min <- mean_val - 4 * sd_val
    # x_max <- mean_val + 4 * sd_val
    # x_seq <- seq(x_min, x_max, length.out = 1000)
    # p_i <- p_i + annotate("line", x = x_seq, y = dnorm(x_seq, mean = mean_val, sd = sd_val), linewidth = 0.8, lty = "dashed")


    norm_params <- res_for_plot2 %>%
      group_by(DVID, residual_type) %>%
      summarise(
        mean_val = mean(value, na.rm = TRUE),
        sd_val = sd(value, na.rm = TRUE),
        min_val = min(value, na.rm = TRUE),
        max_val = max(value, na.rm = TRUE),
        .groups = "drop"
      )

    norm_curves <- norm_params %>%
      group_by(DVID, residual_type) %>%
      summarise(
        x = list(seq(min_val - 0.5 * sd_val, max_val + 0.5 * sd_val, length.out = 1000)),
        y = list(dnorm(x[[1]], mean = mean_val, sd = sd_val)),
        .groups = "drop"
      ) %>%
      unnest(c(x, y))

    p_i <- p_i +  geom_line(data = norm_curves, aes(x = x, y = y),
                            color = "black", linewidth = 0.8, linetype = "dashed")
  }

  if (plot_type == 'QQ') {

    # Создаем базовый QQ-plot
    p_i <- ggplot(res_for_plot2, aes(sample = value)) +
      stat_qq(size = 1.75, color = MSDcol[1], alpha = 0.8) +
      stat_qq_line(col = "firebrick") +
      labs(x = "Theoretical quantiles", y = "Sample quantiles") +
      geom_abline(linewidth = 0.5, col = "black", linetype = "dashed") +
      scale_x_continuous(breaks = scales::pretty_breaks(7)) +
      scale_y_continuous(breaks = scales::pretty_breaks(7)) +
      theme(panel.grid.minor = element_blank()) +
      theme_bw()+
      facet_wrap(DVID ~ residual_type, scales = "free")
  }
  return(p_i)



}

#' Build multistart scenarios with varying initial values
#'
#' @description
#' Generates multiple scenarios with identical model structure
#' (model, RUV, RE, OCC, COVS) and different sampled initial values for
#' population parameters.
#'
#' For each start, initial values of estimated parameters are sampled
#' uniformly within specified intervals, and a corresponding `.mlxtran` file
#' is generated. A summary table describing all multistart scenarios is written
#' to disk.
#'
#' @param mod Model file path or model identifier used for scenario generation.
#' @param data Character string. Path to the dataset used in model fitting or simulation.
#' @param headers Predefined list of dataset column names (e.g., ID, TIME, DV, etc.) used by the modeling framework.
#' @param ruv Residual unexplained variability (RUV) structure.
#'   Each element contains RUV properties (e.g., `YNAME`, `ERR`) and mapping to data.
#' @param theta Tibble describing population parameter properties.
#'   Tibble should contain columns such as `NAME`, `INIT`, and `EST`.
#' @param re Random effects (RE) specification.
#'   Includes matrices `init` and `est` defining initial and estimated covariance structures.
#' @param occ Occasion effect matrices.
#'   Includes `init` and `est` matrices defining inter-occasion variability.
#' @param covs_lst Optional list describing covariate-parameter relationships.
#'   Each element should include fields such as `PAR`, `COVNAME`, and `EST` indicating inclusion in the model.
#' @param n_starts Number of multistart runs (number of initial value scenarios).
#' @param theta_intervals Optional data frame specifying sampling
#'   intervals for initial values. Must contain columns
#'   `NAME`, `lower`, and `upper`.
#'   If `NULL`, intervals are derived from `interval_factor`.
#' @param interval_factor Numeric vector of length 2 specifying
#'   multiplicative lower and upper bounds factors relative to original
#'   `INIT` values (default `c(0.2, 5)`).
#' @param vary_params List of parameters which inital values are varied.
#' Default value implies all parameters are varied. If `theta_interval` is specified,
#' list of parameters is taken from `theta_interval`
#' @param opt_name Character. Optimization engine name (e.g., `"Monolix"`, `"Simurg"`). Default `"Simurg"`.
#' @param path Character. Directory path where output files (CSV summary and model files) will be written. Default is current working directory.
#' @param project_name Character. Base name for the exported project files. Default `"my_project"`.
#' @param seed Optional random seed for reproducible sampling of
#'   initial values. If `NULL`, the global RNG state is not modified.
#'
#' @return
#' The function writes two types of outputs to the specified `path`:
#' \itemize{
#'   \item A CSV file (`scenarios_info.csv`) summarizing all generated scenarios with columns:
#'     \describe{
#'       \item{scenario_number}{Unique index of the scenario.}
#'       \item{model}{Model file or path used in the scenario.}
#'       \item{data}{Path to the dataset used.}
#'       \item{theta}{Active population parameters and initial values used.}
#'       \item{RUV}{Residual error structure(s) used.}
#'       \item{RE}{Random effect and correlation terms included.}
#'       \item{OCC}{Active occasion effects.}
#'       \item{COVS}{Included covariate-parameter relationships.}
#'     }
#'   \item Individual `.mlxtran` model files for each generated scenario,
#'   named according to the pattern `<project_name>_<i>.mlxtran`.
#' }
#'
#' @details
#' For each start:
#' \itemize{
#'   \item Population parameter initial values `INIT` for
#'   estimated parameters are sampled uniformly.
#'   \item A scenario summary is stored.
#'   \item The corresponding `.mlxtran` project files are written.
#' }
#'
#' If `seed` is provided, the function temporarily sets the
#' random number generator state and restores it upon exit.
#'
#' @examples
#' \donttest{
#' library(dplyr)
#' folder_path <- system.file("extdata", package = "SimuRg")
#'
#' mod <- paste(folder_path, "/models/model_PK_1c.txt", sep = "/")
#'
#' data <- paste(folder_path, "datasets", "dspk-warf.csv", sep = "/")
#' re <- list(init = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
#'                                     1, 0, 0, 0, 0,
#'                                     0, 1, 0, 0, 0,
#'                                     0, 0, 1, 0, 0,
#'                                     0, 0, 0, 1, 0,
#'                                     0, 0, 0, 0, 1) %>% as.matrix(),
#'                     est = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
#'                                     TRUE, NA, NA, NA, NA,
#'                                     NA, TRUE, NA, NA, NA,
#'                                     NA, NA, TRUE, NA, NA,
#'                                     NA, NA, NA, TRUE, NA,
#'                                     NA, NA, NA, NA, TRUE) %>% as.matrix(),
#'                     block = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
#'                                       FALSE, NA, NA, NA, NA,
#'                                       NA, FALSE, NA, NA, NA,
#'                                       NA, NA, TRUE, NA, NA,
#'                                       NA, NA, NA, FALSE, NA,
#'                                       NA, NA, NA, NA, FALSE) %>% as.matrix())
#'
#' headers <- list(list(name = "ID", use = "identifier", type = NULL),
#'                list(name = "TIME", use = "time", type = NULL),
#'                list(name = "DV", use = "observation", type = "continuous"),
#'                list(name = "DVID", use = "observationtype", type = NULL),
#'                list(name = "ADM", use = "administration", type = NULL),
#'                list(name = "AMT", use = "amount", type = NULL),
#'                list(name = "EVID", use = "eventidentifier", type = NULL),
#'                list(name = "MDV", use = "missingdependentvariable", type = NULL),
#'                list(name = "AGE", use = "covariate", type = "continuous"),
#'                list(name = "AGE_centered", use = "covariate", type = "continuous"),
#'                list(name = "SEX", use = "covariate", type = "categorical"),
#'                list(name = "WEIGHT", use = "covariate", type = "continuous"),
#'                list(name = "BMI", use = "covariate", type = "continuous"))
#'
#' theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
#'                    "Cl", "logNormal", 0.2, NA, NA, TRUE,
#'                    "Vd", "logNormal", 20, NA, NA, TRUE,
#'                    "ka", "logNormal", 0.2, NA, NA, TRUE)
#'
#' theta_intervals <- list(
#'   Cl = c(0.25*theta$INIT[theta$NAME == "Cl"], 4*theta$INIT[theta$NAME == "Cl"]),
#'   #Vd = c(0.5*theta$INIT[theta$NAME == "Vd"], 2*theta$INIT[theta$NAME == "Vd"]),
#'   ka   = c(1.25*theta$INIT[theta$NAME == "ka"], 1.5*theta$INIT[theta$NAME == "ka"])
#' )
#'
#' ruv <- list(YNAME = "y1",
#'             DVID = 1,
#'             TRANS = "normal",
#'             PRED = "Cc",
#'             ERR = "proportional",
#'             INIT = 1,
#'             EST = TRUE)
#'
#' n_starts <- 20
#'
#' path <- tempdir()
#' sg_multistart(
#'   mod = mod,
#'   data = data,
#'   headers = headers,
#'   ruv = ruv,
#'   theta = theta,
#'   re = re,
#'   occ = re,
#'   n_starts = n_starts,
#'   theta_intervals = theta_intervals,
#'   path = path,
#'   project_name = "multistart_test_project"
#' )
#' clr_files <- list.files(path, full.names = TRUE)
#' unlink(clr_files, recursive = TRUE, force = TRUE)
#' }
#'
#' @import dplyr
#' @importFrom readr write_csv
#' @export
#' @importFrom jsonlite fromJSON
#' @importFrom stats runif
#' @import dplyr
#' @export

sg_multistart <- function(mod, data, headers, ruv, theta, re, occ, path,
                          covs_lst=NULL, opt_name = "Simurg",
                          project_name="multistart_project",
                          n_starts=10, theta_intervals=NULL,
                          interval_factor=c(0.2,5), vary_params=NULL,
                          seed = NULL) {
  path <- gsub("[/\\\\]+$", "", path)

  make_theta_one <- function(theta_lst, val, trans_idx, init_idx) {
    for (i in c(1:length(trans_idx))) {
      theta_lst$TRANS[trans_idx[i]] <- val[i]
    }
    for (i in c(1:length(init_idx))) {
      theta_lst$INIT[init_idx[i]] <- val[i + length(trans_idx)]
    }
    return(theta_lst %>% mutate(TRANS = as.character(TRANS),
                                INIT = as.numeric(INIT)))
  }
  make_theta_lst <- function(theta_lst) {
    theta_lst_ <- theta_lst %>% group_by(NAME) %>%
      mutate(TRANS_ln = length(TRANS %>% unlist()),
             INIT_ln = length(INIT %>% unlist()))
    trans_idx <- which(theta_lst_$TRANS_ln > 1)
    init_idx <- which(theta_lst_$INIT_ln > 1)
    if (length(trans_idx) + length(init_idx) == 0) {
      return(list(theta_lst))
    }
    trans_val <- expand.grid(c(theta_lst$TRANS[trans_idx],
                               theta_lst$INIT[init_idx]),
                             stringsAsFactors = FALSE)
    return(apply(trans_val, 1,function(x){make_theta_one(theta_lst, x, trans_idx, init_idx)}))
  }

  # Setup variation parameters for multistart
  if (is.null(vary_params)) {
    vary_params <- theta$NAME
  }
  if (is.null(theta_intervals)) {
    theta_intervals <- list()
    for (p in vary_params) {
      init_val <- theta$INIT[theta$NAME == p]
      theta_intervals[[p]] <- c(init_val * interval_factor[1], init_val * interval_factor[2])
    }
  }

  # Sample n_starts initial theta values
  theta_samples <- list()
  for (p in names(theta_intervals)) {
    bounds <- theta_intervals[[p]]
    if (is.null(seed)) {
      theta_samples[[p]] <- runif(n_starts, bounds[1], bounds[2])
    } else {
      withr::with_seed(seed, {
        theta_samples[[p]] <- runif(n_starts, bounds[1], bounds[2])
      })
    }
  }
  # Create list of theta tables with varied INIT for chosen parameters
  theta_list <- vector("list", n_starts)
  for (i in seq_len(n_starts)) {
    th <- theta
    for (p in names(theta_samples)) {
      th$INIT[th$NAME == p] <- theta_samples[[p]][i]
    }
    theta_list[[i]] <- th
  }
  # Expand if any transformation or list values
  theta_lst_exp <- lapply(theta_list, function(x) make_theta_lst(x)[[1]])

  summary_list <- vector("list", n_starts)
  for (i in seq_len(n_starts)) {
    model_path <- mod
    # Theta info string of EST parameters
    theta_true <- theta_lst_exp[[i]] %>%
      mutate(NAME_INIT = paste0(NAME,'=',INIT)) %>%
      filter(EST == TRUE) %>%
      pull(NAME_INIT) %>%
      str_c(collapse = ", ")
    # RUV info
    ruv_info <- ruv
    ruv_full <- paste0(ruv_info$YNAME, "_", ruv_info$ERR)
    # RE info
    re_mat <- re$init
    re_names <- colnames(re_mat)
    diag_idx <- which(diag(re_mat) == 1)
    re_active <- if (length(diag_idx) > 0) re_names[diag_idx] else character(0)
    corr_list <- c()
    for (d_idx in diag_idx) {
      diag_name <- re_names[d_idx]
      for (c in seq_len(ncol(re_mat))) {
        if (c != d_idx && re_mat[d_idx, c] == 1) {
          corr_list <- c(corr_list, paste0("corr_", diag_name, "_", re_names[c]))
        }
      }
      for (r in seq_len(nrow(re_mat))) {
        if (r != d_idx && re_mat[r, d_idx] == 1) {
          corr_list <- c(corr_list, paste0("corr_", diag_name, "_", re_names[r]))
        }
      }
    }
    re_full <- paste(unique(c(re_active, corr_list)), collapse = ", ")
    # OCC info
    occ_mat <- occ$init
    occ_active <- colnames(occ_mat)[colSums(occ_mat == 1, na.rm = TRUE) > 0] %>%
      str_c(collapse = ", ")
    # COVS info
    covs_active <- covs_lst %>%
      keep(~ .x$EST == TRUE) %>%
      map_chr(~ str_c(.x$PAR, "_", .x$COVNAME)) %>%
      str_c(collapse = ", ")
    summary_list[[i]] <- tibble(
      scenario_number = i,
      model = model_path,
      data = data,
      theta = theta_true,
      RUV = ruv_full,
      RE = re_full,
      OCC = occ_active,
      COVS = covs_active
    )
  }
  summary_df <- bind_rows(summary_list)
  write_csv(summary_df, file.path(path, 'multistart_info.csv'))

  for (i in seq_len(n_starts)) {
    sg_result_mod <- sg_fit(mod, data, headers, theta_lst_exp[[i]],
                            ruv, re, occ,
                            project_name = paste0(project_name,'_',i),
                            covs_lst, opt_name = "Monolix",
                            path_to_save_output = path)
    write(sg_result_mod, file.path(path, paste0(project_name,'_',i,'.mlxtran')))
  }
}

test_that("sg_multistart creates expected number of files", {
  # str_c("V:/Collaborative_working/SimuRg_as_R_lib/SimuRg/scripts/nlme/1.4-sg-modbuild/")
  folder_path <- system.file("extdata", package = "SimuRg")

  mod <- paste(folder_path, "/models/model_PK_1c.txt", sep = "/")
  ### path to the dataset
  data <- paste(folder_path, "datasets", "dspk-warf.csv", sep = "/")

  re <- list(init = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                            1, 0, 0, 0, 0,
                            0, 1, 0, 0, 0,
                            0, 0, 1, 0, 0,
                            0, 0, 0, 1, 0,
                            0, 0, 0, 0, 1) %>% as.matrix(),
             est = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                           T, NA, NA, NA, NA,
                           NA, T, NA, NA, NA,
                           NA, NA, T, NA, NA,
                           NA, NA, NA, T, NA,
                           NA, NA, NA, NA, T) %>% as.matrix(),
             block =tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                            F, NA, NA, NA, NA,
                            NA, F, NA, NA, NA,
                            NA, NA, T, NA, NA,
                            NA, NA, NA, F, NA,
                            NA, NA, NA, NA, F) %>% as.matrix())

  headers <- list(list(name = "ID", use = "identifier", type = NULL),
                  list(name = "TIME", use = "time", type = NULL),
                  list(name = "DV", use = "observation", type = "continuous"),
                  list(name = "DVID", use = "observationtype", type = NULL),
                  list(name = "ADM", use = "administration", type = NULL),
                  list(name = "AMT", use = "amount", type = NULL),
                  list(name = "EVID", use = "eventidentifier", type = NULL),
                  list(name = "MDV", use = "missingdependentvariable", type = NULL),
                  list(name = "AGE", use = "covariate", type = "continuous"),
                  list(name = "AGE_centered", use = "covariate", type = "continuous"),
                  list(name = "SEX", use = "covariate", type = "categorical"),
                  list(name = "WEIGHT", use = "covariate", type = "continuous"),
                  list(name = "BMI", use = "covariate", type = "continuous"))

  theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
                   "Cl", "logNormal", 0.2, NA, NA, T,
                   "Vd", "logNormal", 20, NA, NA, T,
                   "ka", "logNormal", 0.2, NA, NA, T)

  theta_intervals <- list(
    Cl = c(0.25*theta$INIT[theta$NAME == "Cl"], 4*theta$INIT[theta$NAME == "Cl"]),
    #Vd = c(0.5*theta$INIT[theta$NAME == "Vd"], 2*theta$INIT[theta$NAME == "Vd"]),
    ka   = c(1.25*theta$INIT[theta$NAME == "ka"], 1.5*theta$INIT[theta$NAME == "ka"])
  )

  path <- tempdir()

  ruv <- list(YNAME = "y1",
              DVID = 1,
              TRANS = "normal",
              PRED = "Cc",
              ERR = "proportional",
              INIT = 1,
              EST = T)

  n_starts <- 20

  sg_multistart(
    mod = mod,
    data = data,
    headers = headers,
    ruv = ruv,
    theta = theta,
    re = re,
    occ = re,
    n_starts = n_starts,
    theta_intervals = theta_intervals,
    covs_lst = NULL,
    path = path,
    project_name = "multistart_test_project",
    seed = 126
  )

  # Проверяем что создался CSV файл с информацией о сценариях
  expect_true(file.exists(paste0(path, "/multistart_info.csv")))

  # Проверяем наличие первого файла
  mlxtran_files <- list.files(path, pattern = "\\.mlxtran$")
  expect_true("multistart_test_project_1.mlxtran" %in% mlxtran_files)

  # Проверяем что CSV файл не пустой
  csv_content <- read_csv(paste0(path, "/multistart_info.csv"))
  # expect_gt(nrow(csv_content), 0)
  expect_equal(nrow(csv_content), n_starts)
  # Clear tempdir
  clr_files <- list.files(path, full.names = TRUE)
  unlink(clr_files, recursive = TRUE, force = TRUE)


})

## Author: Mikhailova Anna
## First created: 2025-08-25
## Description: functions for common internal data structures generation
## Keywords: SimuRg, internal data

MSDcol <- c("#1a1866", "#f2b93b", "#b73b58", "#a2d620", "#14D98E", "#9c4ec7",
            "#3a6eba", "#efdd3c", "#69686d", '#844538', '#D91477', '#F3A9FF')

funSum_sim <- list(mean   = ~mean(., na.rm = TRUE),
                   median = ~median(., na.rm = TRUE),
                   min    = ~min(., na.rm = TRUE),
                   max    = ~max(., na.rm = TRUE),
                   sd     = ~sd(., na.rm = TRUE),
                   P025   = ~quantile(., 0.025, na.rm = TRUE),
                   P05    = ~quantile(., 0.05, na.rm = TRUE),
                   P10    = ~quantile(., 0.10, na.rm = TRUE),
                   P15   = ~quantile(., 0.15, na.rm = TRUE),
                   P25    = ~quantile(., 0.25, na.rm = TRUE),
                   P75    = ~quantile(., 0.75, na.rm = TRUE),
                   P85   = ~quantile(., 0.85, na.rm = TRUE),
                   P90    = ~quantile(., 0.90, na.rm = TRUE),
                   P95    = ~quantile(., 0.95, na.rm = TRUE),
                   P975   = ~quantile(., 0.975, na.rm = TRUE),
                   geom_mean = ~exp(mean(log(.[. > 0]), na.rm = TRUE)),
                   CV     = ~sd(., na.rm = TRUE)/mean(., na.rm = TRUE)*100)

#' Warfarin PKPD dataset
#'
#' Generated dataset with warfarin PK/PD measurements and covariates for 100 patients.
#'
#' @format ## `warfarin`
#' A data frame with 1700 rows and 17 columns:
#' \describe{
#'   \item{ID}{identifier of the individual}
#'   \item{TIME}{time of the dose or observation record (nominal)}
#'   \item{DV}{records the measurement data}
#'   \item{DVID}{identifier for the observation type (to distinguish different types of observations, e.g PK and PD)}
#'   \item{DVNAME}{name for the observation type (to distinguish different types of observations, e.g PK and PD)}
#'   \item{CMT}{compartment of the event}
#'   \item{ADM}{identifier for the type of dose}
#'   \item{AMT}{dose amount}
#'   \item{EVID}{identifier to indicate if the line is a dose-line or a response-line}
#'   \item{MDV}{identifier to ignore the observation information of that line}
#'   \item{AGE}{age of the individual}
#'   \item{SEX}{sex of the individual}
#'   \item{WEIGHT}{weight of the individual}
#'   \item{BMI}{BMI of the individual}
#'   \item{CLCR}{creatinine clearance of the individual}
#'   \item{CYP2C9_gentyp}{CYP2C9 genotype of the individual}
#'   \item{VKORC1_gentyp}{VKORC1 genotype of the individual}
#' }
"warfarin"

#' Primary Biliary Cirrhosis (PBC) dataset
#'
#' Clinical trial dataset with Primary Biliary Cirrhosis patient data including survival outcomes and covariates.
#'
#' @format ## `data_pbc`
#' A data frame with 280 rows and 15 columns:
#' \describe{
#'   \item{id}{identifier of the individual}
#'   \item{years}{time in years (survival time)}
#'   \item{status2}{survival status (0 = censored, 1 = event)}
#'   \item{drug}{treatment group ("D-penicil" or "placebo")}
#'   \item{age}{age of the individual in years}
#'   \item{sex}{sex of the individual ("male" or "female")}
#'   \item{ascites}{presence of ascites ("Yes" or "No")}
#'   \item{hepatomegaly}{presence of hepatomegaly ("Yes" or "No")}
#'   \item{spiders}{presence of spider angiomas ("Yes" or "No")}
#'   \item{edema}{edema status ("No edema", "edema no diuretics", or "edema despite diuretics")}
#'   \item{serBilir}{serum bilirubin level (mg/dl)}
#'   \item{serChol}{serum cholesterol level (mg/dl)}
#'   \item{albumin}{albumin level (g/dl)}
#'   \item{platelets}{platelet count (per cubic mm/1000)}
#' }
"data_pbc"

#' Warfarin population PK parameter estimates
#'
#' Final parameter estimates from a warfarin population pharmacokinetic model,
#' including typical values, covariate effects, random effects, and residual
#' error parameters with associated uncertainty measures.
#'
#' @format ## `parest`
#' A data frame with 15 rows and 12 columns:
#' \describe{
#'   \item{parameter}{parameter name as used in the model}
#'   \item{value}{final parameter estimate}
#'   \item{TYPE}{parameter category:`"Typical values"`, `"Covariate effects"`, `"Random effects"`, or `"Residual error model"`}
#'   \item{EST}{estimation status (`"ESTIMATED"`)}
#'   \item{SE}{standard error of the estimate}
#'   \item{RSE}{relative standard error (%)}
#'   \item{LCI}{lower bound of the 95 % confidence interval}
#'   \item{UCI}{upper bound of the 95 % confidence interval}
#'   \item{ETAshrinkage_var}{ETA shrinkage on the variance scale (%); `NA` for non-random-effect parameters}
#'   \item{ETAshrinkage_sd}{ETA shrinkage on the SD scale (%); `NA` for non-random-effect parameters}
#'   \item{EPSshrinkage_sd}{EPS shrinkage on the SD scale (%); `NA` for non-residual-error parameters}
#' }
#'@source Derived from \code{par_fin_i.csv} — warfarin PopPK model estimation output.
"parest"

#' Warfarin covariate values dataset
#'
#' Individual covariate values for 100 patients used in the warfarin
#' covariate sensitivity simulation, including demographic and pharmacogenomic variables.
#'
#' @format ## `ds_covval`
#' A data frame with 100 rows and 9 columns:
#' \describe{
#'   \item{ID}{individual patient identifier}
#'   \item{AGE}{age in years}
#'   \item{WEIGHT}{body weight in kg}
#'   \item{BMI}{body mass index (kg/m\ifelse{html}{\out{<sup>2</sup>}}{\eqn{^2}})}
#'   \item{LG_WEIGHT}{log-transformed body weight (log10 scale), used as continuous covariate in the PK model}
#'   \item{LG_AGE}{log-transformed age (log10 scale), used as continuous covariate in the PK model}
#'   \item{SEX}{sex of the individual (0 = female, 1 = male)}
#'   \item{CYP2C9}{CYP2C9 genotype encoded as integer: 0 = \emph{*1/*1} (reference), 1 = \emph{*1/*2}, 2 = \emph{*1/*3}, 3 = \emph{*2/*2}, 4 = \emph{*2/*3}, 5 = \emph{*3/*3}}
#'   \item{VKORC1}{VKORC1 genotype (\code{"GG"}, \code{"AG"}, or \code{"AA"})}
#' }
#'@source Derived from \code{data_fin_i.csv} — individual covariate dataset for the warfarin PopPK covariate sensitivity analysis.
"ds_covval"

#' Warfarin PopPK model fitting output (4-covariate model)
#'
#' A list containing the full output of a warfarin population pharmacokinetic
#' model fit with four covariates (SEX, LG_WEIGHT, CYP2C9, VKORC1), including
#' parameter estimates, model diagnostics, individual predictions, and covariate data.
#'
#' @format ## `gfo4cov`
#' A named list with 12 elements:
#' \describe{
#'   \item{PROJNAME}{character string identifying the project run (\code{"run_4cov"})}
#'   \item{SUMTAB}{data frame with 15 rows and 11 columns containing the parameter
#'     summary table. Columns: \code{PAR} (parameter name), \code{VALUE} (estimate),
#'     \code{TYPE} (category: \code{"Typical values"}, \code{"Covariate effects"},
#'     \code{"Random effects"}, or \code{"Residual error model"}), \code{EST}
#'     (estimation status), \code{SE} (standard error), \code{RSE} (relative SE, \%),
#'     \code{LCI} and \code{UCI} (95\% confidence interval bounds),
#'     \code{ETAshrinkage_var}, \code{ETAshrinkage_sd}, \code{EPSshrinkage_sd} (shrinkage metrics).}
#'   \item{SDTAB}{data frame with 1600 rows and 11 columns — the standard table of
#'     observations and model predictions. Columns: \code{ID}, \code{TIME}, \code{DV}
#'     (observed), \code{DVID}, \code{PRED} (population prediction), \code{IPRED}
#'     (individual prediction), \code{RES} (residual), \code{IRES} (individual residual),
#'     \code{WRES} (weighted residual), \code{IWRES} (individual weighted residual), \code{MDV}.}
#'   \item{PATAB}{data frame with 100 rows and 7 columns — individual parameter table.
#'     Columns: \code{ID}, \code{eta_ka}, \code{eta_Vd}, \code{eta_CL}
#'     (individual ETA random effects), \code{ka}, \code{Vd}, \code{CL}
#'     (individual post-hoc PK parameter estimates).}
#'   \item{COVMAT}{15 \eqn{\times} 15 numeric matrix — covariance matrix of the
#'     population parameter estimates.}
#'   \item{CORRMAT}{15 \eqn{\times} 15 numeric matrix — correlation matrix of the
#'     population parameter estimates.}
#'   \item{OFV}{data frame with 1 row and 4 columns — model fit criteria:
#'     \code{LL} (log-likelihood), \code{AIC}, \code{BIC}, \code{BICc}.}
#'   \item{OMEGAMAT}{3 \eqn{\times} 3 numeric matrix — OMEGA variance-covariance
#'     matrix of the inter-individual random effects (\code{eta_ka}, \code{eta_Vd}, \code{eta_CL}).}
#'   \item{SIGMAMAT}{1 \eqn{\times} 1 numeric matrix — SIGMA residual error
#'     variance matrix.}
#'   \item{EVTAB}{data frame with 100 rows and 6 columns — dosing/event table.
#'     Columns: \code{ID}, \code{TIME}, \code{EVID}, \code{AMT}, \code{ADM}, \code{CMT}.}
#'   \item{COTAB}{data frame with 100 rows and 6 columns — continuous covariate table.
#'     Columns: \code{ID}, \code{AGE}, \code{WEIGHT}, \code{BMI},
#'     \code{LG_WEIGHT} (log10-transformed weight), \code{LG_AGE} (log10-transformed age).}
#'   \item{CATAB}{data frame with 100 rows and 4 columns — categorical covariate table.
#'     Columns: \code{ID}, \code{SEX} (0 = female, 1 = male),
#'     \code{CYP2C9} (genotype integer code, 0–5),
#'     \code{VKORC1} (genotype: \code{"GG"}, \code{"AG"}, or \code{"AA"}).}
#' }
#' @source Generated from the warfarin PopPK model fitting run \code{"run_4cov"} used
#' in the covariate sensitivity analysis workflow.
"gfo4cov"

## Author: Ugolkov Yaroslav
## First created: 2025-09-12
## Description: Local sensitivity analysis simulations for RxODE models
## Keywords: SimuRg, local sensitivity

#' Perform local sensitivity analysis simulations
#'
#' @description
#' Generates simulation datasets for a model varying each specified parameter
#' within provided bounds or relative percentage. Supports multiple outputs and covariates.
#'
#' @param model An RxODE model object.
#' @param params Character vector of parameter names to vary in the analysis.
#' @param stimes Numeric vector of time points for the simulation.
#' @param output Character vector of model outputs (variables) to keep. Default is NULL (all outputs).
#' @param perc Numeric. If lb/ub not provided, defines ±percentage range around parameter base value. Default 0.2.
#' @param lb Numeric vector of lower bounds for each parameter. Length must match `params`. Default NULL.
#' @param ub Numeric vector of upper bounds for each parameter. Length must match `params`. Default NULL.
#' @param cov Character vector of covariate names to include in the simulation. Default NULL.
#' @param et Event table for the simulation.
#' @param theta Named numeric vector of baseline parameters. Default NULL.
#' @param n_sim Integer. Number of points to simulate between LB and UB for each parameter. Default 10.
#'
#' @return A tibble containing the simulated results with the following columns:
#' \describe{
#'   \item{NPOP}{Optional. Population identification number. From simulation.}
#'   \item{ID}{Optional. Individual subject ID. From simulation.}
#'   \item{TIME}{Time points of simulation.}
#'   \item{VAR}{Name of the output variable (model compartment or biomarker).}
#'   \item{VALUE}{Simulated value of the variable.}
#'   \item{mean, median, min, max, sd, etc.}{Optional. Aggregated statistics, if aggregation applied.}
#'   \item{COV1,...COVn}{Optional. Covariate values, if `cov` provided.}
#'   \item{PARNAME}{Name of the parameter being varied in this simulation.}
#'   \item{PARVAL}{Value of the parameter used in this simulation.}
#'   \item{PARVAL_NORM}{Normalized parameter value between 0 and 1 relative to LB and UB. Useful for plotting color gradients.}
#' }
#'
#' @examples
#' \donttest{
#' library(rxode2)
#' library(tibble)
#' mod_ex <- RxODE({
#'   # Doses in mg
#'   # Time in hours
#'   ### Parameter values
#'   # Typical
#'   POPCL = 5;
#'   POPVC = 180;
#'   POPQ = 7;
#'   POPVP = 52;
#'   POPKTR = 6;
#'   FBIOPAR = 1;

#'   # Covariate coefficients
#'   POPIGFRCOV = 0.8;
#'   POPPATCOVCLGR1 = 0.85;
#'   POPPATCOVCLGR2 = 0.85;
#'   POPPATCOVCLGR3 = 0.85;
#'   POPPATCOVCLGR4 = 0.85;

#'   # Random effects
#'   PPVCL = 0;
#'   PPVVC = 0;
#'   PPVKTR = 0;

#'   # Residual error
#'   RUV = 0;

#'   ### Covariates
#'   PATCOVCL = 1
#'   if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
#'   if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
#'   if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
#'   if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
#'   IGFRCOV = (IGFR/112)^POPIGFRCOV

#'   ## Parameters
#'   CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
#'   VC = POPVC * exp(PPVVC);
#'   Q = POPQ;
#'   VP = POPVP;
#'   KTR = POPKTR * exp(PPVKTR);

#'   ### Explicit functions
#'   Cc = Ac/VC;                 # nmol/L
#'   Cp = Ap/VP;                 # nmol/L

#'   ### Initial conditions
#'   At1(0) = 0;         # mg
#'   At2(0) = 0;         # mg
#'   At3(0) = 0;         # mg
#'   At4(0) = 0;         # mg
#'   At5(0) = 0;         # mg
#'   At6(0) = 0;         # mg
#'   Ad(0) = 0;          # mg
#'   Ac(0) = 0;          # mg
#'   Ap(0) = 0;          # mg

#'   ### ODEs
#'   d/dt(At1) = - KTR*At1;
#'   d/dt(At2) = KTR*At1 - KTR*At2;
#'   d/dt(At3) = KTR*At2 - KTR*At3;
#'   d/dt(At4) = KTR*At3 - KTR*At4;
#'   d/dt(At5) = KTR*At4 - KTR*At5;
#'   d/dt(At6) = KTR*At5 - KTR*At6;
#'   d/dt(Ad) = KTR*At6 - KTR*Ad;
#'   d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
#'   d/dt(Ap) = Q*(Cc - Cp);

#'   FBIO = FBIOPAR
#'   f(At1) = FBIO*1000000/505;
#'   CHECKRUV = RUV;
#'   Cc_ResErr = Cc + RUV*Cc;
#' })
#' et_base <- tribble(
#'   ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
#'   1,   0,     1,     1,    10,   2,     24,  112,   1
#' )
#' sens_loc <- sg_localsens_sim(
#'   model = mod_ex,
#'   params = c("POPCL", "POPVC"),
#'   stimes = seq(0, 168, 0.1),
#'   output = "Cc",
#'   perc = 0.9,
#'   cov = c("IGFR", "POPN"),
#'   et = et_base
#' )
#' }
#'
#' @import dplyr
#' @import purrr
#' @export


sg_localsens_sim <- function(model, params, stimes, output = NULL,
                             perc = 0.2, lb = NULL, ub = NULL, cov = NULL, et, theta = NULL, n_sim = 10) {

  inits <- rxInits(model)
  params_df <- data.frame(
    Parameter = names(inits),
    Value = inits,
    row.names = NULL
  )

  if(!is.null(lb) & length(lb) != length(params)) stop("The length of lb must match the number of params.")
  if(!is.null(ub) & length(ub) != length(params)) stop("The length of ub must match the number of params.")

  par_bounds <- tibble(
    PAR = as.character(params),
    LB = if(!is.null(lb)) lb else params_df$Value[match(params, params_df$Parameter)] * (1 - perc),
    UB = if(!is.null(ub)) ub else params_df$Value[match(params, params_df$Parameter)] * (1 + perc)
  )

  sens_loc <- par_bounds %>% pmap_dfr(function(PAR, LB, UB) {
    par_range <- seq(LB, UB, length.out = n_sim)

    map_dfr(par_range, function(p) {
      theta_i <- if(is.null(theta)) c() else theta
      theta_i[PAR] <- p

      sim_i <- sg_sim(
        model = model,
        stimes = stimes,
        outputs = output,
        theta = theta_i,
        covs = cov,
        et = et
      ) %>%
        mutate(
          PARNAME = PAR,
          PARVAL = p,
          PARVAL_NORM = (p - LB) / (UB - LB)
        )

      return(sim_i)
    })
  })

  return(sens_loc)
}

## Author: Ugolkov Yaroslav
## First created: 2025-31-10
## Description: formal testing of sg-localsens-sim function and its helpers
## Keywords: SimuRg, sg-modcomp, model building, comparison
mod_ex <- RxODE({
  # Doses in mg
  # Time in hours

  ### Parameter values
  # Typical
  POPCL = 5;
  POPVC = 180;
  POPQ = 7;
  POPVP = 52;
  POPKTR = 6;
  FBIOPAR = 1;

  # Covariate coefficients
  POPIGFRCOV = 0.8;
  POPPATCOVCLGR1 = 0.85;
  POPPATCOVCLGR2 = 0.85;
  POPPATCOVCLGR3 = 0.85;
  POPPATCOVCLGR4 = 0.85;

  # Random effects
  PPVCL = 0;
  PPVVC = 0;
  PPVKTR = 0;

  # Residual error
  RUV = 0;

  ### Covariates
  PATCOVCL = 1
  if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
  if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
  if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
  if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
  IGFRCOV = (IGFR/112)^POPIGFRCOV

  ### Parameters
  CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
  VC = POPVC * exp(PPVVC);
  Q = POPQ;
  VP = POPVP;
  KTR = POPKTR * exp(PPVKTR);

  ### Explicit functions
  Cc = Ac/VC;                 # nmol/L
  Cp = Ap/VP;                 # nmol/L

  ### Initial conditions
  At1(0) = 0;         # mg
  At2(0) = 0;         # mg
  At3(0) = 0;         # mg
  At4(0) = 0;         # mg
  At5(0) = 0;         # mg
  At6(0) = 0;         # mg
  Ad(0) = 0;          # mg
  Ac(0) = 0;          # mg
  Ap(0) = 0;          # mg

  ### ODEs
  d/dt(At1) = - KTR*At1;
  d/dt(At2) = KTR*At1 - KTR*At2;
  d/dt(At3) = KTR*At2 - KTR*At3;
  d/dt(At4) = KTR*At3 - KTR*At4;
  d/dt(At5) = KTR*At4 - KTR*At5;
  d/dt(At6) = KTR*At5 - KTR*At6;
  d/dt(Ad) = KTR*At6 - KTR*Ad;
  d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
  d/dt(Ap) = Q*(Cc - Cp);

  FBIO = FBIOPAR
  f(At1) = FBIO*1000000/505;
  CHECKRUV = RUV;
  Cc_ResErr = Cc + RUV*Cc;
})
et_base <- tribble(
  ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
  1,   0,     1,     1,    10,   2,     24,  112,   1
)
test_that("sg_localsens_sim works and contains all elements", {
  # Просто вызываем функцию и проверяем результат
  result <- sg_localsens_sim(
    model = mod_ex,
    params = c("POPCL", "POPVC"),
    stimes = seq(0, 168, 0.1),
    output = "Cc",
    perc = 0.9,
    cov = c("IGFR", "POPN"),
    et = et_base
  )

  # Проверяем что функция отрабатывает и возвращает tibble
  expect_s3_class(result, "tbl_df")

  # Проверяем что все основные элементы присутствуют
  required_columns <- c("TIME", "VAR", "VALUE", "PARNAME", "PARVAL", "PARVAL_NORM")
  expect_true(all(required_columns %in% names(result)))

  # Проверяем что таблица не пустая
  expect_gt(nrow(result), 0)

  # Проверяем что все параметры из списка params присутствуют
  expect_equal(sort(unique(result$PARNAME)), sort(c("POPCL", "POPVC")))

  # Проверяем что выходная переменная правильная
  expect_equal(unique(result$VAR), "Cc")
})

test_that("sg_localsens_sim fails gracefully", {
  # Ожидаем error для невалидных входных данных
  expect_error(
    sg_localsens_sim(
      model = "invalid_model",
      params = c("POPCL", "POPVC"),
      stimes = seq(0, 168, 0.1),
      et = et_base
    )
  )

  # Ожидаем error для несовпадающих длин lb/ub
  expect_error(
    sg_localsens_sim(
      model = mod_ex,
      params = c("POPCL", "POPVC"),
      stimes = seq(0, 168, 0.1),
      lb = c(1),  # Длина 1 вместо 2
      ub = c(10), # Длина 1 вместо 2
      et = et_base
    )
  )
})

## Author: Ugolkov Yaroslav, Victoria Kulesh, Mikhailova Anna
## First created: 2025-10-17
## Description: Local sensitivity analysis simulations for RxODE models
## Keywords: SimuRg, model build

#' Build estimation model scenarios
#'
#' @description
#' Constructs and exports multiple model configuration scenarios by combining population parameters,
#' random effects, residual variability, occasion effects, and covariate assignments.
#' Each scenario is defined by unique combinations of these model components and exported as `.mlxtran` files.
#' Additionally, a summary CSV file describing all scenarios is generated.
#'
#' @param mod_lst List of model file paths or model identifiers used for scenario generation.
#' @param data Character string. Path to the dataset used in model fitting or simulation.
#' @param headers Predefined list of dataset column names (e.g., ID, TIME, DV, etc.) used by the modeling framework.
#' @param ruv_lst List specifying residual unexplained variability (RUV) structures.
#'   Each element contains RUV properties (e.g., `YNAME`, `ERR`) and mapping to data.
#' @param theta_lst List of tibbles describing population parameter properties.
#'   Each tibble should contain columns such as `NAME`, `INIT`, and `EST`.
#' @param re_lst List of random effects (RE) specifications.
#'   Each element includes matrices `init` and `est` defining initial and estimated covariance structures.
#' @param occ_lst List of occasion effect matrices.
#'   Each element includes `init` and `est` matrices defining inter-occasion variability.
#' @param covs_lst Optional list describing covariate-parameter relationships.
#'   Each element should include fields such as `PAR`, `COVNAME`, and `EST` indicating inclusion in the model.
#' @param task_lst Optional list defining additional tasks or configuration options for each scenario. Default is NULL.
#' @param opt_name Character. Optimization engine name (e.g., `"Monolix"`, `"Simurg"`). Default `"Simurg"`.
#' @param path Character. Directory path where output files (CSV summary and model files) will be written. Default is current working directory.
#' @param project_name Character. Base name for the exported project files. Default `"my_project"`.
#'
#' @return
#' The function writes two types of outputs to the specified `path`:
#' \itemize{
#'   \item A CSV file (`scenarios_info.csv`) summarizing all generated scenarios with columns:
#'     \describe{
#'       \item{scenario_number}{Unique index of the scenario.}
#'       \item{model}{Model file or path used in the scenario.}
#'       \item{data}{Path to the dataset used.}
#'       \item{theta}{Active population parameters and initial values used.}
#'       \item{RUV}{Residual error structure(s) used.}
#'       \item{RE}{Random effect and correlation terms included.}
#'       \item{OCC}{Active occasion effects.}
#'       \item{COVS}{Included covariate-parameter relationships.}
#'     }
#'   \item Individual `.mlxtran` model files for each generated scenario,
#'   named according to the pattern `<project_name>_<i>.mlxtran`.
#' }
#'
#' @details
#' This function automates the construction of multiple model configurations for model evaluation,
#' sensitivity analysis, or scenario testing. It expands parameter and variability definitions,
#' constructs all possible combinations, and exports model specifications for external fitting tools.
#'
#' @examples
#' \donttest{
#' library(dplyr)
#' folder_path <- system.file("extdata", package = "SimuRg")
#' mod_lst <- list(paste(folder_path, "/models/model_PK_1c.txt", sep = "/"),
#'                   paste(folder_path, "/models/model_PK_2c.txt", sep = ""))
#'
#' ### path to the dataset
#' data <- paste(folder_path, "datasets", "dspk-warf.csv", sep = "/")
#' re_lst_1 <- list(
#'               list(init = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
#'                                     1, 0, 0, 0, 0,
#'                                     0, 1, 0, 0, 0,
#'                                     0, 0, 1, 0, 0,
#'                                     0, 0, 0, 1, 0,
#'                                     0, 0, 0, 0, 1) %>% as.matrix(),
#'                     est = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
#'                                     TRUE, NA, NA, NA, NA,
#'                                     NA, TRUE, NA, NA, NA,
#'                                     NA, NA, TRUE, NA, NA,
#'                                     NA, NA, NA, TRUE, NA,
#'                                     NA, NA, NA, NA, TRUE) %>% as.matrix(),
#'                     block = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
#'                                       FALSE, NA, NA, NA, NA,
#'                                       NA, FALSE, NA, NA, NA,
#'                                       NA, NA, TRUE, NA, NA,
#'                                       NA, NA, NA, FALSE, NA,
#'                                       NA, NA, NA, NA, FALSE) %>% as.matrix())
#'            )
#' headers <- list(list(name = "ID", use = "identifier", type = NULL),
#'                list(name = "TIME", use = "time", type = NULL),
#'                list(name = "DV", use = "observation", type = "continuous"),
#'                list(name = "DVID", use = "observationtype", type = NULL),
#'                list(name = "ADM", use = "administration", type = NULL),
#'                list(name = "AMT", use = "amount", type = NULL),
#'                list(name = "EVID", use = "eventidentifier", type = NULL),
#'                list(name = "MDV", use = "missingdependentvariable", type = NULL),
#'                list(name = "AGE", use = "covariate", type = "continuous"),
#'                list(name = "AGE_centered", use = "covariate", type = "continuous"),
#'                list(name = "SEX", use = "covariate", type = "categorical"),
#'                list(name = "WEIGHT", use = "covariate", type = "continuous"),
#'                list(name = "BMI", use = "covariate", type = "continuous"))
#'
#' ruv_lst_2 <- list(
#'   # structural model 1
#'  list(
#'    list(YNAME = "y1",
#'         DVID = 1,
#'         TRANS = "normal",
#'         PRED = "Cc",
#'         ERR = list("constant", "proportional", "combined1"),
#'          # options to test (can be length = 1, i.e., "constant" or "combined1")
#'         INIT = list(1, 1, c(1, 1)),
#'          # options to test (can be length = 1, i.e., 1 or c(1, 1))
#'         EST = list(TRUE, TRUE, c(TRUE, TRUE)))
#'         # options to test (can be length = 1, i.e., T or c(T, T))BLQM = NULL))
#'  ),
#'  # structural model 2
#'  list(
#'    list(YNAME = "y1",
#'         DVID = 1,
#'         TRANS = "normal",
#'         PRED = "Cc",
#'         ERR = list("constant", "proportional"),
#'         # options to test (can be length = 1, i.e., "constant" or "combined1")
#'         INIT = list(1, 1), # options to test (can be length = 1, i.e., 1 or c(1, 1))
#'         EST = list(TRUE, TRUE))
#'          # options to test (can be length = 1, i.e., T or c(T, T))BLQM = NULL))
#'  )
#' )
#'
#' theta_lst_2 <- list(
#'  # structural model 1
#'  tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
#'          "Cl", "logNormal", 0.2, NA, NA, TRUE,
#'          "Vd", "logNormal", 20, NA, NA, TRUE,
#'          "ka", "logNormal", 0.2, NA, NA, TRUE),
#'  # structural model 2
#'  tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
#'          "Cl", "logNormal", 0.2, NA, NA, TRUE,
#'          "Vd", c("Normal", "logNormal"), c(10, 20, 30), NA, NA, TRUE,
#'          "ka", "logNormal", c(0.2, 0.1, 0.5), NA, NA, TRUE,
#'          # INIT could be length > 1
#'          "Vp", c("Normal", "logNormal"), 10, NA, NA, TRUE,
#'          "Q", "logNormal", 5, NA, NA, TRUE))
#'
#' path <- tempdir()
#' sg_modbuild(
#'  mod_lst = mod_lst[1],
#'  data = data,
#'  headers = headers,
#'  ruv_lst = ruv_lst_2,
#'  theta_lst = theta_lst_2,
#'  re_lst = re_lst_1,
#'  occ_lst = re_lst_1,
#'  covs_lst = NULL,
#'  path = path,
#'  project_name = "tests_test_project"
#' )
#' clr_files <- list.files(path, full.names = TRUE)
#' unlink(clr_files, recursive = TRUE, force = TRUE)
#' }
#'
#' @import dplyr
#' @importFrom readr write_csv
#' @export
#' @importFrom jsonlite fromJSON
#' @import dplyr
#' @export
sg_modbuild <- function(mod_lst, data, headers, ruv_lst, theta_lst, re_lst, path,
                        occ_lst, covs_lst=NULL, task_lst = NULL, opt_name = "Simurg",
                        project_name = "my_project") {
  path <- gsub("[/\\\\]+$", "", path)


  make_re_obj <- function(changes, est, init, variable_num, init0) {
    for (j in changes) {
      if (j == 0) {
        next
      }
      init[variable_num[j, 'row'],
           variable_num[j, 'col']]  <- init0[variable_num[j, 'row'],
                                             variable_num[j, 'col']]
      est[variable_num[j, 'row'],
          variable_num[j, 'col']]  <- T
    }
    return(list(init = init, est = est))
  }
  make_re_lst <- function(re_lst) {
    variable <- re_lst[["est"]] & !re_lst[["block"]]
    variable_num <- which(variable, arr.ind=TRUE)
    variants <- combn(c(rep(0, times = nrow(variable_num)), 1:nrow(variable_num)),
                      m = nrow(variable_num), simplify = FALSE) %>% unique()
    init <- ifelse(!variable | is.na(variable), re_lst[["init"]], 0)
    est <- ifelse(variable, NA, !variable)
    fin_re_lst <- map(variants, function(x){
      make_re_obj(x, est, init, variable_num, re_lst[["init"]])})
    return(fin_re_lst)
  }

  make_ruv_one <- function(i, j, ruv_lst) {
    ruv_lst$EST <- ruv_lst$EST[[i]]
    ruv_lst$INIT <- ruv_lst$INIT[[i]]
    ruv_lst$ERR <- ruv_lst$ERR[[i]]
    ruv_lst$TRANS <- ruv_lst$TRANS[[j]]
    return(ruv_lst)
  }
  make_ruv_lst <- function(ruv_lst) {
    i_s <- c(1:length(ruv_lst$ERR))
    j_s <- c(1:length(ruv_lst$TRANS))
    inxs <- expand_grid(i_s, j_s)
    return(pmap(inxs, function(i_s, j_s) {make_ruv_one(i_s, j_s, ruv_lst)}))
  }
  make_ruv_all <- function(ruv_lst) {
    if (length(ruv_lst) == 1){
      return(make_ruv_lst(ruv_lst[[1]]))
    } else {
      return(map(ruv_lst, make_ruv_lst))
    }
  }

  make_theta_one <- function(theta_lst, val, trans_idx, init_idx) {
    for (i in c(1:length(trans_idx))) {
      theta_lst$TRANS[trans_idx[i]] <- val[i]
    }
    for (i in c(1:length(init_idx))) {
      theta_lst$INIT[init_idx[i]] <- val[i + length(trans_idx)]
    }
    return(theta_lst %>% mutate(TRANS = as.character(TRANS),
                                INIT = as.numeric(INIT)))
  }
  make_theta_lst <- function(theta_lst) {
    theta_lst_ <- theta_lst %>% group_by(NAME) %>%
      mutate(TRANS_ln = length(TRANS %>% unlist()),
             INIT_ln = length(INIT %>% unlist()))
    trans_idx <- which(theta_lst_$TRANS_ln > 1)
    init_idx <- which(theta_lst_$INIT_ln > 1)
    if (length(trans_idx) + length(init_idx) == 0) {
      return(list(theta_lst))
    }
    trans_val <- expand.grid(c(theta_lst$TRANS[trans_idx],
                               theta_lst$INIT[init_idx]),
                             stringsAsFactors = FALSE)
    return(apply(trans_val, 1,function(x){make_theta_one(theta_lst, x, trans_idx, init_idx)}))
  }


  theta_lst_exp <- map(theta_lst, make_theta_lst)
  re_lst_exp <- map(re_lst, make_re_lst)
  occ_lst_exp <- map(occ_lst, make_re_lst)
  ruv_lst_exp <- map(ruv_lst, function(x)  {map(x, make_ruv_lst)})

  sc_lst <- tibble()
  for (i in c(1:length(mod_lst[1]))) {
    if (length(ruv_lst_exp[[i]]) > 1) {
      ruv_lst_sev_dv <- do.call(expand.grid, ruv_lst_exp[[i]]) %>% pmap(., list) # Several DVID
    } else{
      ruv_lst_sev_dv <- ruv_lst_exp[[i]][[1]]
    }

    all_variants <- expand_grid(theta_lst_exp[[i]], re_lst_exp[[i]],
                                ruv_lst_sev_dv, occ_lst_exp[[i]])
    colnames(all_variants) <- c('theta', 're', 'ruv', 'occ')
    all_variants['mod'] <- mod_lst[[i]]
    sc_lst <- rbind(sc_lst, all_variants)
  }
  # return(sc_lst)
  n_iter <- length(sc_lst$theta)
  summary_list <- vector("list", n_iter)

  for (i in seq_len(n_iter)) {
    model_path <- sc_lst$mod[i]
    # --- theta ---
    theta_true <- sc_lst$theta[[i]] %>% mutate(NAME_INIT = paste0(NAME,'=',INIT)) %>%
      filter(EST == TRUE) %>%
      pull(NAME_INIT) %>%
      str_c(collapse = ", ")

    # --- RUV ---
    ruv_info <- sc_lst$ruv[[i]]
    ruv_full <- paste0(ruv_info$YNAME, "_", ruv_info$ERR)

    # --- RE ---
    re_mat <- sc_lst$re[[i]]$init
    re_names <- colnames(re_mat)
    diag_idx <- which(diag(re_mat) == 1)
    re_active <- if (length(diag_idx) > 0) re_names[diag_idx] else character(0)

    corr_list <- c()
    for (d_idx in diag_idx) {
      diag_name <- re_names[d_idx]

      for (c in seq_len(ncol(re_mat))) {
        if (c != d_idx && re_mat[d_idx, c] == 1) {
          corr_list <- c(corr_list, paste0("corr_", diag_name, "_", re_names[c]))
        }
      }
      for (r in seq_len(nrow(re_mat))) {
        if (r != d_idx && re_mat[r, d_idx] == 1) {
          corr_list <- c(corr_list, paste0("corr_", diag_name, "_", re_names[r]))
        }
      }
    }

    re_full <- paste(unique(c(re_active, corr_list)), collapse = ", ")

    # --- OCC ---
    occ_mat <- sc_lst$occ[[i]]$init
    occ_active <- colnames(occ_mat)[colSums(occ_mat == 1, na.rm = TRUE) > 0] %>%
      str_c(collapse = ", ")

    # --- COVS ---
    covs_active <- covs_lst %>%
      keep(~ .x$EST == TRUE) %>%
      map_chr(~ str_c(.x$PAR, "_", .x$COVNAME)) %>%
      str_c(collapse = ", ")

    # --- tibble ---
    summary_list[[i]] <- tibble(
      scenario_number = i,
      model = model_path,
      data = data,
      theta = theta_true,
      RUV = ruv_full,
      RE = re_full,
      OCC = occ_active,
      COVS = covs_active
    )

  }
  summary_df <- bind_rows(summary_list)
  write_csv(summary_df, file.path(path, 'scenarios_info.csv'))
  for (i in seq(1,length(sc_lst$mod))){
    sg_result_mod <- sg_fit(sc_lst$mod[i], data, headers, sc_lst$theta[[i]],
                            sc_lst$ruv[[i]], sc_lst$re[[i]],  sc_lst$occ[[i]],
                            project_name = paste0(project_name,'_',i),
                            covs_lst, opt_name = "Monolix",
                            path_to_save_output = path)
    write(sg_result_mod, file.path(path, paste0(project_name,'_',i,'.mlxtran')))
  }
}

## Author: Ugolkov Yaroslav
## First created: 2025-31-10
## Description: formal testing of sg-modbuild function
## Keywords: SimuRg, sg-modbuild, model building

test_that("sg_modbuild creates expected number of files", {

  folder_path <- system.file("extdata", package = "SimuRg")

  ### list of paths to structural models
  mod_lst <- list(paste(folder_path, "/models/model_PK_1c.txt", sep = "/"),
                  paste(folder_path, "/models/model_PK_2c.txt", sep = "/"))

  ### path to the dataset
  data <- paste(folder_path, "datasets", "dspk-warf.csv", sep = "/")
  re_lst_1 <- list(
    list(init = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                        1, 0, 0, 0, 0,
                        0, 1, 0, 0, 0,
                        0, 0, 1, 0, 0,
                        0, 0, 0, 1, 0,
                        0, 0, 0, 0, 1) %>% as.matrix(),
         est = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                       T, NA, NA, NA, NA,
                       NA, T, NA, NA, NA,
                       NA, NA, T, NA, NA,
                       NA, NA, NA, T, NA,
                       NA, NA, NA, NA, T) %>% as.matrix(),
         block =tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                        F, NA, NA, NA, NA,
                        NA, F, NA, NA, NA,
                        NA, NA, T, NA, NA,
                        NA, NA, NA, F, NA,
                        NA, NA, NA, NA, F) %>% as.matrix())
  )
  headers <- list(list(name = "ID", use = "identifier", type = NULL),
                  list(name = "TIME", use = "time", type = NULL),
                  list(name = "DV", use = "observation", type = "continuous"),
                  list(name = "DVID", use = "observationtype", type = NULL),
                  list(name = "ADM", use = "administration", type = NULL),
                  list(name = "AMT", use = "amount", type = NULL),
                  list(name = "EVID", use = "eventidentifier", type = NULL),
                  list(name = "MDV", use = "missingdependentvariable", type = NULL),
                  list(name = "AGE", use = "covariate", type = "continuous"),
                  list(name = "AGE_centered", use = "covariate", type = "continuous"),
                  list(name = "SEX", use = "covariate", type = "categorical"),
                  list(name = "WEIGHT", use = "covariate", type = "continuous"),
                  list(name = "BMI", use = "covariate", type = "continuous"))

  ruv_lst_2 <- list(
    # structural model 1
    list(
      list(YNAME = "y1",
           DVID = 1,
           TRANS = "normal",
           PRED = "Cc",
           ERR = list("constant", "proportional", "combined1"), # options to test (can be length = 1, i.e., "constant" or "combined1")
           INIT = list(1, 1, c(1, 1)), # options to test (can be length = 1, i.e., 1 or c(1, 1))
           EST = list(T, T, c(T, T))) # options to test (can be length = 1, i.e., T or c(T, T))BLQM = NULL))
    ),
    # structural model 2
    list(
      list(YNAME = "y1",
           DVID = 1,
           TRANS = "normal",
           PRED = "Cc",
           ERR = list("constant", "proportional"), # options to test (can be length = 1, i.e., "constant" or "combined1")
           INIT = list(1, 1), # options to test (can be length = 1, i.e., 1 or c(1, 1))
           EST = list(T, T)) # options to test (can be length = 1, i.e., T or c(T, T))BLQM = NULL))
    )
  )

  theta_lst_2 <- list(
    # structural model 1
    tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
            "Cl", "logNormal", 0.2, NA, NA, T,
            "Vd", "logNormal", 20, NA, NA, T,
            "ka", "logNormal", 0.2, NA, NA, T),
    # structural model 2
    tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
            "Cl", "logNormal", 0.2, NA, NA, T,
            "Vd", c("Normal", "logNormal"), c(10, 20, 30), NA, NA, T,
            "ka", "logNormal", c(0.2, 0.1, 0.5), NA, NA, T, # INIT could be length > 1
            "Vp", c("Normal", "logNormal"), 10, NA, NA, T,
            "Q", "logNormal", 5, NA, NA, T))

  path <- tempdir()
  sg_modbuild(
    mod_lst = mod_lst[1],
    data = data,
    headers = headers,
    ruv_lst = ruv_lst_2,
    theta_lst = theta_lst_2,
    re_lst = re_lst_1,
    occ_lst = re_lst_1,
    covs_lst = NULL,
    path = path,
    project_name = "tests_test_project"
  )

  # Проверяем что создался CSV файл с информацией о сценариях
  expect_true(file.exists(paste0(path, "/scenarios_info.csv")))

  # Проверяем что создались mlxtran файлы
  # В данном случае ожидаем 2 файла (2 варианта RUV: constant + proportional)
  mlxtran_files <- list.files(path, pattern = "\\.mlxtran$")
  expect_length(mlxtran_files, 768)

  # Проверяем имена файлов
  expect_true(all(c("tests_test_project_1.mlxtran", "tests_test_project_768.mlxtran") %in% mlxtran_files))

  # Проверяем что CSV файл не пустой
  csv_content <- read_csv(paste0(path, "/scenarios_info.csv"))
  # expect_gt(nrow(csv_content), 0)
  expect_equal(nrow(csv_content), 768) # 2 сценария
  clr_files <- list.files(path, full.names = TRUE)
  unlink(clr_files, recursive = TRUE, force = TRUE)


})

## Author: Mikhailova Anna
## First created: 2025-08-25
## Description: sg-gof-tp and its helper functions
## Keywords: SimuRg, sg-gof-tp, goodness-of-fit

#' Plot time profiles of the fitted data
#'
#' @inheritParams sg_dummy
#' @param pop Logical. If `TRUE` (default), adds population predictions (PRED) as dashed lines and points.
#' @param cap String. Plot caption. Default is "empty circles - observed data solid lines with point - individual predictions dashed grey lines with point - population predictions"
#' @param lab_x String. X-ax label. Default is "Time since first dose, h"
#' @param lab_y String. Y-ax label. Default is "Plasma concentration, mmol/L"
#' @param sort_by Character vector of column names to sort ID facets by.
#'        Use "DOSE" to sort by last dose. Other names (e.g. covariates) must exist in SimuRg object.
#' @param desc Logical. If TRUE, sort in descending order for all columns in `sort_by`.
#' @returns A list of plots with predicted time profiles, faceted by id
#' @examples
#' \donttest{
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData",
#'                         package = "SimuRg")
#' plot_list <- sg_gof_tp(fpath_i)
#' print(plot_list[[1]])
#' }
#' @import dplyr
#' @import ggplot2
#' @importFrom scales pretty_breaks trans_format
#' @importFrom forcats fct_inorder
#' @importFrom zoo na.locf
#' @importFrom purrr map
#' @import stringr
#' @export
sg_gof_tp <- function(fpath_i, pop = TRUE, filt = "T", cov_cols = NULL, col_i = NULL,
                      DVID = 1, tsld = FALSE, f_scales = "free", sort_by = NULL,
                      desc = FALSE, log_y = FALSE, lab_x = "Time since first dose, h",
                      lab_y = "Plasma concentration, mmol/L",
                      cap = str_c("empty circles - observed data\n",
                                  "solid lines with point - ",
                                  "individual predictions\n",
                                  "dashed grey lines with ",
                                  "point - population predictions"),
                      n_quantiles = 3, levels_discrete = 10){

  # --- helper functions ---
  is_discrete <- function(x, max_levels = levels_discrete) {
    n_unique <- length(unique(na.omit(x)))
    n_unique <= max_levels
  }

  continuous_to_categories <- function(x, n_quant = 3) {
    cut(
      x,
      breaks = quantile(x, probs = seq(0, 1, length.out = n_quant + 1), na.rm = TRUE),
      include.lowest = TRUE,
      labels = paste0("Q", 1:n_quant)
    )
  }

  if (!is_null(read_smrg_obj(fpath_i)$GFO))
    smrg_obj <- read_smrg_obj(fpath_i)$GFO
  else
    smrg_obj <- read_smrg_obj(fpath_i)

  if (is.null(smrg_obj$SDTAB)) {
    stop("sg_fit object must contain SDTAB component")
  }

  sdtab <- smrg_obj$SDTAB

  if (is.data.frame(sdtab) && nrow(sdtab) == 0) {
    stop("SDTAB is empty (no rows)")
  }
  if (is.list(sdtab) && length(sdtab) == 0) {
    stop("SDTAB is empty (no elements)")
  }
  if (is.data.frame(sdtab)) {
    ds_i <- sdtab
  } else if (is.list(sdtab)) {
    ds_i <- as.data.frame(do.call(rbind, sdtab))
  } else {
    stop("SDTAB must be a data frame or a list of data frames")
  }

  ds_i <- filter_sdtab_by_DVID(ds_i, DVID)

  ds_tp_pre <- ds_i
  if ("MDV" %in% colnames(ds_i)) {
    ds_tp_pre <- ds_tp_pre %>% filter(.data$MDV != 1)
  }

  continuous_factors <- list()

  if (!is.null(cov_cols)) {
    suppressMessages({
      ds_covs <- left_join(smrg_obj$COTAB, smrg_obj$CATAB)
    })
    ds_covs_i <- ds_covs %>% select(ID, one_of(cov_cols)) %>% unique()
    stopifnot(n_distinct(ds_covs_i$ID) == nrow(ds_covs_i))
    suppressMessages({
      ds_tp_pre <- ds_tp_pre %>% left_join(ds_covs_i, by = "ID")
    })

    for (col in cov_cols) {
      if (!is_discrete(ds_tp_pre[[col]])) {
        cat_col <- paste0(col, "_cat")
        ds_tp_pre[[cat_col]] <- continuous_to_categories(ds_tp_pre[[col]], n_quant = n_quantiles)
        #message(paste(col, "was converted into", n_quantiles, "quantile-based categories."))
        continuous_factors[[col]] <- cat_col
      } else {
        ds_tp_pre[[col]] <- factor(ds_tp_pre[[col]])
      }
    }
  }

  if (!is.null(sort_by) && "DOSE" %in% sort_by) {
    if (is.null(smrg_obj$EVTAB)) {
      stop("EVTAB not found in smrg_obj; cannot sort by DOSE.")
    }
    evtab <- smrg_obj$EVTAB %>%
      filter(EVID == 1) %>%
      group_by(ID) %>%
      arrange(TIME) %>%
      summarise(DOSE = first(AMT), .groups = "drop")
    ds_tp_pre <- ds_tp_pre %>% left_join(evtab, by = "ID")
  }

  ds_tp_pre <- ds_tp_pre %>% filter(eval(rlang::parse_expr(filt)))

  if (!is.null(col_i)) {
    actual_col <- if (col_i %in% names(continuous_factors)) continuous_factors[[col_i]] else col_i
    if (!is.factor(ds_tp_pre[[actual_col]])) {
      ds_tp_pre[[actual_col]] <- factor(ds_tp_pre[[actual_col]])
    }
    col_levels <- levels(ds_tp_pre[[actual_col]])
    n_cols <- length(col_levels)
    col_values <- rep(MSDcol, length.out = n_cols)   # or use a better color palette
    col_labels <- paste0(col_i, ": ", col_levels)
    color_column <- actual_col
  }

  if (!is.null(sort_by) && length(sort_by) > 0) {
    # Check that all sort_by columns exist
    missing <- setdiff(sort_by, colnames(ds_tp_pre))
    if (length(missing) > 0) {
      stop("sort_by column(s) not found: ", paste(missing, collapse = ", "))
    }

    # Obtain desired ID order
    if (desc) {
      id_order <- ds_tp_pre %>%
        distinct(ID, !!!syms(sort_by)) %>%
        arrange(across(all_of(sort_by), desc)) %>%
        pull(ID)
    } else {
      id_order <- ds_tp_pre %>%
        distinct(ID, !!!syms(sort_by)) %>%
        arrange(across(all_of(sort_by))) %>%
        pull(ID)
    }

    # Reorder ID factor in the whole dataset
    ds_tp_pre <- ds_tp_pre %>%
      mutate(ID = factor(ID, levels = id_order))

    # Build facet labels: "ID <id>; col1=val1; col2=val2"
    label_lookup <- ds_tp_pre %>%
      distinct(ID, !!!syms(sort_by)) %>%
      rowwise() %>%
      mutate(
        label_part = paste0(sort_by, "=", c_across(all_of(sort_by)), collapse = "; "),
        facet_label = paste0("ID ", ID, "; ", label_part)
      ) %>%
      ungroup() %>%
      select(ID, facet_label) %>%
      mutate(facet_label = factor(facet_label, levels = unique(facet_label[order(ID)])))

    ds_tp_pre <- ds_tp_pre %>%
      left_join(label_lookup, by = "ID")
  } else {
    ds_tp_pre <- ds_tp_pre %>%
      mutate(ID = fct_inorder(as.character(ID))) %>%
      mutate(facet_label = as.character(ID)) %>%
      mutate(facet_label = factor(facet_label, levels = levels(ID)))
  }

  if ((tsld) & !("ATSLD" %in% colnames(ds_i))) {
    stop("No column specified for time since last dose")
  } else if (tsld) {
    ds_tp_pre <- ds_tp_pre %>%
      mutate(TSLD = ATSLD) %>% group_by(ID) %>% # TSLD calculations should be implemented
      mutate(OCCMAN = ifelse(lag(TSLD) > TSLD | is.na(lag(TSLD)),
                             row_number(), NA), OCCMAN = zoo::na.locf(OCCMAN)) %>% ungroup() %>%
      mutate(DAYMAN = ceiling(TIME/24), Period = case_when(DAYMAN == 1 ~ "Day 1",
                                                           DAYMAN == 2 ~ "Day 2",
                                                           DAYMAN == 3 ~ "Day 3",
                                                           DAYMAN > 3 ~ "Day 3+"))
  }

  ds_tp_filt <- ds_tp_pre %>%
    arrange(ID) %>%
    mutate(NTILE = ceiling(as.numeric(ID) / 20))
    # arrange(DVID) %>%
    # mutate(NTILE = ceiling(as.numeric(fct_inorder(as.character(ID)))/20),
    #        ID = fct_inorder(as.character(ID)))

  TVAR <- "TIME";
  if(tsld){ TVAR <- "TSLD";}

  p_i <- unique(ds_tp_filt$NTILE) %>% map(function(n){
    set_n <- ds_tp_filt %>% filter(NTILE == n)

    p_n <- ggplot(data = set_n, aes(x = !!sym(TVAR), y = DV)) + # , group = OCCMAN)) +
      geom_point(size = 1.5, shape = 21, stroke = 0.8, fill = "white", col = "grey15")

    if(pop) {   # add PRED layers only when pop is TRUE
      p_n <- p_n +
        geom_line(aes(y = PRED), lty = "dashed", col = "grey75") +
        geom_point(aes(y = PRED), size = 1, col = "grey75")
    }

    if(!is.null(col_i)) {
      p_n <- p_n +
        geom_line(aes(y = IPRED, color = .data[[color_column]])) +
        geom_point(aes(y = IPRED, color = .data[[color_column]]), size = 1)
    } else {
      p_n <- p_n +
        geom_line(aes(y = IPRED)) +
        geom_point(aes(y = IPRED), size = 1)
    }

    # Rest of the layers (scales, facets, theme)
    p_n <- p_n +
      scale_x_continuous(name = lab_x, breaks = scales::pretty_breaks(7)) +
      scale_y_continuous(name = lab_y, breaks = scales::pretty_breaks(7))

    # Add manual color scale if needed
    if(!is.null(col_i)) {
      # col_values and col_labels should have been prepared earlier (as in your code)
      p_n <- p_n + scale_color_manual(
        name = col_i,
        values = col_values,
        labels = col_levels,
        drop = FALSE
      )
      # Show legend, hide grid minor
      p_n <- p_n + theme(legend.position = "right", panel.grid.minor = element_blank())
    } else {
      # Original scale_color_manual? You had scale_color_manual(values = MSDcol) but no mapping – remove it
      # Actually, better to keep legend none and no color scale
      p_n <- p_n + theme(legend.position = "none", panel.grid.minor = element_blank())
    }

    p_n <- p_n +
      facet_wrap(~facet_label, scales = f_scales) +
      theme_bw() +
      labs(subtitle = str_c(" (part ", n, " out of ", max(ds_tp_filt$NTILE), ")"),
           caption = cap)

    # p_n <- p_n +
    #   geom_line(aes(y = IPRED)) +
    #   geom_point(aes(y = IPRED), size = 1) + #, shape = Period
    #   scale_x_continuous(name = lab_x, breaks = scales::pretty_breaks(7)) +
    #   scale_y_continuous(name = lab_y, breaks = scales::pretty_breaks(7)) +
    #   scale_color_manual(values = MSDcol) +
    #   facet_wrap(~ID, scales = f_scales) +
    #   theme_bw() +
    #   theme(legend.position = "none", panel.grid.minor = element_blank()) +
    #   labs(subtitle = str_c( " (part ", n, " out of ", max(ds_tp_filt$NTILE), ")"),
    #        caption = cap)
    if(log_y){
      p_n <- p_n + scale_y_log10(
        name = lab_y,
        breaks = scales::trans_breaks("log10", function(x) 10^x),
        labels = scales::trans_format("log10", scales::math_format(10^.x))
      )
    }
    return(p_n)
  })
  return(p_i)
}

## Author: Mikhailova Anna
## First created: 2025-08-25
## Description: formal testing of sg-gof-tp function and its helpers
## Keywords: SimuRg, sg-gof-tp, goodness-of-fit


test_that("sg-tp-gof output is correct", {
  x <- sg_gof_tp(obj1)
  expect_true(inherits(x[[1]], "ggplot"))
  expect_snapshot(ggplot2::layer_data(x[[1]]))
})
test_that("sg-gof-tp does not work", {
  expect_error(sg_gof_tp(data.frame()))
  expect_error(sg_gof_tp(obj1, tsld = T))
})
test_that("sg-tp-gof file load", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  x <- sg_gof_tp(fpath_i, log_y = T)
  expect_true(inherits(x[[1]], "ggplot"))
  expect_snapshot(ggplot2::layer_data(x[[1]]))
})

## Author: Victor Sokolov
## First created: 2026-03-05
## Description: Translate structural models between rxode2 and MLXTRAN syntax
## Keywords: SimuRg, monolix, rxode2, translator, converter

#' Translate structural models between rxode2 and MLXTRAN syntax
#'
#' @description
#' Converts structural pharmacometric model files between rxode2 (R package)
#' and MLXTRAN (Monolix) syntax. Supports ODE-based models and Monolix pkmodel macros.
#'
#' @param input_path Character string. Path to the input structural model file (.txt).
#' @param to Character string. Target format: \code{"mlxtran"} or \code{"rxode"}.
#' @param output_path Character string. Path to write the translated model file (.txt).
#' @param dm_list List of dosing macros with \code{cmt} and \code{adm} vectors.
#'   For example, \code{list(cmt = 2, adm = 1)} or \code{list(cmt = c(2, 1), adm = c(1, 2))}.
#'   Used to generate \code{iv()} dosing macros in MLXTRAN output.
#' @param regressors Character vector of regressor variable names.
#'   For example, \code{c("WTBL", "ADAN")} generates \code{WTBL = \{use = regressor\}} in MLXTRAN.
#' @param output_vars Character vector of output variable names for the MLXTRAN \code{OUTPUT:} section.
#'   For example, \code{c("Cc_mgL", "CSF1_ngmL")} generates \code{output = \{Cc_mgL, CSF1_ngmL\}}.
#'   Required when \code{to = "mlxtran"}; ignored when \code{to = "rxode"}.
#' @param macros Logical. If \code{TRUE}, attempt to convert rxode2 model to pkmodel macro
#'   format in MLXTRAN (only for simple 1- or 2-compartment oral PK models). Default \code{TRUE}.
#' @param stiff Logical. If \code{TRUE}, add \code{odeType = stiff} to the MLXTRAN EQUATION section.
#'   Default \code{TRUE}.
#'
#' @return Invisibly returns the translated model as a character string.
#'   The translated model is also written to \code{output_path}.
#'
#' @details
#' The function detects the source format automatically from file content.
#'
#' \strong{rxode2 format} uses \code{# [INPUT]}, \code{# [MODEL]}, \code{# [OUTPUT]} section markers,
#' \code{d/dt(X)} ODE notation, \code{f(X)} for bioavailability, and \code{alag(X)} for lag time.
#'
#' \strong{MLXTRAN format} uses \code{[LONGITUDINAL]}, \code{PK:}/\code{EQUATION:}/\code{OUTPUT:} sections,
#' \code{ddt_X} ODE notation, \code{compartment()}/\code{iv()} dosing macros, and optionally
#' \code{pkmodel()} macros.
#'
#' When converting rxode2 to MLXTRAN with \code{macros = TRUE}, the function checks if the model
#' structure matches a simple \code{pkmodel()} pattern (1- or 2-compartment with first-order oral
#' absorption). If not, it falls back to full ODE format with \code{compartment()}/\code{iv()} macros.
#'
#' MLXTRAN models using \code{absorption()}/\code{elimination()}/\code{peripheral()} macros (type 2)
#' are expanded to explicit ODEs when converted to rxode2.
#'
#' @examples
#' \donttest{
#' # Convert rxode2 model to MLXTRAN ODE format
#' rxode_model_hv_iv <- system.file("extdata", "models", "rxode",
#'                                  "model_PK_hv_iv.txt", package = "SimuRg")
#' rxode_model_1c <- system.file("extdata", "models", "rxode",
#'                               "model_PK_1c.txt", package = "SimuRg")
#' monolix_model_hv_iv <- system.file("extdata", "models", "monolix",
#'                                    "model_PK_hv_iv.txt", package = "SimuRg")
#' monolix_model_1c <- system.file("extdata", "models", "monolix",
#'                                 "model_PK_1c.txt", package = "SimuRg")
#'
#' path_to_save <- tempdir()
#'
#' sg_translator(rxode_model_hv_iv, to = "mlxtran",
#'               output_path = normalizePath(file.path(path_to_save,
#'                                                     "model_PK_hv_iv_mlx.txt"),
#'                                           mustWork = FALSE),
#'               output_vars = "Cc_mgL",
#'               dm_list = list(cmt = 1, adm = 1), stiff = TRUE)
#'
#' # Convert rxode2 model to MLXTRAN with pkmodel macros
#' sg_translator(rxode_model_1c, to = "mlxtran",
#'               output_path = normalizePath(file.path(path_to_save,
#'                                                     "model_PK_1c_mlx.txt"),
#'                                           mustWork = FALSE),
#'               output_vars = "Cc_nM", macros = TRUE)
#'
#' # Convert MLXTRAN model to rxode2
#' sg_translator(monolix_model_hv_iv, to = "rxode",
#'               output_path = normalizePath(file.path(path_to_save,
#'                                                     "model_PK_hv_iv_rx.txt"),
#'                                            mustWork = FALSE))
#' }
#'
#' @export
sg_translator <- function(input_path, to, output_path, dm_list = NULL,
                          regressors = NULL, output_vars = NULL,
                          macros = TRUE, stiff = TRUE) {

  input_path <- normalizePath(input_path, mustWork = FALSE)
  if(!file.exists(input_path)) {
    stop("Input model file does not exist. Check file existance or try to use absolute path")
  }
  raw <- readLines(input_path, warn = FALSE)
  lines <- stringr::str_trim(raw, side = "right")
  is_mlxtran <- any(stringr::str_detect(lines, "^\\s*\\[LONGITUDINAL\\]"))

  if (to == "mlxtran" && !is_mlxtran && is.null(output_vars)) {
    stop("'output_vars' is required when converting to MLXTRAN (e.g., output_vars = c(\"Cc_mgL\"))")
  }

  if (to == "rxode" && is_mlxtran) {
    result <- .tr_mlxtran_to_rxode(lines)
  } else if (to == "mlxtran" && !is_mlxtran) {
    result <- .tr_rxode_to_mlxtran(lines, dm_list, regressors, output_vars, macros, stiff)
  } else {
    stop("Source format already matches target or format is unrecognized")
  }

  output <- paste(result, collapse = "\n")
  writeLines(output, output_path)
  invisible(output)
}


# =============================================================================
# COMMENT AND LINE CONVERSION HELPERS
# =============================================================================

.tr_comment_mlx_to_rx <- function(line) {
  line %>%
    stringr::str_replace(";;;", "###") %>%
    stringr::str_replace(";;", "##") %>%
    stringr::str_replace(";", "#")
}

.tr_comment_rx_to_mlx <- function(line) {
  line %>%
    stringr::str_replace("###", ";;;") %>%
    stringr::str_replace("##", ";;") %>%
    stringr::str_replace("#", ";")
}

.tr_mlx_code_to_rx <- function(line, add_semicolon = TRUE) {
  trimmed <- stringr::str_trim(line)
  if (trimmed == "") return("")

  sc_pos <- stringr::str_locate(trimmed, ";")[1, "start"]

  if (!is.na(sc_pos) && sc_pos == 1) {
    return(.tr_comment_mlx_to_rx(trimmed))
  }

  if (!is.na(sc_pos)) {
    code_part <- stringr::str_sub(trimmed, 1, sc_pos - 1) %>% stringr::str_trim("right")
    comment_part <- stringr::str_sub(trimmed, sc_pos) %>% .tr_comment_mlx_to_rx()
    sep <- if (add_semicolon) ";\t\t\t\t" else "\t\t\t\t"
    return(paste0(code_part, sep, comment_part))
  }

  if (add_semicolon) {
    return(paste0(trimmed, ";"))
  }
  trimmed
}

.tr_rx_code_to_mlx <- function(line) {
  trimmed <- stringr::str_trim(line)
  if (trimmed == "") return("")

  if (stringr::str_detect(trimmed, "^#")) {
    return(.tr_comment_rx_to_mlx(trimmed))
  }

  hash_pos <- stringr::str_locate(trimmed, "#")[1, "start"]

  if (!is.na(hash_pos) && hash_pos > 1) {
    code_part <- stringr::str_sub(trimmed, 1, hash_pos - 1) %>%
      stringr::str_trim("right") %>%
      stringr::str_remove(";$")
    comment_part <- stringr::str_sub(trimmed, hash_pos) %>%
      .tr_comment_rx_to_mlx()
    return(paste0(code_part, "\t\t\t\t", comment_part))
  }

  stringr::str_remove(trimmed, ";\\s*$")
}

.tr_extract_macro_arg <- function(macro_str, arg_name) {
  pattern <- paste0("(?:,|\\()\\s*", arg_name, "\\s*=\\s*")
  m <- regexpr(pattern, macro_str, perl = TRUE)
  if (m == -1) return(NA_character_)

  start <- m + attr(m, "match.length")
  depth <- 0
  chars <- strsplit(macro_str, "")[[1]]
  end <- start

  for (j in start:length(chars)) {
    ch <- chars[j]
    if (ch == "(") depth <- depth + 1
    if (ch == ")") {
      if (depth == 0) { end <- j - 1; break }
      depth <- depth - 1
    }
    if (ch == "," && depth == 0) { end <- j - 1; break }
    end <- j
  }

  stringr::str_trim(substr(macro_str, start, end))
}

.tr_extract_rhs <- function(line_trimmed) {
  full_rhs <- stringr::str_match(line_trimmed, "=\\s*(.*)")[, 2]
  hash_pos <- stringr::str_locate(full_rhs, "#")[1, "start"]
  sc_pos <- stringr::str_locate(full_rhs, ";")[1, "start"]

  if (!is.na(hash_pos) && hash_pos > 1) {
    rhs <- stringr::str_sub(full_rhs, 1, hash_pos - 1) %>%
      stringr::str_trim() %>% stringr::str_remove(";$") %>% stringr::str_trim()
    comment <- stringr::str_sub(full_rhs, hash_pos)
  } else if (!is.na(sc_pos)) {
    rhs <- stringr::str_sub(full_rhs, 1, sc_pos - 1) %>% stringr::str_trim()
    remaining <- stringr::str_sub(full_rhs, sc_pos + 1) %>% stringr::str_trim()
    comment <- if (nchar(remaining) > 0) paste0("; ", remaining) else ""
  } else {
    rhs <- stringr::str_trim(full_rhs)
    comment <- ""
  }

  list(rhs = rhs, comment = comment)
}


# =============================================================================
# MLXTRAN → RXODE2 CONVERSION
# =============================================================================

.tr_mlxtran_to_rxode <- function(lines) {
  n <- length(lines)

  idx_desc <- which(stringr::str_detect(lines, "^\\s*DESCRIPTION:"))[1]
  idx_long <- which(stringr::str_detect(lines, "^\\s*\\[LONGITUDINAL\\]"))[1]
  idx_pk   <- which(stringr::str_detect(lines, "^\\s*PK:"))[1]
  idx_eq   <- which(stringr::str_detect(lines, "^\\s*EQUATION:"))[1]
  idx_out  <- which(stringr::str_detect(lines, "^\\s*OUTPUT:"))[1]

  # --- Description ---
  desc_rx <- character(0)
  if (!is.na(idx_desc)) {
    desc_end <- ifelse(!is.na(idx_long), idx_long - 1, n)
    if (idx_desc + 1 <= desc_end) {
      desc_raw <- lines[(idx_desc + 1):desc_end]
      desc_raw <- desc_raw[stringr::str_trim(desc_raw) != ""]
      desc_rx <- vapply(desc_raw, .tr_comment_mlx_to_rx, character(1), USE.NAMES = FALSE)
    }
  }

  # --- Input parameters and regressors ---
  input_idx <- which(stringr::str_detect(lines, "^\\s*input\\s*="))[1]
  input_str <- stringr::str_extract(lines[input_idx], "\\{[^}]+\\}") %>%
    stringr::str_remove_all("[{}]") %>%
    stringr::str_split(",") %>% .[[1]] %>% stringr::str_trim()

  reg_idx <- which(stringr::str_detect(lines, "=\\s*\\{\\s*use\\s*=\\s*regressor"))
  reg_vars <- if (length(reg_idx) > 0) {
    stringr::str_extract(lines[reg_idx], "^\\s*\\w+") %>% stringr::str_trim()
  } else {
    character(0)
  }
  model_params <- setdiff(input_str, reg_vars)

  # --- Parse PK section ---
  compartments <- list()
  iv_macros <- list()
  pk_equations <- character(0)
  pkmodel_line <- NULL
  pkmodel_output_var <- NULL
  absorption_macros <- list()
  peripheral_macros <- list()
  elimination_macros <- list()

  if (!is.na(idx_pk)) {
    pk_end <- min(c(idx_eq, idx_out, n + 1)[!is.na(c(idx_eq, idx_out, n + 1))]) - 1
    pk_raw <- lines[(idx_pk + 1):pk_end]

    for (pl in pk_raw) {
      pl_t <- stringr::str_trim(pl)
      if (pl_t == "" || stringr::str_detect(pl_t, "^;")) next

      if (stringr::str_detect(pl_t, "^compartment\\s*\\(")) {
        cmt <- as.integer(stringr::str_extract(pl_t, "(?<=cmt\\s{0,5}=\\s{0,5})\\d+"))
        amt <- stringr::str_match(pl_t, "amount\\s*=\\s*(\\w+)")[, 2]
        vol <- stringr::str_match(pl_t, "volume\\s*=\\s*(\\w+)")[, 2]
        conc <- stringr::str_match(pl_t, "concentration\\s*=\\s*(\\w+)")[, 2]
        compartments[[length(compartments) + 1]] <- list(
          cmt = cmt, amount = amt, volume = vol, concentration = conc
        )

      } else if (stringr::str_detect(pl_t, "^iv\\s*\\(")) {
        macro_part <- stringr::str_remove(pl_t, ";.*$")
        cmt <- as.integer(stringr::str_extract(macro_part, "(?<=cmt\\s{0,5}=\\s{0,5})\\d+"))
        adm <- as.integer(stringr::str_extract(macro_part, "(?<=adm\\s{0,5}=\\s{0,5})\\d+"))
        p_val <- .tr_extract_macro_arg(macro_part, "p")
        tlag_val <- .tr_extract_macro_arg(macro_part, "Tlag")
        inline_comment <- stringr::str_extract(pl_t, ";.*$")
        iv_macros[[length(iv_macros) + 1]] <- list(
          cmt = cmt, adm = adm, p = p_val, Tlag = tlag_val, comment = inline_comment
        )

      } else if (stringr::str_detect(pl_t, "pkmodel\\s*\\(")) {
        pkmodel_line <- pl_t
        pkmodel_output_var <- stringr::str_extract(pl_t, "^\\w+")

      } else if (stringr::str_detect(pl_t, "^absorption\\s*\\(")) {
        cmt <- as.integer(stringr::str_extract(pl_t, "(?<=cmt\\s{0,5}=\\s{0,5})\\d+"))
        tk0 <- .tr_extract_macro_arg(pl_t, "Tk0")
        ka_val <- .tr_extract_macro_arg(pl_t, "ka")
        tlag_val <- .tr_extract_macro_arg(pl_t, "Tlag")
        p_val <- .tr_extract_macro_arg(pl_t, "p")
        absorption_macros[[length(absorption_macros) + 1]] <- list(
          cmt = cmt,
          Tk0 = if (!is.na(tk0)) tk0 else NULL,
          ka  = if (!is.na(ka_val)) ka_val else NULL,
          Tlag = if (!is.na(tlag_val)) tlag_val else NULL,
          p   = if (!is.na(p_val)) p_val else NULL
        )

      } else if (stringr::str_detect(pl_t, "^peripheral\\s*\\(")) {
        args <- stringr::str_extract(pl_t, "\\([^)]+\\)") %>%
          stringr::str_remove_all("[()]") %>%
          stringr::str_split(",") %>% .[[1]] %>% stringr::str_trim()
        peripheral_macros[[length(peripheral_macros) + 1]] <- list(k12 = args[1], k21 = args[2])

      } else if (stringr::str_detect(pl_t, "^elimination\\s*\\(")) {
        cmt <- as.integer(stringr::str_extract(pl_t, "(?<=cmt\\s{0,5}=\\s{0,5})\\d+"))
        inner <- stringr::str_extract(pl_t, "\\([^)]+\\)") %>% stringr::str_remove_all("[()]")
        parts <- stringr::str_split(inner, ",")[[1]] %>% stringr::str_trim()
        k_val <- parts[!stringr::str_detect(parts, "cmt")]
        if (length(k_val) > 0) {
          k_val <- stringr::str_remove(k_val, "^k\\s*=\\s*") %>% stringr::str_trim()
        }
        elimination_macros[[length(elimination_macros) + 1]] <- list(cmt = cmt, k = k_val)

      } else {
        pk_equations <- c(pk_equations, pl_t)
      }
    }
  }

  # --- Parse EQUATION section ---
  eq_raw <- character(0)
  if (!is.na(idx_eq)) {
    eq_end <- ifelse(!is.na(idx_out), idx_out - 1, n)
    eq_raw <- lines[(idx_eq + 1):eq_end]
  }

  # --- Parse OUTPUT ---
  output_line <- NULL
  if (!is.na(idx_out)) {
    out_lines <- lines[(idx_out + 1):n]
    out_match <- out_lines[stringr::str_detect(out_lines, "output\\s*=")]
    if (length(out_match) > 0) {
      braced <- stringr::str_extract(out_match[1], "\\{[^}]+\\}")
      if (!is.na(braced)) {
        ov <- stringr::str_remove_all(braced, "[{}]") %>% stringr::str_trim()
      } else {
        ov <- stringr::str_match(out_match[1], "output\\s*=\\s*(.+)")[, 2] %>%
          stringr::str_trim()
      }
      output_line <- paste0("# output = {", ov, "}")
    }
  }

  # --- Build rxode2 output ---
  result <- character(0)

  if (length(desc_rx) > 0) result <- c(result, desc_rx, "")

  result <- c(result, "# [INPUT]")
  for (p in model_params) result <- c(result, paste0(p, " = 1;"))
  result <- c(result, "")

  result <- c(result, "# [MODEL]")

  if (!is.null(pkmodel_line)) {
    result <- c(result, .tr_expand_pkmodel(pkmodel_line, pkmodel_output_var,
                                           pk_equations, eq_raw))
  } else if (length(absorption_macros) > 0) {
    result <- c(result, .tr_expand_type2(
      absorption_macros, peripheral_macros, elimination_macros,
      pk_equations, eq_raw, compartments
    ))
  } else {
    for (eq in pk_equations) result <- c(result, .tr_mlx_code_to_rx(eq))
    result <- c(result, .tr_convert_eq_section(eq_raw))

    f_alag <- character(0)
    for (iv in iv_macros) {
      comp_idx <- which(vapply(compartments, function(x) x$cmt, integer(1)) == iv$cmt)
      if (length(comp_idx) == 0) next
      state_var <- compartments[[comp_idx[1]]]$amount

      if (!is.na(iv$p)) {
        cmt_str <- ""
        if (!is.na(iv$comment)) {
          cmt_str <- paste0(" ", .tr_comment_mlx_to_rx(iv$comment))
        }
        f_alag <- c(f_alag, paste0("f(", state_var, ") = ", iv$p, ";", cmt_str))
      }
      if (!is.na(iv$Tlag)) {
        f_alag <- c(f_alag, paste0("alag(", state_var, ") = ", iv$Tlag, ";"))
      }
    }
    if (length(f_alag) > 0) result <- c(result, "", f_alag)
  }

  if (!is.null(output_line)) {
    result <- c(result, "", "# [OUTPUT]", output_line)
  }

  result
}


# =============================================================================
# EQUATION SECTION CONVERTER (MLXTRAN → RXODE2)
# =============================================================================

.tr_convert_eq_section <- function(eq_lines) {
  result <- character(0)
  i <- 1
  n <- length(eq_lines)

  ddt_states <- stringr::str_extract(
    eq_lines[stringr::str_detect(eq_lines, "^\\s*ddt_\\w+")],
    "(?<=ddt_)\\w+"
  )

  while (i <= n) {
    trimmed <- stringr::str_trim(eq_lines[i])

    if (trimmed == "") {
      result <- c(result, "")
      i <- i + 1
      next
    }

    if (stringr::str_detect(trimmed, "^\\s*odeType\\s*=")) {
      i <- i + 1
      next
    }

    if (stringr::str_detect(trimmed, "^if\\s+") &&
        !stringr::str_detect(trimmed, "^if\\s*\\(")) {
      block <- .tr_parse_mlx_if_block(eq_lines, i)
      result <- c(result, block$rx_lines)
      i <- block$next_i
      next
    }

    if (stringr::str_detect(trimmed, "^ddt_\\w+\\s*=")) {
      state <- stringr::str_extract(trimmed, "(?<=^ddt_)\\w+")
      rhs_raw <- stringr::str_match(trimmed, "=\\s*(.*)")[, 2]
      rhs_converted <- .tr_mlx_code_to_rx(paste0("X = ", rhs_raw)) %>%
        stringr::str_remove("^X = ")
      result <- c(result, paste0("d/dt(", state, ") = ", rhs_converted))
      i <- i + 1
      next
    }

    if (stringr::str_detect(trimmed, "^\\w+_0\\s*=")) {
      var_name <- stringr::str_extract(trimmed, "^\\w+(?=_0\\s*=)")
      if (var_name %in% ddt_states) {
        rhs_raw <- stringr::str_match(trimmed, "_0\\s*=\\s*(.*)")[, 2]
        rhs_converted <- .tr_mlx_code_to_rx(paste0("X = ", rhs_raw)) %>%
          stringr::str_remove("^X = ")
        result <- c(result, paste0(var_name, "(0) = ", rhs_converted))
        i <- i + 1
        next
      }
    }

    result <- c(result, .tr_mlx_code_to_rx(trimmed))
    i <- i + 1
  }

  result
}


# =============================================================================
# IF/ELSE BLOCK CONVERTERS
# =============================================================================

.tr_parse_mlx_if_block <- function(lines, start_i) {
  n <- length(lines)
  condition <- stringr::str_trim(lines[start_i]) %>% stringr::str_remove("^if\\s+")

  i <- start_i + 1
  if_body <- character(0)
  else_body <- character(0)
  has_else <- FALSE

  while (i <= n) {
    lt <- stringr::str_trim(lines[i])
    if (lt == "else") { has_else <- TRUE; i <- i + 1; next }
    if (lt == "end")  { i <- i + 1; break }
    if (!has_else) {
      if_body <- c(if_body, lt)
    } else {
      else_body <- c(else_body, lt)
    }
    i <- i + 1
  }

  rx <- character(0)
  if (length(if_body) == 1 && length(else_body) <= 1) {
    rx <- c(rx, paste0("if(", condition, ")"))
    rx <- c(rx, paste0("{ ", .tr_mlx_code_to_rx(if_body[1], add_semicolon = FALSE), " }"))
    if (has_else && length(else_body) == 1) {
      rx <- c(rx, "else")
      rx <- c(rx, paste0("{ ", .tr_mlx_code_to_rx(else_body[1], add_semicolon = FALSE), " };"))
    }
  } else {
    rx <- c(rx, paste0("if(", condition, ") {"))
    for (bl in if_body) rx <- c(rx, paste0("  ", .tr_mlx_code_to_rx(bl, add_semicolon = FALSE)))
    rx <- c(rx, "}")
    if (has_else) {
      rx <- c(rx, "else {")
      for (bl in else_body) rx <- c(rx, paste0("  ", .tr_mlx_code_to_rx(bl, add_semicolon = FALSE)))
      rx <- c(rx, "};")
    }
  }

  list(rx_lines = rx, next_i = i)
}

.tr_convert_inline_if_rx_to_mlx <- function(line) {
  var_name <- stringr::str_extract(line, "^\\w+")
  cond <- stringr::str_match(line, "if\\s*\\((.+?)\\)\\s*\\{")[, 2]
  if_expr <- stringr::str_match(line, "\\{\\s*([^}]+?)\\s*\\}")[, 2] %>%
    stringr::str_trim()
  else_expr <- stringr::str_match(line, "else\\s*\\{\\s*([^}]+?)\\s*\\}")[, 2] %>%
    stringr::str_trim()

  c(
    paste0("if ", cond),
    paste0("  ", var_name, " = ", if_expr),
    "else",
    paste0("  ", var_name, " = ", else_expr),
    "end"
  )
}


# =============================================================================
# PKMODEL MACRO EXPANSION (MLXTRAN → RXODE2)
# =============================================================================

.tr_expand_pkmodel <- function(pkmodel_line, output_var, pk_equations, eq_raw) {
  args_str <- stringr::str_extract(pkmodel_line, "(?<=pkmodel\\s{0,3}\\().*(?=\\))") %>%
    stringr::str_trim()
  all_args <- stringr::str_split(args_str, ",")[[1]] %>% stringr::str_trim()

  named_args <- all_args[stringr::str_detect(all_args, "=")]
  positional <- all_args[!stringr::str_detect(all_args, "=")]

  p_val <- NULL
  for (na in named_args) {
    if (stringr::str_detect(na, "^p\\s*=")) {
      p_val <- stringr::str_remove(na, "^p\\s*=\\s*") %>% stringr::str_trim()
    }
  }

  n_pos <- length(positional)
  has_tlag <- n_pos %in% c(4, 6)
  is_2c <- n_pos >= 5

  tlag_var <- NULL
  if (is_2c) {
    if (has_tlag) {
      tlag_var <- positional[1]; ka <- positional[2]; v <- positional[3]
      cl <- positional[4]; k12 <- positional[5]; k21 <- positional[6]
    } else {
      ka <- positional[1]; v <- positional[2]; cl <- positional[3]
      k12 <- positional[4]; k21 <- positional[5]
    }
  } else {
    if (has_tlag) {
      tlag_var <- positional[1]; ka <- positional[2]; v <- positional[3]; cl <- positional[4]
    } else {
      ka <- positional[1]; v <- positional[2]; cl <- positional[3]
    }
  }

  result <- character(0)

  for (eq in pk_equations) result <- c(result, .tr_mlx_code_to_rx(eq))

  eq_converted <- .tr_convert_eq_section(eq_raw)
  if (length(eq_converted) > 0) {
    eq_converted <- eq_converted[stringr::str_trim(eq_converted) != ""]
    if (length(eq_converted) > 0) result <- c(result, eq_converted)
  }

  result <- c(result, paste0(output_var, " = Ac/", v, ";"))

  result <- c(result, "", "### Initial conditions")
  result <- c(result, "Ad(0) = 0;")
  result <- c(result, "Ac(0) = 0;")
  if (is_2c) result <- c(result, "Ap(0) = 0;")

  result <- c(result, "", "### Differential equations")
  result <- c(result, paste0("d/dt(Ad) = -", ka, "*Ad;"))
  if (is_2c) {
    result <- c(result, paste0("d/dt(Ac) = ", ka, "*Ad - ", cl, "*Ac/", v,
                               " - ", k12, "*Ac + ", k21, "*Ap;"))
    result <- c(result, paste0("d/dt(Ap) = ", k12, "*Ac - ", k21, "*Ap;"))
  } else {
    result <- c(result, paste0("d/dt(Ac) = ", ka, "*Ad - ", cl, "*Ac/", v, ";"))
  }

  if (!is.null(p_val)) result <- c(result, "", paste0("f(Ad) = ", p_val, ";"))
  if (!is.null(tlag_var)) result <- c(result, paste0("alag(Ad) = ", tlag_var, ";"))

  result
}


# =============================================================================
# MACROS TYPE 2 EXPANSION (MLXTRAN → RXODE2)
# =============================================================================

.tr_expand_type2 <- function(absorption_macros, peripheral_macros, elimination_macros,
                             pk_equations, eq_raw, compartments) {
  result <- character(0)

  for (eq in pk_equations) result <- c(result, .tr_mlx_code_to_rx(eq))

  if (length(compartments) > 0) {
    comp1 <- compartments[[1]]
    if (!is.na(comp1$concentration) && !is.na(comp1$volume)) {
      result <- c(result, paste0(comp1$concentration, " = Ac/", comp1$volume, ";"))
    }
  }

  eq_converted <- .tr_convert_eq_section(eq_raw)
  if (length(eq_converted) > 0) result <- c(result, eq_converted)

  depot_names <- character(0)
  central_inflow <- character(0)
  f_lines_out <- character(0)
  alag_lines_out <- character(0)
  zero_order_defs <- character(0)
  depot_odes <- character(0)

  for (j in seq_along(absorption_macros)) {
    am <- absorption_macros[[j]]

    if (!is.null(am$Tk0)) {
      dn <- paste0("Ad", j - 1)
      depot_names <- c(depot_names, dn)
      rate_var <- paste0("ka", j - 1)
      dose_expr <- if (!is.null(am$p)) paste0("amtDose*", am$p) else "amtDose"
      zero_order_defs <- c(zero_order_defs,
        paste0(rate_var, " = if(", dn, " > 0){(", dose_expr, ")/", am$Tk0, "} else {0};"))
      central_inflow <- c(central_inflow, rate_var)
      depot_odes <- c(depot_odes, paste0("d/dt(", dn, ") = -", rate_var, ";"))
      if (!is.null(am$p)) f_lines_out <- c(f_lines_out, paste0("f(", dn, ") = ", am$p, ";"))
      if (!is.null(am$Tlag)) alag_lines_out <- c(alag_lines_out, paste0("alag(", dn, ") = ", am$Tlag, ";"))

    } else if (!is.null(am$ka)) {
      dn <- "Ad"
      depot_names <- c(depot_names, dn)
      central_inflow <- c(central_inflow, paste0(am$ka, "*", dn))
      depot_odes <- c(depot_odes, paste0("d/dt(", dn, ") = -", am$ka, "*", dn, ";"))
      if (!is.null(am$p)) f_lines_out <- c(f_lines_out, paste0("f(", dn, ") = ", am$p, ";"))
      if (!is.null(am$Tlag)) alag_lines_out <- c(alag_lines_out, paste0("alag(", dn, ") = ", am$Tlag, ";"))
    }
  }

  elim_term <- ""
  if (length(elimination_macros) > 0) {
    elim_term <- paste0(" - ", elimination_macros[[1]]$k, "*Ac")
  }

  periph_central <- ""
  periph_ode <- character(0)
  if (length(peripheral_macros) > 0) {
    pm <- peripheral_macros[[1]]
    periph_central <- paste0(" - ", pm$k12, "*Ac + ", pm$k21, "*Ap")
    periph_ode <- paste0("d/dt(Ap) = ", pm$k12, "*Ac - ", pm$k21, "*Ap;")
  }

  if (length(zero_order_defs) > 0) result <- c(result, zero_order_defs)

  result <- c(result, "", "### Initial conditions")
  for (dn in depot_names) result <- c(result, paste0(dn, "(0) = 0;"))
  result <- c(result, "Ac(0) = 0;")
  if (length(peripheral_macros) > 0) result <- c(result, "Ap(0) = 0;")

  result <- c(result, "", "### Differential equations")
  result <- c(result, depot_odes)

  inflow_str <- paste(central_inflow, collapse = " + ")
  result <- c(result, paste0("d/dt(Ac) = ", inflow_str, elim_term, periph_central, ";"))
  if (length(periph_ode) > 0) result <- c(result, periph_ode)

  if (length(f_lines_out) > 0 || length(alag_lines_out) > 0) {
    result <- c(result, "", f_lines_out, alag_lines_out)
  }

  result
}


# =============================================================================
# RXODE2 → MLXTRAN CONVERSION
# =============================================================================

.tr_rxode_to_mlxtran <- function(lines, dm_list, regressors, output_vars, macros, stiff) {
  n <- length(lines)

  idx_input  <- which(stringr::str_detect(lines, "^\\s*#?\\s*\\[INPUT\\]"))[1]
  idx_model  <- which(stringr::str_detect(lines, "^\\s*#?\\s*\\[MODEL\\]"))[1]
  idx_output <- which(stringr::str_detect(lines, "^\\s*#?\\s*\\[OUTPUT\\]"))[1]

  # --- Description ---
  desc_mlx <- character(0)
  if (!is.na(idx_input) && idx_input > 1) {
    desc_raw <- lines[1:(idx_input - 1)]
    desc_raw <- desc_raw[stringr::str_trim(desc_raw) != ""]
    if (length(desc_raw) > 0) {
      desc_mlx <- vapply(desc_raw, .tr_comment_rx_to_mlx, character(1), USE.NAMES = FALSE)
    }
  }

  # --- Input parameters ---
  model_params <- character(0)
  if (!is.na(idx_input) && !is.na(idx_model)) {
    input_lines <- lines[(idx_input + 1):(idx_model - 1)]
    for (il in input_lines) {
      il_t <- stringr::str_trim(il)
      if (il_t == "" || stringr::str_detect(il_t, "^#")) next
      pname <- stringr::str_extract(il_t, "^\\w+")
      if (!is.na(pname)) model_params <- c(model_params, pname)
    }
  }

  # --- Model section ---
  model_end <- ifelse(!is.na(idx_output), idx_output - 1, n)
  model_section <- lines[(idx_model + 1):model_end]

  # First pass: extract structure
  ode_states <- character(0)
  f_map <- list()
  alag_map <- list()
  f_comments <- list()

  for (line in model_section) {
    trimmed <- stringr::str_trim(line)

    if (stringr::str_detect(trimmed, "^d/dt\\(\\w+\\)\\s*=")) {
      state <- stringr::str_extract(trimmed, "(?<=d/dt\\()\\w+")
      ode_states <- c(ode_states, state)
    }

    if (stringr::str_detect(trimmed, "^f\\(\\w+\\)\\s*=")) {
      state <- stringr::str_extract(trimmed, "(?<=f\\()\\w+")
      parsed <- .tr_extract_rhs(trimmed)
      f_map[[state]] <- parsed$rhs
      f_comments[[state]] <- parsed$comment
    }

    if (stringr::str_detect(trimmed, "^alag\\(\\w+\\)\\s*=")) {
      state <- stringr::str_extract(trimmed, "(?<=alag\\()\\w+")
      parsed <- .tr_extract_rhs(trimmed)
      alag_map[[state]] <- parsed$rhs
    }
  }

  # Check pkmodel eligibility
  use_pkmodel <- macros && length(ode_states) %in% c(2, 3) &&
    .tr_can_use_pkmodel(ode_states, model_section)

  # Second pass: convert model body lines for EQUATION section
  eq_body <- character(0)
  i <- 1
  n_model <- length(model_section)

  while (i <= n_model) {
    trimmed <- stringr::str_trim(model_section[i])

    if (trimmed == "") { eq_body <- c(eq_body, ""); i <- i + 1; next }

    # Skip f() and alag() (handled via iv macros)
    if (stringr::str_detect(trimmed, "^f\\(\\w+\\)\\s*=") ||
        stringr::str_detect(trimmed, "^alag\\(\\w+\\)\\s*=")) {
      i <- i + 1
      next
    }

    # d/dt(X) = expr; → ddt_X = expr
    if (stringr::str_detect(trimmed, "^d/dt\\(\\w+\\)\\s*=")) {
      state <- stringr::str_extract(trimmed, "(?<=d/dt\\()\\w+")
      rhs_raw <- stringr::str_match(trimmed, "=\\s*(.*)")[, 2]
      rhs_mlx <- .tr_rx_code_to_mlx(paste0("X = ", rhs_raw)) %>%
        stringr::str_remove("^X = ")
      eq_body <- c(eq_body, paste0("ddt_", state, " = ", rhs_mlx))
      i <- i + 1
      next
    }

    # X(0) = expr; → X_0 = expr
    if (stringr::str_detect(trimmed, "^\\w+\\(0\\)\\s*=")) {
      state <- stringr::str_extract(trimmed, "^\\w+(?=\\(0\\))")
      rhs_raw <- stringr::str_match(trimmed, "=\\s*(.*)")[, 2]
      rhs_mlx <- .tr_rx_code_to_mlx(paste0("X = ", rhs_raw)) %>%
        stringr::str_remove("^X = ")
      eq_body <- c(eq_body, paste0(state, "_0 = ", rhs_mlx))
      i <- i + 1
      next
    }

    # Inline if: var = if(cond){expr} else {expr};
    if (stringr::str_detect(trimmed, "^\\w+\\s*=\\s*if\\s*\\(")) {
      eq_body <- c(eq_body, .tr_convert_inline_if_rx_to_mlx(trimmed))
      i <- i + 1
      next
    }

    # Multi-line if(condition) block
    if (stringr::str_detect(trimmed, "^if\\s*\\(") &&
        !stringr::str_detect(trimmed, "^\\w+\\s*=\\s*if")) {
      block <- .tr_parse_rx_if_block(model_section, i)
      eq_body <- c(eq_body, block$mlx_lines)
      i <- block$next_i
      next
    }

    # Regular line
    eq_body <- c(eq_body, .tr_rx_code_to_mlx(model_section[i]))
    i <- i + 1
  }

  # output_vars is passed from the caller and validated there

  # --- Build MLXTRAN output ---
  result <- character(0)

  if (length(desc_mlx) > 0) result <- c(result, "DESCRIPTION:", desc_mlx, "")

  result <- c(result, "[LONGITUDINAL]")
  all_input <- model_params
  if (!is.null(regressors)) all_input <- c(all_input, regressors)
  result <- c(result, paste0("input = {", paste(all_input, collapse = ", "), "}"))
  if (!is.null(regressors)) {
    for (r in regressors) result <- c(result, paste0(r, " = {use = regressor}"))
  }
  result <- c(result, "")

  if (use_pkmodel) {
    result <- c(result, "PK:")
    result <- c(result, .tr_build_pkmodel(ode_states, model_section, f_map,
                                          alag_map, eq_body))
  } else {
    result <- c(result, "PK:")
    for (j in seq_along(ode_states)) {
      result <- c(result, paste0("compartment(cmt = ", j, ", amount = ",
                                 ode_states[j], ")"))
    }

    if (!is.null(dm_list)) {
      result <- c(result, "")
      for (j in seq_along(dm_list$cmt)) {
        cmt_num <- dm_list$cmt[j]
        adm_num <- dm_list$adm[j]
        state <- ode_states[cmt_num]

        iv_str <- paste0("iv(cmt = ", cmt_num, ", adm = ", adm_num)
        if (!is.null(f_map[[state]])) iv_str <- paste0(iv_str, ", p = ", f_map[[state]])
        if (!is.null(alag_map[[state]])) iv_str <- paste0(iv_str, ", Tlag = ",
                                                          alag_map[[state]])
        iv_str <- paste0(iv_str, ")")

        if (!is.null(f_comments[[state]]) && f_comments[[state]] != "") {
          iv_str <- paste0(iv_str, "\t\t", .tr_comment_rx_to_mlx(f_comments[[state]]))
        }
        result <- c(result, iv_str)
      }
    }

    result <- c(result, "", "")
    result <- c(result, "EQUATION:")
    if (stiff) result <- c(result, "odeType = stiff")
    result <- c(result, eq_body)
  }

  result <- c(result, "", "OUTPUT:")
  result <- c(result, paste0("output = {", paste(output_vars, collapse = ", "), "}"))

  result
}


# =============================================================================
# MULTI-LINE IF/ELSE PARSER (RXODE2 → MLXTRAN)
# =============================================================================

.tr_parse_rx_if_block <- function(lines, start_i) {
  n <- length(lines)
  trimmed <- stringr::str_trim(lines[start_i])
  condition <- stringr::str_match(trimmed, "^if\\s*\\((.+?)\\)")[, 2]

  i <- start_i + 1
  if_body <- character(0)
  else_body <- character(0)
  has_else <- FALSE

  while (i <= n) {
    lt <- stringr::str_trim(lines[i])

    if (lt == "else" || stringr::str_detect(lt, "^\\}\\s*else")) {
      has_else <- TRUE
      i <- i + 1
      next
    }

    content <- lt %>%
      stringr::str_remove("^\\{\\s*") %>%
      stringr::str_remove("\\s*\\}\\s*;?$") %>%
      stringr::str_trim()

    has_closing_brace <- stringr::str_detect(lt, "\\}\\s*;?\\s*$")

    if (content != "" && content != "{" && content != "}") {
      if (!has_else) {
        if_body <- c(if_body, content)
      } else {
        else_body <- c(else_body, content)
      }
    }

    i <- i + 1
    if (has_closing_brace && has_else) break
    if (has_closing_brace && !has_else) {
      if (i <= n && stringr::str_detect(stringr::str_trim(lines[i]), "^else")) next
      break
    }
  }

  mlx <- c(paste0("if ", condition))
  for (bl in if_body) mlx <- c(mlx, paste0("  ", .tr_rx_code_to_mlx(bl)))
  if (has_else) {
    mlx <- c(mlx, "else")
    for (bl in else_body) mlx <- c(mlx, paste0("  ", .tr_rx_code_to_mlx(bl)))
  }
  mlx <- c(mlx, "end")

  list(mlx_lines = mlx, next_i = i)
}


# =============================================================================
# PKMODEL ELIGIBILITY CHECK (RXODE2 → MLXTRAN MACROS)
# =============================================================================

.tr_can_use_pkmodel <- function(ode_states, model_lines) {
  n_states <- length(ode_states)
  if (!(n_states %in% c(2, 3))) return(FALSE)

  ddt_rhs <- character(0)
  for (line in model_lines) {
    trimmed <- stringr::str_trim(line)
    if (stringr::str_detect(trimmed, "^d/dt\\(")) {
      rhs <- stringr::str_match(trimmed, "=\\s*(.*)")[, 2] %>%
        stringr::str_remove("#.*") %>% stringr::str_remove(";") %>%
        stringr::str_trim()
      ddt_rhs <- c(ddt_rhs, rhs)
    }
  }
  if (length(ddt_rhs) != n_states) return(FALSE)

  depot <- ode_states[1]
  if (!stringr::str_detect(ddt_rhs[1], paste0("^-\\s*\\w+\\s*\\*\\s*", depot, "$"))) return(FALSE)

  central <- ode_states[2]
  if (!stringr::str_detect(ddt_rhs[2], paste0("\\w+\\s*\\*\\s*", depot))) return(FALSE)
  if (!stringr::str_detect(ddt_rhs[2], paste0("\\w+\\s*\\*\\s*", central, "\\s*/\\s*\\w+"))) return(FALSE)

  if (n_states == 3) {
    periph <- ode_states[3]
    if (!stringr::str_detect(ddt_rhs[3], paste0("\\w+\\s*\\*\\s*", central))) return(FALSE)
    if (!stringr::str_detect(ddt_rhs[3], paste0("\\w+\\s*\\*\\s*", periph))) return(FALSE)
  }

  TRUE
}


# =============================================================================
# PKMODEL BUILDER (RXODE2 → MLXTRAN MACROS)
# =============================================================================

.tr_build_pkmodel <- function(ode_states, model_section, f_map, alag_map, eq_body) {
  depot <- ode_states[1]
  central <- ode_states[2]

  ka <- NULL; v <- NULL; cl <- NULL; k12 <- NULL; k21 <- NULL

  for (line in model_section) {
    trimmed <- stringr::str_trim(line)
    if (stringr::str_detect(trimmed, paste0("^d/dt\\(", depot, "\\)"))) {
      rhs <- stringr::str_match(trimmed, "=\\s*(.*)")[, 2] %>%
        stringr::str_remove("#.*") %>% stringr::str_remove(";") %>% stringr::str_trim()
      ka <- stringr::str_extract(rhs, "(?<=^-\\s{0,5})\\w+")
      if (is.na(ka)) ka <- stringr::str_extract(rhs, "(?<=^-)\\w+")
    }
    if (stringr::str_detect(trimmed, paste0("^d/dt\\(", central, "\\)"))) {
      rhs <- stringr::str_match(trimmed, "=\\s*(.*)")[, 2] %>%
        stringr::str_remove("#.*") %>% stringr::str_remove(";") %>% stringr::str_trim()
      cl_v_match <- stringr::str_match(rhs, "(\\w+)\\s*\\*\\s*\\w+\\s*/\\s*(\\w+)")
      cl <- cl_v_match[, 2]
      v <- cl_v_match[, 3]
    }
    if (length(ode_states) == 3) {
      periph <- ode_states[3]
      if (stringr::str_detect(trimmed, paste0("^d/dt\\(", periph, "\\)"))) {
        rhs <- stringr::str_match(trimmed, "=\\s*(.*)")[, 2] %>%
          stringr::str_remove("#.*") %>% stringr::str_remove(";") %>% stringr::str_trim()
        pm <- stringr::str_match(rhs, "(\\w+)\\s*\\*\\s*\\w+\\s*-\\s*(\\w+)")
        k12 <- pm[, 2]
        k21 <- pm[, 3]
      }
    }
  }

  tlag_val <- alag_map[[depot]]
  p_val <- f_map[[depot]]

  conc_var <- "Cc"
  if (!is.null(v)) {
    for (line in model_section) {
      trimmed <- stringr::str_trim(line)
      if (stringr::str_detect(trimmed, paste0("^\\w+\\s*=\\s*", central, "\\s*/\\s*", v))) {
        conc_var <- stringr::str_extract(trimmed, "^\\w+")
        break
      }
    }
  }

  result <- character(0)

  # Include non-special equation body lines, skip ODE/init headers and content
  skip_next_empty <- FALSE
  for (idx in seq_along(eq_body)) {
    el_t <- stringr::str_trim(eq_body[idx])

    if (stringr::str_detect(el_t, "^ddt_")) { skip_next_empty <- TRUE; next }
    if (stringr::str_detect(el_t, "^\\w+_0\\s*=")) { skip_next_empty <- TRUE; next }
    if (!is.null(v) && stringr::str_detect(el_t, paste0("^", conc_var, "\\s*=\\s*", central, "\\s*/\\s*", v))) next

    if (stringr::str_detect(el_t, "^;;;\\s*(Initial conditions|Differential equations|ODEs)")) next

    if (el_t == "" && skip_next_empty) { skip_next_empty <- FALSE; next }
    skip_next_empty <- FALSE

    result <- c(result, eq_body[idx])
  }

  # Build pkmodel() call
  pkm_args <- character(0)
  if (!is.null(tlag_val)) pkm_args <- c(pkm_args, tlag_val)
  pkm_args <- c(pkm_args, ka, v, cl)
  if (!is.null(k12)) pkm_args <- c(pkm_args, k12, k21)

  call_str <- paste(pkm_args, collapse = ", ")
  if (!is.null(p_val)) call_str <- paste0(call_str, ", p = ", p_val)

  result <- c(result, paste0(conc_var, " = pkmodel(", call_str, ")"))

  result
}

## Author: Anna Mikhailova
## First created: 2026-05-04
## Description: Basic testing of sg-converter function
## Keywords: SimuRg, sg-converter

test_that("sg-translator from monolix to rxode2 works", {
  path_to_save <- tempdir()
  monolix_model_hv_iv <- system.file("extdata", "models", "monolix",
                                     "model_PK_hv_iv.txt", package = "SimuRg")
  monolix_model_1c <- system.file("extdata", "models", "monolix",
                                  "model_PK_1c.txt", package = "SimuRg")



  # Convert MLXTRAN model to rxode2
  sg_translator(monolix_model_hv_iv, to = "rxode",
                output_path = normalizePath(file.path(path_to_save,
                                                      "model_PK_hv_iv_rx.txt"),
                                             mustWork = FALSE))
  sg_translator(monolix_model_1c, to = "rxode",
                output_path = normalizePath(file.path(path_to_save,
                                                      "model_PK_1c_rx.txt"),
                                            mustWork = FALSE))
  expect_true(file.exists(file.path(path_to_save, "model_PK_hv_iv_rx.txt")))
  expect_true(file.exists(file.path(path_to_save, "model_PK_1c_rx.txt")))
  clr_files <- list.files(path_to_save, full.names = TRUE)
  unlink(clr_files, recursive = TRUE, force = TRUE)
})

## Author: Mikhailova Anna
## First created: 2025-12-03
## Description: functions for vpc visualisation
## Keywords: SimuRg, vpc, diagnostics

add_percentile_lines <- function(plot_i, line_data, ci_data, show_ci, interpolation) {

  if (interpolation) {
    # Use smooth lines
    plot_i <- plot_i +
      geom_line(data = line_data,
                aes(x = TIME_BIN, y = OBS,
                    group = interaction(PI, TYPE), lty = TYPE),
                col = "royalblue", linewidth = 0.8)

    if (show_ci) {
      plot_i <- plot_i +
        geom_ribbon(data = ci_data %>% filter(PI != "median"),
                    aes(x = TIME_BIN, ymin = theor_LCI,
                        ymax = theor_UCI, group = PI,
                        fill = CI_label),
                    col = NA, linewidth = 1, alpha = 0.3) +
        geom_ribbon(data = ci_data %>% filter(PI == "median"),
                    aes(x = TIME_BIN, ymin = theor_LCI,
                        ymax = theor_UCI, group = PI,
                        fill = CI_label),
                    col = NA, linewidth = 1, alpha = 0.3)
    }

  } else {
    # Use step rectangles
    plot_i <- plot_i +
      geom_line(data = line_data,
                aes(x = TIME_BIN, y = OBS,
                    group = interaction(PI, TYPE), lty = TYPE),
                col = "royalblue", linewidth = 0.8)

    if (show_ci) {
      plot_i <- plot_i +
        geom_rect(data = ci_data %>% filter(PI != "median"),
                  aes(xmin = TIME_BIN_min, xmax = TIME_BIN_max,
                      ymin = theor_LCI, ymax = theor_UCI,
                      group = PI, fill = CI_label),
                  col = NA, linewidth = 1, alpha = 0.3) +
        geom_rect(data = ci_data %>% filter(PI == "median"),
                  aes(xmin = TIME_BIN_min, xmax = TIME_BIN_max,
                      ymin = theor_LCI, ymax = theor_UCI,
                      group = PI, fill = CI_label),
                  col = NA, linewidth = 1, alpha = 0.3)
    }
  }

  return(plot_i)
}

create_vpc_plot <- function(pred_int_median_emp_mod,
                            pred_int_ci_mod,
                            data_obs,
                            pred_int_ci,
                            data_i_var,
                            theor_perc,
                            emp_perc,
                            theor_percCI,
                            interpolation,
                            dt_obs_fl,
                            strat_by_dose,
                            log_y,
                            pred.corr,
                            lab_y,
                            lab_x,
                            legend_fl) {

  # Base plot aesthetics
  p_char <- list(
    theme_bw(),
    theme(
      panel.background = element_rect(fill = "transparent", colour = "black"),
      strip.text = element_text(size = 18, colour = "black"),
      plot.title = element_text(size = 18, face = "bold"),
      axis.text = element_text(size = 14),
      axis.title = element_text(size = 18)
    ),
    scale_fill_manual(values = c("royalblue", "firebrick3")),
    scale_linetype_manual(values = c(
      `empirical percentiles` = "solid",
      `theoretical percentiles` = "dashed"
    )),
    scale_x_continuous(name = lab_x)
  )

  # Legend settings
  p_char_leg_T <- list(
    theme(legend.position = c(0.8, 0.85)),
    theme(legend.box.background = element_rect(fill = "white", color = "black", linetype = "solid")),
    theme(legend.box.margin = margin(0.5, 0.5, 0.5, 0.5, "cm")),
    theme(legend.text = element_text(size = 18)),
    theme(legend.title = element_blank()),
    theme(legend.key.size = unit(1, "cm")),
    theme(legend.margin = margin(0, 0, 0, 0, "cm")),
    #theme(legend.margin = unit(0, "cm")),
    labs(lty = "", fill = "", color = "", shape = "")
  )

  p_char_leg_F <- list(theme(legend.position = "none"))

  # Initialize plot
  vpc_plot <- ggplot()

  # Add percentile lines and confidence intervals
  if (theor_perc && emp_perc) {
    vpc_plot <- add_percentile_lines(vpc_plot, pred_int_median_emp_mod,
                                     pred_int_ci_mod, theor_percCI, interpolation)
  } else if (theor_perc) {
    data_filtered <- pred_int_median_emp_mod %>%
      filter(TYPE == "theoretical percentiles")
    vpc_plot <- add_percentile_lines(vpc_plot, data_filtered,
                                     pred_int_ci_mod, theor_percCI, interpolation)
  } else if (emp_perc) {
    data_filtered <- pred_int_median_emp_mod %>%
      filter(TYPE == "empirical percentiles")
    vpc_plot <- add_percentile_lines(vpc_plot, data_filtered,
                                     pred_int_ci_mod, theor_percCI, interpolation)
  }

  # Add stratification by dose
  if (!is.null(strat_by_dose)) {
    vpc_plot <- vpc_plot + facet_grid(~COV_add)
  }

  # Add observed data points
  if (dt_obs_fl) {
    vpc_plot <- vpc_plot +
      geom_point(data = data_obs, aes(x = TIME, y = DV, shape = DT_label),
                 col = "royalblue4", size = 1.5)
  }

  # Y-axis scaling
  y_label <- paste0(if_else(pred.corr, "Prediction corrected ", ""), lab_y)

  x_range <- c(
    min(c(pred_int_ci$TIME_BIN, data_i_var$TIME)),
    max(c(pred_int_ci$TIME_BIN, data_i_var$TIME))
  )

  if (log_y) {
    vpc_plot <- vpc_plot +
      scale_y_log10(
        name = y_label,
        breaks = 10^seq(-2, 4, 1),
        labels = scales::trans_format("log10", scales::math_format(10^.x))
      ) +
      coord_cartesian(xlim = x_range)
  } else {
    vpc_plot <- vpc_plot +
      scale_y_continuous(name = y_label) +
      coord_cartesian(xlim = x_range)
  }

  # Apply styling
  vpc_plot <- vpc_plot + p_char

  # Apply legend settings
  if (legend_fl) {
    vpc_plot <- vpc_plot + p_char_leg_T
  } else {
    vpc_plot <- vpc_plot + p_char_leg_F
  }

  return(vpc_plot)
}

fun_Bin_smrg <- function(ds, n_bins, method = c("kmeans", "ntile", "equal_x", "custom")){

  #browser()

  if(method == "kmeans"){

    if(n_bins > length(unique(ds$TIME))){
      n_bins <- length(unique(ds$TIME))
    }

    ds_bin <- ds %>%
      mutate(BIN = as.factor(kmeans(TIME, n_bins)$cluster))

    sorted_bins <- ds_bin %>%
      group_by(BIN) %>%
      summarise_at(vars(TIME), list(min_BIN_time = ~min(., na.rm=TRUE))) %>%
      ungroup() %>%
      select(BIN, min_BIN_time) %>%
      unique() %>%
      arrange(min_BIN_time) %>%
      mutate(BIN_sort = 1:nrow(.)) %>%
      select(-min_BIN_time)

    ds_bin <- ds_bin %>%
      left_join(sorted_bins, by='BIN') %>%
      select(-BIN) %>%
      rename(BIN=BIN_sort)

  } else if(method == "ntile"){
    ds_bin <- ds %>%
      mutate(BIN = as.factor(ntile(TIME, n_bins)))
  } else if(method == "equal_x"){


    #browser()

    #browser()
    n_bins <- seq(min(ds$TIME, na.rm = TRUE),
                max(ds$TIME, na.rm = TRUE),
                length.out = n_bins + 1)
    n_bins <- n_bins[-1]

    time_bin <- ds %>%
      select(TIME) %>%
      filter(!is.na(TIME)) %>%
      mutate(BIN = 0) %>%
      unique()

    time_bin <- time_bin %>%
      mutate(nrow = 1:nrow(.)) %>%
      group_by(nrow) %>%
      mutate(BIN = min((1:length(n_bins))[TIME <= n_bins], na.rm = TRUE)) %>%
      ungroup() %>%
      select(-nrow)

    time_bin <- time_bin %>%
      mutate(BIN = as.factor(BIN))

    ds_bin <- ds %>%
      left_join(time_bin, by = "TIME")

  } else if(method == "custom"){
    n_bins <- c(n_bins, max(ds$TIME, na.rm = TRUE))

    time_bin <- ds %>%
      select(TIME) %>%
      filter(!is.na(TIME)) %>%
      mutate(BIN = 0)

    for (i in 1:nrow(time_bin)){
      time_bin[i,]$BIN <- min((1:length(n_bins))[time_bin[i,]$TIME <= n_bins], na.rm = TRUE)
    }

    time_bin <- time_bin %>%
      mutate(BIN = as.factor(BIN))

    ds_bin <- ds %>%
      left_join(time_bin, by = "TIME")
  }

  ds_bin <- ds_bin %>%
    group_by(BIN) %>%
    mutate_at(vars(TIME), list(TIME_BIN = ~mean(., na.rm = TRUE))) %>%
    ungroup()

  bin_stats <- ds_bin %>%
    group_by(BIN) %>%
    summarise(TIME_BIN_min = min(TIME),
              TIME_BIN_max = max(TIME)) %>%
    mutate(TIME_BIN_max_prev = lag(TIME_BIN_max),
           TIME_BIN_min_next = lead(TIME_BIN_min)) %>%
    ungroup() %>%
    mutate(TIME_BIN_min = (TIME_BIN_min + TIME_BIN_max_prev) / 2,
           TIME_BIN_max = (TIME_BIN_max + TIME_BIN_min_next) / 2) %>%
    select(-TIME_BIN_max_prev, -TIME_BIN_min_next)

  bin_stats$TIME_BIN_min[is.na(bin_stats$TIME_BIN_min)] <- -Inf
  bin_stats$TIME_BIN_max[is.na(bin_stats$TIME_BIN_max)] <- Inf

  ds_bin <- ds_bin %>%
    left_join(bin_stats, by = "BIN")

  return(ds_bin)
}

#' Visual Predictive Check (VPC) Function
#'
#' Generates VPC plots to assess model performance by comparing observed data
#' with prediction intervals derived from simulated data.
#'
#' @inheritParams sg_dummy
#' @param ds_sim A data frame with simulated data containing columns:
#'   - ID: simulation replicate identifier
#'   - TIME: time points
#'   - VAR: output variable name
#'   - VALUE: simulated values
#' @param data_i A data frame with observed data containing columns:
#'   - ID: patient identifier
#'   - TIME: time points
#'   - DV: observed dependent variable values
#'   - DVID: dependent variable identifier
#' @param output_names A data frame mapping `VAR` values (from `ds_sim`)
#'  to `DVID` values (from `data_i`). Must contain columns:
#'   - output: character, `VAR` values (e.g., "Cc", "Cp")
#'   - dvid: numeric, corresponding DVIDs (e.g., 1, 2)
#' @param lab_x String. X-axis label. Default is "TIME, h".
#' @param lab_y String. Y-axis label. Default is "DV".
#' @param method Character. Binning method: "kmeans", "equal", "quantile" (default: "kmeans").
#' @param n_bins Integer. Number of bins to use in the histogram. Default is 10.
#' @param interpolation Logical. Use line interpolation vs rectangles (default: FALSE).
#' @param strat_by_dose Character. Variable name for dose stratification (default: NULL).
#'
#' @return List of ggplot objects, one for each output variable
#' @details
#' For now, the model in the example is NOT the generalized model object (GMO),
#' as the parameters in generalized fit object are backtransformed, and,
#' therefore, not transformed in the model. This issue will be fixed in the next
#' versions of SimuRg package
#'
#' @examples
#' \donttest{
#' library(rxode2)
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
#' mod <- rxode2::rxode2({
#'   ka_pop = 0.1;
#'   Vd_pop = 10;
#'   Cl_pop = 0.5;
#'
#'   omega_ka = 0;
#'   omega_Vd = 0;
#'   omega_Cl = 0;
#'
#'   Cc_b = 0;
#'   ka_tv = exp(log(ka_pop));
#'   Vd_tv = exp(log(Vd_pop));
#'   Cl_tv = exp(log(Cl_pop));
#'
#'   ka = ka_tv * exp(omega_ka);
#'   Vd = Vd_tv * exp(omega_Vd);
#'   Cl = Cl_tv * exp(omega_Cl);
#'
#'   Cc = Ac / Vd;
#'
#'   Ad(0) = 0;
#'   Ac(0) = 0;
#'
#'   d/dt(Ad) = -ka * Ad;
#'   d/dt(Ac) = ka * Ad - Cl * Cc;
#'
#'   Cc_ResErr = Cc * (1 + Cc_b);
#' })
#'
#' sim_data <- sg_vpc_sim(fpath_i, model=mod, outputs = "Cc_ResErr")
#' outp_nms <- data.frame(dvid = 1, output = "Cc")
#' vpc_plots <- sg_vpc_vis(
#'   ds_sim = sim_data,
#'   data_i = warfarin,
#'   output_names = outp_nms,
#'   lab_x = "Time (hours)",
#'   lab_y = "Concentration (ng/mL)",
#'   n_bins = 8,
#'   pred.corr = TRUE
#' )
#' }
#' @import dplyr tidyr ggplot2 purrr stringr
#' @export
sg_vpc_vis <- function(ds_sim,
                       data_i,
                       output_names,
                       time_col = "TIME",
                       dv_col = "DV",
                       log_y = FALSE,
                       piLow = 0.10,
                       piUp = 0.90,
                       ciLow = 0.025,
                       ciUp = 0.975,
                       pred.corr = FALSE,
                       lab_y = "DV",
                       lab_x = "Time, h",
                       theor_perc = TRUE,
                       theor_percCI = TRUE,
                       emp_perc = TRUE,
                       dt_obs_fl = FALSE,
                       legend_fl = FALSE,
                       n_bins = 10,
                       method = "kmeans",
                       interpolation = TRUE,
                       strat_by_dose = NULL) {

  # Check input dataset for required columns
  required_ds_sim_cols <- c("sim.id", "ID", "TIME", "VAR", "VALUE")
  missing_ds_sim_cols <- setdiff(required_ds_sim_cols, colnames(ds_sim))
  if (length(missing_ds_sim_cols) > 0) {
    stop(
      "ds_sim is missing required columns: ",
      paste(missing_ds_sim_cols, collapse = ", "),
      call. = FALSE
    )
  }

  # Validate output_names: must be provided and contain required columns
  if (is.null(output_names)) {
    stop("output_names must be provided and map VAR to DVID", call. = FALSE)
  }
  required_output_cols <- c("output", "dvid")
  missing_output_cols <- setdiff(required_output_cols, colnames(output_names))
  if (length(missing_output_cols) > 0) {
    stop(
      "output_names is missing required columns: ",
      paste(missing_output_cols, collapse = ", "),
      call. = FALSE
    )
  }

  # Check if the method is valid
  valid_methods <- c("kmeans", "ntile", "equal_x", "custofm")
  if (!method %in% valid_methods) {
    message(sprintf("Invalid binning method. Valid methods are: %s \nUsing default method: kmeans\n", paste(valid_methods, collapse = ", ")))
    method <- "kmeans"
  }

  # Get unique output variables
  unique_vars <- unique(ds_sim$VAR)

  # Generate VPC for each output variable
  vpc_plots <- purrr::map(unique_vars, function(var) {

    # Filter data for current variable
    ds_sim_var <- ds_sim %>%
      filter(VAR == var)

    # Match observed data to simulation variable
    var_clean <- stringr::str_remove(var, "_ResErr")
    target_dvid <- output_names$dvid[output_names$output == var_clean]

    data_i_var <- data_i %>%
      filter(DVID == target_dvid) %>%
      rename(TIME = all_of(time_col), DV = all_of(dv_col))

    # Store original observed data for overlay
    data_obs <- data_i_var %>%
      mutate(DT_label = "Data")

    # ===== TIME BINNING =====
    data_i_var <- fun_Bin_smrg(data_i_var, n_bins = n_bins, method = method)

    # Extract binning assignments
    bin_assignments <- data_i_var %>%
      select(ID, TIME, BIN, TIME_BIN, TIME_BIN_min, TIME_BIN_max)

    # Apply bins to simulated data
    ds_sim_var <- ds_sim_var %>%
      left_join(bin_assignments, by = c("ID", "TIME"))

    # ===== PREDICTION CORRECTION =====
    if (pred.corr) {

      # Calculate population prediction (PPRED) for each individual observation
      ppred <- ds_sim_var %>%
        group_by(ID, TIME) %>%
        summarise(PPRED = mean(VALUE, na.rm = TRUE), .groups = "drop")

      # Calculate mean PPRED per bin
      ppred_bin <- ds_sim_var %>%
        group_by(BIN) %>%
        summarise(PPRED_bin = mean(VALUE, na.rm = TRUE), .groups = "drop")

      # Join PPREDs to observed data
      data_i_var <- data_i_var %>%
        left_join(ppred, by = c("ID", "TIME")) %>%
        left_join(ppred_bin, by = "BIN")

      # Join PPREDs to simulated data
      ds_sim_var <- ds_sim_var %>%
        left_join(ppred, by = c("ID", "TIME")) %>%
        left_join(ppred_bin, by = "BIN")

      # Apply prediction correction: DV_corr = DV * (mean_PPRED_bin / PPRED_ij)
      data_i_var <- data_i_var %>%
        mutate(DV = DV * PPRED_bin / PPRED)

      ds_sim_var <- ds_sim_var %>%
        mutate(VALUE = VALUE * PPRED_bin / PPRED)
    }

    # ===== CALCULATE PERCENTILES =====

    # Define grouping variables
    vec_group <- if (!is.null(strat_by_dose)) {
      c(strat_by_dose, "TIME_BIN", "TIME_BIN_min", "TIME_BIN_max")
    } else {
      c("TIME_BIN", "TIME_BIN_min", "TIME_BIN_max")
    }

    # Summary functions
    summ_pi <- list(
      median = ~median(., na.rm = TRUE),
      pi_l = ~quantile(., piLow, na.rm = TRUE),
      pi_u = ~quantile(., piUp, na.rm = TRUE)
    )

    summ_ci <- list(
      median = ~median(., na.rm = TRUE),
      ci_l = ~quantile(., ciLow, na.rm = TRUE),
      ci_u = ~quantile(., ciUp, na.rm = TRUE)
    )

    # Empirical percentiles from observed data
    emp_int <- data_i_var %>%
      group_by(across(all_of(vec_group))) %>%
      summarise(across(DV, summ_pi), .groups = "drop")
    colnames(emp_int) <- str_remove(colnames(emp_int), "DV_")

    # Prediction intervals from simulated data (per simulation)
    pred_int <- ds_sim_var %>%
      mutate(ID = as.numeric(ID)) %>%
      group_by(across(all_of(c(vec_group, "ID")))) %>%
      summarise(across(VALUE, summ_pi), .groups = "drop")
    colnames(pred_int) <- str_remove(colnames(pred_int), "VALUE_")
    # Confidence intervals around prediction intervals
    conf_int <- pred_int %>%
      group_by(across(all_of(vec_group))) %>%
      summarise(across(c(median, pi_l, pi_u), summ_ci), .groups = "drop")

    # ===== PREPARE DATA FOR PLOTTING =====

    # Theoretical percentiles (median of prediction intervals)
    pred_int_median <- conf_int %>%
      select(all_of(vec_group), median_median, pi_l_median, pi_u_median) %>%
      pivot_longer(
        cols = c("median_median", "pi_l_median", "pi_u_median"),
        values_to = "theor_median",
        names_to = "PI"
      ) %>%
      mutate(PI = case_when(
        str_detect(PI, "pi_l") ~ "lower",
        str_detect(PI, "median_") ~ "median",
        str_detect(PI, "pi_u") ~ "upper"
      ))

    # Lower confidence interval
    pred_int_l_ci <- conf_int %>%
      select(all_of(vec_group), median_ci_l, pi_l_ci_l, pi_u_ci_l) %>%
      pivot_longer(
        cols = c("median_ci_l", "pi_l_ci_l", "pi_u_ci_l"),
        values_to = "theor_LCI",
        names_to = "PI"
      ) %>%
      mutate(PI = case_when(
        str_detect(PI, "pi_l") ~ "lower",
        str_detect(PI, "median_") ~ "median",
        str_detect(PI, "pi_u") ~ "upper"
      ))

    # Upper confidence interval
    pred_int_u_ci <- conf_int %>%
      select(all_of(vec_group), median_ci_u, pi_l_ci_u, pi_u_ci_u) %>%
      pivot_longer(
        cols = c("median_ci_u", "pi_l_ci_u", "pi_u_ci_u"),
        values_to = "theor_UCI",
        names_to = "PI"
      ) %>%
      mutate(PI = case_when(
        str_detect(PI, "pi_l") ~ "lower",
        str_detect(PI, "median_") ~ "median",
        str_detect(PI, "pi_u") ~ "upper"
      ))

    # Empirical percentiles
    emp_w <- emp_int %>%
      pivot_longer(
        cols = c("median", "pi_l", "pi_u"),
        values_to = "OBS",
        names_to = "PI"
      ) %>%
      mutate(PI = case_when(
        str_detect(PI, "pi_l") ~ "lower",
        str_detect(PI, "median") ~ "median",
        str_detect(PI, "pi_u") ~ "upper"
      ))

    # Combine all elements
    pred_int_ci <- pred_int_median %>%
      left_join(pred_int_l_ci, by = c(vec_group, "PI")) %>%
      left_join(pred_int_u_ci, by = c(vec_group, "PI")) %>%
      left_join(emp_w, by = c(vec_group, "PI"))

    # Prepare datasets for legend distinction
    emp_w_mod <- emp_w %>%
      mutate(TYPE = "empirical percentiles")

    pred_int_median_mod <- pred_int_median %>%
      rename(OBS = theor_median) %>%
      mutate(TYPE = "theoretical percentiles")

    pred_int_median_emp_mod <- bind_rows(pred_int_median_mod, emp_w_mod) %>%
      mutate(TYPE = as.factor(TYPE))

    # Add CI labels
    pred_int_l_ci_mod <- pred_int_l_ci %>%
      mutate(CI_label = ifelse(
        PI == "median",
        "CI for the median",
        paste0("CI for the ", piLow, " and ", piUp, " percentiles")
      ))

    pred_int_u_ci_mod <- pred_int_u_ci %>%
      mutate(CI_label = ifelse(
        PI == "median",
        "CI for the median",
        paste0("CI for the ", piLow, " and ", piUp, " percentiles")
      ))

    pred_int_ci_mod <- left_join(pred_int_l_ci_mod, pred_int_u_ci_mod,
                                 by = c(vec_group, "PI", "CI_label"))

    # ===== CREATE PLOT =====

    vpc_plot <- create_vpc_plot(
      pred_int_median_emp_mod = pred_int_median_emp_mod,
      pred_int_ci_mod = pred_int_ci_mod,
      data_obs = data_obs,
      pred_int_ci = pred_int_ci,
      data_i_var = data_i_var,
      theor_perc = theor_perc,
      emp_perc = emp_perc,
      theor_percCI = theor_percCI,
      interpolation = interpolation,
      dt_obs_fl = dt_obs_fl,
      strat_by_dose = strat_by_dose,
      log_y = log_y,
      pred.corr = pred.corr,
      lab_y = lab_y,
      lab_x = lab_x,
      legend_fl = legend_fl
    )

    return(vpc_plot)
  })

  names(vpc_plots) <- unique_vars
  return(vpc_plots)
}

## Author: Melnikova Alina
## First created: 2025-12-04
## Description: formal testing of sg-vpc-vis function and its helpers
## Keywords: SimuRg, sg-vpc-vis, vpc, diagnostics

# Helper function to create mock simulated data for VPC
create_mock_sim_data <- function(n_ids = 2, n_times = 5, n_sims = 10, var_name = "Cc") {
  data.frame(
    sim.id = rep(1:n_sims, each = n_ids * n_times),
    ID = rep(rep(1:n_ids, each = n_times), n_sims),
    TIME = rep(rep(c(0, 1, 2, 4, 8), n_ids), n_sims),
    VAR = var_name,
    VALUE = rnorm(n_ids * n_times * n_sims, mean = 10, sd = 2)
  )
}

# Helper function to create mock observed data
create_mock_obs_data <- function(n_ids = 2, n_times = 5, dvid = 1) {
  data.frame(
    ID = rep(1:n_ids, each = n_times),
    TIME = rep(c(0, 1, 2, 4, 8), n_ids),
    DV = rnorm(n_ids * n_times, mean = 10, sd = 2),
    DVID = dvid
  )
}

# Helper function to create output_names mapping
create_mock_output_names <- function(var_name = "Cc", dvid = 1) {
  data.frame(
    output = var_name,
    dvid = dvid
  )
}

# Error handling tests
test_that("sg_vpc_vis errors on missing required columns in ds_sim", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- data.frame(id = 1:10, time = 1:10)  # Missing VAR and VALUE
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  expect_error(
    sg_vpc_vis(ds_sim = ds_sim, data_i = data_i, output_names = output_names),
    regexp = "VAR|VALUE"
  )
})

test_that("sg_vpc_vis errors on missing required columns in data_i", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- data.frame(ID = 1:5, TIME = 1:5)  # Missing DV and DVID
  output_names <- create_mock_output_names()

  expect_error(
    sg_vpc_vis(ds_sim = ds_sim, data_i = data_i, output_names = output_names),
    regexp = "DV|DVID"
  )
})

test_that("sg_vpc_vis errors on missing output_names", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()

  expect_error(
    sg_vpc_vis(ds_sim = ds_sim, data_i = data_i, output_names = NULL),
    regexp = "output_names"
  )
})

test_that("sg_vpc_vis errors on invalid binning method", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # This should error or use default method
  result <- tryCatch({
    sg_vpc_vis(ds_sim = ds_sim, data_i = data_i, output_names = output_names,
               method = "invalid_method")
  }, error = function(e) NULL)


  # If it doesn't error, it should still produce output (method validation may be in helper)
  if (!is.null(result)) {
    expect_true(is.list(result))
  }
})

# Basic functionality tests
test_that("sg_vpc_vis returns list of ggplot objects", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i, output_names = output_names)

  expect_type(result, "list")
  expect_true(length(result) > 0)
  expect_true(all(vapply(result, function(x) inherits(x, "ggplot"), logical(1))))
})

test_that("sg_vpc_vis handles multiple output variables", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  # Create data with two variables
  ds_sim1 <- create_mock_sim_data(var_name = "Cc")
  ds_sim2 <- create_mock_sim_data(var_name = "Cp")
  ds_sim <- rbind(ds_sim1, ds_sim2)

  data_i1 <- create_mock_obs_data(dvid = 1)
  data_i2 <- create_mock_obs_data(dvid = 2)
  data_i <- rbind(data_i1, data_i2)

  output_names <- data.frame(
    output = c("Cc", "Cp"),
    dvid = c(1, 2)
  )

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i, output_names = output_names)

  expect_equal(length(result), 2)
  expect_named(result, c("Cc", "Cp"))
})

test_that("sg_vpc_vis handles different binning methods", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # Test kmeans method
  result_kmeans <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                              output_names = output_names, method = "kmeans", n_bins = 5)
  expect_true(is.list(result_kmeans))
  expect_true(all(vapply(result_kmeans, function(x) inherits(x, "ggplot"), logical(1))))

  # Test ntile method
  result_ntile <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                             output_names = output_names, method = "ntile", n_bins = 5)
  expect_true(is.list(result_ntile))

  # Test equal_x method
  result_equal <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                             output_names = output_names, method = "equal_x", n_bins = 5)
  expect_true(is.list(result_equal))
})

test_that("sg_vpc_vis handles custom n_bins parameter", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                       output_names = output_names, n_bins = 5)

  expect_true(is.list(result))
  expect_true(all(vapply(result, function(x) inherits(x, "ggplot"), logical(1))))
})

test_that("sg_vpc_vis handles interpolation parameter", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # With interpolation
  result_interp <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                              output_names = output_names, interpolation = TRUE)
  expect_true(is.list(result_interp))

  # Without interpolation
  result_no_interp <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                                 output_names = output_names, interpolation = FALSE)
  expect_true(is.list(result_no_interp))
})

test_that("sg_vpc_vis handles log_y parameter", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # Linear scale
  result_linear <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                              output_names = output_names, log_y = FALSE)
  expect_true(is.list(result_linear))

  # Log scale
  result_log <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                           output_names = output_names, log_y = TRUE)
  expect_true(is.list(result_log))
})

test_that("sg_vpc_vis handles prediction correction", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # Without prediction correction
  result_no_corr <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                               output_names = output_names, pred.corr = FALSE)
  expect_true(is.list(result_no_corr))

  # With prediction correction
  result_corr <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                            output_names = output_names, pred.corr = TRUE)
  expect_true(is.list(result_corr))
})

test_that("sg_vpc_vis handles percentile display flags", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # Only theoretical percentiles
  result_theor <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                             output_names = output_names,
                             theor_perc = TRUE, emp_perc = FALSE)
  expect_true(is.list(result_theor))

  # Only empirical percentiles
  result_emp <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                           output_names = output_names,
                           theor_perc = FALSE, emp_perc = TRUE)
  expect_true(is.list(result_emp))

  # Both
  result_both <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                            output_names = output_names,
                            theor_perc = TRUE, emp_perc = TRUE)
  expect_true(is.list(result_both))
})

test_that("sg_vpc_vis handles confidence interval display", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # With CI
  result_ci <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                          output_names = output_names, theor_percCI = TRUE)
  expect_true(is.list(result_ci))

  # Without CI
  result_no_ci <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                             output_names = output_names, theor_percCI = FALSE)
  expect_true(is.list(result_no_ci))
})

test_that("sg_vpc_vis handles observed data points display", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # With observed points
  result_points <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                              output_names = output_names, dt_obs_fl = TRUE)
  expect_true(is.list(result_points))

  # Without observed points
  result_no_points <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                                 output_names = output_names, dt_obs_fl = FALSE)
  expect_true(is.list(result_no_points))
})

test_that("sg_vpc_vis handles custom axis labels", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                       output_names = output_names,
                       lab_x = "Time (hours)",
                       lab_y = "Concentration (ng/mL)")

  expect_true(is.list(result))
  expect_true(all(vapply(result, function(x) inherits(x, "ggplot"), logical(1))))
})

test_that("sg_vpc_vis handles custom prediction interval bounds", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                       output_names = output_names,
                       piLow = 0.05, piUp = 0.95)

  expect_true(is.list(result))
})

test_that("sg_vpc_vis handles custom confidence interval bounds", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                       output_names = output_names,
                       ciLow = 0.01, ciUp = 0.99)

  expect_true(is.list(result))
})

test_that("sg_vpc_vis handles legend display", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # With legend
  result_legend <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                              output_names = output_names, legend_fl = TRUE)
  expect_true(is.list(result_legend))

  # Without legend
  result_no_legend <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                                 output_names = output_names, legend_fl = FALSE)
  expect_true(is.list(result_no_legend))
})

test_that("sg_vpc_vis handles custom time and DV column names", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  # Create data with custom column names
  data_i_custom <- create_mock_obs_data()
  names(data_i_custom)[names(data_i_custom) == "TIME"] <- "TAD"
  names(data_i_custom)[names(data_i_custom) == "DV"] <- "CONC"

  output_names <- create_mock_output_names()

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i_custom,
                       output_names = output_names,
                       time_col = "TAD", dv_col = "CONC")

  expect_true(is.list(result))
  expect_true(all(vapply(result, function(x) inherits(x, "ggplot"), logical(1))))
})

test_that("sg_vpc_vis handles variables with _ResErr suffix", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  # Create data with _ResErr suffix
  ds_sim <- create_mock_sim_data(var_name = "Cc_ResErr")
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names(var_name = "Cc")

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                       output_names = output_names)

  expect_true(is.list(result))
  expect_true("Cc_ResErr" %in% names(result))
})

test_that("sg_vpc_vis returns named list matching VAR values", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data(var_name = "Cc")
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names(var_name = "Cc")

  result <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                       output_names = output_names)

  expect_named(result, "Cc")
  expect_true(is.list(result))
})

test_that("sg_vpc_vis plot changes when bins change", {
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("purrr")
  skip_if_not_installed("stringr")

  ds_sim <- create_mock_sim_data()
  data_i <- create_mock_obs_data()
  output_names <- create_mock_output_names()

  # Use deterministic kmeans binning (fun_Bin_smrg sets seed internally)
  p_bins5 <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                        output_names = output_names, method = "kmeans",
                        n_bins = 5, interpolation = FALSE)[[1]]
  p_bins3 <- sg_vpc_vis(ds_sim = ds_sim, data_i = data_i,
                        output_names = output_names, method = "kmeans",
                        n_bins = 3, interpolation = FALSE)[[1]]

  b5 <- ggplot2::ggplot_build(p_bins5)
  b3 <- ggplot2::ggplot_build(p_bins3)

  # Compare number of unique x positions in the first layer (lines/steps)
  x_unique_5 <- length(unique(b5$data[[1]]$x))
  x_unique_3 <- length(unique(b3$data[[1]]$x))

  expect_false(identical(x_unique_5, x_unique_3))
})

## Author: Melnikova Alina,  Mikhailova Anna
## First created: 2025-09-04
## Description: sg-parsum and its helper functions
## Keywords: SimuRg, sg-parsum, goodness-of-fit

#' Extract parameter summary (theta, eta, SE, RSE, CI, CV, shrinkage)
#'
#' @inheritParams sg_dummy
#' @returns A table with parameter summary
#' @examples
#' fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
#' sum_tab <- sg_parsum(fpath_i)
#' print(sum_tab)
#' @import dplyr
#' @import stringr
#' @export
sg_parsum <- function(fpath_i, addOFV = TRUE){
  if (inherits(fpath_i, "character")) {
    obj <- get(load(fpath_i))
  } else if (inherits(fpath_i, "list")) {
    obj <- fpath_i
  } else {
    stop("fpath_i object should be either an sg_fit object, or a path to saved sg_fit object")
  }
  obj_partab_i <- obj$SUMTAB
  par_i <- obj_partab_i %>% select(Parameter = PAR, TYPE, Estimate = VALUE, SE, RSE, CV, LCI, UCI, Shrinkage_var = ETAshrinkage_var) %>%
    mutate(`95% CI` = paste0(signif(LCI,3)," - ", signif(UCI,3)), Estimate = signif(Estimate,3), SE = signif(SE,3), RSE = signif(RSE,3), `Shrinkage (var), %` = signif(Shrinkage_var,3))
  par_out <- par_i %>% mutate( `CV % (95% CI)` = ifelse(TYPE == "Random effects", str_c(round(CV, 3), " (", signif(100*sqrt(exp(LCI^2) - 1), 3), ", ", signif(100*sqrt(exp(UCI^2) - 1), 3), ")"), NA)) %>%
    #select(-UCI, -LCI, -CV, -TYPE)
    select(Parameter, Estimate, SE, RSE, `95% CI`, `CV % (95% CI)`, `Shrinkage (var), %`)

  par_out <- par_out %>%
    mutate(Parameter = str_remove_all(Parameter, "_pop")) %>% mutate(Parameter = str_replace_all(Parameter, "_", " "))

  addOFV = T
  if(addOFV){
    ofv_lst <- obj$OFV

    par_out <- par_out %>% mutate(OFV = -2*ofv_lst[["LL"]], AIC = ofv_lst[["AIC"]])
  }

  return(par_out)
}

## Author: Mikhailova Anna
## First created: 2025-09-05
## Description: formal testing of sg-parsum function and its helpers
## Keywords: SimuRg, sg-parsum, goodness-of-fit

test_that("sg-parsum file load works", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
  sum_tab <- sg_parsum(fpath_i)
  expect_true(inherits(sum_tab, "data.frame"))
  expect_snapshot(sum_tab)
  sum_tab <- sg_parsum(obj1)
  expect_true(inherits(sum_tab, "data.frame"))
  expect_snapshot(sum_tab)
})
test_that("sg-parsum file load works dataset with covariates", {
  fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK_cov.RData", package = "SimuRg")
  sum_tab <- sg_parsum(fpath_i)
  expect_true(inherits(sum_tab, "data.frame"))
  expect_snapshot(sum_tab)
  sum_tab <- sg_parsum(obj1)
  expect_true(inherits(sum_tab, "data.frame"))
  expect_snapshot(sum_tab)
})
test_that("sg-parsum does not work", {
  expect_error(sg_parsum(data.frame()))
})

## Author: Ugolkov Yaroslav
## First created: 2026-02-24
## Description: Global sensitivity analysis simulations for RxODE models
## Keywords: SimuRg, gloval sensitivity

#' Global sensitivity analysis via PRCC or eFAST
#'
#' @description
#' Performs global sensitivity analysis using either PRCC (Partial Rank
#' Correlation Coefficients with LHS sampling) or eFAST (extended Fourier
#' Amplitude Sensitivity Test). For each sampled parameter set, the function
#' runs simulations via `sg_sim()`, reduces trajectories to scalar summary
#' statistics, and computes sensitivity indices for each output–statistic pair.
#'
#' @inheritParams sg_dummy
#' @return List with:
#' \describe{
#'   \item{result}{Tibble containing sensitivity analysis results.
#'   For `PRCC` method columns include: `PAR`, `VAR`, `STAT`, `estimate`, `p.value`.
#'   For `eFAST` method columns include: `PAR`, `VAR`, `STAT`, `TYPE`, `VALUE`, `METHOD`.}
#' }
#'
#'   \item{summary}{
#'     Tibble of scalar summaries used for sensitivity computation:
#'     `sim.id`, `VAR`, `STAT`, `value`.
#'   }
#'
#'   \item{design}{
#'     Data.frame of sampled parameter values (real scale) with `sim.id`.
#'     For PRCC this corresponds to LHS samples.
#'     For eFAST this corresponds to the FAST design matrix.
#'   }
#'
#'   \item{bounds}{
#'     Parameter bounds used in the analysis.
#'   }
#' @examples
#' \donttest{
#' # Example model
#' library(rxode2)
#' library(tibble)
#' source(system.file("extdata", "RxODE_model", "example_rxode_model.R", package = "SimuRg")) # mod_ex
#'
#'
#' # Set up event table
#' et_base <- tribble(
#'   ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
#'   1,   0,     1,     1,    10,   2,     24,  112,   1
#' )
#'
#' # Define parameter bounds
#' inits <- rxInits(mod_ex)
#' par_bounds <- tibble::tibble(
#'   PAR = c("POPCL", "POPVC"),
#'   LB  = inits[c("POPCL", "POPVC")] * (1 - 0.9),
#'   UB  = inits[c("POPCL", "POPVC")] * (1 + 0.9)
#' )
#'
#' # Run eFAST sensitivity analysis
#' res_efast <- sg_globalsens_sim(
#'   method = c("eFAST"),
#'   model = mod_ex,
#'   params = c("POPCL"),
#'   par_bounds = par_bounds,
#'   n_sim = 100,
#'   stimes = seq(0, 168, 10),
#'   output = "Cc",
#'   cov = c("IGFR", "POPN"),
#'   et = et_base,
#'   stat_comp = c("mean")
#' )
#'
#' # Run PRCC sensitivity analysis
#' res_prcc <- sg_globalsens_sim(
#'   method = c("PRCC"),
#'   model = mod_ex,
#'   params = c("POPCL", "POPVC"),
#'   par_bounds = par_bounds,
#'   n_sim = 100,
#'   stimes = seq(0, 168, 10),
#'   output = "Cc",
#'   cov = c("IGFR", "POPN"),
#'   et = et_base,
#'   stat_comp = c("mean")
#' )
#' }
#'
#'
#'
#' @import dplyr tidyr purrr ppcor
#' @importFrom sensitivity fast99 tell
#' @importFrom lhs randomLHS
#' @export

sg_globalsens_sim <- function(method = c("PRCC", "eFAST"),
                              model,
                              params,
                              par_bounds,
                              n_sim,
                              stimes,
                              output,
                              stat_comp = c("mean", "min", "max"),
                              et,
                              theta = NULL,
                              cov = NULL) {

  method <- match.arg(method)


  req_cols <- c("PAR", "LB", "UB")
  if (!all(req_cols %in% colnames(par_bounds))) {
    stop("par_bounds must contain columns: PAR, LB, UB")
  }

  params <- as.character(params)

  bounds <- par_bounds %>%
    dplyr::filter(PAR %in% params) %>%
    dplyr::slice(match(params, PAR))

  if (nrow(bounds) != length(params))
    stop("Some parameters missing in par_bounds")


  # DESIGN GENERATION


  if (method == "PRCC") {

    X_norm <- lhs::randomLHS(n_sim, length(params)) %>% as.data.frame()
    colnames(X_norm) <- params
    X_norm$sim.id <- seq_len(n_sim)

    X_real <- purrr::imap_dfc(X_norm %>% dplyr::select(-sim.id), function(x, nm) {
      LB <- bounds$LB[match(nm, bounds$PAR)]
      UB <- bounds$UB[match(nm, bounds$PAR)]
      (UB - LB) * x + LB
    }) %>%
      dplyr::mutate(sim.id = X_norm$sim.id)

  } else {

    # eFAST design
    q <- rep("qunif", length(params))
    q.arg <- lapply(seq_len(nrow(bounds)), function(i) {
      list(min = bounds$LB[i], max = bounds$UB[i])
    })

    fast_obj <- sensitivity::fast99(
      model = NULL,
      factors = params,
      n = n_sim,
      q = q,
      q.arg = q.arg
    )

    X_real <- as.data.frame(fast_obj$X)
    X_real$sim.id <- seq_len(nrow(X_real))
    X_norm <- NULL
  }


  # SIMULATIONS


  sim_all <- purrr::map_dfr(seq_len(nrow(X_real)), function(i) {

    theta_i <- if (is.null(theta)) c() else theta
    theta_i[params] <- as.numeric(X_real[i, params])

    sg_sim(
      model = model,
      stimes = stimes,
      outputs = output,
      theta = theta_i,
      covs = cov,
      et = et
    ) %>%
      dplyr::mutate(sim.id = i)
  })

  if (!all(c("TIME","VAR","VALUE","sim.id") %in% colnames(sim_all)))
    stop("sg_sim() must return TIME, VAR, VALUE, sim.id")


  # SUMMARY STATISTICS

  summary_df <- sim_all %>%
    dplyr::group_by(sim.id, VAR) %>%
    dplyr::summarise(
      mean = mean(VALUE, na.rm = TRUE),
      median = median(VALUE, na.rm = TRUE),
      min = min(VALUE, na.rm = TRUE),
      max = max(VALUE, na.rm = TRUE),
      sd = sd(VALUE, na.rm = TRUE),
      cmax = max(VALUE, na.rm = TRUE),
      SS = VALUE[TIME == max(stimes)][1],
      .groups = "drop"
    ) %>%
    dplyr::select(sim.id, VAR, dplyr::all_of(stat_comp)) %>%
    tidyr::pivot_longer(
      cols = -c(sim.id, VAR),
      names_to = "STAT",
      values_to = "value"
    )


  # METHOD-SPECIFIC PART

  if (method == "PRCC") {

    prcc_df <- summary_df %>%
      dplyr::left_join(X_real, by = "sim.id")

    result <- prcc_df %>%
      dplyr::group_by(VAR, STAT) %>%
      dplyr::group_modify(function(dat, key) {

        dat2 <- dat %>%
          dplyr::select("value", dplyr::all_of(params)) %>%
          dplyr::mutate(dplyr::across(dplyr::everything(), as.numeric)) %>%
          tidyr::drop_na()


        if (nrow(dat2) < (length(params) + 2) || stats::sd(dat2$value) == 0) {
          return(tibble::tibble(
            PAR = params,
            estimate = NA_real_,
            p.value = NA_real_
          ))
        }


        good_par <- params[sapply(dat2[, params, drop = FALSE], stats::sd) > 0]

        if (length(good_par) == 0) {
          return(tibble::tibble(
            PAR = params,
            estimate = NA_real_,
            p.value = NA_real_
          ))
        }

        # ---- pcor
        pc <- suppressWarnings(ppcor::pcor(dat2[, c("value", good_par), drop = FALSE],
                                           method = "spearman"))


        if (is.null(dimnames(pc$estimate))) {
          dimnames(pc$estimate) <- list(colnames(dat2[, c("value", good_par)]),
                                        colnames(dat2[, c("value", good_par)]))
        }
        if (is.null(dimnames(pc$p.value))) {
          dimnames(pc$p.value) <- list(colnames(dat2[, c("value", good_par)]),
                                       colnames(dat2[, c("value", good_par)]))
        }

        est <- pc$estimate["value", good_par, drop = TRUE]
        pvl <- pc$p.value["value", good_par, drop = TRUE]


        out <- tibble::tibble(
          PAR = params,
          estimate = NA_real_,
          p.value = NA_real_
        )

        out$estimate[out$PAR %in% good_par] <- as.numeric(est)
        out$p.value[out$PAR %in% good_par] <- as.numeric(pvl)

        out
      }) %>%
      dplyr::ungroup() %>%
      dplyr::mutate(PAR = factor(.data$PAR, levels = params))

  }
  else {

    result <- purrr::map_dfr(unique(summary_df$VAR), function(v) {
      purrr::map_dfr(unique(summary_df$STAT), function(s) {

        y <- summary_df %>%
          dplyr::filter(VAR == v, STAT == s) %>%
          dplyr::pull(value)

        if (sd(y) == 0)
          return(NULL)

        fast_obj <- sensitivity::fast99(
          model = NULL,
          factors = params,
          n = n_sim,
          q = rep("qunif", length(params)),
          q.arg = lapply(seq_len(nrow(bounds)),
                         function(i) list(min = bounds$LB[i],
                                          max = bounds$UB[i]))
        )

        fast_obj$y <- y
        fast_obj <- sensitivity::tell(fast_obj)

        tibble::tibble(
          PAR = params,
          VAR = v,
          STAT = s,
          `First order` = fast_obj$D1 / fast_obj$V,
          `Total order` = 1 - fast_obj$Dt / fast_obj$V
        ) %>%
          tidyr::pivot_longer(
            cols = c(`First order`, `Total order`),
            names_to = "TYPE",
            values_to = "VALUE"
          ) %>%
          dplyr::mutate(METHOD = "eFAST")
      })
    })
  }

  return(list(
    result = result,
    summary = summary_df,
    design = X_real,
    bounds = bounds
  ))
}

## Author: Ugolkov Yaroslav
## First created: 2026-02-24
## Description: formal testing of sg-globalsens-sim function and its helpers
## Keywords: SimuRg, sg-modcomp, model building, comparison
mod_ex <- RxODE({
  # Doses in mg
  # Time in hours

  ### Parameter values
  # Typical
  POPCL = 5;
  POPVC = 180;
  POPQ = 7;
  POPVP = 52;
  POPKTR = 6;
  FBIOPAR = 1;

  # Covariate coefficients
  POPIGFRCOV = 0.8;
  POPPATCOVCLGR1 = 0.85;
  POPPATCOVCLGR2 = 0.85;
  POPPATCOVCLGR3 = 0.85;
  POPPATCOVCLGR4 = 0.85;

  # Random effects
  PPVCL = 0;
  PPVVC = 0;
  PPVKTR = 0;

  # Residual error
  RUV = 0;

  ### Covariates
  PATCOVCL = 1
  if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
  if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
  if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
  if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
  IGFRCOV = (IGFR/112)^POPIGFRCOV

  ### Parameters
  CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
  VC = POPVC * exp(PPVVC);
  Q = POPQ;
  VP = POPVP;
  KTR = POPKTR * exp(PPVKTR);

  ### Explicit functions
  Cc = Ac/VC;                 # nmol/L
  Cp = Ap/VP;                 # nmol/L

  ### Initial conditions
  At1(0) = 0;         # mg
  At2(0) = 0;         # mg
  At3(0) = 0;         # mg
  At4(0) = 0;         # mg
  At5(0) = 0;         # mg
  At6(0) = 0;         # mg
  Ad(0) = 0;          # mg
  Ac(0) = 0;          # mg
  Ap(0) = 0;          # mg

  ### ODEs
  d/dt(At1) = - KTR*At1;
  d/dt(At2) = KTR*At1 - KTR*At2;
  d/dt(At3) = KTR*At2 - KTR*At3;
  d/dt(At4) = KTR*At3 - KTR*At4;
  d/dt(At5) = KTR*At4 - KTR*At5;
  d/dt(At6) = KTR*At5 - KTR*At6;
  d/dt(Ad) = KTR*At6 - KTR*Ad;
  d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
  d/dt(Ap) = Q*(Cc - Cp);

  FBIO = FBIOPAR
  f(At1) = FBIO*1000000/505;
  CHECKRUV = RUV;
  Cc_ResErr = Cc + RUV*Cc;
})
et_base <- tribble(
  ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
  1,   0,     1,     1,    10,   2,     24,  112,   1
)
inits <- rxInits(mod_ex)
par_bounds <- tibble::tibble(
  PAR = c("POPCL", "POPVC"),
  LB  = inits[c("POPCL", "POPVC")] * (1 - 0.9),
  UB  = inits[c("POPCL", "POPVC")] * (1 + 0.9)
)
result <- sg_globalsens_sim(
  method = "PRCC",
  model = mod_ex,
  params = c("POPCL","POPVC"),
  par_bounds = par_bounds,
  n_sim = 100,
  stimes = seq(0, 168, 10),
  output = "Cc",
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean")
)
test_that("sg_globalsens_sim PRCC output structure is correct", {


  # List output
  expect_type(result, "list")

  # Required list elements
  expect_true(all(c("result", "summary", "design", "bounds") %in% names(result)))

  # Result table
  expect_s3_class(result$result, "tbl_df")

  # Summary table
  expect_s3_class(result$summary, "tbl_df")

  # Design table
  expect_s3_class(result$design, "tbl_df")

  # Bounds table
  expect_s3_class(result$bounds, "tbl_df")
})
test_that("sg_globalsens_sim PRCC returns correct parameter set", {



  res_tbl <- result$result

  expect_true(all(c("VAR","STAT","PAR","estimate","p.value") %in% names(res_tbl)))

  expect_true(all(sort(unique(res_tbl$PAR)) %in%
                    c("POPCL","POPVC")))
})
result <- sg_globalsens_sim(
  method = "eFAST",
  model = mod_ex,
  params = c("POPCL","POPVC"),
  par_bounds = par_bounds,
  n_sim = 100,
  stimes = seq(0, 168, 10),
  output = "Cc",
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean")
)
test_that("sg_globalsens_sim eFAST output structure is correct", {



  # List output
  expect_type(result, "list")

  # Required list elements
  expect_true(all(c("result", "summary", "design", "bounds") %in% names(result)))

  # Result table
  expect_s3_class(result$result, "tbl_df")

  # Summary table
  expect_s3_class(result$summary, "tbl_df")


  # Bounds table
  expect_s3_class(result$bounds, "tbl_df")
})
test_that("sg_globalsens_sim eFAST returns correct parameter set", {



  res_tbl <- result$result

  expect_true(all(c("VAR","STAT","PAR","TYPE","VALUE") %in% names(res_tbl)))

  expect_true(all(sort(unique(res_tbl$PAR)) %in%
                    c("POPCL","POPVC")))
})
test_that("sg_globalsens_sim handles invalid model input", {

  expect_error(
    sg_globalsens_sim(
      method = "PRCC",
      model = "invalid_model",
      params = c("POPCL","POPVC"),
      par_bounds = par_bounds,
      n_sim = 50,
      stimes = seq(0, 168, 10),
      et = et_base
    )
  )
})

## Author: Alina Melnikova
## First created: 2026-04-22
## Description: formal testing of sg_fit file generation and validation
## Keywords: SimuRg, sg-fit

# ---- Shared fixtures -------------------------------------------------------

.fit_model <- normalizePath(
  system.file("extdata", "models", "model_PK_1c.txt", package = "SimuRg"),
  winslash = "/",
  mustWork = TRUE
)
.fit_data <- normalizePath(
  system.file("extdata", "datasets", "dspk-warf.csv", package = "SimuRg"),
  winslash = "/",
  mustWork = TRUE
)

.fit_headers <- list(
  list(name = "ID", use = "identifier", type = NULL),
  list(name = "TIME", use = "time", type = NULL),
  list(name = "DV", use = "observation", type = "continuous"),
  list(name = "DVID", use = "observationtype", type = NULL),
  list(name = "ADM", use = "administration", type = NULL),
  list(name = "AMT", use = "amount", type = NULL),
  list(name = "EVID", use = "eventidentifier", type = NULL),
  list(name = "MDV", use = "missingdependentvariable", type = NULL),
  list(name = "AGE", use = "covariate", type = "continuous")
)

.fit_theta <- data.frame(
  NAME = c("ka", "Vd", "CL"),
  TRANS = c("logNormal", "logNormal", "logNormal"),
  INIT = c(0.5, 20, 0.2),
  LB = c(NA_real_, NA_real_, NA_real_),
  UB = c(NA_real_, NA_real_, NA_real_),
  EST = c(TRUE, FALSE, TRUE),
  stringsAsFactors = FALSE
)

.fit_ruv <- list(
  YNAME = "y1",
  DVID = 1,
  TRANS = "normal",
  PRED = "Cc",
  ERR = "combined1",
  INIT = c(1, 0.3),
  EST = c(TRUE, FALSE),
  BLQM = NULL
)

.fit_re <- list(
  init = matrix(c(
    0.1, 0,   0,
    0,   0,   0,
    0,   0, 0.2
  ), nrow = 3, byrow = TRUE),
  est = matrix(c(
    TRUE, NA,   NA,
    NA,   NA,   NA,
    NA,   NA, TRUE
  ), nrow = 3, byrow = TRUE)
)

.fit_occ <- list(
  init = matrix(0, nrow = 3, ncol = 3),
  est = matrix(NA, nrow = 3, ncol = 3)
)

.fit_covs <- list(
  list(PAR = "CL", COVNAME = "AGE", FUNC = "linear",
       TRANS = "median", INIT = 0.01, EST = TRUE)
)

.sg_fit_args <- function(project_name, output_dir) {
  list(
    model = .fit_model,
    data = .fit_data,
    headers = .fit_headers,
    theta = .fit_theta,
    ruv = .fit_ruv,
    re = .fit_re,
    occ = .fit_occ,
    covs = .fit_covs,
    project_name = project_name,
    fit = FALSE,
    path_to_save_output = output_dir
  )
}

.find_monolix_batch <- function() {
  env_candidates <- c(
    Sys.getenv("SIMURG_MONOLIX_PATH", unset = ""),
    Sys.getenv("MONOLIX_PATH", unset = "")
  )

  lixoft_root <- "C:/ProgramData/Lixoft"
  install_dirs <- if (dir.exists(lixoft_root)) {
    list.dirs(lixoft_root, recursive = FALSE, full.names = TRUE)
  } else {
    character()
  }

  file_candidates <- c(
    env_candidates,
    file.path(install_dirs, "bin", "monolix.bat"),
    "C:/ProgramData/Lixoft/MonolixSuite2023R1/bin/monolix.bat"
  )

  file_candidates <- unique(file_candidates[nzchar(file_candidates)])
  existing <- file_candidates[file.exists(file_candidates)]

  if (length(existing) == 0) {
    return(NA_character_)
  }

  normalizePath(existing[[1]], winslash = "/", mustWork = TRUE)
}

# ============================================================
# 1. Input validation
# ============================================================

test_that("sg_fit errors when the model file is missing", {
  args <- .sg_fit_args(
    project_name = "missing-model",
    output_dir = tempdir()
  )
  args$model <- file.path(tempdir(), "does-not-exist-model.txt")

  expect_error(
    do.call(sg_fit, args),
    regexp = "model file does not exist"
  )
})

test_that("sg_fit errors when the data file is missing", {
  args <- .sg_fit_args(
    project_name = "missing-data",
    output_dir = tempdir()
  )
  args$data <- file.path(tempdir(), "does-not-exist-data.csv")

  expect_error(
    do.call(sg_fit, args),
    regexp = "data file does not exist"
  )
})

# ============================================================
# 2. Monolix project generation with fit = FALSE
# ============================================================

test_that("sg_fit writes an mlxtran project with the expected sections", {
  output_dir <- tempfile("sg-fit-write-")
  dir.create(output_dir, recursive = TRUE)
  on.exit(unlink(output_dir, recursive = TRUE, force = TRUE), add = TRUE)

  project_name <- "unit_fit"
  result <- do.call(sg_fit, .sg_fit_args(project_name, output_dir))
  mlxtran_path <- file.path(output_dir, sprintf("%s.mlxtran", project_name))

  expect_null(result)
  expect_true(file.exists(mlxtran_path))

  mlxtran_text <- paste(readLines(mlxtran_path, warn = FALSE), collapse = "\n")

  expect_match(mlxtran_text, "<DATAFILE>", fixed = TRUE)
  expect_match(mlxtran_text, "[CONTENT]", fixed = TRUE)
  expect_match(mlxtran_text, "[INDIVIDUAL]", fixed = TRUE)
  expect_match(mlxtran_text, "[LONGITUDINAL]", fixed = TRUE)
  expect_match(mlxtran_text, "<FIT>", fixed = TRUE)
  expect_match(mlxtran_text, "<MONOLIX>", fixed = TRUE)
})

test_that("sg_fit mlxtran output reflects core headers and parameterization", {
  output_dir <- tempfile("sg-fit-content-")
  dir.create(output_dir, recursive = TRUE)
  on.exit(unlink(output_dir, recursive = TRUE, force = TRUE), add = TRUE)

  project_name <- "content_fit"
  do.call(sg_fit, .sg_fit_args(project_name, output_dir))
  mlxtran_path <- file.path(output_dir, sprintf("%s.mlxtran", project_name))
  mlxtran_text <- paste(readLines(mlxtran_path, warn = FALSE), collapse = "\n")
  observed_dvid <- sort(unique(read.csv(.fit_data)[["DVID"]]))
  expected_yname <- paste0("'", observed_dvid, "'", collapse = ", ")
  expected_data_type <- paste0("'", observed_dvid, "'=plasma", collapse = ", ")

  expect_match(mlxtran_text, "header=\\{ID, TIME, DV, DVID", perl = TRUE)
  expect_match(mlxtran_text, "ID = \\{use=identifier\\}", perl = TRUE)
  expect_match(
    mlxtran_text,
    sprintf("DV = {use=observation, yname=%s, type=continuous}", expected_yname),
    fixed = TRUE
  )
  expect_match(
    mlxtran_text,
    sprintf("dataType = {%s}", expected_data_type),
    fixed = TRUE
  )
  expect_match(mlxtran_text, "\\[COVARIATE\\]\\ninput = \\{AGE\\}", perl = TRUE)
  expect_match(
    mlxtran_text,
    "ka = \\{distribution=logNormal, typical=ka_pop, sd=omega_ka\\}",
    perl = TRUE
  )
  expect_match(
    mlxtran_text,
    "Vd = \\{distribution=logNormal, typical=Vd_pop, no-variability\\}",
    perl = TRUE
  )
  expect_match(
    mlxtran_text,
    "CL = \\{distribution=logNormal, typical=CL_pop, covariate=AGE, coefficient=beta_CL_AGE, sd=omega_CL\\}",
    perl = TRUE
  )
  expect_match(
    mlxtran_text,
    "y1 = \\{distribution=normal, prediction=Cc, errorModel=combined1\\(Cc_a, Cc_b\\)\\}",
    perl = TRUE
  )
  expect_match(mlxtran_text, "ka_pop = \\{value=0.5, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "Vd_pop = \\{value=20, method=FIXED\\}", perl = TRUE)
  expect_match(mlxtran_text, "CL_pop = \\{value=0.2, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "Cc_a = \\{value=1, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "Cc_b = \\{value=0.3, method=FIXED\\}", perl = TRUE)
  expect_match(mlxtran_text, "omega_ka = \\{value=0.1, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "omega_CL = \\{value=0.2, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "beta_CL_AGE = \\{value=0.01, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "exportpath = 'content_fit'", perl = TRUE)
})
# ============================================================
# 3. Optional Monolix integration
# ============================================================

test_that("sg_fit fit = TRUE skips cleanly when Monolix is unavailable", {
  monolix_path <- .find_monolix_batch()
  skip_if(
    is.na(monolix_path),
    "Monolix executable not available on this machine."
  )

  output_dir <- tempfile("sg-fit-monolix-")
  dir.create(output_dir, recursive = TRUE)
  on.exit(unlink(output_dir, recursive = TRUE, force = TRUE), add = TRUE)

  project_name <- "integration_fit"
  args <- .sg_fit_args(project_name, output_dir)
  args$fit <- TRUE
  args$path_to_fitter <- monolix_path
  args$max_wait_time <- 120

  result <- do.call(sg_fit, args)
  project_dir <- file.path(output_dir, project_name)
  mlxtran_path <- file.path(output_dir, sprintf("%s.mlxtran", project_name))

  expect_true(file.exists(mlxtran_path))
  expect_true(dir.exists(project_dir))
  expect_false(is.null(result))
})

test_that("sg_fit errors when the data file is missing", {
  args <- .sg_fit_args(
    project_name = "missing-data",
    output_dir = tempdir()
  )
  args$data <- file.path(tempdir(), "does-not-exist-data.csv")

  expect_error(
    do.call(sg_fit, args),
    regexp = "data file does not exist"
  )
})

# ============================================================
# 2. Monolix project generation with fit = FALSE
# ============================================================

test_that("sg_fit writes an mlxtran project with the expected sections", {
  output_dir <- tempfile("sg-fit-write-")
  dir.create(output_dir, recursive = TRUE)
  on.exit(unlink(output_dir, recursive = TRUE, force = TRUE), add = TRUE)

  project_name <- "unit_fit"
  result <- do.call(sg_fit, .sg_fit_args(project_name, output_dir))
  mlxtran_path <- file.path(output_dir, sprintf("%s.mlxtran", project_name))

  expect_null(result)
  expect_true(file.exists(mlxtran_path))

  mlxtran_text <- paste(readLines(mlxtran_path, warn = FALSE), collapse = "\n")

  expect_match(mlxtran_text, "<DATAFILE>", fixed = TRUE)
  expect_match(mlxtran_text, "[CONTENT]", fixed = TRUE)
  expect_match(mlxtran_text, "[INDIVIDUAL]", fixed = TRUE)
  expect_match(mlxtran_text, "[LONGITUDINAL]", fixed = TRUE)
  expect_match(mlxtran_text, "<FIT>", fixed = TRUE)
  expect_match(mlxtran_text, "<MONOLIX>", fixed = TRUE)
})

test_that("sg_fit mlxtran output reflects core headers and parameterization", {
  output_dir <- tempfile("sg-fit-content-")
  dir.create(output_dir, recursive = TRUE)
  on.exit(unlink(output_dir, recursive = TRUE, force = TRUE), add = TRUE)

  project_name <- "content_fit"
  do.call(sg_fit, .sg_fit_args(project_name, output_dir))
  mlxtran_path <- file.path(output_dir, sprintf("%s.mlxtran", project_name))
  mlxtran_text <- paste(readLines(mlxtran_path, warn = FALSE), collapse = "\n")
  observed_dvid <- sort(unique(read.csv(.fit_data)[["DVID"]]))
  expected_yname <- paste0("'", observed_dvid, "'", collapse = ", ")
  expected_data_type <- paste0("'", observed_dvid, "'=plasma", collapse = ", ")

  expect_match(mlxtran_text, "header=\\{ID, TIME, DV, DVID", perl = TRUE)
  expect_match(mlxtran_text, "ID = \\{use=identifier\\}", perl = TRUE)
  expect_match(
    mlxtran_text,
    sprintf("DV = {use=observation, yname=%s, type=continuous}", expected_yname),
    fixed = TRUE
  )
  expect_match(
    mlxtran_text,
    sprintf("dataType = {%s}", expected_data_type),
    fixed = TRUE
  )
  expect_match(mlxtran_text, "\\[COVARIATE\\]\\ninput = \\{AGE\\}", perl = TRUE)
  expect_match(
    mlxtran_text,
    "ka = \\{distribution=logNormal, typical=ka_pop, sd=omega_ka\\}",
    perl = TRUE
  )
  expect_match(
    mlxtran_text,
    "Vd = \\{distribution=logNormal, typical=Vd_pop, no-variability\\}",
    perl = TRUE
  )
  # expect_match(
  #   mlxtran_text,
  #   "CL = \\{distribution=logNormal, typical=CL_pop, covariate=AGE, coefficient=beta_CL_AGE, sd=omega_CL\\}",
  #   perl = TRUE
  # )
  expect_match(
    mlxtran_text,
    "y1 = \\{distribution=normal, prediction=Cc, errorModel=combined1\\(Cc_a, Cc_b\\)\\}",
    perl = TRUE
  )
  expect_match(mlxtran_text, "ka_pop = \\{value=0.5, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "Vd_pop = \\{value=20, method=FIXED\\}", perl = TRUE)
  expect_match(mlxtran_text, "CL_pop = \\{value=0.2, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "Cc_a = \\{value=1, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "Cc_b = \\{value=0.3, method=FIXED\\}", perl = TRUE)
  expect_match(mlxtran_text, "omega_ka = \\{value=0.1, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "omega_CL = \\{value=0.2, method=MLE\\}", perl = TRUE)
  # expect_match(mlxtran_text, "beta_CL_AGE = \\{value=0.01, method=MLE\\}", perl = TRUE)
  expect_match(mlxtran_text, "exportpath = 'content_fit'", perl = TRUE)
})
# ============================================================
# 3. Optional Monolix integration
# ============================================================

test_that("sg_fit fit = TRUE skips cleanly when Monolix is unavailable", {
  monolix_path <- .find_monolix_batch()
  skip_if(
    is.na(monolix_path),
    "Monolix executable not available on this machine."
  )

  output_dir <- tempfile("sg-fit-monolix-")
  dir.create(output_dir, recursive = TRUE)
  on.exit(unlink(output_dir, recursive = TRUE, force = TRUE), add = TRUE)

  project_name <- "integration_fit"
  args <- .sg_fit_args(project_name, output_dir)
  args$fit <- TRUE
  args$path_to_fitter <- monolix_path
  args$max_wait_time <- 120

  result <- do.call(sg_fit, args)
  project_dir <- file.path(output_dir, project_name)
  mlxtran_path <- file.path(output_dir, sprintf("%s.mlxtran", project_name))

  expect_true(file.exists(mlxtran_path))
  expect_true(dir.exists(project_dir))
  expect_false(is.null(result))
})

# sg-modcomp works

    Code
      sum_tab
    Output
        Run ID Project name      ETAs                        Covariates
      1      3   proj-r-cov ka, V, Cl AGE~V, CYP2C9_gentyp(5 levels)~Cl
        Identifiability Shrinkage Max RSE, % Max shrinkage, %      OFV       AIC
      1             Yes       Yes       35.5             6.04 20668.24 -10308.12

---

    Code
      sum_tab
    Output
        Run ID Project name   ETAs Covariates Identifiability Shrinkage Max RSE, %
      1      4    proj-solo ka, Cl                        Yes       Yes       7.36
        Max shrinkage, %      OFV      AIC
      1             2.86 19064.22 -9520.11

# sg-gof-res output is correct

    Code
      ggplot2::layer_data(x)
    Output
        yintercept PANEL group colour linewidth linetype alpha
      1          0     1    -1  black       0.7   dashed    NA
      2          0     2    -1  black       0.7   dashed    NA
      3          0     3    -1  black       0.7   dashed    NA

# sg-gof-res file load

    Code
      ggplot2::layer_data(x)
    Output
        yintercept PANEL group colour linewidth linetype alpha
      1          0     1    -1  black       0.7   dashed    NA

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-translator.R
\name{sg_translator}
\alias{sg_translator}
\title{Translate structural models between rxode2 and MLXTRAN syntax}
\usage{
sg_translator(
  input_path,
  to,
  output_path,
  dm_list = NULL,
  regressors = NULL,
  output_vars = NULL,
  macros = TRUE,
  stiff = TRUE
)
}
\arguments{
\item{input_path}{Character string. Path to the input structural model file (.txt).}

\item{to}{Character string. Target format: \code{"mlxtran"} or \code{"rxode"}.}

\item{output_path}{Character string. Path to write the translated model file (.txt).}

\item{dm_list}{List of dosing macros with \code{cmt} and \code{adm} vectors.
For example, \code{list(cmt = 2, adm = 1)} or \code{list(cmt = c(2, 1), adm = c(1, 2))}.
Used to generate \code{iv()} dosing macros in MLXTRAN output.}

\item{regressors}{Character vector of regressor variable names.
For example, \code{c("WTBL", "ADAN")} generates \code{WTBL = \{use = regressor\}} in MLXTRAN.}

\item{output_vars}{Character vector of output variable names for the MLXTRAN \code{OUTPUT:} section.
For example, \code{c("Cc_mgL", "CSF1_ngmL")} generates \code{output = \{Cc_mgL, CSF1_ngmL\}}.
Required when \code{to = "mlxtran"}; ignored when \code{to = "rxode"}.}

\item{macros}{Logical. If \code{TRUE}, attempt to convert rxode2 model to pkmodel macro
format in MLXTRAN (only for simple 1- or 2-compartment oral PK models). Default \code{TRUE}.}

\item{stiff}{Logical. If \code{TRUE}, add \code{odeType = stiff} to the MLXTRAN EQUATION section.
Default \code{TRUE}.}
}
\value{
Invisibly returns the translated model as a character string.
The translated model is also written to \code{output_path}.
}
\description{
Converts structural pharmacometric model files between rxode2 (R package)
and MLXTRAN (Monolix) syntax. Supports ODE-based models and Monolix pkmodel macros.
}
\details{
The function detects the source format automatically from file content.

\strong{rxode2 format} uses \code{# [INPUT]}, \code{# [MODEL]}, \code{# [OUTPUT]} section markers,
\code{d/dt(X)} ODE notation, \code{f(X)} for bioavailability, and \code{alag(X)} for lag time.

\strong{MLXTRAN format} uses \code{[LONGITUDINAL]}, \code{PK:}/\code{EQUATION:}/\code{OUTPUT:} sections,
\code{ddt_X} ODE notation, \code{compartment()}/\code{iv()} dosing macros, and optionally
\code{pkmodel()} macros.

When converting rxode2 to MLXTRAN with \code{macros = TRUE}, the function checks if the model
structure matches a simple \code{pkmodel()} pattern (1- or 2-compartment with first-order oral
absorption). If not, it falls back to full ODE format with \code{compartment()}/\code{iv()} macros.

MLXTRAN models using \code{absorption()}/\code{elimination()}/\code{peripheral()} macros (type 2)
are expanded to explicit ODEs when converted to rxode2.
}
\examples{
\donttest{
# Convert rxode2 model to MLXTRAN ODE format
rxode_model_hv_iv <- system.file("extdata", "models", "rxode",
                                 "model_PK_hv_iv.txt", package = "SimuRg")
rxode_model_1c <- system.file("extdata", "models", "rxode",
                              "model_PK_1c.txt", package = "SimuRg")
monolix_model_hv_iv <- system.file("extdata", "models", "monolix",
                                   "model_PK_hv_iv.txt", package = "SimuRg")
monolix_model_1c <- system.file("extdata", "models", "monolix",
                                "model_PK_1c.txt", package = "SimuRg")

path_to_save <- tempdir()

sg_translator(rxode_model_hv_iv, to = "mlxtran",
              output_path = normalizePath(file.path(path_to_save,
                                                    "model_PK_hv_iv_mlx.txt"),
                                          mustWork = FALSE),
              output_vars = "Cc_mgL",
              dm_list = list(cmt = 1, adm = 1), stiff = TRUE)

# Convert rxode2 model to MLXTRAN with pkmodel macros
sg_translator(rxode_model_1c, to = "mlxtran",
              output_path = normalizePath(file.path(path_to_save,
                                                    "model_PK_1c_mlx.txt"),
                                          mustWork = FALSE),
              output_vars = "Cc_nM", macros = TRUE)

# Convert MLXTRAN model to rxode2
sg_translator(monolix_model_hv_iv, to = "rxode",
              output_path = normalizePath(file.path(path_to_save,
                                                    "model_PK_hv_iv_rx.txt"),
                                           mustWork = FALSE))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-converter.R
\name{sg_converter}
\alias{sg_converter}
\title{Convert Monolix project output to R objects}
\usage{
sg_converter(folder_path, proj_name, save_file = FALSE)
}
\arguments{
\item{folder_path}{Character string. Path to the directory containing Monolix project files.}

\item{proj_name}{Character string. Name of the Monolix project (without file extension).}

\item{save_file}{Logical. If \code{TRUE}, saves \code{GCO} and \code{GFO} JSON files
to \code{folder_path} with names \code{<proj_name>_GCO.json} and \code{<proj_name>_GFO.json},
and also saves two RData files: \code{<proj_name>_GCO.RData} (object \code{gco})
and \code{<proj_name>_GFO.RData} (object \code{gfo}).}
}
\value{
Returns a list with the following components:
\itemize{
\item \code{GFO}: SimuRg generalized fit object output object with:
\itemize{
\item \code{SDTAB}: A tibble containing data used for fitting
\item \code{SUMTAB}: A tibble with parameter summary statistics containing
\item \code{SIGMAMAT}: Residual variability matrix
\item \code{OMEGAMAT}: Inter-individual variability matrix
\item \code{OCCMAT}: Inter-occasion variability matrix (NA if not present)
\item \code{EVTAB}: A tibble with event information
\item \code{PATAB}: A tibble with individual parameter estimates
\item \code{COTAB}: A tibble with continuous covariates
\item \code{CATAB}: A tibble with categorical covariates
\item \code{REGTAB}: Regression parameters (empty data.frame if not present)
\item \code{OFV}: A tibble with objective function values
\item \code{COVMAT}: Variance-covariance matrix of parameter estimates
\item \code{CORRMAT}: Correlation matrix of parameter estimates
\item \code{OPTIONS}: Model options (NULL if not present)
\item \code{PROJNAME}: Project name
}
\item \code{GCO}: SimuRg generalized control object parsed from the mlxtran project with:
\itemize{
\item \code{headers}: List of dataset column descriptors (\code{name}, \code{use}, \code{type})
\item \code{data}: Path to source data file
\item \code{model}: Path to model file
\item \code{task_opt}: Task options placeholder (empty object)
\item \code{covs}: Covariate names detected
\item \code{project_name}: Project name
\item \code{theta}: List of population parameter definitions (\code{NAME}, \code{INIT}, \code{EST}, \code{TRANS})
\item \code{ruv}: Residual error model definition (\code{YNAME}, \code{DVID}, \code{TRANS}, \code{PRED}, \code{ERR}, \code{INIT}, \code{EST})
\item \code{re}: Between-subject variability matrices (\code{init}, \code{est})
\item \code{occ}: Between-occasion variability matrices (\code{init}, \code{est})
\item \code{modelText}: Text content of the model file
}
}
}
\description{
Reads and parses Monolix project output files (.mlxtran and associated data files)
into a structured list of SimuRg objects including parameter estimates, individual predictions,
residuals, and diagnostic information.
}
\details{
This function serves as a bridge between Monolix output and R by parsing the .mlxtran file
and associated data files to create a comprehensive R object containing all relevant model
outputs. This facilitates further analysis, visualization, and reporting in R.

The function automatically detects and imports various components of Monolix output
including population parameters, individual parameters, covariates, and diagnostic
metrics.

If \code{save_file = TRUE}, the function additionally writes \code{GCO} and \code{GFO}
JSON files and \code{.RData} files to \code{folder_path}.
}
\examples{
\donttest{
library(stringr)
# Convert Monolix project results
test_folder <- system.file("extdata", "Monolix_objects", package = "SimuRg")
if (substr(test_folder, nchar(test_folder), nchar(test_folder)) != "/")
  test_folder <- str_c(test_folder, "/")
pro_name <- "proj-solo"
result <- sg_converter(folder_path = test_folder, proj_name = pro_name)
# save(results, file = "./models/simurg_object/Warfarin_PK.RData")
# Access individual predictions
head(result$GFO$SDTAB)

# View parameter estimates
print(result$GFO$SUMTAB)

# Check objective function value
print(result$GFO$OFV)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{warfarin}
\alias{warfarin}
\title{Warfarin PKPD dataset}
\format{
\subsection{\code{warfarin}}{

A data frame with 1700 rows and 17 columns:
\describe{
\item{ID}{identifier of the individual}
\item{TIME}{time of the dose or observation record (nominal)}
\item{DV}{records the measurement data}
\item{DVID}{identifier for the observation type (to distinguish different types of observations, e.g PK and PD)}
\item{DVNAME}{name for the observation type (to distinguish different types of observations, e.g PK and PD)}
\item{CMT}{compartment of the event}
\item{ADM}{identifier for the type of dose}
\item{AMT}{dose amount}
\item{EVID}{identifier to indicate if the line is a dose-line or a response-line}
\item{MDV}{identifier to ignore the observation information of that line}
\item{AGE}{age of the individual}
\item{SEX}{sex of the individual}
\item{WEIGHT}{weight of the individual}
\item{BMI}{BMI of the individual}
\item{CLCR}{creatinine clearance of the individual}
\item{CYP2C9_gentyp}{CYP2C9 genotype of the individual}
\item{VKORC1_gentyp}{VKORC1 genotype of the individual}
}
}
}
\usage{
warfarin
}
\description{
Generated dataset with warfarin PK/PD measurements and covariates for 100 patients.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gfo4cov}
\alias{gfo4cov}
\title{Warfarin PopPK model fitting output (4-covariate model)}
\format{
\subsection{\code{gfo4cov}}{

A named list with 12 elements:
\describe{
\item{PROJNAME}{character string identifying the project run (\code{"run_4cov"})}
\item{SUMTAB}{data frame with 15 rows and 11 columns containing the parameter
summary table. Columns: \code{PAR} (parameter name), \code{VALUE} (estimate),
\code{TYPE} (category: \code{"Typical values"}, \code{"Covariate effects"},
\code{"Random effects"}, or \code{"Residual error model"}), \code{EST}
(estimation status), \code{SE} (standard error), \code{RSE} (relative SE, \\%),
\code{LCI} and \code{UCI} (95\\% confidence interval bounds),
\code{ETAshrinkage_var}, \code{ETAshrinkage_sd}, \code{EPSshrinkage_sd} (shrinkage metrics).}
\item{SDTAB}{data frame with 1600 rows and 11 columns — the standard table of
observations and model predictions. Columns: \code{ID}, \code{TIME}, \code{DV}
(observed), \code{DVID}, \code{PRED} (population prediction), \code{IPRED}
(individual prediction), \code{RES} (residual), \code{IRES} (individual residual),
\code{WRES} (weighted residual), \code{IWRES} (individual weighted residual), \code{MDV}.}
\item{PATAB}{data frame with 100 rows and 7 columns — individual parameter table.
Columns: \code{ID}, \code{eta_ka}, \code{eta_Vd}, \code{eta_CL}
(individual ETA random effects), \code{ka}, \code{Vd}, \code{CL}
(individual post-hoc PK parameter estimates).}
\item{COVMAT}{15 \eqn{\times} 15 numeric matrix — covariance matrix of the
population parameter estimates.}
\item{CORRMAT}{15 \eqn{\times} 15 numeric matrix — correlation matrix of the
population parameter estimates.}
\item{OFV}{data frame with 1 row and 4 columns — model fit criteria:
\code{LL} (log-likelihood), \code{AIC}, \code{BIC}, \code{BICc}.}
\item{OMEGAMAT}{3 \eqn{\times} 3 numeric matrix — OMEGA variance-covariance
matrix of the inter-individual random effects (\code{eta_ka}, \code{eta_Vd}, \code{eta_CL}).}
\item{SIGMAMAT}{1 \eqn{\times} 1 numeric matrix — SIGMA residual error
variance matrix.}
\item{EVTAB}{data frame with 100 rows and 6 columns — dosing/event table.
Columns: \code{ID}, \code{TIME}, \code{EVID}, \code{AMT}, \code{ADM}, \code{CMT}.}
\item{COTAB}{data frame with 100 rows and 6 columns — continuous covariate table.
Columns: \code{ID}, \code{AGE}, \code{WEIGHT}, \code{BMI},
\code{LG_WEIGHT} (log10-transformed weight), \code{LG_AGE} (log10-transformed age).}
\item{CATAB}{data frame with 100 rows and 4 columns — categorical covariate table.
Columns: \code{ID}, \code{SEX} (0 = female, 1 = male),
\code{CYP2C9} (genotype integer code, 0–5),
\code{VKORC1} (genotype: \code{"GG"}, \code{"AG"}, or \code{"AA"}).}
}
}
}
\source{
Generated from the warfarin PopPK model fitting run \code{"run_4cov"} used
in the covariate sensitivity analysis workflow.
}
\usage{
gfo4cov
}
\description{
A list containing the full output of a warfarin population pharmacokinetic
model fit with four covariates (SEX, LG_WEIGHT, CYP2C9, VKORC1), including
parameter estimates, model diagnostics, individual predictions, and covariate data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-localsens-sim.R
\name{sg_localsens_sim}
\alias{sg_localsens_sim}
\title{Perform local sensitivity analysis simulations}
\usage{
sg_localsens_sim(
  model,
  params,
  stimes,
  output = NULL,
  perc = 0.2,
  lb = NULL,
  ub = NULL,
  cov = NULL,
  et,
  theta = NULL,
  n_sim = 10
)
}
\arguments{
\item{model}{An RxODE model object.}

\item{params}{Character vector of parameter names to vary in the analysis.}

\item{stimes}{Numeric vector of time points for the simulation.}

\item{output}{Character vector of model outputs (variables) to keep. Default is NULL (all outputs).}

\item{perc}{Numeric. If lb/ub not provided, defines ±percentage range around parameter base value. Default 0.2.}

\item{lb}{Numeric vector of lower bounds for each parameter. Length must match \code{params}. Default NULL.}

\item{ub}{Numeric vector of upper bounds for each parameter. Length must match \code{params}. Default NULL.}

\item{cov}{Character vector of covariate names to include in the simulation. Default NULL.}

\item{et}{Event table for the simulation.}

\item{theta}{Named numeric vector of baseline parameters. Default NULL.}

\item{n_sim}{Integer. Number of points to simulate between LB and UB for each parameter. Default 10.}
}
\value{
A tibble containing the simulated results with the following columns:
\describe{
\item{NPOP}{Optional. Population identification number. From simulation.}
\item{ID}{Optional. Individual subject ID. From simulation.}
\item{TIME}{Time points of simulation.}
\item{VAR}{Name of the output variable (model compartment or biomarker).}
\item{VALUE}{Simulated value of the variable.}
\item{mean, median, min, max, sd, etc.}{Optional. Aggregated statistics, if aggregation applied.}
\item{COV1,...COVn}{Optional. Covariate values, if \code{cov} provided.}
\item{PARNAME}{Name of the parameter being varied in this simulation.}
\item{PARVAL}{Value of the parameter used in this simulation.}
\item{PARVAL_NORM}{Normalized parameter value between 0 and 1 relative to LB and UB. Useful for plotting color gradients.}
}
}
\description{
Generates simulation datasets for a model varying each specified parameter
within provided bounds or relative percentage. Supports multiple outputs and covariates.
}
\examples{
\donttest{
library(rxode2)
library(tibble)
mod_ex <- RxODE({
  # Doses in mg
  # Time in hours
  ### Parameter values
  # Typical
  POPCL = 5;
  POPVC = 180;
  POPQ = 7;
  POPVP = 52;
  POPKTR = 6;
  FBIOPAR = 1;
  # Covariate coefficients
  POPIGFRCOV = 0.8;
  POPPATCOVCLGR1 = 0.85;
  POPPATCOVCLGR2 = 0.85;
  POPPATCOVCLGR3 = 0.85;
  POPPATCOVCLGR4 = 0.85;
  # Random effects
  PPVCL = 0;
  PPVVC = 0;
  PPVKTR = 0;
  # Residual error
  RUV = 0;
  ### Covariates
  PATCOVCL = 1
  if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
  if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
  if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
  if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
  IGFRCOV = (IGFR/112)^POPIGFRCOV
  ## Parameters
  CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
  VC = POPVC * exp(PPVVC);
  Q = POPQ;
  VP = POPVP;
  KTR = POPKTR * exp(PPVKTR);
  ### Explicit functions
  Cc = Ac/VC;                 # nmol/L
  Cp = Ap/VP;                 # nmol/L
  ### Initial conditions
  At1(0) = 0;         # mg
  At2(0) = 0;         # mg
  At3(0) = 0;         # mg
  At4(0) = 0;         # mg
  At5(0) = 0;         # mg
  At6(0) = 0;         # mg
  Ad(0) = 0;          # mg
  Ac(0) = 0;          # mg
  Ap(0) = 0;          # mg
  ### ODEs
  d/dt(At1) = - KTR*At1;
  d/dt(At2) = KTR*At1 - KTR*At2;
  d/dt(At3) = KTR*At2 - KTR*At3;
  d/dt(At4) = KTR*At3 - KTR*At4;
  d/dt(At5) = KTR*At4 - KTR*At5;
  d/dt(At6) = KTR*At5 - KTR*At6;
  d/dt(Ad) = KTR*At6 - KTR*Ad;
  d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
  d/dt(Ap) = Q*(Cc - Cp);
  FBIO = FBIOPAR
  f(At1) = FBIO*1000000/505;
  CHECKRUV = RUV;
  Cc_ResErr = Cc + RUV*Cc;
})
et_base <- tribble(
  ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
  1,   0,     1,     1,    10,   2,     24,  112,   1
)
sens_loc <- sg_localsens_sim(
  model = mod_ex,
  params = c("POPCL", "POPVC"),
  stimes = seq(0, 168, 0.1),
  output = "Cc",
  perc = 0.9,
  cov = c("IGFR", "POPN"),
  et = et_base
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-fit.R
\name{sg_fit}
\alias{sg_fit}
\title{Run fit with monolix/simurg/nonmem fitter}
\usage{
sg_fit(
  model,
  data,
  headers,
  theta,
  ruv,
  re,
  occ,
  covs,
  project_name,
  task_opt = NULL,
  opt_name = "Monolix",
  fit = FALSE,
  path_to_save_output = NULL,
  path_to_fitter = NULL,
  max_wait_time = 3600
)
}
\arguments{
\item{model}{string. Path to a txt file file with model structure in Monolix syntax. This should be a valid file path pointing to a model file that defines the pharmacokinetic/pharmacodynamic model structure}

\item{data}{String. Path to the dataset used to fit a model. Should be a CSV file containing the pharmacokinetic/pharmacodynamic data with appropriate column structure matching the headers specification}

\item{headers}{List. A list specifying the data frame column headers.
Each element should be a list containing column information with the following structure:
\itemize{
\item \code{name} - string, column name in the dataset
\item \code{use} - string, column usage type. Valid values include:
\itemize{
\item "identifier" for subject ID columns
\item "time" for time columns
\item "observation" for dependent variable columns
\item "observationtype" for observation type identifier
\item "administration" for administration route
\item "amount" for dose amount
\item "eventidentifier" for event ID
\item "missingdependentvariable" for missing DV flag
\item "covariate" for covariate columns
}
\item \code{type} - string or NULL, data type specification. For observations use
"continuous", "count/categorical" or "event", depending on the nature of
observations. For covariates use "continuous" or "categorical". Can be NULL for non-covariate columns.
}}

\item{theta}{tibble or data.frame. Should contain the following columns:
\itemize{
\item \code{NAME} - string, parameter name. Should match names specified in the model file
\item \code{TRANS} - string, parameter transformation type. Can be: \code{"normal"}, \code{"logNormal"}, \code{"logitNormal"}
\item \code{INIT} - numeric, initial value for the parameter
\item \code{LB} - numeric, lower bound for logit transformation, \code{NA} for other transformations
\item \code{UB} - numeric, upper bound for logit transformation,  \code{NA} for other transformation
\item \code{EST} - logical, whether to estimate this parameter.
}}

\item{ruv}{A list specifying options for residual error used in model fit, containing:
\itemize{
\item \code{YNAME} - string, output name. Typically \code{"y1"}, \code{"y2"},...
\item \code{DVID} - numeric, observation type identifier corresponding to DVID column values
\item \code{TRANS} - string, residual error distribution. Can be: \code{"normal"}, \code{"logNormal"}, \code{"logitNormal"}
\item \code{PRED} - string, prediction variable name from the model
\item \code{ERR} - string, error model type. Options include:
\itemize{
\item "constant" for additive error
\item "proportional" for proportional error
\item "combined1" for combined additive and proportional error
}
\item \code{INIT} - numeric vector, initial values for error parameters (length depends on error model)
\item \code{EST} - logical vector, whether to estimate each error parameter (same length as INIT)
\item \code{BLQM} - below limit of quantification method (can be NULL)
For several observation types list of lists should be provided:
one residual error (ruv) object for each observations.
}}

\item{re}{List. A list specifying options for random effects used in model fit, containing:
\itemize{
\item \code{init} - matrix, initial values for the variance-covariance matrix of random
effects. Rows and columns should correspond to parameters defined in theta.
Diagonal elements represent variances, off-diagonal elements represent covariances. Use 0 for no variability
\item \code{est} - matrix, logical matrix of same dimensions as \code{init} specifying
which variance-covariance elements to estimate. Use \code{TRUE} to estimate,
\code{FALSE} to fix, \code{NA} to not use this random effect.
}}

\item{occ}{A list specifying interoccasion variability properties, containing:
\itemize{
\item \code{init} - matrix, initial values for the interoccasion variance-covariance matrix.
Same structure as \code{re$init} but for occasion-to-occasion variability. Use 0 for no interoccasion variability
\item \code{est} - matrix, logical matrix specifying which interoccasion variance-covariance
elements to estimate. Use \code{TRUE} to estimate, \code{FALSE} to fix, \code{NA} for
elements not applicable. Typically all elements are NA when no interoccasion variability is modeled.
}}

\item{covs}{A list of covariate structures. Each element should be a list containing covariate relationship definitions with the following structure:
\itemize{
\item \code{PAR} - string, name of the parameter to which covariate effect is applied
\item \code{COVNAME} - string, name of the covariate column in the dataset
\item \code{FUNC} - string, functional form of the covariate effect
\item \code{TRANS} - string, transformation applied to covariate
\item \code{REF} - numeric, reference value for categorical covariates
\item \code{INIT} - numeric, initial value for the covariate effect parameter
\item \code{EST} - logical, whether to estimate the covariate effect
Can be \code{NULL}, if no covariates are used. Default is \code{NULL}
}}

\item{project_name}{String. The name of the Monolix project without file extension.
This will be used as the base name for output files and directories.}

\item{task_opt}{String. Additional task options to be passed to the fitting software.
For Monolix, this can include specific task configurations or optimization settings.
When \code{NULL}, default tasks (populationParameters, individualParameters, fim, logLikelihood)
will be used.  Default is \code{NULL}.}

\item{opt_name}{String. Specify the optimizer/fitter to use for model fitting.
Currently supported options:
\itemize{
\item \code{"Monolix"} - uses Monolix Suite for population pharmacokinetic modeling (generates .mlxtran files)
\item \code{"Simurg"} - uses SimuRg internal fitter (generates .R files with JSON control structure)
Default is \code{"Monolix"}.
}}

\item{fit}{Logical. If \code{TRUE}, the model fitting will be executed immediately
using the specified fitter. If \code{FALSE}, only the fit configuration file will
be generated without running the fit. Set to \code{FALSE} for file preparation only,
or \code{TRUE} to run the complete fitting process. Default is \code{FALSE}.}

\item{path_to_save_output}{String. Path to save fit output files.
Should be a valid directory path where the fit results and project files
will be saved. If \code{NULL}, current working directory will be used. Default is \code{NULL}.}

\item{path_to_fitter}{String. The path to the program fitter executable.
For Monolix, this should point to the monolix.bat file.
If \code{NULL}, "C:/ProgramData/Lixoft/MonolixSuite2023R1/bin/monolix.bat" will be used as default. Default is \code{NULL}.}

\item{max_wait_time}{numeric. Maximum time in seconds to wait for fit results to complete. Default is 3600 seconds (1 hour). Set to \code{Inf} for no timeout.}
}
\value{
if option \code{fit = TRUE}, generalized simurg output object is returned. Otherwise, the file for fit is written and no output is returned
}
\description{
Run fit with monolix/simurg/nonmem fitter
}
\examples{
\donttest{
library(tibble)
library(dplyr)
library(stringr)
# Specify structural model path. For fit, should be in Monolix syntax
model <- system.file("extdata", "models", "model_PK_1c.txt", package = "SimuRg")
# Specify data path. Should be ADPPK-like format
data  <- system.file("extdata", "datasets", "dspk-warf.csv", package = "SimuRg")

# Specify headers. Should be a list of lists with the following elements:
# name - string, name of the column
# use - string, use of the column from Monolix documentation
# type - string, type of the column. for use = "covariate", should be
# "continuous" or "categorical", for other uses should be NULL
 headers <- list(list(name = "ID", use = "identifier", type = NULL),
                 list(name = "TIME", use = "time", type = NULL),
                 list(name = "DV", use = "observation", type = "continuous"),
                 list(name = "DVID", use = "observationtype", type = NULL),
                 list(name = "ADM", use = "administration", type = NULL),
                 list(name = "AMT", use = "amount", type = NULL),
                 list(name = "EVID", use = "eventidentifier", type = NULL),
                 list(name = "MDV", use = "missingdependentvariable", type = NULL),
                 list(name = "AGE", use = "covariate", type = "continuous"),
                 list(name = "AGE_centered", use = "covariate", type = "continuous"),
                 list(name = "SEX", use = "covariate", type = "categorical"),
                 list(name = "WEIGHT", use = "covariate", type = "continuous"),
                 list(name = "BMI", use = "covariate", type = "continuous"))

# Dataset with the parameters properties. Should be a tibble with the following columns:
# NAME - string, name of the parameter
# TRANS - string, distribution of the parameter. Should be one of the following:
# "normal", "logNormal", "logitNormal"
# INIT - numeric, initial value of the parameter or its fixed value
# LB - numeric, lower bound of the parameter for logit transformation
# UB - numeric, upper bound of the parameter for logit transformation
# EST - logical, estimation status of the parameter. For estimation, should
# be TRUE, for fixed, should be FALSE
 theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
                  "Cl", "logNormal", 0.2, NA, NA, TRUE,
                  "V", "logNormal", 20, NA, NA, TRUE,
                  "ka", "logNormal", 0.2, NA, NA, TRUE
 )
 # Examples of random effect model specification
 # Examples of random effect model specification for fit
 # Single observation (legacy format):
 # YNAME - string, name of the observation, ususally y1, y2, ...
 # DVID - numeric, observation type identifier corresponding to DVID column values
 # TRANS - string, residual error distribution. Can be: "normal", "logNormal",
 # "logitNormal"
 # PRED - string, prediction variable name from the model
 # ERR - string, error model type. Options include: "constant" for additive
 # error, "proportional" for proportional error, "combined1" for combined
 # additive and proportional error
 # INIT - numeric vector, initial values for error parameters
 # (length depends on error model)
 # EST - logical vector, whether to estimate each error parameter
 # (same length as INIT)
 # BLQM - below limit of quantification method (can be NULL)
 # Single observation (legacy format):
 ruv <- list(YNAME = "y1", DVID = 1, TRANS = "normal", PRED = "Cc",
             ERR = "combined1", INIT = c(1, 1), EST = c(TRUE, TRUE), BLQM = NULL)

 # Multiple observations (recommended format):
ruv <- list(
  list(YNAME = "y1", DVID = 1, TRANS = "normal", PRED = "Cc",
       ERR = "combined1", INIT = c(1, 1), EST = c(TRUE, TRUE), BLQM = NULL),
  list(YNAME = "y2", DVID = 2, TRANS = "normal", PRED = "EFF",
      ERR = "proportional", INIT = c(0.1), EST = c(TRUE), BLQM = NULL)
)

 # Example of random effects (RE) specification.
 #
 # init matrix: provides the initial values for the random effects.
 # est matrix: controls how each random effect is handled:
 #   TRUE  - the random effect will be estimated,
 #   FALSE - the random effect will be fixed at its initial value,
 #   NA    - no random effect will be applied.
 #
 # To fit a model without random effects, set all entries in the
 # est matrix to NA.
 #
 # The same logic applies to the between-occasion variability
 # matrix (occ).

 re <- list(init = tribble(~Cl, ~V, ~ka,
                           1, 0, 0,
                           0, 0, 0,
                           0, 0, 1) \%>\% as.matrix(),
            est = tribble(~Cl, ~V, ~ka,
                          TRUE, NA, NA,
                          NA, NA, NA,
                          NA, NA, TRUE) \%>\% as.matrix())
# Example of between-occasion variability (BOV) specification. The structure
# is the same as for RE, but for BOV

 occ <- list(init = tribble(~Cl, ~V, ~ka,
                            0, 0, 0,
                            0, 0, 0,
                            0, 0, 0) \%>\% as.matrix(),
             est = tribble(~Cl, ~V, ~ka,
                           NA, NA, NA,
                           NA, NA, NA,
                           NA, NA, NA) \%>\% as.matrix())
# Example of covariate specification. Should be a list of lists with the following elements:
# PAR - string, name of the parameter to which the covariate is applied
# COVNAME - string, name of the covariate
# FUNC - string, function to apply to the covariate. Should be "linear" for
# continuous covariates, "categorical" for categorical covariates
# TRANS - string, transformation of the covariate. Should be "median" for
# continuous covariates, "reference" for categorical covariates
# INIT - numeric, initial value of the covariate
# EST - logical, estimation status of the covariate. For estimation, should
#be TRUE, for fixed, should be FALSE
 covs <- list(list(PAR = "V", COVNAME = "AGE", FUNC = "linear",
                   TRANS = "median", INIT = 1, EST = TRUE),
              list(PAR = "ka", COVNAME = "SEX", REF = 0, INIT = 1, EST = TRUE))
 output_path <- str_c(tempdir(), "/")
 fitter_path <- "C:/ProgramData/Lixoft/MonolixSuite2023R1/bin/monolix.bat"
 # Examples of task_opt parameter

 task_opt <-  paste("populationParameters()", "individualParameters()",
                    "logLikelihood()", sep = "\n")
 task_opt_lin <-  paste("populationParameters()", "individualParameters()",
                        "fim(method = Linearization)",
                        "logLikelihood(method = Linearization)", sep = "\n")
# Generalized control object
# gco <-list(headers = headers,
#            data = data,
#            model = model,
#            task_opt = task_opt
#            covs = covs,
#            project_name = "test-proj",
#            theta = theta,
#            ruv = ruv,
#            re = re,
#            occ = occ,
#            modelText = "")
 result <- sg_fit(model, data, headers, theta, ruv, re, occ, covs,
                  project_name = "my_project", fit = FALSE, # set fit = TRUE for fit
                  path_to_save_output =  output_path,
                  path_to_fitter = fitter_path)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-modbuild.R
\name{sg_modbuild}
\alias{sg_modbuild}
\title{Build estimation model scenarios}
\usage{
sg_modbuild(
  mod_lst,
  data,
  headers,
  ruv_lst,
  theta_lst,
  re_lst,
  path,
  occ_lst,
  covs_lst = NULL,
  task_lst = NULL,
  opt_name = "Simurg",
  project_name = "my_project"
)
}
\arguments{
\item{mod_lst}{List of model file paths or model identifiers used for scenario generation.}

\item{data}{Character string. Path to the dataset used in model fitting or simulation.}

\item{headers}{Predefined list of dataset column names (e.g., ID, TIME, DV, etc.) used by the modeling framework.}

\item{ruv_lst}{List specifying residual unexplained variability (RUV) structures.
Each element contains RUV properties (e.g., \code{YNAME}, \code{ERR}) and mapping to data.}

\item{theta_lst}{List of tibbles describing population parameter properties.
Each tibble should contain columns such as \code{NAME}, \code{INIT}, and \code{EST}.}

\item{re_lst}{List of random effects (RE) specifications.
Each element includes matrices \code{init} and \code{est} defining initial and estimated covariance structures.}

\item{path}{Character. Directory path where output files (CSV summary and model files) will be written. Default is current working directory.}

\item{occ_lst}{List of occasion effect matrices.
Each element includes \code{init} and \code{est} matrices defining inter-occasion variability.}

\item{covs_lst}{Optional list describing covariate-parameter relationships.
Each element should include fields such as \code{PAR}, \code{COVNAME}, and \code{EST} indicating inclusion in the model.}

\item{task_lst}{Optional list defining additional tasks or configuration options for each scenario. Default is NULL.}

\item{opt_name}{Character. Optimization engine name (e.g., \code{"Monolix"}, \code{"Simurg"}). Default \code{"Simurg"}.}

\item{project_name}{Character. Base name for the exported project files. Default \code{"my_project"}.}
}
\value{
The function writes two types of outputs to the specified \code{path}:
\itemize{
\item A CSV file (\code{scenarios_info.csv}) summarizing all generated scenarios with columns:
\describe{
\item{scenario_number}{Unique index of the scenario.}
\item{model}{Model file or path used in the scenario.}
\item{data}{Path to the dataset used.}
\item{theta}{Active population parameters and initial values used.}
\item{RUV}{Residual error structure(s) used.}
\item{RE}{Random effect and correlation terms included.}
\item{OCC}{Active occasion effects.}
\item{COVS}{Included covariate-parameter relationships.}
}
\item Individual \code{.mlxtran} model files for each generated scenario,
named according to the pattern \verb{<project_name>_<i>.mlxtran}.
}
}
\description{
Constructs and exports multiple model configuration scenarios by combining population parameters,
random effects, residual variability, occasion effects, and covariate assignments.
Each scenario is defined by unique combinations of these model components and exported as \code{.mlxtran} files.
Additionally, a summary CSV file describing all scenarios is generated.
}
\details{
This function automates the construction of multiple model configurations for model evaluation,
sensitivity analysis, or scenario testing. It expands parameter and variability definitions,
constructs all possible combinations, and exports model specifications for external fitting tools.
}
\examples{
\donttest{
library(dplyr)
folder_path <- system.file("extdata", package = "SimuRg")
mod_lst <- list(paste(folder_path, "/models/model_PK_1c.txt", sep = "/"),
                  paste(folder_path, "/models/model_PK_2c.txt", sep = ""))

### path to the dataset
data <- paste(folder_path, "datasets", "dspk-warf.csv", sep = "/")
re_lst_1 <- list(
              list(init = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                                    1, 0, 0, 0, 0,
                                    0, 1, 0, 0, 0,
                                    0, 0, 1, 0, 0,
                                    0, 0, 0, 1, 0,
                                    0, 0, 0, 0, 1) \%>\% as.matrix(),
                    est = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                                    TRUE, NA, NA, NA, NA,
                                    NA, TRUE, NA, NA, NA,
                                    NA, NA, TRUE, NA, NA,
                                    NA, NA, NA, TRUE, NA,
                                    NA, NA, NA, NA, TRUE) \%>\% as.matrix(),
                    block = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                                      FALSE, NA, NA, NA, NA,
                                      NA, FALSE, NA, NA, NA,
                                      NA, NA, TRUE, NA, NA,
                                      NA, NA, NA, FALSE, NA,
                                      NA, NA, NA, NA, FALSE) \%>\% as.matrix())
           )
headers <- list(list(name = "ID", use = "identifier", type = NULL),
               list(name = "TIME", use = "time", type = NULL),
               list(name = "DV", use = "observation", type = "continuous"),
               list(name = "DVID", use = "observationtype", type = NULL),
               list(name = "ADM", use = "administration", type = NULL),
               list(name = "AMT", use = "amount", type = NULL),
               list(name = "EVID", use = "eventidentifier", type = NULL),
               list(name = "MDV", use = "missingdependentvariable", type = NULL),
               list(name = "AGE", use = "covariate", type = "continuous"),
               list(name = "AGE_centered", use = "covariate", type = "continuous"),
               list(name = "SEX", use = "covariate", type = "categorical"),
               list(name = "WEIGHT", use = "covariate", type = "continuous"),
               list(name = "BMI", use = "covariate", type = "continuous"))

ruv_lst_2 <- list(
  # structural model 1
 list(
   list(YNAME = "y1",
        DVID = 1,
        TRANS = "normal",
        PRED = "Cc",
        ERR = list("constant", "proportional", "combined1"),
         # options to test (can be length = 1, i.e., "constant" or "combined1")
        INIT = list(1, 1, c(1, 1)),
         # options to test (can be length = 1, i.e., 1 or c(1, 1))
        EST = list(TRUE, TRUE, c(TRUE, TRUE)))
        # options to test (can be length = 1, i.e., T or c(T, T))BLQM = NULL))
 ),
 # structural model 2
 list(
   list(YNAME = "y1",
        DVID = 1,
        TRANS = "normal",
        PRED = "Cc",
        ERR = list("constant", "proportional"),
        # options to test (can be length = 1, i.e., "constant" or "combined1")
        INIT = list(1, 1), # options to test (can be length = 1, i.e., 1 or c(1, 1))
        EST = list(TRUE, TRUE))
         # options to test (can be length = 1, i.e., T or c(T, T))BLQM = NULL))
 )
)

theta_lst_2 <- list(
 # structural model 1
 tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
         "Cl", "logNormal", 0.2, NA, NA, TRUE,
         "Vd", "logNormal", 20, NA, NA, TRUE,
         "ka", "logNormal", 0.2, NA, NA, TRUE),
 # structural model 2
 tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
         "Cl", "logNormal", 0.2, NA, NA, TRUE,
         "Vd", c("Normal", "logNormal"), c(10, 20, 30), NA, NA, TRUE,
         "ka", "logNormal", c(0.2, 0.1, 0.5), NA, NA, TRUE,
         # INIT could be length > 1
         "Vp", c("Normal", "logNormal"), 10, NA, NA, TRUE,
         "Q", "logNormal", 5, NA, NA, TRUE))

path <- tempdir()
sg_modbuild(
 mod_lst = mod_lst[1],
 data = data,
 headers = headers,
 ruv_lst = ruv_lst_2,
 theta_lst = theta_lst_2,
 re_lst = re_lst_1,
 occ_lst = re_lst_1,
 covs_lst = NULL,
 path = path,
 project_name = "tests_test_project"
)
clr_files <- list.files(path, full.names = TRUE)
unlink(clr_files, recursive = TRUE, force = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ds_covval}
\alias{ds_covval}
\title{Warfarin covariate values dataset}
\format{
\subsection{\code{ds_covval}}{

A data frame with 100 rows and 9 columns:
\describe{
\item{ID}{individual patient identifier}
\item{AGE}{age in years}
\item{WEIGHT}{body weight in kg}
\item{BMI}{body mass index (kg/m\ifelse{html}{\out{<sup>2</sup>}}{\eqn{^2}})}
\item{LG_WEIGHT}{log-transformed body weight (log10 scale), used as continuous covariate in the PK model}
\item{LG_AGE}{log-transformed age (log10 scale), used as continuous covariate in the PK model}
\item{SEX}{sex of the individual (0 = female, 1 = male)}
\item{CYP2C9}{CYP2C9 genotype encoded as integer: 0 = \emph{*1/*1} (reference), 1 = \emph{*1/*2}, 2 = \emph{*1/*3}, 3 = \emph{*2/*2}, 4 = \emph{*2/*3}, 5 = \emph{*3/*3}}
\item{VKORC1}{VKORC1 genotype (\code{"GG"}, \code{"AG"}, or \code{"AA"})}
}
}
}
\source{
Derived from \code{data_fin_i.csv} — individual covariate dataset for the warfarin PopPK covariate sensitivity analysis.
}
\usage{
ds_covval
}
\description{
Individual covariate values for 100 patients used in the warfarin
covariate sensitivity simulation, including demographic and pharmacogenomic variables.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sg_dummy}
\alias{sg_dummy}
\title{The function to store common parameters description}
\usage{
sg_dummy(
  ...,
  abreaks = scales::pretty_breaks(7),
  addcov = TRUE,
  addOFV = TRUE,
  addline = TRUE,
  aggr = NULL,
  alpha_i = 0.5,
  atol = 1e-08,
  cap,
  cat_cov_l,
  ci_band_alpha = 0.2,
  ci_band_col = "firebrick",
  ci_limits = c(0.8, 1.25),
  ci_quantiles = c("P025", "P975"),
  cont_cov_l,
  cov_cols,
  cov,
  covint = "locf",
  covsens_res,
  ciLow = 0.025,
  ciUp = 0.975,
  col_i,
  col_lab,
  covs,
  data,
  dens,
  ds_covs,
  ds_i,
  ds_parest = NULL,
  dt_obs_fl = FALSE,
  DVID = 1,
  dv_col = "DV",
  emp_perc = TRUE,
  errorbar_width = 0.2,
  est_covmat,
  et,
  eta_seq = NULL,
  excl_col = NULL,
  exclude_vars = NULL,
  f_scales = "fixed",
  facet_i,
  fill_i = NULL,
  filt = "T",
  fit = FALSE,
  fpath_i,
  free_stat = "free",
  group_i = "VAR",
  headers,
  ncores = 1,
  id_col = NULL,
  indiv = TRUE,
  inits = NULL,
  keep = NULL,
  lab_x,
  lab_y,
  legend_fl = FALSE,
  levels_discrete = 10,
  log_x = FALSE,
  log_y = FALSE,
  log_axes = FALSE,
  lty_i = NULL,
  max_x = NULL,
  max_y = NULL,
  method = "lsoda",
  min_x = NULL,
  min_y = NULL,
  model,
  n_bins = 30,
  n_quantiles = 3,
  n_sim,
  no_leg = FALSE,
  npop = 1,
  nsub = 1,
  occ,
  opt_name = "Monolix",
  omega,
  outputs = NULL,
  output = NULL,
  path_to_fitter = NULL,
  path_to_save_output = NULL,
  par_bounds,
  par_seq = NULL,
  par_type = "Ind",
  params = NULL,
  piLow = 0.1,
  piUp = 0.9,
  plot_type = "DIST",
  point_size = 2.5,
  pred.corr = FALSE,
  project_name,
  quantiles = c(0.1, 0.9),
  re,
  ref_line_col = "grey25",
  rtol = 1e-06,
  run_id = 1,
  ruv,
  sc_factor = 1,
  scale = NULL,
  seed = 123,
  sigma = NULL,
  shp_i = NULL,
  smooth = TRUE,
  stat_comp = NULL,
  stimes = NULL,
  task_opt = NULL,
  tdist = TRUE,
  time_col = "TIME",
  theor_perc = TRUE,
  theor_percCI = TRUE,
  theta = NULL,
  thetamat = NULL,
  tsld = FALSE,
  val_col = "VALUE",
  wrap_i = NULL,
  wrap_ncol = NULL,
  wrap_nrow = NULL
)
}
\arguments{
\item{...}{Other arguments that will be passed to rxSolve function.}

\item{abreaks}{A function that generates axis breaks. Default is \code{scales::pretty_breaks(7)}.}

\item{addcov}{Logical. If \code{TRUE}, columns with covariate values will be added
to the resulting dataset. Default is \code{TRUE}.}

\item{addOFV}{Logical. If \code{TRUE}, information about OFV and AIC of the model
will be added. Default is \code{TRUE}.}

\item{addline}{Logical. If \code{TRUE}, lines connecting observations of
individual subjects will be added. Default is \code{TRUE}.}

\item{aggr}{A string that specifies the aggregation type. Set to \code{NULL} for
no aggregation, \code{ID} for aggregation by time, population and ID, \code{NPOP} for
aggregation by population and time, \code{TIME} for aggregation by time.
Default is \code{NULL}.}

\item{alpha_i}{Numeric. Transparency level (from 0 to 1) for points/lines.
Default is 0.5.}

\item{atol}{A numeric absolute tolerance used by the ODE solver to determine
if a good solution has been achieved. This is also used in the solved linear
model to check if prior doses do no add anything to the solution.
Default is 1e-8.}

\item{cap}{A string that specifies the plot caption.}

\item{cat_cov_l}{A named list. Each element defines one categorical covariate
and must itself be a list with components:
\describe{
\item{\code{NAME}}{Character. Column name of the covariate
(e.g. \code{"SEX"}).}
\item{\code{NICENAME}}{Character or \code{NULL}. Display label.}
\item{\code{REF}}{Character or \code{NULL}. Reference category value.
If \code{NULL}, the first factor level (alphabetically) is used.}
\item{\code{par_vec}}{Character vector. Model parameter(s) affected by
this covariate (e.g. \code{c("ka")}).}
}}

\item{ci_band_alpha}{Numeric in \eqn{[0,1]}: transparency of the shaded band.
Default \code{0.2}.}

\item{ci_band_col}{Color for the band fill and dotted limit lines.
Default \code{"firebrick"}.}

\item{ci_limits}{Numeric vector of length 2: lower and upper bounds of the
shaded acceptance band and of the dotted horizontal guides.  Default
\code{c(0.8, 1.25)} is a common bioequivalence-style window on the ratio
scale.}

\item{ci_quantiles}{Character vector of length 2: names of the lower and
upper uncertainty columns in the sensitivity table, in that order.
Defaults \code{c("P025", "P975")} to match the default percentiles in
\code{sg_covsens_sim}.  Use other names (e.g. \code{c("P05", "P95")}) if
you changed \code{quantiles} in the simulation and the columns exist.}

\item{cont_cov_l}{A named list. Each element defines one continuous covariate
and must itself be a list with components:
\describe{
\item{\code{NAME}}{Character. Column name of the (transformed)
covariate in the dataset (e.g. \code{"LG_AGE"}).}
\item{\code{UTNAME}}{Character or \code{NULL}. Column name of the
untransformed (back-transformed) covariate
(e.g. \code{"AGE"}).  If \code{NULL} or \code{NA}, defaults to
\code{NAME}.}
\item{\code{REF}}{Character or numeric. Reference value for the
covariate.  Use \code{"median"} to derive from data, or a numeric
value.}
\item{\code{NICENAME}}{Character or \code{NULL}. Display label for
plots and tables (e.g. \code{"Age, years"}).}
\item{\code{par_vec}}{Character vector. Model parameter(s) affected by
this covariate (e.g. \code{c("CL")}).}
}}

\item{cov_cols}{A character vector specifying the names of the columns with covariates.}

\item{cov}{Covariates passed to \code{sg_sim()}. Default is \code{NULL}.}

\item{covint}{String. Specifies the interpolation method for time-varying
covariates. When solving ODEs it often samples times outside the sampling
time specified in event table. When this happens, the time varying
covariates are interpolated. Currently this can be:
\itemize{
\item \code{"linear"} interpolation, which interpolates the covariate by solving the
line between the observed covariates and extrapolating the new covariate value
\item \code{"locf"} last observation carried forward
\item \code{"NOCB"} next observation carried backward.
This is the same method that NONMEM uses
\item \code{"midpoint"} last observation carried forward to midpoint;
next observation carried backward to midpoint
}

Default is \code{"locf"}}

\item{covsens_res}{Named list as returned by \code{sg_covsens_sim()}.  Must
contain the element selected by \code{type} (\code{PARSENS} and/or
\code{EXPSENS} data.frames with columns \code{LAB}, \code{VAR},
\code{mean}, \code{Type}, and the interval columns named by
\code{ci_quantiles}).}

\item{ciLow}{Numeric. Lower confidence interval bound. Default is 0.025}

\item{ciUp}{Numeric. Upper confidence interval bound. Default is 0.975}

\item{col_i}{String. Column name for color}

\item{col_lab}{String. Label for color legend}

\item{covs}{A list of covariate structures. Each element should be a list containing covariate relationship definitions with the following structure:
\itemize{
\item \code{PAR} - string, name of the parameter to which covariate effect is applied
\item \code{COVNAME} - string, name of the covariate column in the dataset
\item \code{FUNC} - string, functional form of the covariate effect
\item \code{TRANS} - string, transformation applied to covariate
\item \code{REF} - numeric, reference value for categorical covariates
\item \code{INIT} - numeric, initial value for the covariate effect parameter
\item \code{EST} - logical, whether to estimate the covariate effect
Can be \code{NULL}, if no covariates are used. Default is \code{NULL}
}}

\item{data}{String. Path to the dataset used to fit a model. Should be a CSV file containing the pharmacokinetic/pharmacodynamic data with appropriate column structure matching the headers specification}

\item{dens}{Logical. If \code{TRUE}, plot histogram/density of residuals instead of scatter}

\item{ds_covs}{Data.frame. The dataframe with covariates}

\item{ds_i}{Data.frame. The data frame with source data.}

\item{ds_parest}{Data.frame. Parameter estimates table with columns
\code{parameter} and \code{value}.
Required when \code{fpath_i} is \code{NULL}; must be provided together
with \code{ds_covs}.  Default is \code{NULL}.}

\item{dt_obs_fl}{Logical. Show observed data points. Default is \code{FALSE}}

\item{DVID}{Restrict \code{SDTAB} to one observation type. Numeric values
select the \code{DVID} column (default \code{1}). If \code{SDTAB} has
\code{DVNAME}, a character or factor is matched to \code{DVNAME} first;
otherwise a digit-only string is coerced to numeric \code{DVID}.}

\item{dv_col}{Character. Name of DV column in data_i. Default is \code{DV}}

\item{emp_perc}{Logical. Show empirical percentiles. Default is \code{TRUE}}

\item{errorbar_width}{Numeric. Width argument for \code{geom_errorbar}.  Default
\code{0.2}.}

\item{est_covmat}{Data.frame. Parameter estimation covariance matrix.  The
first column (\code{X1}) must list parameter names; remaining columns
(named identically) form the symmetric variance–covariance matrix.}

\item{et}{An event table passed to \code{sg_sim()}.}

\item{eta_seq}{Vector of strings. Character vector of parameter names to be plotted. If \code{NULL}, all parameters be included. Default is \code{NULL}}

\item{excl_col}{Character vector. Contains column names to exclude from synthesis. Default: \code{NULL}}

\item{exclude_vars}{Character vector. Contains \code{VAR} levels to omit (e.g.
\code{"Cc_Cmin"}).  \code{NULL} keeps all rows.}

\item{f_scales}{String, one of \code{"fixed"}, \code{"free"}, \code{"free_x"}, \code{"free_y"}. User can specify whether the scales (x and y axes) should be fixed across all panels (\code{"fixed"}), free for each panel (\code{"free"}), or free only in one dimension (\code{"free_x"} or \code{"free_y"}). Default is \code{"fixed"}}

\item{facet_i}{String. Column name for facet}

\item{fill_i}{String. Column name for fill aesthetic. Default is \code{NULL}}

\item{filt}{String. Provide a filter to apply. Default is \code{"T"}}

\item{fit}{Logical. If \code{TRUE}, the model fitting will be executed immediately
using the specified fitter. If \code{FALSE}, only the fit configuration file will
be generated without running the fit. Set to \code{FALSE} for file preparation only,
or \code{TRUE} to run the complete fitting process. Default is \code{FALSE}.}

\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{free_stat}{String. Facet scaling option. One of \code{"free"}, \code{"free_x"},
\code{"free_y"}, or \code{"fixed"}. Default is \code{'free'}.}

\item{group_i}{String. Primary grouping variable for lines. Default is \code{'VAR'}.}

\item{headers}{List. A list specifying the data frame column headers.
Each element should be a list containing column information with the following structure:
\itemize{
\item \code{name} - string, column name in the dataset
\item \code{use} - string, column usage type. Valid values include:
\itemize{
\item "identifier" for subject ID columns
\item "time" for time columns
\item "observation" for dependent variable columns
\item "observationtype" for observation type identifier
\item "administration" for administration route
\item "amount" for dose amount
\item "eventidentifier" for event ID
\item "missingdependentvariable" for missing DV flag
\item "covariate" for covariate columns
}
\item \code{type} - string or NULL, data type specification. For observations use
"continuous", "count/categorical" or "event", depending on the nature of
observations. For covariates use "continuous" or "categorical". Can be NULL for non-covariate columns.
}}

\item{ncores}{Integer. Number of cores used for calculations. Default is 1.}

\item{id_col}{Character string. Specify the name of the identifier column
to exclude from synthesis. Default: \code{NULL}.}

\item{indiv}{Logical. If \code{TRUE}, use individual predictions (\code{"IPRED"});
otherwise use population predictions (\code{"PRED"}). Default is \code{TRUE}.}

\item{inits}{A named vector specifying initial conditions for model variables; Default is \code{NULL}.}

\item{keep}{Vector of strings. Columns of event table to keep in the output data frame. Default is \code{NULL}.}

\item{lab_x}{String. X-axis label.}

\item{lab_y}{String. Y-axis label.}

\item{legend_fl}{Logical. Show legend. Default is \code{FALSE}.}

\item{levels_discrete}{Integer. Maximum unique values to consider a variable discrete. Default is 10.}

\item{log_x}{Logical. If \code{TRUE}, a logarithmic scale is applied to x-axis. Default is \code{FALSE}.}

\item{log_y}{Logical. If \code{TRUE}, a logarithmic scale is applied to y-axis. Default is \code{FALSE}.}

\item{log_axes}{Logical. If \code{TRUE}, a logarithmic scale is applied to all axes. Default is \code{FALSE}.}

\item{lty_i}{String. Column name for linetype aesthetic. Default is \code{NULL}.}

\item{max_x}{Numeric. X-axis maximum limit. Default is \code{NA}.}

\item{max_y}{Numeric. Y-axis maximum limit. Default is \code{NA}.}

\item{method}{Character string. \code{"PRCC"} or \code{"eFAST"}.}

\item{min_x}{Numeric. X-axis minimum limit. Default is \code{NA}.}

\item{min_y}{Numeric. Y-axis minimum limit. Default is \code{NA}.}

\item{model}{A model object passed to \code{sg_sim()}.}

\item{n_bins}{Integer. Number of bins to use in the histogram. Default is 30.}

\item{n_quantiles}{Integer. Number of quantile groups for continuous variables in \code{col_i}. Default is 3.}

\item{n_sim}{Integer. Number of samples (LHS size for PRCC, base frequency size for eFAST).}

\item{no_leg}{Logical. If \code{TRUE}, the legend is not shown. Default is \code{FALSE}}

\item{npop}{Integer. Number of population replicates. Default is 1.}

\item{nsub}{Integer. Number of subjects sampled per population (omega/sigma matrices per ID). Default is 1.}

\item{occ}{A list specifying interoccasion variability properties, containing:
\itemize{
\item \code{init} - matrix, initial values for the interoccasion variance-covariance matrix.
Same structure as \code{re$init} but for occasion-to-occasion variability. Use 0 for no interoccasion variability
\item \code{est} - matrix, logical matrix specifying which interoccasion variance-covariance
elements to estimate. Use \code{TRUE} to estimate, \code{FALSE} to fix, \code{NA} for
elements not applicable. Typically all elements are NA when no interoccasion variability is modeled.
}}

\item{opt_name}{String. Specify the optimizer/fitter to use for model fitting.
Currently supported options:
\itemize{
\item \code{"Monolix"} - uses Monolix Suite for population pharmacokinetic modeling (generates .mlxtran files)
\item \code{"Simurg"} - uses SimuRg internal fitter (generates .R files with JSON control structure)
Default is \code{"Monolix"}.
}}

\item{omega}{A named matrix or vector.}

\item{outputs}{Vector of strings. Names of the model variables to output.
If \code{NULL}, all variables returned. Default is \code{NULL}.}

\item{output}{A character vector of outputs to keep. Passed to \code{sg_sim()}.}

\item{path_to_fitter}{String. The path to the program fitter executable.
For Monolix, this should point to the monolix.bat file.
If \code{NULL}, "C:/ProgramData/Lixoft/MonolixSuite2023R1/bin/monolix.bat" will be used as default. Default is \code{NULL}.}

\item{path_to_save_output}{String. Path to save fit output files.
Should be a valid directory path where the fit results and project files
will be saved. If \code{NULL}, current working directory will be used. Default is \code{NULL}.}

\item{par_bounds}{A tibble or data frame with columns \code{PAR}, \code{LB}, \code{UB}.}

\item{par_seq}{Vector of strings. Character vector of parameter names to be plotted. If \code{NULL}, all parameters be included. Default is \code{NULL}}

\item{par_type}{String. A character string specifying the type of parameters used for
theoretical distribution overlay (only relevant when \code{plot_type = 'DIST'}
and \code{tdist = TRUE}). If 'Ind' - Individual (default), distributions are shown on the
natural parameter scale assuming log-normal variability. If 'RE' - Random Effect, distributions are shown on the ETA scale,
assuming a normal distribution with mean zero and covariance defined
by \verb{$OMEGAMAT}, without transformation.}

\item{params}{Character vector of parameter names to vary.}

\item{piLow}{Numeric. Lower prediction interval bound. Default is 0.10.}

\item{piUp}{Numeric. Upper prediction interval bound Default is 0.90.}

\item{plot_type}{Character. Type of plot to produce:
\itemize{
\item \code{"DIST"} (default) - histogram of individual parameters,
\item \code{"QQ"} - QQ-plot of individual parameters.
}}

\item{point_size}{Point size for \code{geom_point}.  Default \code{2.5}.}

\item{pred.corr}{Logical. Apply prediction correction. Default is \code{FALSE}.}

\item{project_name}{String. The name of the Monolix project without file extension.
This will be used as the base name for output files and directories.}

\item{quantiles}{A numeric vector of length 2. Lower and upper quantiles of
the continuous covariate distribution to test.
Default is \code{c(0.1, 0.9)}.}

\item{re}{List. A list specifying options for random effects used in model fit, containing:
\itemize{
\item \code{init} - matrix, initial values for the variance-covariance matrix of random
effects. Rows and columns should correspond to parameters defined in theta.
Diagonal elements represent variances, off-diagonal elements represent covariances. Use 0 for no variability
\item \code{est} - matrix, logical matrix of same dimensions as \code{init} specifying
which variance-covariance elements to estimate. Use \code{TRUE} to estimate,
\code{FALSE} to fix, \code{NA} to not use this random effect.
}}

\item{ref_line_col}{Color for the dashed horizontal line at \code{y = 1}
(no change from reference).  Default \code{"grey25"}.}

\item{rtol}{Numeric. A numeric relative tolerance used by the ODE solver to determine
if a good solution has been achieved. This is also used in the solved linear
model to check if prior doses do not add anything to the solution. Default is 1e-6.}

\item{run_id}{Integer. Tested model ID. Default is 1.}

\item{ruv}{A list specifying options for residual error used in model fit, containing:
\itemize{
\item \code{YNAME} - string, output name. Typically \code{"y1"}, \code{"y2"},...
\item \code{DVID} - numeric, observation type identifier corresponding to DVID column values
\item \code{TRANS} - string, residual error distribution. Can be: \code{"normal"}, \code{"logNormal"}, \code{"logitNormal"}
\item \code{PRED} - string, prediction variable name from the model
\item \code{ERR} - string, error model type. Options include:
\itemize{
\item "constant" for additive error
\item "proportional" for proportional error
\item "combined1" for combined additive and proportional error
}
\item \code{INIT} - numeric vector, initial values for error parameters (length depends on error model)
\item \code{EST} - logical vector, whether to estimate each error parameter (same length as INIT)
\item \code{BLQM} - below limit of quantification method (can be NULL)
For several observation types list of lists should be provided:
one residual error (ruv) object for each observations.
}}

\item{sc_factor}{Numeric. Scaling factor for DV/PRED/IPRED values. Default is 1 (no scaling).}

\item{scale}{A numeric named vector. Scaling for ODE parameters of the system.
The names must correspond to the parameter identifiers in the ODE specification.
Each of the ODE variables will be divided by the scaling factor. Default is \code{NULL}.}

\item{seed}{Integer. Random seed for synthetic data generation reproducibility. Default is \code{123}.}

\item{sigma}{A named matrix representing a sigma covariance matrix or its
Cholesky decomposition; Default is \code{NULL}.}

\item{shp_i}{String. Column name for shape aesthetic. Default is \code{NULL}.}

\item{smooth}{Logical. Add LOESS smooth line. Default is \code{TRUE}.}

\item{stat_comp}{A character vector of summary statistics to compute.
Supported internally: \verb{"mean","median","min","max","sd","cmax","SS"}.}

\item{stimes}{A numeric vector of simulation times.}

\item{task_opt}{String. Additional task options to be passed to the fitting software.
For Monolix, this can include specific task configurations or optimization settings.
When \code{NULL}, default tasks (populationParameters, individualParameters, fim, logLikelihood)
will be used.  Default is \code{NULL}.}

\item{tdist}{Logical. If \code{TRUE}, overlay theoretical parameter distributions
based on population mean and OMEGA matrix. Default is \code{TRUE}.}

\item{time_col}{String. The column to use as a time column. Currently, can be only \code{TIME}. Default is \code{TIME}.}

\item{theor_perc}{Logical. Show theoretical percentiles. Default is \code{TRUE}.}

\item{theor_percCI}{Logical. Show CI around theoretical percentiles. Default is \code{TRUE}.}

\item{theta}{A named numeric vector of baseline parameters. Default is \code{NULL}.
Parameters listed in \code{params} are replaced by sampled values.}

\item{thetamat}{A named variance-covariance matrix (for parameters brought to normal distribution).
Default is \code{NULL}.}

\item{tsld}{Logical. If \code{TRUE}, uses time since last dose instead of time from first dose. Default is \code{FALSE}.}

\item{val_col}{String. Name of value column. Default is \code{VALUE}.}

\item{wrap_i}{String. Faceting formula for \code{facet_wrap}. Default is \code{NULL}.}

\item{wrap_ncol}{Integer. Number of columns for \code{facet_wrap}. Default is \code{NULL}.}

\item{wrap_nrow}{Integer. Number of rows for \code{facet_wrap}. Default is \code{NULL}.}
}
\description{
The function to store common parameters description
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-gof-res.R
\name{sg_gof_res}
\alias{sg_gof_res}
\title{Create residual diagnostic plots}
\usage{
sg_gof_res(
  fpath_i,
  DVID = 1,
  cov_cols = NULL,
  indiv = TRUE,
  vs_time = TRUE,
  weighted = TRUE,
  addline = TRUE,
  alpha_i = 0.5,
  smooth = TRUE,
  log_x = FALSE,
  abreaks = scales::pretty_breaks(7),
  lab_y = NULL,
  lab_x = NULL,
  col_i = NULL,
  col_lab = NULL,
  facet_i = NULL,
  f_scales = "fixed",
  n_bins = 50,
  min_y = NA,
  max_y = NA,
  min_x = NA,
  max_x = NA,
  legend_fl = FALSE,
  n_quantiles = 3,
  levels_discrete = 10
)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{DVID}{Restrict \code{SDTAB} to one observation type. Numeric values
select the \code{DVID} column (default \code{1}). If \code{SDTAB} has
\code{DVNAME}, a character or factor is matched to \code{DVNAME} first;
otherwise a digit-only string is coerced to numeric \code{DVID}.}

\item{cov_cols}{A character vector specifying the names of the columns with covariates.}

\item{indiv}{Logical. If \code{TRUE}, use individual predictions (\code{"IPRED"});
otherwise use population predictions (\code{"PRED"}). Default is \code{TRUE}.}

\item{vs_time}{Logical. If \code{TRUE}, plot residuals vs TIME; otherwise, plot
residuals vs predictions (IPRED/PRED).}

\item{weighted}{Logical. If \code{TRUE}, use weighted residuals; otherwise, use
RES/IRES}

\item{addline}{Logical. If \code{TRUE}, lines connecting observations of
individual subjects will be added. Default is \code{TRUE}.}

\item{alpha_i}{Numeric. Transparency level (from 0 to 1) for points/lines.
Default is 0.5.}

\item{smooth}{Logical. Add LOESS smooth line. Default is \code{TRUE}.}

\item{log_x}{Logical. If \code{TRUE}, a logarithmic scale is applied to x-axis. Default is \code{FALSE}.}

\item{abreaks}{A function that generates axis breaks. Default is \code{scales::pretty_breaks(7)}.}

\item{lab_y}{String. Y-axis label.}

\item{lab_x}{String. X-axis label.}

\item{col_i}{String. Column name for color}

\item{col_lab}{String. Label for color legend}

\item{facet_i}{String. Column name for facet}

\item{f_scales}{String, one of \code{"fixed"}, \code{"free"}, \code{"free_x"}, \code{"free_y"}. User can specify whether the scales (x and y axes) should be fixed across all panels (\code{"fixed"}), free for each panel (\code{"free"}), or free only in one dimension (\code{"free_x"} or \code{"free_y"}). Default is \code{"fixed"}}

\item{n_bins}{Integer. Number of bins to use in the histogram. Default is 30.}

\item{min_y}{Numeric. Y-axis minimum limit. Default is \code{NA}.}

\item{max_y}{Numeric. Y-axis maximum limit. Default is \code{NA}.}

\item{min_x}{Numeric. X-axis minimum limit. Default is \code{NA}.}

\item{max_x}{Numeric. X-axis maximum limit. Default is \code{NA}.}

\item{legend_fl}{Logical. Show legend. Default is \code{FALSE}.}

\item{n_quantiles}{Integer. Number of quantile groups for continuous variables in \code{col_i}. Default is 3.}

\item{levels_discrete}{Integer. Maximum unique values to consider a variable discrete. Default is 10.}
}
\value{
A \code{ggplot2} object
}
\description{
Generates residual diagnostic plots versus time/predictions.
Supports faceting, covariate coloring, quantile binning for
continuous covariates, and optional smoothing.
}
\examples{
# Basic example with mock data
set.seed(123)  # For reproducibility
n_subjects <- 50
mock_obj <- list(
  SDTAB = do.call(rbind, lapply(1:n_subjects, function(id) {
    n_obs <- 6
    times <- sort(runif(n_obs, min = 0, max = 24))  # random times between 0 and 24h
    data.frame(
      ID    = id,
      TIME  = times,
      DV    = rnorm(n_obs, mean = 10, sd = 2),
      PRED  = rnorm(n_obs, mean = 10, sd = 1.5),
      IPRED = rnorm(n_obs, mean = 10, sd = 1.2),
      IWRES = rnorm(n_obs, mean = 0, sd = 0.8),
      IRES  = rnorm(n_obs, mean = 0, sd = 1.2),
      MDV   = 0
    )
  })),
  COTAB = data.frame(
    ID  = 1:n_subjects,
    AGE = sample(20:80, n_subjects, replace = TRUE)
  ),
  CATAB = data.frame(
    ID   = 1:n_subjects,
    RACE = sample(c("Hispanic", "Asian", "Caucasian"), n_subjects, replace = TRUE)
  )
)

# Basic plot: individual weighted residuals vs TIME (weighted = TRUE, vs_time = TRUE)
p <- sg_gof_res(mock_obj, smooth = FALSE)

# With covariates and faceting (use RACE as facet and AGE as color)
p <- sg_gof_res(
  mock_obj,
  smooth = TRUE,
  cov_cols = c("RACE","AGE"),
  col_i = "RACE",
  facet_i = "AGE",
  indiv = TRUE,
  weighted = TRUE,
  vs_time = FALSE,
  legend_fl = TRUE
)
p

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-gof-par-cov.R
\name{sg_gof_par_cov}
\alias{sg_gof_par_cov}
\title{Plot random effects or individual parameters vs covariates}
\usage{
sg_gof_par_cov(fpath_i, ptype = "REvsCov", cat_cov = NULL, cont_cov = NULL)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{ptype}{Character. Type of plot: \code{"REvsCov"} for random effects or \code{"IndParvsCov"} for individual parameters.}

\item{cat_cov}{Optional tibble with categorical covariates. Must have columns \code{COV} and optionally \code{COVNAME} for labels.}

\item{cont_cov}{Optional tibble with continuous covariates. Must have columns \code{COV} and optionally \code{COVNAME} for labels.}
}
\value{
A list of ggplot objects. Returns versus continious covariates
\code{vs_contcov} and versus categorical covariates \code{vs_catcov} plots.
}
\description{
Generates ggplot2 visualizations for either random effects (RE) or individual parameters (IndPar)
versus continuous or categorical covariates from a Simurg object.
}
\examples{
library(tibble)
cont_cov <- tibble(
  COV = c("AGE", "WEIGHT"),
  COVNAME = c("Age, years", "Body weight, kg")
)
cat_cov <- tibble(
  COV = c("SEX", "VKORC1_gentyp"),
  COVNAME = c("Sex, M/F", "VKORC1 genotype")
)
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
p <- sg_gof_par_cov(
  fpath_i = fpath_i,
  ptype = "IndParvsCov",
  cont_cov = cont_cov,
  cat_cov = cat_cov
)
p$vs_contcov
p$vs_catcov

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-vpc-sim.R
\name{sg_vpc_sim}
\alias{sg_vpc_sim}
\title{Perform simulations for VPC plot}
\usage{
sg_vpc_sim(
  fpath_i,
  gco = NULL,
  model = NULL,
  time_col = "TIME",
  outputs = NULL,
  npop = 100
)
}
\arguments{
\item{fpath_i}{Either a character string specifying the file path to a saved \code{sg_fit} object (R data file), or a list object containing the \code{sg_fit} results directly. The object must contain: \code{SDTAB}, \code{EVTAB}, \code{SUMTAB}, \code{OMEGAMAT}, \code{SIGMAMAT}, and optionally \code{COTAB} and \code{CATAB}}

\item{gco}{Generalized control object. Either an R list or path to Rdata/json file. Either \code{gco} or \code{model} file should be specified for simulations model specification. Defauls is \code{NULL}}

\item{model}{A model object passed to \code{sg_sim()}.}

\item{time_col}{String. The column to use as a time column. Currently, can be only \code{TIME}. Default is \code{TIME}.}

\item{outputs}{Vector of strings. Names of the model variables to output.
If \code{NULL}, all variables returned. Default is \code{NULL}.}

\item{npop}{Integer specifying the number of virtual subjects to simulate per original individual. Higher values provide more robust percentile estimates but increase computation time. Default is \code{100}}
}
\value{
A dataset with simulation results
}
\description{
Perform simulations for VPC plot
}
\examples{
\donttest{
library(rxode2)
library(tibble)
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
# Simulate VPC from with generalized model object
mod <- rxode2::rxode2({
  ka_pop = 0.1;
  Vd_pop = 10;
  Cl_pop = 0.5;

  omega_ka = 0;
  omega_Vd = 0;
  omega_Cl = 0;

  Cc_b = 0;
  ka_tv = exp(ka_pop);
  Vd_tv = exp(Vd_pop);
  Cl_tv = exp(Cl_pop);

  ka = ka_tv * exp(omega_ka);
  Vd = Vd_tv * exp(omega_Vd);
  Cl = Cl_tv * exp(omega_Cl);

  Cc = Ac / Vd;

  Ad(0) = 0;
  Ac(0) = 0;

  d/dt(Ad) = -ka * Ad;
  d/dt(Ac) = ka * Ad - Cl * Cc;

  Cc_ResErr = Cc * (1 + Cc_b);
})

sg_vpc_sim(fpath_i, model=mod, outputs = "Cc_ResErr")
# Make VPC plot with the generalized control object
model <- system.file("extdata", "models", "rxode", "model_PK_1c.txt", package = "SimuRg")
data  <- system.file("extdata", "datasets", "dspk-warf.csv", package = "SimuRg")
headers <- list(list(name = "ID", use = "identifier", type = NULL),
             list(name = "TIME", use = "time", type = NULL),
                          list(name = "DV", use = "observation", type = "continuous"),
                          list(name = "DVID", use = "observationtype", type = NULL),
                          list(name = "ADM", use = "administration", type = NULL),
                          list(name = "AMT", use = "amount", type = NULL),
                          list(name = "EVID", use = "eventidentifier", type = NULL),
                          list(name = "MDV", use = "missingdependentvariable", type = NULL),
                          list(name = "AGE", use = "covariate", type = "continuous"),
                          list(name = "AGE_centered", use = "covariate", type = "continuous"),
                          list(name = "SEX", use = "covariate", type = "categorical"),
                          list(name = "WEIGHT", use = "covariate", type = "continuous"),
                          list(name = "BMI", use = "covariate", type = "continuous"))

theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
                  "Cl", "logNormal", 0.2, NA, NA, TRUE,
                "V", "logNormal", 20, NA, NA, TRUE,
                "ka", "logNormal", 0.2, NA, NA, TRUE
)

ruv <- list(YNAME = "y1", DVID = 1, TRANS = "normal", PRED = "Cc",
           ERR = "combined1", INIT = c(1, 1), EST = c(TRUE, TRUE), BLQM = NULL)

re <- list(init = tribble(~Cl, ~V, ~ka,
                            1, 0, 0,
                            0, 1, 0,
                            0, 0, 1) \%>\% as.matrix(),
            est = tribble(~Cl, ~V, ~ka,
                         TRUE, NA, NA,
                           NA, TRUE, NA,
                          NA, NA, TRUE) \%>\% as.matrix())
occ <- list(init = tribble(~Cl, ~V, ~ka,
                        0, 0, 0,
                          0, 0, 0,
                          0, 0, 0) \%>\% as.matrix(),
           est = tribble(~Cl, ~V, ~ka,
                         NA, NA, NA,
                         NA, NA, NA,
                         NA, NA, NA) \%>\% as.matrix())
covs <- list(list(PAR = "V", COVNAME = "AGE", FUNC = "linear",
                 TRANS = "median", INIT = 1, EST = TRUE),
            list(PAR = "ka", COVNAME = "SEX", REF = 0, INIT = 1, EST = TRUE))

gco <-list(headers = headers, data = data, model = model, task_opt = "",
          covs = covs, project_name = "test-proj", theta = theta,
          ruv = ruv, re = re, occ = occ, modelText = "")

res <- sg_vpc_sim(fpath_i, gco = gco, output = "Cc")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-vpc-vis.R
\name{sg_vpc_vis}
\alias{sg_vpc_vis}
\title{Visual Predictive Check (VPC) Function}
\usage{
sg_vpc_vis(
  ds_sim,
  data_i,
  output_names,
  time_col = "TIME",
  dv_col = "DV",
  log_y = FALSE,
  piLow = 0.1,
  piUp = 0.9,
  ciLow = 0.025,
  ciUp = 0.975,
  pred.corr = FALSE,
  lab_y = "DV",
  lab_x = "Time, h",
  theor_perc = TRUE,
  theor_percCI = TRUE,
  emp_perc = TRUE,
  dt_obs_fl = FALSE,
  legend_fl = FALSE,
  n_bins = 10,
  method = "kmeans",
  interpolation = TRUE,
  strat_by_dose = NULL
)
}
\arguments{
\item{ds_sim}{A data frame with simulated data containing columns:
\itemize{
\item ID: simulation replicate identifier
\item TIME: time points
\item VAR: output variable name
\item VALUE: simulated values
}}

\item{data_i}{A data frame with observed data containing columns:
\itemize{
\item ID: patient identifier
\item TIME: time points
\item DV: observed dependent variable values
\item DVID: dependent variable identifier
}}

\item{output_names}{A data frame mapping \code{VAR} values (from \code{ds_sim})
to \code{DVID} values (from \code{data_i}). Must contain columns:
\itemize{
\item output: character, \code{VAR} values (e.g., "Cc", "Cp")
\item dvid: numeric, corresponding DVIDs (e.g., 1, 2)
}}

\item{time_col}{String. The column to use as a time column. Currently, can be only \code{TIME}. Default is \code{TIME}.}

\item{dv_col}{Character. Name of DV column in data_i. Default is \code{DV}}

\item{log_y}{Logical. If \code{TRUE}, a logarithmic scale is applied to y-axis. Default is \code{FALSE}.}

\item{piLow}{Numeric. Lower prediction interval bound. Default is 0.10.}

\item{piUp}{Numeric. Upper prediction interval bound Default is 0.90.}

\item{ciLow}{Numeric. Lower confidence interval bound. Default is 0.025}

\item{ciUp}{Numeric. Upper confidence interval bound. Default is 0.975}

\item{pred.corr}{Logical. Apply prediction correction. Default is \code{FALSE}.}

\item{lab_y}{String. Y-axis label. Default is "DV".}

\item{lab_x}{String. X-axis label. Default is "TIME, h".}

\item{theor_perc}{Logical. Show theoretical percentiles. Default is \code{TRUE}.}

\item{theor_percCI}{Logical. Show CI around theoretical percentiles. Default is \code{TRUE}.}

\item{emp_perc}{Logical. Show empirical percentiles. Default is \code{TRUE}}

\item{dt_obs_fl}{Logical. Show observed data points. Default is \code{FALSE}}

\item{legend_fl}{Logical. Show legend. Default is \code{FALSE}.}

\item{n_bins}{Integer. Number of bins to use in the histogram. Default is 10.}

\item{method}{Character. Binning method: "kmeans", "equal", "quantile" (default: "kmeans").}

\item{interpolation}{Logical. Use line interpolation vs rectangles (default: FALSE).}

\item{strat_by_dose}{Character. Variable name for dose stratification (default: NULL).}
}
\value{
List of ggplot objects, one for each output variable
}
\description{
Generates VPC plots to assess model performance by comparing observed data
with prediction intervals derived from simulated data.
}
\details{
For now, the model in the example is NOT the generalized model object (GMO),
as the parameters in generalized fit object are backtransformed, and,
therefore, not transformed in the model. This issue will be fixed in the next
versions of SimuRg package
}
\examples{
\donttest{
library(rxode2)
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
mod <- rxode2::rxode2({
  ka_pop = 0.1;
  Vd_pop = 10;
  Cl_pop = 0.5;

  omega_ka = 0;
  omega_Vd = 0;
  omega_Cl = 0;

  Cc_b = 0;
  ka_tv = exp(log(ka_pop));
  Vd_tv = exp(log(Vd_pop));
  Cl_tv = exp(log(Cl_pop));

  ka = ka_tv * exp(omega_ka);
  Vd = Vd_tv * exp(omega_Vd);
  Cl = Cl_tv * exp(omega_Cl);

  Cc = Ac / Vd;

  Ad(0) = 0;
  Ac(0) = 0;

  d/dt(Ad) = -ka * Ad;
  d/dt(Ac) = ka * Ad - Cl * Cc;

  Cc_ResErr = Cc * (1 + Cc_b);
})

sim_data <- sg_vpc_sim(fpath_i, model=mod, outputs = "Cc_ResErr")
outp_nms <- data.frame(dvid = 1, output = "Cc")
vpc_plots <- sg_vpc_vis(
  ds_sim = sim_data,
  data_i = warfarin,
  output_names = outp_nms,
  lab_x = "Time (hours)",
  lab_y = "Concentration (ng/mL)",
  n_bins = 8,
  pred.corr = TRUE
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-vpop-est.R
\name{sg_vpop_est}
\alias{sg_vpop_est}
\title{Perform generation of synthetic datasets for an empirical distribution}
\usage{
sg_vpop_est(
  data_i,
  nobj = NA,
  id_col = NULL,
  minnumlev = 3,
  npop = 1,
  excl_col = NULL,
  seed = NA,
  seed_umap = 123,
  palette = NULL,
  diag_plots = FALSE,
  remove_duplicates = TRUE,
  noise_level = 0.1,
  nds = 1,
  tg_corrdif = 0.1
)
}
\arguments{
\item{data_i}{data frame. Input data frame containing the original dataset to be synthesized (required)}

\item{nobj}{integer. Specify the exact number of rows to generate in the synthetic dataset. When provided, overrides \code{npop} (optional, default: \code{NA})}

\item{id_col}{Character string. Specify the name of the identifier column
to exclude from synthesis. Default: \code{NULL}.}

\item{minnumlev}{integer. Threshold; numeric variables with <= \code{minnumlev} unique values are converted to factors (optional, default: \code{3})}

\item{npop}{Integer. Number of population replicates. Default is 1.}

\item{excl_col}{Character vector. Contains column names to exclude from synthesis. Default: \code{NULL}}

\item{seed}{integer. Random seed for synthetic data generation. If provided (not \code{NA}), generates a single dataset with this seed (fixed seed mode). If \code{NA}, uses search mode to find \code{nds} datasets meeting correlation threshold (optional, default: \code{NA})}

\item{seed_umap}{integer. Random seed for UMAP algorithm reproducibility (optional, default: \code{123})}

\item{palette}{character vector. Contains color codes (hex format) for custom plot color schemes. If provided, should contain at least 2 colors. Used for histograms, bar plots, and UMAP visualizations (optional, default: \code{c("#3a6eba", "#efdd3c", "#1a1866", "#f2b93b")})}

\item{diag_plots}{logical flag. If \code{TRUE}, generates diagnostic plots and UMAP visualizations (optional, default: \code{TRUE})}

\item{remove_duplicates}{logical flag. If \code{TRUE}, automatically removes exact duplicates between original and synthetic data by adding controlled noise (optional, default: \code{TRUE})}

\item{noise_level}{numeric. Proportion of standard deviation to use when adding noise to continuous variables for duplicate removal. E.g., 0.10 means 10\% of SD (optional, default: \code{0.10})}

\item{nds}{integer. Number of synthetic datasets to generate in search mode. Ignored in fixed seed mode (optional, default: \code{1})}

\item{tg_corrdif}{numeric. Target maximum absolute correlation difference threshold for dataset selection in search mode. Ignored in fixed seed mode (optional, default: \code{0.1})}
}
\value{
A list of lists, where each element contains results for one generated dataset:
\itemize{
\item \code{datagen} - Synthetic dataset returned by \code{synthpop::syn()} (data.frame)
\item \code{seed} - Random seed used to generate this dataset (integer or NA)
\item \code{exact_dupl_check} - Logical indicating if exact duplicates exist between original and synthetic data
\item \code{dplot_umap} - ggplot object with combined UMAP visualization comparing original and synthetic data (or \code{NULL} if diag_plots=FALSE)
\item \code{ks_test} - Tibble with Kolmogorov-Smirnov p-values and statuses for continuous variables (or \code{NULL} if no continuous variables)
\item \code{jsd_res} - Weighted mean Jensen-Shannon divergence (JSD) value for categorical variables (numeric or \code{NULL} if no categorical variables)
\item \code{corr_diff_mean} - Mean absolute difference between original and synthetic correlation matrices (numeric or \code{NULL} if no continuous variables)
\item \code{corr_diff_max} - Maximum absolute difference between original and synthetic correlation matrices (numeric or \code{NULL} if no continuous variables)
\item \code{dplot_corr_diff} - ggplot heatmap object showing correlation difference matrix (Synthetic - Original) (or \code{NULL} if diag_plots=FALSE or no continuous variables)
\item \code{dplot_cont} - List of ggplot histograms for continuous variables (or \code{NULL} if diag_plots=FALSE or no continuous variables)
\item \code{dplot_cat} - List of ggplot barplots for categorical variables (or \code{NULL} if diag_plots=FALSE or no categorical variables)
}
}
\description{
The function operates in two modes:
\itemize{
\item \strong{Fixed seed mode} (when \code{seed} is specified): Generates a single dataset using the provided seed
\item \strong{Search mode} (when \code{seed = NA}): Iteratively searches for \code{nds} datasets that meet the correlation difference threshold
}
}
\examples{
\donttest{
library(dplyr)

# Generate example dataset
data <- data.frame(
  id = 1:150,
  ALB = rnorm(150, mean = 3.5, sd = 0.5),
  ALT = rnorm(150, mean = 50, sd = 20),
  SEX = factor(sample(c("M", "F"), 150, replace = TRUE)),
  RACE = factor(sample(c("White", "Black", "Asian", "Other"), 150, replace = TRUE))
)

# EXAMPLE 1: Fixed seed mode - generate single dataset with specified seed
output_fixed <- sg_vpop_est(data_i = data,
                            id_col = "id",
                            seed = 123,        # Fixed seed mode
                            seed_umap = 40,
                            diag_plots = TRUE)

# Access the dataset and its diagnostics
print(head(output_fixed[[1]]$datagen, 10))
print(output_fixed[[1]]$ks_test)
print(output_fixed[[1]]$seed)           # Will be 123
print(output_fixed[[1]]$corr_diff_max)
print(output_fixed[[1]]$dplot_umap)

# EXAMPLE 2: Search mode - generate 3 datasets meeting correlation threshold
output_search <- sg_vpop_est(data_i = data,
                             id_col = "id",
                             seed = NA,        # Search mode (default)
                             seed_umap = 40,
                             diag_plots = TRUE,
                             nds = 3,          # Generate 3 datasets
                             tg_corrdif = 0.1) # Target correlation difference

# Compare metrics across all datasets
sapply(output_search, function(x) x$corr_diff_max)
sapply(output_search, function(x) x$jsd_res)
sapply(output_search, function(x) x$seed)  # Different seeds for each dataset

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_pbc}
\alias{data_pbc}
\title{Primary Biliary Cirrhosis (PBC) dataset}
\format{
\subsection{\code{data_pbc}}{

A data frame with 280 rows and 15 columns:
\describe{
\item{id}{identifier of the individual}
\item{years}{time in years (survival time)}
\item{status2}{survival status (0 = censored, 1 = event)}
\item{drug}{treatment group ("D-penicil" or "placebo")}
\item{age}{age of the individual in years}
\item{sex}{sex of the individual ("male" or "female")}
\item{ascites}{presence of ascites ("Yes" or "No")}
\item{hepatomegaly}{presence of hepatomegaly ("Yes" or "No")}
\item{spiders}{presence of spider angiomas ("Yes" or "No")}
\item{edema}{edema status ("No edema", "edema no diuretics", or "edema despite diuretics")}
\item{serBilir}{serum bilirubin level (mg/dl)}
\item{serChol}{serum cholesterol level (mg/dl)}
\item{albumin}{albumin level (g/dl)}
\item{platelets}{platelet count (per cubic mm/1000)}
}
}
}
\usage{
data_pbc
}
\description{
Clinical trial dataset with Primary Biliary Cirrhosis patient data including survival outcomes and covariates.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-covsens-sim.R
\name{sg_covsens_sim}
\alias{sg_covsens_sim}
\title{Covariate sensitivity analysis via simulation}
\usage{
sg_covsens_sim(
  fpath_i = NULL,
  ds_parest = NULL,
  ds_covs = NULL,
  model,
  stimes,
  et,
  est_covmat,
  npop = 5,
  cont_cov_l,
  cat_cov_l,
  quantiles = c(0.1, 0.9),
  outputs = "Cc",
  aggr = c("min", "max", "mean")
)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{ds_parest}{Data.frame. Parameter estimates table with columns
\code{parameter} and \code{value}.
Required when \code{fpath_i} is \code{NULL}; must be provided together
with \code{ds_covs}.  Default is \code{NULL}.}

\item{ds_covs}{data.frame. Subject-level covariate dataset (one row per
subject) containing both continuous and categorical covariate columns.
Required when \code{fpath_i} is \code{NULL}; must be provided together
with \code{ds_parest}.  Default is \code{NULL}.}

\item{model}{A model object passed to \code{sg_sim()}.}

\item{stimes}{numeric vector. Sampling time points for steady-state
simulation (e.g. generated by a cycling function over dosing intervals).}

\item{et}{data.frame. Event (dosing) table.  Must contain at least
columns \code{id}, \code{time}, \code{amt}, \code{evid} and
\code{Regimen}.  Additional columns such as \code{ii}, \code{addl},
\code{dur} and \code{Dose} are used when present.}

\item{est_covmat}{Data.frame. Parameter estimation covariance matrix.  The
first column (\code{X1}) must list parameter names; remaining columns
(named identically) form the symmetric variance–covariance matrix.}

\item{npop}{integer. Number of population-level simulation replicates drawn
from the parameter uncertainty distribution.  Default is \code{5}.}

\item{cont_cov_l}{A named list. Each element defines one continuous covariate
and must itself be a list with components:
\describe{
\item{\code{NAME}}{Character. Column name of the (transformed)
covariate in the dataset (e.g. \code{"LG_AGE"}).}
\item{\code{UTNAME}}{Character or \code{NULL}. Column name of the
untransformed (back-transformed) covariate
(e.g. \code{"AGE"}).  If \code{NULL} or \code{NA}, defaults to
\code{NAME}.}
\item{\code{REF}}{Character or numeric. Reference value for the
covariate.  Use \code{"median"} to derive from data, or a numeric
value.}
\item{\code{NICENAME}}{Character or \code{NULL}. Display label for
plots and tables (e.g. \code{"Age, years"}).}
\item{\code{par_vec}}{Character vector. Model parameter(s) affected by
this covariate (e.g. \code{c("CL")}).}
}}

\item{cat_cov_l}{A named list. Each element defines one categorical covariate
and must itself be a list with components:
\describe{
\item{\code{NAME}}{Character. Column name of the covariate
(e.g. \code{"SEX"}).}
\item{\code{NICENAME}}{Character or \code{NULL}. Display label.}
\item{\code{REF}}{Character or \code{NULL}. Reference category value.
If \code{NULL}, the first factor level (alphabetically) is used.}
\item{\code{par_vec}}{Character vector. Model parameter(s) affected by
this covariate (e.g. \code{c("ka")}).}
}}

\item{quantiles}{A numeric vector of length 2. Lower and upper quantiles of
the continuous covariate distribution to test.
Default is \code{c(0.1, 0.9)}.}

\item{outputs}{character vector. Name(s) of the model output variable(s)
to evaluate (e.g. \code{"Cc"} or \code{c("Cc", "Effect")}).
Default is \code{"Cc"}.}

\item{aggr}{character vector. Exposure aggregation metric(s) applied over
the simulation time grid.
Allowed values: \code{"min"} (Cmin), \code{"max"} (Cmax),
\code{"mean"} (Cavg).  Default is \code{c("min", "max", "mean")}.}
}
\value{
A named list of three elements:
\describe{
\item{\code{$PARSENS}}{data.frame.
Full sensitivity results for model parameters: percent change
statistics (mean, median, percentiles) for each covariate–parameter
combination, with columns \code{NICEN}, \code{VAR}, \code{KEY},
\code{LAB}, \code{Type} and summary statistics
(\code{mean} through \code{P975}).}
\item{\code{$SUMPARSENS}}{data.frame.
Compact summary table with columns \code{Parameter},
\code{Covariate}, \code{Cov. percentile}, \code{Cov. value},
\code{Mean} and \code{90\%CI}.}
\item{\code{$EXPSENS}}{data.frame.
Sensitivity of exposure metrics (Cmin, Cmax, Cavg) to covariates,
structured identically to \code{$PARSENS}.}
}
}
\description{
Evaluate the impact of continuous and categorical covariates on model
parameters and exposure metrics.  For each covariate the function perturbs
its value to selected quantiles (continuous) or observed categories
(categorical) while holding the remaining covariates at their reference
values, simulates from the pharmacometric model under parameter uncertainty,
and summarises the resulting percent change relative to the reference.
}
\details{
Two data-source modes are supported (mutually exclusive):
\itemize{
\item \strong{File mode} — supply \code{fpath_i} (path to a SimuRg JSON /
RData object that contains SUMTAB, CATAB and COTAB).
\item \strong{Table mode} — supply both \code{ds_parest} (parameter
estimates) and \code{ds_covs} (covariate dataset).
}

The analysis proceeds in two stages for each covariate type (continuous and
categorical):
\enumerate{
\item \strong{Parameter sensitivity} — the covariate is set to its
quantile or category value and the model is evaluated at time zero (no
ODE integration) to quantify the direct effect on each parameter.
\item \strong{Exposure sensitivity} — the full time-course is simulated
over \code{stimes} and exposure metrics (\code{aggr}) are computed.
}

Uncertainty is propagated by sampling \code{npop} parameter vectors from a
multivariate normal distribution parameterised by the population estimates
and the estimation covariance matrix (\code{est_covmat}).
}
\examples{
\donttest{
library(dplyr)
library(rxode2)

# --- Covariate definitions ---
cont_cov_l <- list(
  LG_AGE = list(NAME = "LG_AGE", UTNAME = "AGE",
                REF = "median", NICENAME = "Age, years",
                par_vec = c("CL")),
  LG_WEIGHT = list(NAME = "LG_WEIGHT", UTNAME = "WEIGHT",
                   REF = "median", NICENAME = "Weight, kg",
                   par_vec = c("Vd"))
)

cat_cov_l <- list(
  SEX = list(NAME = "SEX", NICENAME = "Sex",
             REF = "0", par_vec = c("ka")),
  CYP2C9 = list(NAME = "CYP2C9", NICENAME = "CYP2C9 genotype",
                REF = NULL, par_vec = c("CL"))
)

# --- Dosing ---
ev_t_input <- tribble(
  ~id, ~time, ~ii, ~amt, ~addl, ~dur, ~evid, ~Regimen,        ~Dose,
  1,   0,     336, 10,   21,    0.5,  1,     "0.3 mg/kg Q2W", 0.3
)
# --- Model ---
model <- RxODE({
  # Doses in mg
  # Time in hours

  ### Parameter values
  # Typical values
  ka_pop = 0.073;
  Vd_pop = 14.8;
  CL_pop = 0.347;

  # Random effects
  omega_ka = 0;
  omega_Vd = 0;
  omega_CL = 0;

  # Covariate effect
  # Continuous
  beta_CL_LG_AGE = 0.49990114;
  beta_Vd_LG_WEIGHT = 0.60529433;

  # Categorical
  beta_CL_CYP2C9_1_2 = -0.339;
  beta_CL_CYP2C9_1_3 = -0.574;
  beta_CL_CYP2C9_2_2 = -1.079;
  beta_CL_CYP2C9_2_3 = -0.745;
  beta_CL_CYP2C9_3_3 = -2.13;

  beta_ka_SEX_1 = -0.12198035;

  # Residual error
  Cc_b = 0;

  # Transformations
  ka_tv = exp(ka_pop);
  Vd_tv = exp(Vd_pop);
  CL_tv = exp(CL_pop);

  CL_multiplier = 1.0;  # Default/reference
  ka_multiplier = 1.0;

  if (SEX == "1") {ka_multiplier = exp(beta_ka_SEX_1)}

  if (CYP2C9 == "1") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_2);
  } else if (CYP2C9 == "2") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_3);
  } else if (CYP2C9 == "3") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_2);
  } else if (CYP2C9 == "4") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_3);
  } else if (CYP2C9 == "5") {
    CL_multiplier = exp(beta_CL_CYP2C9_3_3);
  }

  ka = ka_tv*ka_multiplier*exp(omega_ka);
  Vd = Vd_tv*exp(beta_Vd_LG_WEIGHT * LG_WEIGHT + omega_Vd); #Vd_tv*exp(omega_Vd);
  CL = CL_tv*CL_multiplier*exp(beta_CL_LG_AGE * LG_AGE + omega_CL);


  ### Explicit functions
  Cc = Ac/Vd;

  ### Initial conditions
  Ad(0) = 0;
  Ac(0) = 0;

  ### Differential equations
  d/dt(Ad) = - ka*Ad;
  d/dt(Ac) = ka*Ad - CL*Cc;

  Cc_ResErr = Cc*(1 + Cc_b);
})

# --- Estimation covariance (mock) ---
pnames     <- parest$parameter
npar       <- length(pnames)
set.seed(1)
m_cov      <- matrix(0.02, npar, npar)
diag(m_cov) <- 0.05 + runif(npar, 0, 0.05)
m_cov      <- (m_cov + t(m_cov)) / 2
est_covmat <- as_tibble(cbind(X1 = pnames, as.data.frame(m_cov)))
names(est_covmat)[-1] <- pnames

# --- Simulation times (steady-state cycle 10) ---
ss_cycle <- 10
stimes_ss <- c(
  ss_cycle * 4 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1)),
  ss_cycle * 4 * 7 * 24 + 2 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1))
)

# --- Run ---
result <- sg_covsens_sim(
  fpath_i=NULL, ds_parest = parest, ds_covs = ds_covval,
  model = model, stimes = stimes_ss, et = ev_t_input,
  est_covmat = est_covmat, npop = 10,
  cont_cov_l = cont_cov_l, cat_cov_l = cat_cov_l,
  quantiles = c(0.2, 0.8), aggr = c("max"),
  outputs = "Cc"
)
print(result[["PARSENS"]])
print(result[["SUMPARSENS"]])
print(result[["EXPSENS"]])

}

}
\seealso{
\code{\link{sg_sim}}, \code{\link{read_smrg_obj}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-globalsens-vis.R
\name{sg_globalsens_vis}
\alias{sg_globalsens_vis}
\title{Visualize global sensitivity analysis results}
\usage{
sg_globalsens_vis(
  x,
  type = c("heatmap", "bar"),
  params = NULL,
  vars = NULL,
  stats = NULL
)
}
\arguments{
\item{x}{Result object returned by \code{sg_globalsens_sim()}}

\item{type}{Plot type for PRCC: \code{"heatmap"} or \code{"bar"}}

\item{params}{Optional vector of parameters to display}

\item{vars}{Optional vector of output variables}

\item{stats}{Optional vector of statistics to display}
}
\value{
ggplot object
}
\description{
Generates visualization plots for results produced by
\code{sg_globalsens_sim()}. Supported visualizations:
\itemize{
\item PRCC: heatmap or barplot
\item eFAST: barplot of first and total order indices
}
}
\examples{
\donttest{
# Example model
library(rxode2)
library(tibble)
source(system.file("extdata", "RxODE_model", "example_rxode_model.R", package = "SimuRg")) # mod_ex


# Set up event table
et_base <- tribble(
  ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
  1,   0,     1,     1,    10,   2,     24,  112,   1
)

# Define parameter bounds
inits <- rxInits(mod_ex)
par_bounds <- tibble::tibble(
  PAR = c("POPCL", "POPVC"),
  LB  = inits[c("POPCL", "POPVC")] * (1 - 0.9),
  UB  = inits[c("POPCL", "POPVC")] * (1 + 0.9)
)

# Run eFAST sensitivity analysis
res_efast <- sg_globalsens_sim(
  method = c("eFAST"),
  model = mod_ex,
  params = c("POPCL"),
  par_bounds = par_bounds,
  n_sim = 100,
  stimes = seq(0, 168, 10),
  output = "Cc",
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean")
)

  efast_p <- sg_globalsens_vis(
    res_efast,
    params = c("POPCL"),
    vars = "Cc",
    stats = c("mean")
  )

  efast_p
# Run PRCC sensitivity analysis
res_prcc <- sg_globalsens_sim(
  method = c("PRCC"),
  model = mod_ex,
  params = c("POPCL", "POPVC"),
  par_bounds = par_bounds,
  n_sim = 100,
  stimes = seq(0, 168, 10),
  output = "Cc",
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean")
)

  prcc_p <- sg_globalsens_vis(
    res_prcc,
    type = c("heatmap"),
    params = c("POPCL", "POPVC"),
    vars = "Cc",
    stats = c("mean")
  )

  prcc_p
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-gof-obpr.R
\name{sg_gof_obpr}
\alias{sg_gof_obpr}
\title{Observed vs predicted plot}
\usage{
sg_gof_obpr(
  fpath_i,
  DVID = 1,
  cov_cols = NULL,
  indiv = TRUE,
  addline = TRUE,
  alpha_i = 0.5,
  smooth = TRUE,
  log_axes = FALSE,
  sc_factor = 1,
  abreaks = scales::pretty_breaks(7),
  lab_x = "Model-predicted values",
  lab_y = "Observed values",
  col_i = NULL,
  col_lab = NULL,
  facet_i = NULL,
  f_scales = "fixed",
  no_leg = FALSE,
  n_quantiles = 3,
  levels_discrete = 10
)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{DVID}{Restrict \code{SDTAB} to one observation type. Numeric values
select the \code{DVID} column (default \code{1}). If \code{SDTAB} has
\code{DVNAME}, a character or factor is matched to \code{DVNAME} first;
otherwise a digit-only string is coerced to numeric \code{DVID}.}

\item{cov_cols}{A character vector specifying the names of the columns with covariates.}

\item{indiv}{Logical. If \code{TRUE}, use individual predictions (\code{"IPRED"});
otherwise use population predictions (\code{"PRED"}). Default is \code{TRUE}.}

\item{addline}{Logical. If \code{TRUE}, lines connecting observations of
individual subjects will be added. Default is \code{TRUE}.}

\item{alpha_i}{Numeric. Transparency level (from 0 to 1) for points/lines.
Default is 0.5.}

\item{smooth}{Logical. Add LOESS smooth line. Default is \code{TRUE}.}

\item{log_axes}{Logical. If \code{TRUE}, a logarithmic scale is applied to all axes. Default is \code{FALSE}.}

\item{sc_factor}{Numeric. Scaling factor for DV/PRED/IPRED values. Default is 1 (no scaling).}

\item{abreaks}{A function that generates axis breaks. Default is \code{scales::pretty_breaks(7)}.}

\item{lab_x}{X-axis label. Default "Model-predicted values"}

\item{lab_y}{Y-axis label. Default "Observed values"}

\item{col_i}{String. Column name for color}

\item{col_lab}{String. Label for color legend}

\item{facet_i}{String. Column name for facet}

\item{f_scales}{String, one of \code{"fixed"}, \code{"free"}, \code{"free_x"}, \code{"free_y"}. User can specify whether the scales (x and y axes) should be fixed across all panels (\code{"fixed"}), free for each panel (\code{"free"}), or free only in one dimension (\code{"free_x"} or \code{"free_y"}). Default is \code{"fixed"}}

\item{no_leg}{Logical. If \code{TRUE}, the legend is not shown. Default is \code{FALSE}}

\item{n_quantiles}{Integer. Number of quantile groups for continuous variables in \code{col_i}. Default is 3.}

\item{levels_discrete}{Integer. Maximum unique values to consider a variable discrete. Default is 10.}
}
\value{
A ggplot2 object
}
\description{
Function generates observed versus predicted (OBS vs PRED/IPRED) scatter plots,
a fundamental goodness-of-fit diagnostic tool in pharmacometric modeling.
This visualization assesses model adequacy by comparing observed clinical measurements against model-predicted values,
enabling identification of systematic bias, heteroscedasticity, and model misspecification patterns.
Function contains options for faceting, coloring by covariates, and trend lines.
}
\examples{
\donttest{
# Basic example with mock data
set.seed(123)  # For reproducibility
mock_obj <- list(
 SDTAB = data.frame(
   ID = rep(1:3, each = 5),
   TIME = rep(c(0, 1, 2, 4, 8), 3),
   DV = rnorm(15, mean = 10, sd = 2),
   PRED = rnorm(15, mean = 10, sd = 1.5),
   IPRED = rnorm(15, mean = 10, sd = 1.2),
   MDV = rep(0, 15)
 ),
 COTAB = data.frame(ID = 1:3, AGE = c(30, 45, 60)),
 CATAB = data.frame(ID = 1:3, RACE = c("Hispanic", "Hispanic", "Asian"))
)

# Basic plot
p <- sg_gof_obpr(mock_obj)

# With covariates and faceting
p <- sg_gof_obpr(
  mock_obj,
  cov_cols = "RACE",
  col_i = "RACE",
  facet_i = "RACE"
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-predist-sim.R
\name{sg_predist_sim}
\alias{sg_predist_sim}
\title{Perform simulations for prediction distribution plots}
\usage{
sg_predist_sim(fpath_i, model, time_col = "TIME", outputs = NULL, npop = 500)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{model}{A model object passed to \code{sg_sim()}.}

\item{time_col}{String. The column to use as a time column. Currently, can be only \code{TIME}. Default is \code{TIME}.}

\item{outputs}{Vector of strings. Names of the model variables to output.
If \code{NULL}, all variables returned. Default is \code{NULL}.}

\item{npop}{Integer. Number of population replicates. Default is 1.}
}
\value{
A dataset with simulation results
}
\description{
Perform simulations for prediction distribution plots
}
\examples{
\donttest{
library(rxode2)
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
mod_fin <- rxode2({
  # Differential equations
  d/dt(Ad) = -ka * Ad
  d/dt(Ac) = ka * Ad - Cl/V * Ac
  # Concentration calculations
  Cc = Ac / V
})
sg_predist_sim(fpath_i, mod_fin, outputs = "Cc")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{parest}
\alias{parest}
\title{Warfarin population PK parameter estimates}
\format{
\subsection{\code{parest}}{

A data frame with 15 rows and 12 columns:
\describe{
\item{parameter}{parameter name as used in the model}
\item{value}{final parameter estimate}
\item{TYPE}{parameter category:\code{"Typical values"}, \code{"Covariate effects"}, \code{"Random effects"}, or \code{"Residual error model"}}
\item{EST}{estimation status (\code{"ESTIMATED"})}
\item{SE}{standard error of the estimate}
\item{RSE}{relative standard error (\%)}
\item{LCI}{lower bound of the 95 \% confidence interval}
\item{UCI}{upper bound of the 95 \% confidence interval}
\item{ETAshrinkage_var}{ETA shrinkage on the variance scale (\%); \code{NA} for non-random-effect parameters}
\item{ETAshrinkage_sd}{ETA shrinkage on the SD scale (\%); \code{NA} for non-random-effect parameters}
\item{EPSshrinkage_sd}{EPS shrinkage on the SD scale (\%); \code{NA} for non-residual-error parameters}
}
}
}
\source{
Derived from \code{par_fin_i.csv} — warfarin PopPK model estimation output.
}
\usage{
parest
}
\description{
Final parameter estimates from a warfarin population pharmacokinetic model,
including typical values, covariate effects, random effects, and residual
error parameters with associated uncertainty measures.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-parsum.R
\name{sg_parsum}
\alias{sg_parsum}
\title{Extract parameter summary (theta, eta, SE, RSE, CI, CV, shrinkage)}
\usage{
sg_parsum(fpath_i, addOFV = TRUE)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{addOFV}{Logical. If \code{TRUE}, information about OFV and AIC of the model
will be added. Default is \code{TRUE}.}
}
\value{
A table with parameter summary
}
\description{
Extract parameter summary (theta, eta, SE, RSE, CI, CV, shrinkage)
}
\examples{
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
sum_tab <- sg_parsum(fpath_i)
print(sum_tab)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-gof-par-dist.R
\name{sg_gof_par_dist}
\alias{sg_gof_par_dist}
\title{Plot distributions of individual parameters with optional theoretical density}
\usage{
sg_gof_par_dist(
  fpath_i,
  par_seq = NULL,
  par_type = "Ind",
  n_bins = 30,
  tdist = TRUE,
  plot_type = "DIST"
)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{par_seq}{Vector of strings. Character vector of parameter names to be plotted. If \code{NULL}, all parameters be included. Default is \code{NULL}}

\item{par_type}{String. A character string specifying the type of parameters used for
theoretical distribution overlay (only relevant when \code{plot_type = 'DIST'}
and \code{tdist = TRUE}). If 'Ind' - Individual (default), distributions are shown on the
natural parameter scale assuming log-normal variability. If 'RE' - Random Effect, distributions are shown on the ETA scale,
assuming a normal distribution with mean zero and covariance defined
by \verb{$OMEGAMAT}, without transformation.}

\item{n_bins}{Integer. Number of bins to use in the histogram. Default is 30.}

\item{tdist}{Logical. If \code{TRUE}, overlay theoretical parameter distributions
based on population mean and OMEGA matrix. Default is \code{TRUE}.}

\item{plot_type}{Character string specifying the type of plot to generate.
Options: 'DIST' for parameter distributions (default), 'QQ' for Q-Q plots,
'correlations' for correlation matrix plot.}
}
\value{
A \code{ggplot} object containing histogram(s) of individual parameter distributions, optionally overlaid with theoretical densities.
}
\description{
This function creates histograms of individual parameter estimates,
with optional overlay of the theoretical distribution based on the population parameters and OMEGA matrix.
}
\details{
The function visualizes the distribution of post hoc individual parameter estimates (from \verb{$PATAB}) and optionally compares them with the expected population-level variability using \verb{$SUMTAB$DISTRIBUTION} (see Details).
Shrinkage values are calculated based on \code{ETAshrinkage_var} from \verb{$SUMTAB} and included in facet labels.

Theoretical distributions are generated by sampling from a multivariate normal distribution (mean zero, covariance from \verb{$OMEGAMAT}) and mapping to the individual-parameter scale using typical values  and the \code{DISTRIBUTION} column: \code{logNormal} uses \code{TV * exp(ETA)}, \code{normal} uses \code{TV + ETA}, \code{logitNormal} uses the inverse logit of \code{logit(TV) + ETA}. Parameters with missing or empty \code{DISTRIBUTION} do not get a theoretical curve.
}
\examples{
\donttest{
# Basic usage: distribution plots for all parameters
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
sg_gof_par_dist(fpath_i = fpath_i)

# Distribution plots for selected parameters (natural scale)
sg_gof_par_dist(fpath_i = fpath_i, par_seq = c("ka", "Cl"))

# Distribution plots with theoretical densities disabled
sg_gof_par_dist(fpath_i = fpath_i, tdist = FALSE)

# Distribution plots for ETA
sg_gof_par_dist(fpath_i = fpath_i, par_seq = c("eta_ka", "eta_Cl"), par_type = "RE")

# Q-Q plots for all ETA parameters
sg_gof_par_dist(fpath_i = fpath_i, plot_type = "QQ")

# Q-Q plot for a specific ETA parameter
sg_gof_par_dist(fpath_i = fpath_i, plot_type = "QQ", par_seq = "eta_ka")

# Correlation matrix for selected parameters
sg_gof_par_dist(fpath_i = fpath_i, plot_type = "correlations")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-sim.R
\name{sg_sim}
\alias{sg_sim}
\title{Perform simulations from an rxode2 model}
\usage{
sg_sim(
  model,
  et,
  stimes = NULL,
  outputs = NULL,
  theta = NULL,
  omega = NULL,
  sigma = NULL,
  thetamat = NULL,
  covs = NULL,
  npop = 1,
  nsub = 1,
  aggr = NULL,
  addcov = TRUE,
  keep = NULL,
  scale = NULL,
  covint = "locf",
  inits = NULL,
  byID = NULL,
  byPOP = NULL,
  shared = NULL,
  ncores = 1,
  atol = 1e-08,
  rtol = 1e-06,
  ...
)
}
\arguments{
\item{model}{A model object passed to \code{sg_sim()}.}

\item{et}{An event table passed to \code{sg_sim()}.}

\item{stimes}{A numeric vector of simulation times.}

\item{outputs}{Vector of strings. Names of the model variables to output.
If \code{NULL}, all variables returned. Default is \code{NULL}.}

\item{theta}{A named numeric vector of baseline parameters. Default is \code{NULL}.
Parameters listed in \code{params} are replaced by sampled values.}

\item{omega}{A named matrix or vector.}

\item{sigma}{A named matrix representing a sigma covariance matrix or its
Cholesky decomposition; Default is \code{NULL}.}

\item{thetamat}{A named variance-covariance matrix (for parameters brought to normal distribution).
Default is \code{NULL}.}

\item{covs}{A list of covariate structures. Each element should be a list containing covariate relationship definitions with the following structure:
\itemize{
\item \code{PAR} - string, name of the parameter to which covariate effect is applied
\item \code{COVNAME} - string, name of the covariate column in the dataset
\item \code{FUNC} - string, functional form of the covariate effect
\item \code{TRANS} - string, transformation applied to covariate
\item \code{REF} - numeric, reference value for categorical covariates
\item \code{INIT} - numeric, initial value for the covariate effect parameter
\item \code{EST} - logical, whether to estimate the covariate effect
Can be \code{NULL}, if no covariates are used. Default is \code{NULL}
}}

\item{npop}{Integer. Number of population replicates. Default is 1.}

\item{nsub}{Integer. Number of subjects sampled per population (omega/sigma matrices per ID). Default is 1.}

\item{aggr}{A string that specifies the aggregation type. Set to \code{NULL} for
no aggregation, \code{ID} for aggregation by time, population and ID, \code{NPOP} for
aggregation by population and time, \code{TIME} for aggregation by time.
Default is \code{NULL}.}

\item{addcov}{Logical. If \code{TRUE}, columns with covariate values will be added
to the resulting dataset. Default is \code{TRUE}.}

\item{keep}{Vector of strings. Columns of event table to keep in the output data frame. Default is \code{NULL}.}

\item{scale}{A numeric named vector. Scaling for ODE parameters of the system.
The names must correspond to the parameter identifiers in the ODE specification.
Each of the ODE variables will be divided by the scaling factor. Default is \code{NULL}.}

\item{covint}{String. Specifies the interpolation method for time-varying
covariates. When solving ODEs it often samples times outside the sampling
time specified in event table. When this happens, the time varying
covariates are interpolated. Currently this can be:
\itemize{
\item \code{"linear"} interpolation, which interpolates the covariate by solving the
line between the observed covariates and extrapolating the new covariate value
\item \code{"locf"} last observation carried forward
\item \code{"NOCB"} next observation carried backward.
This is the same method that NONMEM uses
\item \code{"midpoint"} last observation carried forward to midpoint;
next observation carried backward to midpoint
}

Default is \code{"locf"}}

\item{inits}{A named vector specifying initial conditions for model variables; Default is \code{NULL}.}

\item{byID}{logical. If \code{TRUE}, replicate populations/subjects per event-table ID.
Default \code{NULL} is treated as \code{TRUE}.}

\item{byPOP}{logical. When both thetamat and omega are used: if \code{TRUE},
replicate subjects by population. Default \code{NULL} is treated as \code{TRUE}.}

\item{shared}{logical. If \code{TRUE},
one shared population draw for all IDs. Default \code{NULL} is treated as \code{TRUE}.}

\item{ncores}{Integer. Number of cores used for calculations. Default is 1.}

\item{atol}{A numeric absolute tolerance used by the ODE solver to determine
if a good solution has been achieved. This is also used in the solved linear
model to check if prior doses do no add anything to the solution.
Default is 1e-8.}

\item{rtol}{Numeric. A numeric relative tolerance used by the ODE solver to determine
if a good solution has been achieved. This is also used in the solved linear
model to check if prior doses do not add anything to the solution. Default is 1e-6.}

\item{...}{optional arguments passed to \code{rxode2::rxSolve} (e.g. \code{method}, \code{maxsteps}).}
}
\value{
A dataset with simulation results (long format).
}
\description{
Runs simulations using an rxode2 model object with explicit R objects for
parameters (theta), variance-covariance (thetamat), omega, and sigma.
}
\details{
Index columns \code{ID}, \code{POPN}, and \code{sim.id} are included depending on what is used:
\describe{
\item{No uncertainty, no variability}{Only \code{ID} (and \code{TIME}, \code{VAR}, \code{VALUE})}
\item{Only uncertainty (\code{thetamat})}{ \code{POPN} and \code{ID}}
\item{Only variability (\code{omega})}{\code{sim.id} and \code{ID}}
\item{Uncertainty and variability}{\code{POPN}, \code{sim.id}, and \code{ID}}
}
Other columns:
\describe{
\item{TIME}{Timepoint of the simulations}
\item{VAR}{Names of the output variables}
\item{VALUE}{Optional. Simulated value. Returned when no aggregation applied}
\item{mean, median, ...}{Optional. Aggregated statistic of simulated values. Returned when aggregation is applied}
\item{COV1, ...COVn}{Optional. Covariates value. Returned when \code{addcov} is \code{TRUE}}
\item{KEEP1, ...KEEPn}{Optional. Columns that were specified in the \code{keep} argument}
}
Event table can use either \code{id} or \code{ID}. Parameters can be fixed per ID by
including parameter names (e.g. \code{*_pop}, \code{omega_*}) as columns in the event table.
}
\examples{
\donttest{
library(rxode2)
library(dplyr)
library(tibble)
# Base 1-compartment PK model (no covariate)
mod <- rxode2::rxode2({
  ka_pop = 0.1;
  Vd_pop = 10;
  CL_pop = 0.5;

  omega_ka = 0;
  omega_Vd = 0;
  omega_CL = 0;

  Cc_b = 0;
  ka_tv = exp(ka_pop);
  Vd_tv = exp(Vd_pop);
  CL_tv = exp(CL_pop);

  ka = ka_tv * exp(omega_ka);
  Vd = Vd_tv * exp(omega_Vd);
  CL = CL_tv * exp(omega_CL);

  Cc = Ac / Vd;

  Ad(0) = 0;
  Ac(0) = 0;

  d/dt(Ad) = -ka * Ad;
  d/dt(Ac) = ka * Ad - CL * Cc;

  Cc_ResErr = Cc * (1 + Cc_b);
})

# Model with covariate: WTBL on Vd (Vd_tv = exp(Vd_pop) * (WTBL/70)^beta_WTBL_Vd_pop)

mod_cov <- rxode2::rxode2({
  ka_pop = 0.1;
  Vd_pop = 10;
  CL_pop = 0.5;

  beta_WTBL_Vd_pop = 0;

  omega_ka = 0;
  omega_Vd = 0;
  omega_CL = 0;

  Cc_b = 0;

  ka_tv = exp(ka_pop);
  Vd_tv = exp(Vd_pop) * (WTBL/70)^beta_WTBL_Vd_pop;
  CL_tv = exp(CL_pop);

  ka = ka_tv * exp(omega_ka);
  Vd = Vd_tv * exp(omega_Vd);
  CL = CL_tv * exp(omega_CL);

  Cc = Ac / Vd;

  Ad(0) = 0;
  Ac(0) = 0;

  d/dt(Ad) = -ka * Ad;
  d/dt(Ac) = ka * Ad - CL * Cc;

  Cc_ResErr = Cc * (1 + Cc_b);
})

et_test <- tibble::tribble(
  ~ID, ~TIME, ~EVID, ~CMT, ~AMT,
  1, 0, 1, 1, 10,
  2, 0, 1, 1, 50
)

stimes_test <- seq(0, 24, 0.1)
output_test <- c("Ac", "Ad", "Cc", "Cc_ResErr")
theta_test <- c(ka_pop = log(0.1), Vd_pop = log(10), CL_pop = log(0.5))

thetamat_test <- matrix(c(0.05, 0.01, 0, 0.01, 0.05, 0, 0, 0, 0.05), nrow = 3, byrow = TRUE)
rownames(thetamat_test) <- colnames(thetamat_test) <- c("ka_pop", "Vd_pop", "CL_pop")

omega_test <- matrix(c(0.2, 0.05, 0, 0.05, 0.2, 0, 0, 0, 0.2), nrow = 3, byrow = TRUE)
rownames(omega_test) <- colnames(omega_test) <- c("omega_ka", "omega_Vd", "omega_CL")

sigma_test <- matrix(0.1); rownames(sigma_test) <- colnames(sigma_test) <- "Cc_b"

# No variability
sim1 <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
               outputs = output_test)

# Population uncertainty, single scenario (ID)
sim2a <- sg_sim(model = mod, et = dplyr::filter(et_test, ID == 1),
                stimes = stimes_test, theta = theta_test, thetamat = thetamat_test,
                npop = 10)

# Population uncertainty, multiple IDs, byID = TRUE (populations are replicated
# for each scenario (ID)),
# shared = TRUE (the same populations are used for each scenario (ID))
sim2b <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
               thetamat = thetamat_test, npop = 10, byID = TRUE, shared = TRUE)

# Population uncertainty, multiple IDs, byID = FALSE (one solve over full event table;
# npop must equal n(ID); ID - virtual subject)
sim2c <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
               thetamat = thetamat_test, npop = nrow(et_test), byID = FALSE)


# Between-subject variability (BSV), multiple IDs, byID = TRUE (subjects are
# replicated for each scenario (ID)), shared = FALSE (separate subjects per ID)
sim3a <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
                omega = omega_test, nsub = 10, byID = TRUE, shared = FALSE)


# Between-subject variability (BSV), byID = FALSE (one solve over full event table;
# nsub must equal n(ID); ID - virtual subject)
sim3b <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
                omega = omega_test, nsub = nrow(et_test), byID = FALSE)


# BSV + uncertainty: byID = TRUE (populations/subjects are replicated for each
# scenario (ID)), byPOP = TRUE (subjects are replicated for each population),
# shared = FALSE (separate populations/subjects per ID)
sim4a <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
                thetamat = thetamat_test, npop = 10, omega = omega_test, nsub = 5,
                byID = TRUE, byPOP = TRUE, shared = FALSE)

# BSV + uncertainty: byID = TRUE (populations/subjects are replicated for each
# scenario (ID)), byPOP = FALSE (subjects are not replicated between population;
# npop = nsub)
sim4b <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
                thetamat = thetamat_test, npop = nrow(et_test), omega = omega_test,
                nsub = nrow(et_test), byID = TRUE, byPOP = FALSE)

# BSV + uncertainty: byID = FALSE (one solve over full event table; nsub and/or
# npop must equal n(ID); ID - virtual subject), byPOP = TRUE (subjects are
# replicated for each population), shared = FALSE (separate populations/subjects
# per ID)
sim4c <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
                thetamat = thetamat_test, npop = 10, omega = omega_test, nsub = 5,
                byID = FALSE, byPOP = TRUE, shared = FALSE)

# BSV + uncertainty: byID = FALSE (one solve over full event table; nsub and/or
# npop must equal n(ID); ID - virtual subject), byPOP = FALSE (subjects are not
# replicated between population; npop = nsub)
sim4d <- sg_sim(model = mod, et = et_test, stimes = stimes_test, theta = theta_test,
                thetamat = thetamat_test, npop = nrow(et_test), omega = omega_test,
                nsub = nrow(et_test), byID = FALSE, byPOP = FALSE)

# Parameter override in event table
sim4 <- sg_sim(model = mod, et = dplyr::filter(et_test, ID == 1) \%>\%
                                 dplyr::mutate(Vd_pop = 2),
               stimes = stimes_test, theta = theta_test, thetamat = thetamat_test,
               npop = 1)


# Covariate (WTBL): pass covs and keep so WTBL is used and returned
theta_cov <- c(theta_test, beta_WTBL_Vd_pop = 0.5)
sim5 <- sg_sim(model = mod_cov, et = dplyr::filter(et_test, ID == 1) \%>\%
                                     dplyr::mutate(WTBL = 70),
               stimes = stimes_test, theta = theta_cov, covs = c("WTBL"),
               keep = c("WTBL"),
               thetamat = thetamat_test, npop = 5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-globalsens-sim.R
\name{sg_globalsens_sim}
\alias{sg_globalsens_sim}
\title{Global sensitivity analysis via PRCC or eFAST}
\usage{
sg_globalsens_sim(
  method = c("PRCC", "eFAST"),
  model,
  params,
  par_bounds,
  n_sim,
  stimes,
  output,
  stat_comp = c("mean", "min", "max"),
  et,
  theta = NULL,
  cov = NULL
)
}
\arguments{
\item{method}{Character string. \code{"PRCC"} or \code{"eFAST"}.}

\item{model}{A model object passed to \code{sg_sim()}.}

\item{params}{Character vector of parameter names to vary.}

\item{par_bounds}{A tibble or data frame with columns \code{PAR}, \code{LB}, \code{UB}.}

\item{n_sim}{Integer. Number of samples (LHS size for PRCC, base frequency size for eFAST).}

\item{stimes}{A numeric vector of simulation times.}

\item{output}{A character vector of outputs to keep. Passed to \code{sg_sim()}.}

\item{stat_comp}{A character vector of summary statistics to compute.
Supported internally: \verb{"mean","median","min","max","sd","cmax","SS"}.}

\item{et}{An event table passed to \code{sg_sim()}.}

\item{theta}{A named numeric vector of baseline parameters. Default is \code{NULL}.
Parameters listed in \code{params} are replaced by sampled values.}

\item{cov}{Covariates passed to \code{sg_sim()}. Default is \code{NULL}.}
}
\value{
List with:
\describe{
\item{result}{Tibble containing sensitivity analysis results.
For \code{PRCC} method columns include: \code{PAR}, \code{VAR}, \code{STAT}, \code{estimate}, \code{p.value}.
For \code{eFAST} method columns include: \code{PAR}, \code{VAR}, \code{STAT}, \code{TYPE}, \code{VALUE}, \code{METHOD}.}
}

\item{summary}{
Tibble of scalar summaries used for sensitivity computation:
\code{sim.id}, \code{VAR}, \code{STAT}, \code{value}.
}

\item{design}{
Data.frame of sampled parameter values (real scale) with \code{sim.id}.
For PRCC this corresponds to LHS samples.
For eFAST this corresponds to the FAST design matrix.
}

\item{bounds}{
Parameter bounds used in the analysis.
}
}
\description{
Performs global sensitivity analysis using either PRCC (Partial Rank
Correlation Coefficients with LHS sampling) or eFAST (extended Fourier
Amplitude Sensitivity Test). For each sampled parameter set, the function
runs simulations via \code{sg_sim()}, reduces trajectories to scalar summary
statistics, and computes sensitivity indices for each output–statistic pair.
}
\examples{
\donttest{
# Example model
library(rxode2)
library(tibble)
source(system.file("extdata", "RxODE_model", "example_rxode_model.R", package = "SimuRg")) # mod_ex


# Set up event table
et_base <- tribble(
  ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
  1,   0,     1,     1,    10,   2,     24,  112,   1
)

# Define parameter bounds
inits <- rxInits(mod_ex)
par_bounds <- tibble::tibble(
  PAR = c("POPCL", "POPVC"),
  LB  = inits[c("POPCL", "POPVC")] * (1 - 0.9),
  UB  = inits[c("POPCL", "POPVC")] * (1 + 0.9)
)

# Run eFAST sensitivity analysis
res_efast <- sg_globalsens_sim(
  method = c("eFAST"),
  model = mod_ex,
  params = c("POPCL"),
  par_bounds = par_bounds,
  n_sim = 100,
  stimes = seq(0, 168, 10),
  output = "Cc",
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean")
)

# Run PRCC sensitivity analysis
res_prcc <- sg_globalsens_sim(
  method = c("PRCC"),
  model = mod_ex,
  params = c("POPCL", "POPVC"),
  par_bounds = par_bounds,
  n_sim = 100,
  stimes = seq(0, 168, 10),
  output = "Cc",
  cov = c("IGFR", "POPN"),
  et = et_base,
  stat_comp = c("mean")
)
}



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{read_smrg_obj}
\alias{read_smrg_obj}
\title{Read or validate a SimuRg fit object}
\usage{
read_smrg_obj(fpath_i)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}
}
\value{
List. A SimuRg fit object with all table components coerced to
data frames.
}
\description{
Loads a SimuRg fit object from an \code{.RData} or \code{.json} file, or accepts an
already-loaded list and ensures all table components (\code{SDTAB}, \code{EVTAB},
\code{COTAB}, \code{CATAB}, \code{SUMTAB}) are proper data frames.
}
\examples{
# Pass an already-loaded sg-fit object
obj <- read_smrg_obj(gfo4cov)
class(obj$SDTAB)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-sim-tp.R
\name{sg_sim_tp}
\alias{sg_sim_tp}
\title{Time-profile plotting with optional summary bands}
\usage{
sg_sim_tp(
  ds_i,
  time_col = "TIME",
  val_col = "VALUE",
  group_i = "VAR",
  bands_i = NULL,
  cent_i = NULL,
  vrns_i = NULL,
  lperc_i = NULL,
  uperc_i = NULL,
  add_points = 0,
  col_i = NULL,
  fill_i = NULL,
  lty_i = NULL,
  shp_i = NULL,
  grid_i = NULL,
  wrap_i = NULL,
  free_stat = "free",
  wrap_ncol = NULL,
  wrap_nrow = NULL,
  min_x = NA,
  max_x = NA,
  min_y = NA,
  max_y = NA,
  log_y = FALSE,
  log_x = FALSE
)
}
\arguments{
\item{ds_i}{Data.frame. The data frame with source data.}

\item{time_col}{String. The column to use as a time column. Currently, can be only \code{TIME}. Default is \code{TIME}.}

\item{val_col}{String. Name of value column. Default is \code{VALUE}.}

\item{group_i}{String. Primary grouping variable for lines. Default is \code{'VAR'}.}

\item{bands_i}{vector of characters. Character vector of length 2 with column names for custom ymin/ymax ribbons. Default is \code{NULL}}

\item{cent_i}{string. Central tendency measure. One of \code{'mean'}, \code{'median'}, \code{'geom_mean'}, or \code{NULL} for raw values. Default is \code{NULL}}

\item{vrns_i}{string. Variance band type. One of \code{'SD'}, \code{'SE'}, \code{'IQR'}, or \code{NULL}. Default is \code{NULL}}

\item{lperc_i}{numeric. Lower percentile for percentile ribbons (must be provided together with \code{uperc_i}). Default is \code{NULL}}

\item{uperc_i}{numeric. Upper percentile for percentile ribbons (must be provided together with \code{lperc_i}). Default is \code{NULL}}

\item{add_points}{numeric. Size of points to add to the plot. If > 0, points will be added. Default is \code{0}}

\item{col_i}{String. Column name for color}

\item{fill_i}{String. Column name for fill aesthetic. Default is \code{NULL}}

\item{lty_i}{String. Column name for linetype aesthetic. Default is \code{NULL}.}

\item{shp_i}{String. Column name for shape aesthetic. Default is \code{NULL}.}

\item{grid_i}{string. Faceting formula for \code{facet_grid} (e.g., \code{'~VAR'} or \code{'A~B'}). Default is \code{NULL}}

\item{wrap_i}{String. Faceting formula for \code{facet_wrap}. Default is \code{NULL}.}

\item{free_stat}{String. Facet scaling option. One of \code{"free"}, \code{"free_x"},
\code{"free_y"}, or \code{"fixed"}. Default is \code{'free'}.}

\item{wrap_ncol}{Integer. Number of columns for \code{facet_wrap}. Default is \code{NULL}.}

\item{wrap_nrow}{Integer. Number of rows for \code{facet_wrap}. Default is \code{NULL}.}

\item{min_x}{Numeric. X-axis minimum limit. Default is \code{NA}.}

\item{max_x}{Numeric. X-axis maximum limit. Default is \code{NA}.}

\item{min_y}{Numeric. Y-axis minimum limit. Default is \code{NA}.}

\item{max_y}{Numeric. Y-axis maximum limit. Default is \code{NA}.}

\item{log_y}{Logical. If \code{TRUE}, a logarithmic scale is applied to y-axis. Default is \code{FALSE}.}

\item{log_x}{Logical. If \code{TRUE}, a logarithmic scale is applied to x-axis. Default is \code{FALSE}.}
}
\value{
A ggplot object with lines and optional ribbons/points/facets.
}
\description{
Creates a ggplot time-profile plot from longitudinal data with optional
summarization (mean/median/geometric mean), variance bands (SD/SE/IQR),
percentile ribbons, faceting, and log scaling.
}
\examples{
make_extended_mock_data <- function() {
 data.frame(
   TIME = rep(1:4, times = 6),
   VALUE = rnorm(24, mean = 10, sd = 2),
   VAR = rep(rep(c("A", "B"), each = 4), times = 3),
   Regimen = rep(c("R1", "R2", "R3"), each = 8),
   CAT1 = rep(c("Cat1", "Cat2"), each = 12)
 )
}
ds_sim <- make_extended_mock_data()
p <- sg_sim_tp(ds_i = ds_sim, group_i = 'VAR', col_i = 'VAR', fill_i = 'VAR',
               wrap_i = '~VAR', wrap_ncol = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-localsens-vis.R
\name{sg_localsens_vis}
\alias{sg_localsens_vis}
\title{Visualize Local Sensitivity Analysis Results}
\usage{
sg_localsens_vis(
  sens_data,
  tornado_time = NULL,
  metrics = "value",
  ref_data = NULL,
  log_scale = FALSE,
  color_low = "#1f77b4",
  color_high = "#ff7f0e",
  facet_scales = "free"
)
}
\arguments{
\item{sens_data}{A data frame containing sensitivity analysis results from sg_localsens_sim()}

\item{tornado_time}{Numeric value specifying the time point (in the same units as \code{TIME})
at which to evaluate the model output for the tornado plot. Defaults to the maximum time in \code{sens_data} if not specified.}

\item{metrics}{Character vector specifying which metrics to include in the tornado plot.
Supported options are \code{"value"} (value at \code{tornado_time}) and \code{"cmax"} (maximum output). Defaults to \code{"value"}.}

\item{ref_data}{Optional data frame providing reference (baseline) output values,
formatted similarly to \code{sens_data}. If \code{NULL}, the function uses the mid-range parameter value as the baseline.}

\item{log_scale}{Logical. If \code{TRUE}, the y-axis of the family-of-curves plot is displayed on a log scale.}

\item{color_low}{Character. Color used for the lower parameter bound (default: "#1f77b4").}

\item{color_high}{Character. Color used for the upper parameter bound (default: "#ff7f0e").}

\item{facet_scales}{Character string passed to \code{facet_grid()}, defining how y-axis scales behave across facets (default = \code{"free"}).}
}
\value{
A named list containing:
\describe{
\item{\code{family_of_curves}}{A \code{ggplot2} object showing parameter sensitivity curves over time.}
\item{\code{tornado}}{A \code{ggplot2} object showing the tornado plot of relative sensitivity.}
}
}
\description{
This function creates plots to visualize the results of a local sensitivity analysis:
(1) a family of curves plot showing how model outputs vary over time for different parameter perturbations,
and (2) a tornado plot summarizing the relative influence of each parameter on selected summary metrics
(e.g., output value at a chosen time or Cmax).
}
\examples{
\donttest{
library(rxode2)
library(tibble)
mod_ex <- RxODE({
  # Doses in mg
  # Time in hours
  ### Parameter values
  # Typical
  POPCL = 5;
  POPVC = 180;
  POPQ = 7;
  POPVP = 52;
  POPKTR = 6;
  FBIOPAR = 1;
  # Covariate coefficients
  POPIGFRCOV = 0.8;
  POPPATCOVCLGR1 = 0.85;
  POPPATCOVCLGR2 = 0.85;
  POPPATCOVCLGR3 = 0.85;
  POPPATCOVCLGR4 = 0.85;
  # Random effects
  PPVCL = 0;
  PPVVC = 0;
  PPVKTR = 0;
  # Residual error
  RUV = 0;
  ### Covariates
  PATCOVCL = 1
  if(POPN == 1){PATCOVCL = POPPATCOVCLGR1}
  if(POPN == 2){PATCOVCL = POPPATCOVCLGR2}
  if(POPN == 3){PATCOVCL = POPPATCOVCLGR3}
  if(POPN == 4){PATCOVCL = POPPATCOVCLGR4}
  IGFRCOV = (IGFR/112)^POPIGFRCOV
  ## Parameters
  CL = POPCL * IGFRCOV * PATCOVCL * exp(PPVCL);
  VC = POPVC * exp(PPVVC);
  Q = POPQ;
  VP = POPVP;
  KTR = POPKTR * exp(PPVKTR);
  ### Explicit functions
  Cc = Ac/VC;                 # nmol/L
  Cp = Ap/VP;                 # nmol/L
  ### Initial conditions
  At1(0) = 0;         # mg
  At2(0) = 0;         # mg
  At3(0) = 0;         # mg
  At4(0) = 0;         # mg
  At5(0) = 0;         # mg
  At6(0) = 0;         # mg
  Ad(0) = 0;          # mg
  Ac(0) = 0;          # mg
  Ap(0) = 0;          # mg
  ### ODEs
  d/dt(At1) = - KTR*At1;
  d/dt(At2) = KTR*At1 - KTR*At2;
  d/dt(At3) = KTR*At2 - KTR*At3;
  d/dt(At4) = KTR*At3 - KTR*At4;
  d/dt(At5) = KTR*At4 - KTR*At5;
  d/dt(At6) = KTR*At5 - KTR*At6;
  d/dt(Ad) = KTR*At6 - KTR*Ad;
  d/dt(Ac) = KTR*Ad - CL*Cc - Q*(Cc - Cp);
  d/dt(Ap) = Q*(Cc - Cp);
  FBIO = FBIOPAR
  f(At1) = FBIO*1000000/505;
  CHECKRUV = RUV;
  Cc_ResErr = Cc + RUV*Cc;
})
et_base <- tribble(
  ~id, ~time, ~evid, ~cmt, ~amt, ~addl, ~ii, ~IGFR, ~POPN,
  1,   0,     1,     1,    10,   2,     24,  112,   1
)
sens_loc <- sg_localsens_sim(
  model = mod_ex,
  params = c("POPCL", "POPVC"),
  stimes = seq(0, 168, 0.1),
  output = "Cc",
  perc = 0.9,
  cov = c("IGFR", "POPN"),
  et = et_base
)
plots <- sg_localsens_vis(
  sens_data = sens_loc,
  tornado_time = 168,
  metrics = c("value", "cmax"),
  log_scale = TRUE,
  color_low = "#4575b4",
  color_high = "#d73027",
  facet_scales = "free"
)

# Display plots
print(plots$family_of_curves)
print(plots$tornado)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-gof-tp.R
\name{sg_gof_tp}
\alias{sg_gof_tp}
\title{Plot time profiles of the fitted data}
\usage{
sg_gof_tp(
  fpath_i,
  pop = TRUE,
  filt = "T",
  cov_cols = NULL,
  col_i = NULL,
  DVID = 1,
  tsld = FALSE,
  f_scales = "free",
  sort_by = NULL,
  desc = FALSE,
  log_y = FALSE,
  lab_x = "Time since first dose, h",
  lab_y = "Plasma concentration, mmol/L",
  cap = str_c("empty circles - observed data\\n", "solid lines with point - ",
    "individual predictions\\n", "dashed grey lines with ",
    "point - population predictions"),
  n_quantiles = 3,
  levels_discrete = 10
)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{pop}{Logical. If \code{TRUE} (default), adds population predictions (PRED) as dashed lines and points.}

\item{filt}{String. Provide a filter to apply. Default is \code{"T"}}

\item{cov_cols}{A character vector specifying the names of the columns with covariates.}

\item{col_i}{String. Column name for color}

\item{DVID}{Restrict \code{SDTAB} to one observation type. Numeric values
select the \code{DVID} column (default \code{1}). If \code{SDTAB} has
\code{DVNAME}, a character or factor is matched to \code{DVNAME} first;
otherwise a digit-only string is coerced to numeric \code{DVID}.}

\item{tsld}{Logical. If \code{TRUE}, uses time since last dose instead of time from first dose. Default is \code{FALSE}.}

\item{f_scales}{String, one of \code{"fixed"}, \code{"free"}, \code{"free_x"}, \code{"free_y"}. User can specify whether the scales (x and y axes) should be fixed across all panels (\code{"fixed"}), free for each panel (\code{"free"}), or free only in one dimension (\code{"free_x"} or \code{"free_y"}). Default is \code{"fixed"}}

\item{sort_by}{Character vector of column names to sort ID facets by.
Use "DOSE" to sort by last dose. Other names (e.g. covariates) must exist in SimuRg object.}

\item{desc}{Logical. If TRUE, sort in descending order for all columns in \code{sort_by}.}

\item{log_y}{Logical. If \code{TRUE}, a logarithmic scale is applied to y-axis. Default is \code{FALSE}.}

\item{lab_x}{String. X-ax label. Default is "Time since first dose, h"}

\item{lab_y}{String. Y-ax label. Default is "Plasma concentration, mmol/L"}

\item{cap}{String. Plot caption. Default is "empty circles - observed data solid lines with point - individual predictions dashed grey lines with point - population predictions"}

\item{n_quantiles}{Integer. Number of quantile groups for continuous variables in \code{col_i}. Default is 3.}

\item{levels_discrete}{Integer. Maximum unique values to consider a variable discrete. Default is 10.}
}
\value{
A list of plots with predicted time profiles, faceted by id
}
\description{
Plot time profiles of the fitted data
}
\examples{
\donttest{
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData",
                        package = "SimuRg")
plot_list <- sg_gof_tp(fpath_i)
print(plot_list[[1]])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-modcomp.R
\name{sg_modcomp}
\alias{sg_modcomp}
\title{Returns dataframe with summary statistics for model comparison}
\usage{
sg_modcomp(fpath_i, run_id = 1)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{run_id}{Integer. Tested model ID. Default is 1.}
}
\value{
A data.frame with model summary metrics for futher comparison
}
\description{
Returns dataframe with summary statistics for model comparison
}
\examples{
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData", package = "SimuRg")
sum_tab <- sg_modcomp(fpath_i, 3)
print(sum_tab)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-multistart.R
\name{sg_multistart}
\alias{sg_multistart}
\title{Build multistart scenarios with varying initial values}
\usage{
sg_multistart(
  mod,
  data,
  headers,
  ruv,
  theta,
  re,
  occ,
  path,
  covs_lst = NULL,
  opt_name = "Simurg",
  project_name = "multistart_project",
  n_starts = 10,
  theta_intervals = NULL,
  interval_factor = c(0.2, 5),
  vary_params = NULL,
  seed = NULL
)
}
\arguments{
\item{mod}{Model file path or model identifier used for scenario generation.}

\item{data}{Character string. Path to the dataset used in model fitting or simulation.}

\item{headers}{Predefined list of dataset column names (e.g., ID, TIME, DV, etc.) used by the modeling framework.}

\item{ruv}{Residual unexplained variability (RUV) structure.
Each element contains RUV properties (e.g., \code{YNAME}, \code{ERR}) and mapping to data.}

\item{theta}{Tibble describing population parameter properties.
Tibble should contain columns such as \code{NAME}, \code{INIT}, and \code{EST}.}

\item{re}{Random effects (RE) specification.
Includes matrices \code{init} and \code{est} defining initial and estimated covariance structures.}

\item{occ}{Occasion effect matrices.
Includes \code{init} and \code{est} matrices defining inter-occasion variability.}

\item{path}{Character. Directory path where output files (CSV summary and model files) will be written. Default is current working directory.}

\item{covs_lst}{Optional list describing covariate-parameter relationships.
Each element should include fields such as \code{PAR}, \code{COVNAME}, and \code{EST} indicating inclusion in the model.}

\item{opt_name}{Character. Optimization engine name (e.g., \code{"Monolix"}, \code{"Simurg"}). Default \code{"Simurg"}.}

\item{project_name}{Character. Base name for the exported project files. Default \code{"my_project"}.}

\item{n_starts}{Number of multistart runs (number of initial value scenarios).}

\item{theta_intervals}{Optional data frame specifying sampling
intervals for initial values. Must contain columns
\code{NAME}, \code{lower}, and \code{upper}.
If \code{NULL}, intervals are derived from \code{interval_factor}.}

\item{interval_factor}{Numeric vector of length 2 specifying
multiplicative lower and upper bounds factors relative to original
\code{INIT} values (default \code{c(0.2, 5)}).}

\item{vary_params}{List of parameters which inital values are varied.
Default value implies all parameters are varied. If \code{theta_interval} is specified,
list of parameters is taken from \code{theta_interval}}

\item{seed}{Optional random seed for reproducible sampling of
initial values. If \code{NULL}, the global RNG state is not modified.}
}
\value{
The function writes two types of outputs to the specified \code{path}:
\itemize{
\item A CSV file (\code{scenarios_info.csv}) summarizing all generated scenarios with columns:
\describe{
\item{scenario_number}{Unique index of the scenario.}
\item{model}{Model file or path used in the scenario.}
\item{data}{Path to the dataset used.}
\item{theta}{Active population parameters and initial values used.}
\item{RUV}{Residual error structure(s) used.}
\item{RE}{Random effect and correlation terms included.}
\item{OCC}{Active occasion effects.}
\item{COVS}{Included covariate-parameter relationships.}
}
\item Individual \code{.mlxtran} model files for each generated scenario,
named according to the pattern \verb{<project_name>_<i>.mlxtran}.
}
}
\description{
Generates multiple scenarios with identical model structure
(model, RUV, RE, OCC, COVS) and different sampled initial values for
population parameters.

For each start, initial values of estimated parameters are sampled
uniformly within specified intervals, and a corresponding \code{.mlxtran} file
is generated. A summary table describing all multistart scenarios is written
to disk.
}
\details{
For each start:
\itemize{
\item Population parameter initial values \code{INIT} for
estimated parameters are sampled uniformly.
\item A scenario summary is stored.
\item The corresponding \code{.mlxtran} project files are written.
}

If \code{seed} is provided, the function temporarily sets the
random number generator state and restores it upon exit.
}
\examples{
\donttest{
library(dplyr)
folder_path <- system.file("extdata", package = "SimuRg")

mod <- paste(folder_path, "/models/model_PK_1c.txt", sep = "/")

data <- paste(folder_path, "datasets", "dspk-warf.csv", sep = "/")
re <- list(init = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                                    1, 0, 0, 0, 0,
                                    0, 1, 0, 0, 0,
                                    0, 0, 1, 0, 0,
                                    0, 0, 0, 1, 0,
                                    0, 0, 0, 0, 1) \%>\% as.matrix(),
                    est = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                                    TRUE, NA, NA, NA, NA,
                                    NA, TRUE, NA, NA, NA,
                                    NA, NA, TRUE, NA, NA,
                                    NA, NA, NA, TRUE, NA,
                                    NA, NA, NA, NA, TRUE) \%>\% as.matrix(),
                    block = tribble(~Cl, ~Vd, ~ka, ~Vp, ~Q,
                                      FALSE, NA, NA, NA, NA,
                                      NA, FALSE, NA, NA, NA,
                                      NA, NA, TRUE, NA, NA,
                                      NA, NA, NA, FALSE, NA,
                                      NA, NA, NA, NA, FALSE) \%>\% as.matrix())

headers <- list(list(name = "ID", use = "identifier", type = NULL),
               list(name = "TIME", use = "time", type = NULL),
               list(name = "DV", use = "observation", type = "continuous"),
               list(name = "DVID", use = "observationtype", type = NULL),
               list(name = "ADM", use = "administration", type = NULL),
               list(name = "AMT", use = "amount", type = NULL),
               list(name = "EVID", use = "eventidentifier", type = NULL),
               list(name = "MDV", use = "missingdependentvariable", type = NULL),
               list(name = "AGE", use = "covariate", type = "continuous"),
               list(name = "AGE_centered", use = "covariate", type = "continuous"),
               list(name = "SEX", use = "covariate", type = "categorical"),
               list(name = "WEIGHT", use = "covariate", type = "continuous"),
               list(name = "BMI", use = "covariate", type = "continuous"))

theta <- tribble(~NAME, ~TRANS, ~INIT, ~LB, ~UB, ~EST,
                   "Cl", "logNormal", 0.2, NA, NA, TRUE,
                   "Vd", "logNormal", 20, NA, NA, TRUE,
                   "ka", "logNormal", 0.2, NA, NA, TRUE)

theta_intervals <- list(
  Cl = c(0.25*theta$INIT[theta$NAME == "Cl"], 4*theta$INIT[theta$NAME == "Cl"]),
  #Vd = c(0.5*theta$INIT[theta$NAME == "Vd"], 2*theta$INIT[theta$NAME == "Vd"]),
  ka   = c(1.25*theta$INIT[theta$NAME == "ka"], 1.5*theta$INIT[theta$NAME == "ka"])
)

ruv <- list(YNAME = "y1",
            DVID = 1,
            TRANS = "normal",
            PRED = "Cc",
            ERR = "proportional",
            INIT = 1,
            EST = TRUE)

n_starts <- 20

path <- tempdir()
sg_multistart(
  mod = mod,
  data = data,
  headers = headers,
  ruv = ruv,
  theta = theta,
  re = re,
  occ = re,
  n_starts = n_starts,
  theta_intervals = theta_intervals,
  path = path,
  project_name = "multistart_test_project"
)
clr_files <- list.files(path, full.names = TRUE)
unlink(clr_files, recursive = TRUE, force = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-gof-res-dist.R
\name{sg_gof_res_dist}
\alias{sg_gof_res_dist}
\title{Plot distribution or QQ-plot of residuals from NONMEM/Simurg run}
\usage{
sg_gof_res_dist(
  fpath_i,
  res_type = "RES",
  DVID = 1,
  n_bins = 30,
  ndist = TRUE,
  plot_type = "DIST"
)
}
\arguments{
\item{fpath_i}{String or sg-fit object. If the string is given, the path to
\code{.Rdata} or \code{.json} file with sg-fit object is expected.}

\item{res_type}{Character vector. One or several types of residuals to plot.
Values must correspond to column name(s) in \code{smrg_obj$SDTAB}, e.g.
\code{"RES"}, \code{"IWRES"}, \code{"IRES"}. If multiple residual types are provided,
plots will be generated for each of them.}

\item{DVID}{Restrict \code{SDTAB} to one observation type. Numeric values
select the \code{DVID} column (default \code{1}). If \code{SDTAB} has
\code{DVNAME}, a character or factor is matched to \code{DVNAME} first;
otherwise a digit-only string is coerced to numeric \code{DVID}.}

\item{n_bins}{Integer. Number of bins to use in the histogram. Default is 30.}

\item{ndist}{Logical. If \code{TRUE}, overlays the corresponding normal density
curve on the histogram (default: TRUE).}

\item{plot_type}{Character. Type of plot to produce:
\itemize{
\item \code{"DIST"} (default) - histogram of individual parameters,
\item \code{"QQ"} - QQ-plot of individual parameters.
}}
}
\value{
A \code{ggplot} object.
}
\description{
This function visualizes residuals from estimation results
stored in a Simurg output object. The residuals can be plotted either as
histograms with overlaid normal density curves, or as QQ-plots to assess
normality.
}
\examples{
# Plot the distribution of individual weighted residuals (IWRES)
# (default \code{DVID = 1}; pass \code{DVID} or a \code{DVNAME} string to select another endpoint)
fpath_i <- system.file("extdata", "simurg_object", "Warfarin_PK.RData",
                        package = "SimuRg")
sg_gof_res_dist(fpath_i = fpath_i, res_type = "IWRES")

# QQ-plots for the same default endpoint
sg_gof_res_dist(fpath_i = fpath_i, res_type = "RES", plot_type = "QQ")

# Several residual columns at once (still one \code{DVID} unless you change it)
sg_gof_res_dist(fpath_i = fpath_i, res_type = c("IWRES", "IRES"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sg-covsens-vis.R
\name{sg_covsens_vis}
\alias{sg_covsens_vis}
\title{Visualisation of covariate sensitivity analysis results}
\usage{
sg_covsens_vis(
  covsens_res,
  plot_type = c("PARSENS", "EXPSENS"),
  exclude_vars = NULL,
  ci_quantiles = c("P025", "P975"),
  ci_limits = c(0.8, 1.25),
  ci_band_alpha = 0.2,
  ci_band_col = "firebrick",
  ref_line_col = "grey25",
  palette = MSDcol[c(1, 3, 4, 5, 6, 7)],
  point_size = 2.5,
  errorbar_width = 0.2,
  lab_y = "Mean (95\% CI)\\nchange from reference",
  cap = NULL
)
}
\arguments{
\item{covsens_res}{Named list as returned by \code{sg_covsens_sim()}.  Must
contain the element selected by \code{type} (\code{PARSENS} and/or
\code{EXPSENS} data.frames with columns \code{LAB}, \code{VAR},
\code{mean}, \code{Type}, and the interval columns named by
\code{ci_quantiles}).}

\item{plot_type}{Character scalar: \code{"PARSENS"} (default) or \code{"EXPSENS"}.}

\item{exclude_vars}{Character vector. Contains \code{VAR} levels to omit (e.g.
\code{"Cc_Cmin"}).  \code{NULL} keeps all rows.}

\item{ci_quantiles}{Character vector of length 2: names of the lower and
upper uncertainty columns in the sensitivity table, in that order.
Defaults \code{c("P025", "P975")} to match the default percentiles in
\code{sg_covsens_sim}.  Use other names (e.g. \code{c("P05", "P95")}) if
you changed \code{quantiles} in the simulation and the columns exist.}

\item{ci_limits}{Numeric vector of length 2: lower and upper bounds of the
shaded acceptance band and of the dotted horizontal guides.  Default
\code{c(0.8, 1.25)} is a common bioequivalence-style window on the ratio
scale.}

\item{ci_band_alpha}{Numeric in \eqn{[0,1]}: transparency of the shaded band.
Default \code{0.2}.}

\item{ci_band_col}{Color for the band fill and dotted limit lines.
Default \code{"firebrick"}.}

\item{ref_line_col}{Color for the dashed horizontal line at \code{y = 1}
(no change from reference).  Default \code{"grey25"}.}

\item{palette}{Colors for the \code{Type} scale (continuous vs
categorical covariates, etc.).  Recycled if there are more levels than
colors.  Default \code{MSDcol[c(1, 3, 4, 5, 6, 7)]}.}

\item{point_size}{Point size for \code{geom_point}.  Default \code{2.5}.}

\item{errorbar_width}{Numeric. Width argument for \code{geom_errorbar}.  Default
\code{0.2}.}

\item{lab_y}{Axis label for the numeric scale (this becomes the horizontal
axis after \code{coord_flip()}).  Default mentions a 95\\% interval; change
if you use different \code{ci_quantiles}.}

\item{cap}{Optional figure caption, passed to
\code{ggplot2::labs(caption = ...)} (e.g. text describing reference
covariate values).  Default \code{NULL}.}
}
\value{
A \code{ggplot2} object (inactive until printed or saved).  You can
add further layers or themes with the usual \pkg{ggplot2} API.
}
\description{
Draw a forest-style graphic (points with uncertainty intervals) from the
sensitivity tables produced by \code{\link{sg_covsens_sim}}.  Each facet row
corresponds to a model output or exposure metric (\code{VAR}); each point is a
covariate scenario (\code{LAB}), coloured by covariate type (\code{Type}).
}
\details{
Two views are available, matching the named elements of the simulation
output:
\itemize{
\item \strong{\code{PARSENS}} — sensitivity of individual parameters
(simulation at time zero, no ODE time course).
\item \strong{\code{EXPSENS}} — sensitivity of exposure summaries
(e.g. Cmin, Cmax, Cavg) after full simulation over \code{stimes} in
\code{sg_covsens_sim}.
}

Values on the y-axis are ratios relative to the reference scenario: \code{1}
means no change.  In \code{sg_covsens_sim}, percent change relative to
reference is transformed to this scale before tabulation.  The shaded region
between \code{ci_limits} highlights a target interval; points whose
intervals lie largely inside can be read as scenarios consistent with that
criterion, subject to study-specific rules.

The plot layers are drawn in order: reference band, error bars, points,
reference line at 1, dotted lines at \code{ci_limits}, then faceting by
\code{VAR} and flipped coordinates so labels read along the vertical axis.
}
\examples{
\donttest{
library(tibble)
library(dplyr)
library(rxode2)
# Typical workflow: run the simulation (see examples in ?sg_covsens_sim),
# then visualise parameter and exposure sensitivity.

cont_cov_l <- list(
  LG_AGE = list(NAME = "LG_AGE", UTNAME = "AGE",
                REF = "median", NICENAME = "Age, years",
                par_vec = c("CL")),
  LG_WEIGHT = list(NAME = "LG_WEIGHT", UTNAME = "WEIGHT",
                REF = "median", NICENAME = "Weight, kg",
                par_vec = c("Vd"))
)

cat_cov_l <- list(
  SEX = list(NAME = "SEX", NICENAME = "Sex",
             REF = "0", par_vec = c("ka")),
  CYP2C9 = list(NAME = "CYP2C9", NICENAME = "CYP2C9 genotype",
                REF = NULL, par_vec = c("CL"))
)

# --- Dosing ---
ev_t_input <- tribble(
 ~id, ~time, ~ii, ~amt, ~addl, ~dur, ~evid, ~Regimen,        ~Dose,
  1,   0,     336, 10,   21,    0.5,  1,     "0.3 mg/kg Q2W", 0.3
)
# --- Model ---
model <- RxODE({
 # Doses in mg
  # Time in hours

 ### Parameter values
  # Typical values
 ka_pop = 0.073;
  Vd_pop = 14.8;
  CL_pop = 0.347;

  # Random effects
  omega_ka = 0;
  omega_Vd = 0;
  omega_CL = 0;

  # Covariate effect
  # Continuous
  beta_CL_LG_AGE = 0.49990114;
  beta_Vd_LG_WEIGHT = 0.60529433;

  # Categorical
  beta_CL_CYP2C9_1_2 = -0.339;
  beta_CL_CYP2C9_1_3 = -0.574;
  beta_CL_CYP2C9_2_2 = -1.079;
  beta_CL_CYP2C9_2_3 = -0.745;
  beta_CL_CYP2C9_3_3 = -2.13;

  beta_ka_SEX_1 = -0.12198035;

  # Residual error
  Cc_b = 0;

 # Transformations
  ka_tv = exp(ka_pop);
  Vd_tv = exp(Vd_pop);
  CL_tv = exp(CL_pop);

  CL_multiplier = 1.0;  # Default/reference
  ka_multiplier = 1.0;

  if (SEX == "1") {ka_multiplier = exp(beta_ka_SEX_1)}

  if (CYP2C9 == "1") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_2);
  } else if (CYP2C9 == "2") {
    CL_multiplier = exp(beta_CL_CYP2C9_1_3);
  } else if (CYP2C9 == "3") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_2);
  } else if (CYP2C9 == "4") {
    CL_multiplier = exp(beta_CL_CYP2C9_2_3);
  } else if (CYP2C9 == "5") {
    CL_multiplier = exp(beta_CL_CYP2C9_3_3);
  }

  ka = ka_tv*ka_multiplier*exp(omega_ka);
  Vd = Vd_tv*exp(beta_Vd_LG_WEIGHT * LG_WEIGHT + omega_Vd); #Vd_tv*exp(omega_Vd);
  CL = CL_tv*CL_multiplier*exp(beta_CL_LG_AGE * LG_AGE + omega_CL);


  ### Explicit functions
  Cc = Ac/Vd;

  ### Initial conditions
  Ad(0) = 0;
  Ac(0) = 0;

  ### Differential equations
  d/dt(Ad) = - ka*Ad;
  d/dt(Ac) = ka*Ad - CL*Cc;

  Cc_ResErr = Cc*(1 + Cc_b);
})

# --- Estimation covariance (mock) ---
pnames     <- parest$parameter
npar       <- length(pnames)
set.seed(1)
m_cov      <- matrix(0.02, npar, npar)
diag(m_cov) <- 0.05 + runif(npar, 0, 0.05)
m_cov      <- (m_cov + t(m_cov)) / 2
est_covmat <- as_tibble(cbind(X1 = pnames, as.data.frame(m_cov)))
names(est_covmat)[-1] <- pnames

# --- Simulation times (steady-state cycle 10) ---
ss_cycle <- 10
stimes_ss <- c(
  ss_cycle * 4 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1)),
  ss_cycle * 4 * 7 * 24 + 2 * 7 * 24 + c(seq(0, 23.5, 0.5), seq(24, 335, 1))
)
result <- sg_covsens_sim(
  fpath_i = NULL, ds_parest = parest, ds_covs = ds_covval,
  model = model, stimes = stimes_ss, et = ev_t_input,
  est_covmat = est_covmat, npop = 10,
  cont_cov_l = cont_cov_l, cat_cov_l = cat_cov_l,
  quantiles = c(0.1, 0.9), aggr = c("min", "max", "mean"),
  outputs = "Cc"
)

p_par <- sg_covsens_vis(result, plot_type = "PARSENS")
p_exp <- sg_covsens_vis(result, plot_type = "EXPSENS")
print(p_par)
print(p_exp)

# Alternate interval columns (must exist in the sensitivity tables)
sg_covsens_vis(result, ci_quantiles = c("P05", "P95"))

# Drop selected metrics from the exposure panel
sg_covsens_vis(result, plot_type = "EXPSENS", exclude_vars = c("Cc_Cmin"))
}

}
\seealso{
\code{\link{sg_covsens_sim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimuRg.R
\docType{package}
\name{SimuRg-package}
\alias{SimuRg}
\alias{SimuRg-package}
\title{SimuRg: Building, fitting and evaluating PK/PD Modeles}
\description{
SimuRg is an R toolkit for PK/PD modeling and simulation workflows, with a focus on importing, organizing, and exploring nonlinear mixed-effects model results.
}
\author{
\strong{Maintainer}: Anna Mikhailova \email{anna.mikhailova@msdecisions.tech}

Authors:
\itemize{
  \item Victor Sokolov \email{victor.sokolov@msdecisions.tech} [copyright holder]
  \item Yaroslav Ugolkov \email{yaroslav.ugolkov@msdecisions.tech}
  \item Anatoly Pokladyuk \email{anatoly.pokladyuk@msdecisions.tech}
  \item Alina Melnikova \email{alina.melnikova@msdecisions.tech}
  \item Victoria Kulesh \email{victoria.kulesh@msdecisions.tech}
}

}
\keyword{internal}
